@@ -125,7 +125,7 @@ time_t ModMArch::end()
 
 void ModMArch::put( vector<TMess::SRec> &mess )
 {
-    long long t_cnt = TSYS::curTime();
+    int64_t t_cnt = TSYS::curTime();
 
     ResAlloc res(mRes,false);
 
@@ -167,7 +167,7 @@ void ModMArch::put( vector<TMess::SRec> &mess )
 	    }
 	    catch(TError err)
 	    {
-		mess_crit(nodePath().c_str(),_("Error create new archive file <%s>!"),(addr()+f_name).c_str() );
+		mess_crit(nodePath().c_str(),_("Error create new archive file '%s'!"),(addr()+f_name).c_str() );
 		return;
 	    }
 	    //>> Allow parallel read access
@@ -178,18 +178,19 @@ void ModMArch::put( vector<TMess::SRec> &mess )
     tmCalc = TSYS::curTime()-t_cnt;
 }
 
-void ModMArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const string &category, char level )
+void ModMArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const string &category, char level, time_t upTo )
 {
-    if( e_tm <= b_tm ) return;
-    if( !run_st ) throw TError(nodePath().c_str(),_("Archive is not started!"));
+    if(e_tm <= b_tm) return;
+    if(!run_st) throw TError(nodePath().c_str(),_("Archive is not started!"));
+    if(!upTo) upTo = time(NULL)+STD_INTERF_TM;
 
     ResAlloc res(mRes,false);
-    for( int i_arh = arh_s.size()-1; i_arh >= 0 && mess.size() < TArchiveS::max_req_mess; i_arh-- )
+    for(int i_arh = arh_s.size()-1; i_arh >= 0 && time(NULL) < upTo; i_arh--)
     {
-	if( !arh_s[i_arh]->err() &&
-		!( (b_tm < arh_s[i_arh]->begin() && e_tm < arh_s[i_arh]->begin() ) ||
-		   (b_tm > arh_s[i_arh]->end() && e_tm > arh_s[i_arh]->end() ) ) )
-	    arh_s[i_arh]->get(b_tm, e_tm, mess, category, level);
+	if(!arh_s[i_arh]->err() &&
+		!((b_tm < arh_s[i_arh]->begin() && e_tm < arh_s[i_arh]->begin()) ||
+		   (b_tm > arh_s[i_arh]->end() && e_tm > arh_s[i_arh]->end())))
+	    arh_s[i_arh]->get(b_tm, e_tm, mess, category, level, upTo);
     }
 }
 
@@ -204,7 +205,7 @@ void ModMArch::checkArchivator( bool now )
 	if(IdDir == NULL)
 	{
 	    if( mkdir(addr().c_str(),0777) )
-		throw TError(nodePath().c_str(),_("Can not create dir <%s>."),addr().c_str());
+		throw TError(nodePath().c_str(),_("Can not create directory '%s'."),addr().c_str());
 	    IdDir = opendir(addr().c_str());
 	}
 	//> Clean scan flag
@@ -241,7 +242,7 @@ void ModMArch::checkArchivator( bool now )
 		f_arh->scan = true;
 
 		res.request(true);
-		//>>> Oldest and broken arhives to down
+		//>>> Oldest and broken archives to down
 		if( f_arh->err() )	arh_s.push_back( f_arh );
 		else
 		{
@@ -261,31 +262,31 @@ void ModMArch::checkArchivator( bool now )
 
 	//> Check deleting Archives
 	res.request(true);
-	for( unsigned i_arh = 0; i_arh < arh_s.size(); i_arh++ )
-	    if( !arh_s[i_arh]->scan )
+	for(unsigned i_arh = 0; i_arh < arh_s.size(); i_arh++)
+	    if(!arh_s[i_arh]->scan)
 	    {
 		delete arh_s[i_arh];
-		arh_s.erase( arh_s.begin() + i_arh );
+		arh_s.erase(arh_s.begin() + i_arh);
 		i_arh--;
 	    }
 	res.release();
 
 	//> Check file count and delete odd files
-	if( mNumbFiles )
+	if(mNumbFiles && !mod->noArchLimit)
 	{
 	    int f_cnt = 0;	//Work files number
-	    for( unsigned i_arh = 0; i_arh < arh_s.size(); i_arh++)
-		if( !arh_s[i_arh]->err() ) f_cnt++;
-	    if( f_cnt > mNumbFiles )
+	    for(unsigned i_arh = 0; i_arh < arh_s.size(); i_arh++)
+		if(!arh_s[i_arh]->err()) f_cnt++;
+	    if(f_cnt > mNumbFiles)
 	    {
 		//>> Delete oldest files
-		for( int i_arh = arh_s.size()-1; i_arh >= 0; i_arh-- )
-		    if( f_cnt <= mNumbFiles )	break;
-		    else if( !arh_s[i_arh]->err() )
+		for(int i_arh = arh_s.size()-1; i_arh >= 0; i_arh--)
+		    if(f_cnt <= mNumbFiles)	break;
+		    else if(!arh_s[i_arh]->err())
 		    {
 			string f_nm = arh_s[i_arh]->name();	//Deleted file name
 			delete arh_s[i_arh];
-			arh_s.erase( arh_s.begin() + i_arh );
+			arh_s.erase(arh_s.begin() + i_arh);
 			remove(f_nm.c_str());
 			remove((f_nm+".info").c_str());
 			f_cnt--;
@@ -402,7 +403,7 @@ MFileArch::MFileArch( const string &iname, time_t ibeg, ModMArch *iowner, const
     cach_pr.tm = cach_pr.off = 0;
 
     int hd = open( name().c_str(),O_RDWR|O_CREAT|O_TRUNC, 0666 );
-    if(hd <= 0) throw TError(owner().nodePath().c_str(),_("Can not create file: <%s>!"),name().c_str());
+    if(hd <= 0) throw TError(owner().nodePath().c_str(),_("Can not create file: '%s'!"),name().c_str());
 
     if( xmlM() )
     {
@@ -423,7 +424,7 @@ MFileArch::MFileArch( const string &iname, time_t ibeg, ModMArch *iowner, const
     {
 	//> Prepare plain text file
 	char s_buf[STR_BUF_LEN];
-	snprintf(s_buf,sizeof(s_buf),"%s %s %8s %8x %8x\n",
+	snprintf(s_buf,sizeof(s_buf),"%s %s %s %8x %8x\n",
 	    mod->modId().c_str(),mod->modInfo("Version").c_str(),mChars.c_str(),(unsigned int)mBeg,(unsigned int)mEnd);
 	if( write(hd,s_buf,strlen(s_buf)) < 0 )
 	    throw TError(owner().nodePath().c_str(),_("Write to file error: %s"),strerror(errno));
@@ -466,8 +467,8 @@ void MFileArch::attach( const string &iname, bool full )
 		if(rsz > 0 && rsz < (int)sizeof(buf))
 		{
 		    buf[rsz] = 0;
-		    char bChars[21];
-		    if( sscanf(buf,"%lx %lx %20s %hhd",&mBeg,&mEnd,bChars,&mXML) == 4 ) { mChars = bChars; infoOK = true; }
+		    char bChars[100];
+		    if( sscanf(buf,"%lx %lx %99s %hhd",&mBeg,&mEnd,bChars,&mXML) == 4 ) { mChars = bChars; infoOK = true; }
 		}
 		close(hd);
 	    }
@@ -502,11 +503,11 @@ void MFileArch::attach( const string &iname, bool full )
 	f = fopen(name().c_str(),"r");
 	if( f == NULL ) { mErr = true; return; }
 
-	char s_char[9];
+	char s_char[100];
 	//> Check to plain text archive
 	if( fgets(buf,sizeof(buf),f) == NULL )
-	    throw TError(owner().nodePath().c_str(),_("File %s header error!"),name().c_str());
-	string s_tmpl = mod->modId()+"%*s %8s %x %x";
+	    throw TError(owner().nodePath().c_str(),_("File '%s' header error!"),name().c_str());
+	string s_tmpl = mod->modId()+"%*s %99s %x %x";
 	if( sscanf(buf,s_tmpl.c_str(),s_char,&mBeg,&mEnd) == 3 )
 	{
 	    //>> Attach plain text archive file
@@ -541,7 +542,7 @@ void MFileArch::attach( const string &iname, bool full )
 		mNode->load(s_buf);
 		if( mNode->name() != mod->modId() )
 		{
-		    mess_err(owner().nodePath().c_str(),_("Archive file: <%s> is not mine."),name().c_str());
+		    mess_err(owner().nodePath().c_str(),_("Archive file: '%s' is not mine."),name().c_str());
 		    mNode->clear();
 		    mErr = true;
 		    return;
@@ -566,7 +567,7 @@ void MFileArch::attach( const string &iname, bool full )
 		    while((c=fgetc(f)) != '<' && c != EOF);
 		    if(c == EOF)
 		    {
-			mess_err(owner().nodePath().c_str(),_("Archive <%s> file error."),name().c_str());
+			mess_err(owner().nodePath().c_str(),_("Archive '%s' file error."),name().c_str());
 			mErr = true;
 			fclose(f);
 			return;
@@ -575,7 +576,7 @@ void MFileArch::attach( const string &iname, bool full )
 		    while((c=fgetc(f)) != ' ' && c != '\t' && c != '>' && c != EOF) prm += c;
 		    if(c == EOF)
 		    {
-			mess_err(owner().nodePath().c_str(),_("Archive <%s> file error."),name().c_str());
+			mess_err(owner().nodePath().c_str(),_("Archive '%s' file error."),name().c_str());
 			mErr = true;
 			fclose(f);
 			return;
@@ -705,7 +706,7 @@ void MFileArch::put( TMess::SRec mess )
 	    if( mess.time != mEnd )
 	    {
 		mEnd = mess.time;
-		snprintf(buf,sizeof(buf),"%s %s %8s %8x %8x\n",
+		snprintf(buf,sizeof(buf),"%s %s %s %8x %8x\n",
 		    mod->modId().c_str(),mod->modInfo("Version").c_str(),mChars.c_str(),(unsigned int)mBeg,(unsigned int)mEnd);
 		fwrite(buf,strlen(buf),1,f);
 	    }
@@ -768,30 +769,32 @@ void MFileArch::put( TMess::SRec mess )
     }
 }
 
-void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const string &category, char level )
+void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const string &category, char level, time_t upTo )
 {
     TMess::SRec b_rec;
 
-    if( mErr ) throw TError(owner().nodePath().c_str(),_("Getting messages from an error Archive file!"));
+    if(mErr) throw TError(owner().nodePath().c_str(),_("Getting messages from an error Archive file!"));
 
     ResAlloc res(mRes,false);
+    if(!upTo) upTo = time(NULL)+STD_INTERF_TM;
 
-    if( mPack )
+    if(mPack)
     {
 	try{ mName = mod->unPackArch(name()); } catch(TError err) { mErr = true; throw; }
 	mPack = false;
     }
-    if( !mLoad )
+    if(!mLoad)
     {
 	res.release(); attach( mName ); res.request(false);
 	if( mErr || !mLoad )	throw TError(owner().nodePath().c_str(),_("Archive file isn't attached!"));
     }
 
     mAcces = time(NULL);
+    TRegExp re(category, "p");
 
-    if( xmlM() )
+    if(xmlM())
     {
-	for(unsigned i_ch = 0; i_ch < mNode->childSize() && mess.size() < TArchiveS::max_req_mess; i_ch++)
+	for(unsigned i_ch = 0; i_ch < mNode->childSize() && time(NULL) < upTo; i_ch++)
 	{
 	    //> Find messages
 	    b_rec.time  = strtol( mNode->childGet(i_ch)->attr("tm").c_str(),(char **)NULL,16);
@@ -799,18 +802,18 @@ void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const
 	    b_rec.categ = mNode->childGet(i_ch)->attr("cat");
 	    b_rec.level = (TMess::Type)atoi( mNode->childGet(i_ch)->attr("lv").c_str() );
 	    b_rec.mess  = mNode->childGet(i_ch)->text();
-	    if( b_rec.time >= b_tm && b_rec.time <= e_tm && b_rec.level >= level && TMess::chkPattern(b_rec.categ,category) )
+	    if(b_rec.time >= b_tm && b_rec.time <= e_tm && b_rec.level >= level && re.test(b_rec.categ))
 	    {
 		bool equal = false;
 		int i_p = mess.size();
-		for( int i_m = mess.size()-1; i_m >= 0; i_m-- )
+		for(int i_m = mess.size()-1; i_m >= 0; i_m--)
 		{
-		    if( FTM(mess[i_m]) > FTM(b_rec) ) i_p = i_m;
-		    else if( FTM(mess[i_m]) == FTM(b_rec) && b_rec.level == mess[i_m].level && b_rec.mess == mess[i_m].mess )
+		    if(FTM(mess[i_m]) > FTM(b_rec)) i_p = i_m;
+		    else if(FTM(mess[i_m]) == FTM(b_rec) && b_rec.level == mess[i_m].level && b_rec.mess == mess[i_m].mess)
 		    { equal = true; break; }
-		    else if( FTM(mess[i_m]) < FTM(b_rec) ) break;
+		    else if(FTM(mess[i_m]) < FTM(b_rec)) break;
 		}
-		if( !equal ) mess.insert(mess.begin()+i_p,b_rec);
+		if(!equal) mess.insert(mess.begin()+i_p,b_rec);
 	    }
 	}
     }
@@ -828,7 +831,7 @@ void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const
 	//> Check mess records
 	int pass_cnt = 0;
 	time_t last_tm = 0;
-	while( fgets(buf,sizeof(buf),f) != NULL && mess.size() < TArchiveS::max_req_mess )
+	while( fgets(buf,sizeof(buf),f) != NULL && time(NULL) < upTo )
 	{
 	    char stm[51]; int off = 0, bLev;
 	    sscanf(buf,"%50s %d",stm,&bLev);
@@ -843,7 +846,7 @@ void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const
 		sscanf(buf,"%*x:%*d %*d %1000s %1000s",m_cat,m_mess);
 		b_rec.categ = TSYS::strDecode(Mess->codeConvIn(mChars,m_cat),TSYS::HttpURL);
 		b_rec.mess  = TSYS::strDecode(Mess->codeConvIn(mChars,m_mess),TSYS::HttpURL);
-		if( !TMess::chkPattern(b_rec.categ,category) ) continue;
+		if(!re.test(b_rec.categ)) continue;
 		//>> Check to equal messages and inserting
 		bool equal = false;
 		int i_p = mess.size();
