@@ -33,8 +33,8 @@
 #define MOD_NAME	_("DB FireBird")
 #define MOD_TYPE	SDB_ID
 #define VER_TYPE	SDB_VER
-#define MOD_VER		"0.9.6"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"0.9.7"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("DB module. Provides support of the DB FireBird.")
 #define LICENSE		"GPL2"
 //******************************************************************************
@@ -69,7 +69,7 @@ BDMod::BDMod(string name) : TTipBD(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -89,7 +89,7 @@ void BDMod::load_( )
 {
     //> Load parameters from command line
 
-    //> Load parameters from config file
+    //> Load parameters from config-file
 
 }
 
@@ -200,7 +200,7 @@ void MBD::allowList( vector<string> &list )
 TTable *MBD::openTable( const string &inm, bool create )
 {
     if( !enableStat() )
-	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB is disabled."),inm.c_str());
+	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table '%s'. DB is disabled."),inm.c_str());
 
     return new MTable(inm,this,create);
 }
@@ -228,7 +228,7 @@ void MBD::transOpen( )
     if(!htrans)
     {
 	if(isc_start_transaction(status, &htrans, 1, &hdb, 0, NULL))
-	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start trasaction error: %s"),getErr(status).c_str());
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start transaction error: %s"),getErr(status).c_str());
 	trOpenTm = time(NULL);
     }
     reqCnt++;
@@ -258,10 +258,11 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl, char intoTr
     if(tbl) tbl->clear();
     if(!enableStat()) return;
 
-    ResAlloc res(conn_res,true);
     if(intoTrans && intoTrans != EVAL_BOOL)	transOpen();
     else if(!intoTrans && htrans)		transCommit();
 
+    ResAlloc res(conn_res,true);
+
     XSQLDA  *out_sqlda = (XSQLDA *)malloc(XSQLDA_LENGTH(10));
     out_sqlda->version = SQLDA_VERSION1;
     out_sqlda->sqln = 10;
@@ -277,7 +278,7 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl, char intoTr
 	if( isc_dsql_allocate_statement(status, &hdb, &stmt) )
 	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Allocate statement error: %s"),getErr(status).c_str());
 	if( !trans && isc_start_transaction(status, &trans, 1, &hdb, 0, NULL) )
-	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start trasaction error: %s"),getErr(status).c_str());
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start transaction error: %s"),getErr(status).c_str());
 	//> Prepare output data structure
 	if( isc_dsql_prepare(status, &trans, &stmt, 0, Mess->codeConvOut(cd_pg.c_str(),ireq).c_str(), 3, NULL) )
 	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL prepare error: %s"),getErr(status).c_str());
@@ -587,7 +588,7 @@ void MTable::fieldGet( TConfig &cfg )
     mLstUse = time(NULL);
 
     //> Prepare request
-    string req = "SELECT ", req_where, sid;
+    string req = "SELECT ", req_where, sid, first_key;
     //>> Add fields list to queue
     bool first_sel = true, next_wr = false, trPresent = false;
     for(unsigned i_fld = 1; i_fld < tblStrct.size(); i_fld++)
@@ -605,6 +606,7 @@ void MTable::fieldGet( TConfig &cfg )
 	if(u_cfg->fld().flg()&TCfg::Key)
 	{
 	    req_where = req_where + (next_wr?"AND \"":"\"") + mod->sqlReqCode(sid,'"') + "\"='" + mod->sqlReqCode(getVal(*u_cfg)) + "' ";
+	    if(first_key.empty()) first_key = mod->sqlReqCode(sid,'"');
 	    next_wr = true;
 	}
 	else if(u_cfg->view())
@@ -613,6 +615,7 @@ void MTable::fieldGet( TConfig &cfg )
 	    first_sel = false;
 	}
     }
+    if(first_sel) req += "\""+first_key+"\"";
     req = req + " FROM \"" + mod->sqlReqCode(name(),'"') + "\" WHERE " + req_where;
 
     //> Query
@@ -656,7 +659,7 @@ void MTable::fieldSet( TConfig &cfg )
 	if(sid.size() > 3)
 	{
 	    if(!trPresent && sid.substr(0,3) == (Mess->lang2Code()+"#")) trPresent = true;
-	    if(isVarTextTransl && !trDblDef && sid.substr(0,3) == (Mess->lang2CodeBase()+"#")) trDblDef = true;
+	    if(Mess->lang2Code() == Mess->lang2CodeBase() && !trDblDef && sid.compare(0,3,Mess->lang2CodeBase()+"#") == 0) trDblDef = true;
 	}
     }
     if(trDblDef) fieldFix(cfg); 
@@ -779,6 +782,7 @@ void MTable::fieldFix( TConfig &cfg )
 		    case TFld::Integer:	if(tblStrct[i_fld][1] == "8")	isEqual = true;	break;
 		    case TFld::Real:	if(tblStrct[i_fld][1] == "27")	isEqual = true;	break;
 		    case TFld::Boolean:	if(tblStrct[i_fld][1] == "7")	isEqual = true;	break;
+		    default: break;
 		}
 		if(isEqual) break;
 		continue;
@@ -824,6 +828,7 @@ void MTable::fieldFix( TConfig &cfg )
 	    case TFld::Boolean:
 		f_tp = "SMALLINT DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' NOT NULL ";
 		break;
+	    default: break;
 	}
 
 	if(i_fld >= tblStrct.size())
@@ -865,6 +870,7 @@ string MTable::getVal( TCfg &cfg )
 	case TFld::Integer:	return SYS->int2str(cfg.getI());
 	case TFld::Real:	return SYS->real2str(cfg.getR());
 	case TFld::Boolean:	return SYS->int2str(cfg.getB());
+	default: break;
     }
     return "";
 }
@@ -877,5 +883,6 @@ void MTable::setVal( TCfg &cfg, const string &val )
 	case TFld::Integer:	cfg.setI(atoi(val.c_str()));	break;
 	case TFld::Real:	cfg.setR(atof(val.c_str()));	break;
 	case TFld::Boolean:	cfg.setB(atoi(val.c_str()));	break;
+	default: break;
     }
 }
