@@ -34,8 +34,8 @@
 #define MOD_NAME	_("DB PostgreSQL")
 #define MOD_TYPE	SDB_ID
 #define VER_TYPE	SDB_VER
-#define MOD_VER		"0.9.1"
-#define AUTORS		_("Maxim Lysenko")
+#define MOD_VER		"0.9.2"
+#define AUTHORS		_("Maxim Lysenko")
 #define DESCRIPTION	_("BD module. Provides support of the BD PostgreSQL.")
 #define MOD_LICENSE	"GPL2"
 //************************************************
@@ -70,7 +70,7 @@ BDMod::BDMod(string name) : TTipBD(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= MOD_LICENSE;
     mSource	= name;
@@ -213,7 +213,7 @@ void MBD::disable( )
     if(reqCnt) transCommit();
 
     ResAlloc resource(conn_res,true);
-    PQfinish( connection );
+    PQfinish(connection);
 }
 
 void MBD::allowList( vector<string> &list )
@@ -236,7 +236,7 @@ void MBD::allowList( vector<string> &list )
 TTable *MBD::openTable( const string &inm, bool create )
 {
     if( !enableStat() )
-	throw TError(TSYS::DBOpen,nodePath().c_str(),_("Error open table <%s>. DB is disabled."),inm.c_str());
+	throw TError(TSYS::DBOpen,nodePath().c_str(),_("Error open table '%s'. DB is disabled."),inm.c_str());
     return new MTable(inm,this,create);
 }
 
@@ -256,7 +256,7 @@ void MBD::transOpen( )
 	if(!res || PQresultStatus(res) != PGRES_COMMAND_OK)
 	{
 	    PQclear(res);
-	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start trasaction error!"));
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start transaction error!"));
 	}
 	PQclear(res);
 	trOpenTm = time(NULL);
@@ -277,7 +277,7 @@ void MBD::transCommit( )
 	if(!res || PQresultStatus(res) != PGRES_COMMAND_OK)
 	{
 	    PQclear(res);
-	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Stop trasaction error!"));
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Stop transaction error!"));
 	}
 	PQclear(res);
     }
@@ -300,9 +300,10 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl, char intoTr
 
     string req = Mess->codeConvOut(cd_pg.c_str(),ireq);
 
+    if(intoTrans && intoTrans != EVAL_BOOL)	transOpen();
+    else if(!intoTrans && reqCnt)		transCommit();
+
     ResAlloc resource(conn_res,true);
-    if( intoTrans && intoTrans != EVAL_BOOL )	transOpen();
-    else if( !intoTrans && reqCnt )	transCommit();
 
     if( PQstatus( connection ) != CONNECTION_OK  )
     {
@@ -558,32 +559,34 @@ void MTable::fieldGet( TConfig &cfg )
     string sid;
     //> Prepare request
     string req = "SELECT ";
-    string req_where;
+    string req_where, first_key;
     //>> Add fields list to queue
     bool first_sel = true, next_wr = false, trPresent = false;
-    for( unsigned i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
+    for(unsigned i_fld = 1; i_fld < tblStrct.size(); i_fld++)
     {
 	sid = tblStrct[i_fld][0];
 	TCfg *u_cfg = cfg.at(sid,true);
-	if( !cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") )
+	if(!cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#"))
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
-	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText) ) continue;
+	    if(u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText)) continue;
 	    trPresent = true;
 	}
-	if( !u_cfg ) continue;
+	if(!u_cfg) continue;
 
-	if( u_cfg->fld().flg()&TCfg::Key )
+	if(u_cfg->fld().flg()&TCfg::Key)
 	{
 	    req_where = req_where + (next_wr?"AND \"":"\"") + TSYS::strEncode(sid,TSYS::SQL) + "\"='"  +TSYS::strEncode(getVal(*u_cfg),TSYS::SQL) + "' ";
+	    if(first_key.empty()) first_key = TSYS::strEncode(sid,TSYS::SQL);
 	    next_wr = true;
 	}
-	else if( u_cfg->view() )
+	else if(u_cfg->view())
 	{
 	    req = req + (first_sel?"\"":",\"") + TSYS::strEncode(sid,TSYS::SQL) + "\"";
 	    first_sel = false;
 	}
     }
+    if(first_sel) req += "\""+first_key+"\"";
     req = req + " FROM \"" + TSYS::strEncode(name(),TSYS::SQL) + "\" WHERE " + req_where;
 
     //> Query
@@ -625,7 +628,7 @@ void MTable::fieldSet( TConfig &cfg )
 	if( sid.size() > 3 )
 	{
 	    if( !trPresent && sid.substr(0,3) == (Mess->lang2Code()+"#") ) trPresent = true;
-	    if( isVarTextTransl && !trDblDef && sid.substr(0,3) == (Mess->lang2CodeBase()+"#") ) trDblDef = true;
+	    if( Mess->lang2Code() == Mess->lang2CodeBase() && !trDblDef && sid.compare(0,3,Mess->lang2CodeBase()+"#") == 0 ) trDblDef = true;
 	}
     }
     if( trDblDef ) fieldFix(cfg);
@@ -756,12 +759,13 @@ void MTable::fieldFix( TConfig &cfg )
 		    case TFld::Integer:
 			if( u_cfg.fld().flg()&TFld::DateTimeDec && (tblStrct[i_fld][1] == "timestamp with time zone") )
 			    isEqual = true;
-			else if( (tblStrct[i_fld][1] == "integer") || 
+			else if( (tblStrct[i_fld][1] == "integer") ||
 				 (tblStrct[i_fld][1] == "bigint") )
 			    isEqual = true;
 			break;
 		    case TFld::Real:	if( tblStrct[i_fld][1] == "double precision" ) isEqual = true;	break;
 		    case TFld::Boolean:	if( tblStrct[i_fld][1] == "smallint" ) isEqual = true;	break;
+		    default: break;
 		}
 		if( isEqual ) break;
 		continue;
@@ -795,20 +799,21 @@ void MTable::fieldFix( TConfig &cfg )
 	{
 	    case TFld::String:
 		if( u_cfg.fld().len() < 256 || u_cfg.fld().flg()&TCfg::Key )
-		    f_tp="CHARACTER VARYING("+SYS->int2str(vmax(10,vmin(255,u_cfg.fld().len())))+") DEFAULT '"+u_cfg.fld().def()+"' ";
-		else  f_tp="TEXT DEFAULT '"+u_cfg.fld().def()+"' ";
+		    f_tp = "CHARACTER VARYING("+SYS->int2str(vmax(10,vmin(255,u_cfg.fld().len())))+") DEFAULT '"+u_cfg.fld().def()+"' ";
+		else f_tp = "TEXT DEFAULT '"+u_cfg.fld().def()+"' ";
 		break;
 	    case TFld::Integer:
 		if( u_cfg.fld().flg()&TFld::DateTimeDec )
-		    f_tp = req+"TIMESTAMP WITH TIME ZONE DEFAULT '"+UTCtoSQL(atoi(u_cfg.fld().def().c_str()))+"' "; 
-		else f_tp="INTEGER DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' ";
+		    f_tp = "TIMESTAMP WITH TIME ZONE DEFAULT '"+UTCtoSQL(atoi(u_cfg.fld().def().c_str()))+"' ";
+		else f_tp = "INTEGER DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' ";
 		break;
 	    case TFld::Real:
-		f_tp="DOUBLE PRECISION DEFAULT '"+TSYS::real2str(atof(u_cfg.fld().def().c_str()))+"' ";
+		f_tp = "DOUBLE PRECISION DEFAULT '"+TSYS::real2str(atof(u_cfg.fld().def().c_str()))+"' ";
 		break;
 	    case TFld::Boolean:
-		f_tp="SMALLINT DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' ";
+		f_tp = "SMALLINT DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' ";
 		break;
+	    default: break;
 	}
 
 	if( i_fld >= tblStrct.size() )
@@ -853,6 +858,7 @@ string MTable::getVal( TCfg &cfg )
 	    else		return SYS->int2str(cfg.getI());
 	case TFld::Real:	return SYS->real2str(cfg.getR());
 	case TFld::Boolean:	return SYS->int2str(cfg.getB());
+	default: break;
     }
     return "";
 }
@@ -868,6 +874,7 @@ void MTable::setVal( TCfg &cfg, const string &val )
 	    break;
 	case TFld::Real:	cfg.setR(atof(val.c_str()));	break;
 	case TFld::Boolean:	cfg.setB(atoi(val.c_str()));	break;
+	default: break;
     }
 }
 
