@@ -35,8 +35,8 @@
 #define MOD_NAME	_("DB SQLite")
 #define MOD_TYPE	SDB_ID
 #define VER_TYPE	SDB_VER
-#define MOD_VER		"1.6.3"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"1.6.4"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("BD module. Provides support of the BD SQLite.")
 #define LICENSE		"GPL2"
 //******************************************************************************
@@ -71,7 +71,7 @@ BDMod::BDMod(string name) : TTipBD(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -91,7 +91,7 @@ void BDMod::load_( )
 {
     //> Load parameters from command line
 
-    //> Load parameters from config file
+    //> Load parameters from config-file
 
 }
 
@@ -146,15 +146,15 @@ void MBD::enable( )
 
 void MBD::disable( )
 {
-    if( !enableStat() )  return;
+    if(!enableStat())  return;
 
     //> Last commit
-    if( reqCnt ) transCommit();
+    if(reqCnt) transCommit();
 
     TBD::disable( );
 
     //> Close DB
-    ResAlloc res(conn_res,true);
+    ResAlloc res(conn_res, true);
     sqlite3_close(m_db);
 }
 
@@ -164,14 +164,14 @@ void MBD::allowList( vector<string> &list )
     list.clear();
     vector< vector<string> > tbl;
     sqlReq("SELECT name FROM sqlite_master WHERE type IN ('table','view') AND name NOT LIKE 'sqlite_%';",&tbl);
-    for( unsigned i_t = 1; i_t < tbl.size(); i_t++ )
+    for(unsigned i_t = 1; i_t < tbl.size(); i_t++)
 	list.push_back(tbl[i_t][0]);
 }
 
 TTable *MBD::openTable( const string &inm, bool create )
 {
     if( !enableStat() )
-	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB is disabled."),inm.c_str());
+	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table '%s'. DB is disabled."),inm.c_str());
 
     return new MTable(inm,this,create);
 }
@@ -187,10 +187,12 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl, char intoTr
 
     //> Commit set
     string req = ireq;
-    ResAlloc res(conn_res,true);
+
     if(intoTrans && intoTrans != EVAL_BOOL) transOpen();
     else if(!intoTrans && reqCnt) transCommit();
 
+    ResAlloc res(conn_res,true);
+
     //> Put request
     rc = sqlite3_get_table(m_db,Mess->codeConvOut(cd_pg.c_str(),req).c_str(),&result, &nrow, &ncol, &zErrMsg);
     if(rc != SQLITE_OK)
@@ -221,28 +223,31 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl, char intoTr
 void MBD::transOpen( )
 {
     //> Check for limit into one trinsaction
-    if( reqCnt > 1000 ) transCommit( );
+    if(reqCnt > 1000) transCommit();
 
-    ResAlloc resource(conn_res,true);
-    if( !reqCnt )
-    {
-	sqlReq("BEGIN;");
-	trOpenTm = time(NULL);
-    }
+    conn_res.resRequestW();
+    bool begin = !reqCnt;
+    if(begin) trOpenTm = time(NULL);
     reqCnt++;
     reqCntTm = time(NULL);
+    conn_res.resRelease();
+
+    if(begin) sqlReq("BEGIN;");
 }
 
 void MBD::transCommit( )
 {
-    ResAlloc resource(conn_res,true);
-    if( reqCnt ) sqlReq("COMMIT;");
+    conn_res.resRequestW();
+    bool commit = reqCnt;
     reqCnt = reqCntTm = 0;
+    conn_res.resRelease();
+
+    if(commit) sqlReq("COMMIT;");
 }
 
 void MBD::transCloseCheck( )
 {
-    if( enableStat() && reqCnt && ((time(NULL)-reqCntTm) > 10*60 || (time(NULL)-trOpenTm) > 10*60) )
+    if(enableStat() && reqCnt && ((time(NULL)-reqCntTm) > 10*60 || (time(NULL)-trOpenTm) > 10*60))
 	transCommit();
 }
 
@@ -259,7 +264,7 @@ void MBD::cntrCmdProc( XMLNode *opt )
 		      "               Use empty path for a private, temporary on-disk database create.\n"
 		      "               Use \":memory:\" for a private, temporary in-memory database create."));
 	if(reqCnt)
-	    ctrMkNode("comm",opt,-1,"/prm/st/end_tr",_("Close openned transaction"),RWRWRW,"root",SDB_ID);
+	    ctrMkNode("comm",opt,-1,"/prm/st/end_tr",_("Close opened transaction"),RWRWRW,"root",SDB_ID);
 	return;
     }
     //> Process command to page
@@ -391,32 +396,34 @@ void MTable::fieldGet( TConfig &cfg )
     string sid;
     //> Prepare request
     string req = "SELECT ";
-    string req_where;
+    string req_where, first_key;
     //>> Add fields list to queue
     bool first_sel = true, next_wr = false, trPresent = false;
-    for( unsigned i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
+    for(unsigned i_fld = 1; i_fld < tblStrct.size(); i_fld++)
     {
 	sid = tblStrct[i_fld][1];
 	TCfg *u_cfg = cfg.at(sid,true);
-	if( !cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") )
+	if(!cfg.noTransl() && !u_cfg && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#"))
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
-	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText) ) continue;
+	    if(u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText)) continue;
 	    trPresent = true;
 	}
-	if( !u_cfg ) continue;
+	if(!u_cfg) continue;
 
-	if( u_cfg->fld().flg()&TCfg::Key )
+	if(u_cfg->fld().flg()&TCfg::Key)
 	{
 	    req_where = req_where + (next_wr?" AND \"":"\"") + mod->sqlReqCode(sid,'"') + "\"='" + mod->sqlReqCode(getVal(*u_cfg)) + "'";
+	    if(first_key.empty()) first_key = mod->sqlReqCode(sid,'"');
 	    next_wr = true;
 	}
-	else if( u_cfg->view() )
+	else if(u_cfg->view())
 	{
 	    req = req + (first_sel?"\"":",\"") + mod->sqlReqCode(sid,'"') + "\"";
 	    first_sel = false;
 	}
     }
+    if(first_sel) req += "\""+first_key+"\"";
     req = req + " FROM '" + mod->sqlReqCode(name()) + "' WHERE " + req_where + ";";
 
     //> Query
@@ -429,10 +436,10 @@ void MTable::fieldGet( TConfig &cfg )
 	sid = tbl[0][i_fld];
 	TCfg *u_cfg = cfg.at(sid,true);
 	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
-	else if( trPresent && sid.size() > 3 && sid.substr(0,3) == (Mess->lang2Code()+"#") && tbl[1][i_fld].size() )
+	else if( trPresent && sid.compare(0,3,Mess->lang2Code()+"#") == 0 && tbl[1][i_fld].size() )
 	{
 	    u_cfg = cfg.at(sid.substr(3),true);
-	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
+	    if( u_cfg && u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl() ) setVal(*u_cfg,tbl[1][i_fld]);
 	}
     }
 }
@@ -459,7 +466,7 @@ void MTable::fieldSet( TConfig &cfg )
 	if( sid.size() > 3 )
 	{
 	    if( !trPresent && sid.substr(0,3) == (Mess->lang2Code()+"#") ) trPresent = true;
-	    if( isVarTextTransl && !trDblDef && sid.substr(0,3) == (Mess->lang2CodeBase()+"#") ) trDblDef = true;
+	    if( Mess->lang2Code() == Mess->lang2CodeBase() && !trDblDef && sid.compare(0,3,Mess->lang2CodeBase()+"#") == 0 ) trDblDef = true;
 	}
     }
     if( trDblDef ) fieldFix(cfg);
@@ -561,7 +568,7 @@ void MTable::fieldDel( TConfig &cfg )
     catch( TError err )
     {
 	if( (err.cod-100) == SQLITE_READONLY )
-	    throw TError(TSYS::DBReadOnly,nodePath().c_str(),_("Deletion is not permited. Data base is read only."));
+	    throw TError(TSYS::DBReadOnly,nodePath().c_str(),_("Deletion is not permitted. Data base is read only."));
 	throw;
     }
 }
@@ -653,6 +660,7 @@ void MTable::fieldFix( TConfig &cfg )
 	    case TFld::Integer: case TFld::Boolean:
 				tpCfg = "INTEGER DEFAULT '" + mod->sqlReqCode(cf.fld().def()) + "' ";	break;
 	    case TFld::Real:	tpCfg = "DOUBLE DEFAULT '" + mod->sqlReqCode(cf.fld().def()) + "' ";	break;
+	    default:	break;
 	}
 	req += tpCfg;
 
@@ -700,6 +708,7 @@ string MTable::getVal( TCfg &cfg )
 	case TFld::Integer:	return SYS->int2str(cfg.getI());
 	case TFld::Real:	return SYS->real2str(cfg.getR());
 	case TFld::Boolean:	return SYS->int2str(cfg.getB());
+	default: break;
     }
     return "";
 }
@@ -712,5 +721,6 @@ void MTable::setVal( TCfg &cfg, const string &val )
 	case TFld::Integer:	cfg.setI(atoi(val.c_str()));	break;
 	case TFld::Real:	cfg.setR(atof(val.c_str()));	break;
 	case TFld::Boolean:	cfg.setB(atoi(val.c_str()));	break;
+	default: break;
     }
 }
