@@ -33,8 +33,8 @@
 #define MOD_NAME	_("Data sources gate")
 #define MOD_TYPE	SDAQ_ID
 #define VER_TYPE	SDAQ_VER
-#define MOD_VER		"0.9.2"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"0.9.5"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Allow to make gate data sources of remote OpenSCADA station to local OpenSCADA station.")
 #define LICENSE		"GPL2"
 //******************************************************
@@ -68,7 +68,7 @@ TTpContr::TTpContr( string name ) : TTipDAQ(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -90,8 +90,9 @@ void TTpContr::postEnable( int flag )
     TTipDAQ::postEnable(flag);
 
     //> Controler's DB structure
-    fldAdd(new TFld("PRM_BD",_("Parameteres cache table"),TFld::String,TFld::NoFlag,"30",""));
-    fldAdd(new TFld("PERIOD",_("Gather data period (s)"),TFld::Real,TFld::NoFlag,"6.2","1","0.1;100"));
+    fldAdd(new TFld("PRM_BD",_("Parameters cache table"),TFld::String,TFld::NoFlag,"30",""));
+    fldAdd(new TFld("PERIOD",_("Gather data period (s)"),TFld::Integer,TFld::NoFlag,"6.2","1","0.1;100"));	//!!!! Remove at further
+    fldAdd(new TFld("SCHEDULE",_("Acquisition schedule"),TFld::String,TFld::NoFlag,"100",""/* "1" */));
     fldAdd(new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99"));
     fldAdd(new TFld("TM_REST",_("Restore timeout (s)"),TFld::Integer,TFld::NoFlag,"3","30","0;1000"));
     fldAdd(new TFld("TM_REST_DT",_("Restore data depth time (hour). Zero for disable archive access."),TFld::Real,TFld::NoFlag,"6.2","1","0;12"));
@@ -117,10 +118,10 @@ TController *TTpContr::ContrAttach( const string &name, const string &daq_db )
 //******************************************************
 TMdContr::TMdContr( string name_c, const string &daq_db, ::TElem *cfgelem) :
     ::TController(name_c,daq_db,cfgelem),
-    mPer(cfg("PERIOD").getRd()), mSync(cfg("SYNCPER").getRd()), mRestDtTm(cfg("TM_REST_DT").getRd()),
-    mRestTm(cfg("TM_REST").getId()), mPrior(cfg("PRIOR").getId()), mStations(cfg("STATIONS").getSd()),
-    mContrPrm(cfg("CNTRPRM").getSd()),
-    prcSt(false), endrunReq(false), tmGath(0)
+    mSync(cfg("SYNCPER").getRd()), mRestDtTm(cfg("TM_REST_DT").getRd()), mPerOld(cfg("PERIOD").getId()),
+    mRestTm(cfg("TM_REST").getId()), mPrior(cfg("PRIOR").getId()),
+    mSched(cfg("SCHEDULE").getSd()), mStations(cfg("STATIONS").getSd()), mContrPrm(cfg("CNTRPRM").getSd()),
+    prcSt(false), call_st(false), endrunReq(false), mPer(1e9), tmGath(0)
 {
     cfg("PRM_BD").setS(MOD_ID"Prm_"+name_c);
 }
@@ -136,11 +137,14 @@ string TMdContr::getStatus( )
 
     if(startStat() && !redntUse())
     {
+	if(call_st)	val += TSYS::strMess(_("Call now. "));
+	if(period())	val += TSYS::strMess(_("Call by period: %s. "),TSYS::time2str(1e-3*period()).c_str());
+        else val += TSYS::strMess(_("Call next by cron '%s'. "),TSYS::time2str(TSYS::cron(cron()),"%d-%m-%Y %R").c_str());
 	val += TSYS::strMess(_("Spent time: %s. "),TSYS::time2str(tmGath).c_str());
 	bool isWork = false;
 	for(unsigned i_st = 0; i_st < mStatWork.size(); i_st++)
 	    if(mStatWork[i_st].second > -1)
-		val += TSYS::strMess(_("Station '%s' error, restoring in %.6g s."),mStatWork[i_st].first.c_str(),mStatWork[i_st].second);
+		val += TSYS::strMess(_("Station '%s' error, restoring in %.3g s."),mStatWork[i_st].first.c_str(),mStatWork[i_st].second);
 	    else
 	    {
 		val += TSYS::strMess(_("Requests to station '%s': %.6g."),mStatWork[i_st].first.c_str(),-mStatWork[i_st].second);
@@ -157,6 +161,14 @@ TParamContr *TMdContr::ParamAttach( const string &name, int type )
     return new TMdPrm(name,&owner().tpPrmAt(type));
 }
 
+void TMdContr::load_( )
+{
+    TController::load_( );
+
+    //> Check for get old period method value
+    if(mSched.getVal().empty()) mSched = TSYS::real2str(mPerOld);
+}
+
 void TMdContr::enable_( )
 {
     string statv, cp_el, daqtp, cntrnm, prmnm, cntrpath, gPrmLs;
@@ -254,25 +266,30 @@ void TMdContr::start_( )
 {
     if(prcSt) return;
 
+    //> Schedule process
+    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(int64_t)(1e9*atof(mSched.getVal().c_str()))) : 0;
+
     //> Clear stations request counter
     for(unsigned i_st = 0; i_st < mStatWork.size(); i_st++) mStatWork[i_st].second = 0;
 
     //> Start the gathering data task
-    SYS->taskCreate(nodePath('.',true), mPrior, TMdContr::Task, this, &prcSt);
+    SYS->taskCreate(nodePath('.',true), mPrior, TMdContr::Task, this);
 }
 
 void TMdContr::stop_( )
 {
-    if( !prcSt ) return;
+    if(!prcSt) return;
 
     //> Stop the request and calc data task
-    SYS->taskDestroy( nodePath('.',true), &prcSt, &endrunReq );
+    SYS->taskDestroy(nodePath('.',true), &endrunReq);
 }
 
 void *TMdContr::Task( void *icntr )
 {
     map<string,float>::iterator sti;
     TMdContr &cntr = *(TMdContr *)icntr;
+    int64_t t_cnt, t_prev = TSYS::curTime();
+    double syncCnt = 0;
 
     cntr.endrunReq = false;
     cntr.prcSt = true;
@@ -282,7 +299,8 @@ void *TMdContr::Task( void *icntr )
     {
 	if(cntr.redntUse()) { usleep(STD_WAIT_DELAY*1000); continue; }
 
-	long long t_cnt = TSYS::curTime();
+	cntr.call_st = true;
+	t_cnt = TSYS::curTime();
 
 	try
 	{
@@ -291,26 +309,31 @@ void *TMdContr::Task( void *icntr )
 	    //> Allow stations presenting
 	    bool isAccess = false;
 	    for(unsigned i_st = 0; i_st < cntr.mStatWork.size(); i_st++)
-		if(cntr.mStatWork[i_st].second > 0) cntr.mStatWork[i_st].second = vmax(0,cntr.mStatWork[i_st].second-cntr.period());
-		else isAccess = true;
-
-	    if(isAccess)
 	    {
-		unsigned int div = vmax(2,(unsigned int)(cntr.mSync/cntr.period()));
+		if(cntr.mStatWork[i_st].second > 0) cntr.mStatWork[i_st].second = vmax(0,cntr.mStatWork[i_st].second-1e-6*(t_cnt-t_prev));
+		if(cntr.mStatWork[i_st].second <= 0) isAccess = true;
+	    }
+	    if(!isAccess) { t_prev = t_cnt; usleep(1000000); continue; }
+	    else
+	    {
+		unsigned int div = cntr.period() ? vmax(2,(unsigned int)(cntr.syncPer()/(1e-9*cntr.period()))) : 0;
+		if(syncCnt <= 0) syncCnt = cntr.mSync;
+		syncCnt = vmax(0, syncCnt-1e-6*(t_cnt-t_prev));
 		vector<string> pLS;
 		cntr.list(pLS);
 		AutoHD<TMdPrm> prm;
 		string scntr;
 
 		//> Parameters list update
-		if((it_cnt > div && it_cnt%div == 0) || isFirst)
+		if(isFirst || (!div && syncCnt <= 0) || (div && it_cnt > div && (it_cnt%div) == 0))
 		    try { res.release(); cntr.enable_(); res.request(false); }
 		    catch(TError err) { }
 
 		//> Mark no process
 		for(unsigned i_p = 0; i_p < pLS.size(); i_p++)
 		{
-		    if(it_cnt > div && (it_cnt+i_p)%div == 0) cntr.at(pLS[i_p]).at().load_();
+		    if((!div && syncCnt <= 0) || (div && it_cnt > div && (((it_cnt+i_p)%div) == 0)))
+			cntr.at(pLS[i_p]).at().load_();
 		    cntr.at(pLS[i_p]).at().isPrcOK = false;
 		}
 
@@ -333,8 +356,8 @@ void *TMdContr::Task( void *icntr )
 			    prmNd->setAttr( "hostTm", !cntr.restDtTm() ? "1" : "0" );
 
 			    //>> Prepare individual attributes list
-			    bool sepReq = !prm.at().isEVAL && ((it_cnt+i_p)%div);
-			    prmNd->setAttr( "sepReq", sepReq ? "1" : "0" );
+			    bool sepReq = !prm.at().isEVAL && ((!div && syncCnt > 0) || (div && ((it_cnt+i_p)%div)));
+			    prmNd->setAttr("sepReq", sepReq ? "1" : "0");
 			    if(!cntr.restDtTm() && !sepReq) continue;
 
 			    vector<string> listV;
@@ -345,7 +368,7 @@ void *TMdContr::Task( void *icntr )
 				AutoHD<TVal> vl = prm.at().vlAt(listV[iV]);
 				if(sepReq && (!vl.at().arch().freeStat() || vl.at().reqFlg())) { prmNd->childAdd("el")->setAttr("id",listV[iV]); rC++; }
 				if(!vl.at().arch().freeStat())
-				    prmNd->childAdd("ael")->setAttr("id",listV[iV])->setAttr("tm",TSYS::ll2str(vmax(vl.at().arch().at().end(""),TSYS::curTime()-(long long)(3.6e9*cntr.restDtTm()))));
+				    prmNd->childAdd("ael")->setAttr("id",listV[iV])->setAttr("tm",TSYS::ll2str(vmax(vl.at().arch().at().end(""),TSYS::curTime()-(int64_t)(3.6e9*cntr.restDtTm()))));
 			    }
 			    if(sepReq && rC > listV.size()/2)
 			    {
@@ -381,8 +404,8 @@ void *TMdContr::Task( void *icntr )
 			    { vl.at().setS(aNd->text(),cntr.restDtTm()?atoll(aNd->attr("tm").c_str()):0,true); vl.at().setReqFlg(false); }
 			    else if(aNd->name() == "ael" && !vl.at().arch().freeStat() && aNd->childSize())
 			    {
-				long long btm = atoll(aNd->attr("tm").c_str());
-				long long per = atoll(aNd->attr("per").c_str());
+				int64_t btm = atoll(aNd->attr("tm").c_str());
+				int64_t per = atoll(aNd->attr("per").c_str());
 				TValBuf buf(vl.at().arch().at().valType(),0,per,false,true);
 				for(unsigned i_v = 0; i_v < aNd->childSize(); i_v++)
 				    buf.setS(aNd->childGet(i_v)->text(),btm+per*i_v);
@@ -405,7 +428,7 @@ void *TMdContr::Task( void *icntr )
 			AutoHD<TVal> vl = prm.at().vlAt(vLs[i_v]);
 			if(vl.at().getS() == EVAL_STR) continue;
 			vl.at().setS(EVAL_STR, vl.at().arch().freeStat() ? 0 :
-			    vmin(TSYS::curTime(),vmax(vl.at().arch().at().end(""),TSYS::curTime()-(long long)(3.6e9*cntr.restDtTm()))), true);
+			    vmin(TSYS::curTime(),vmax(vl.at().arch().at().end(""),TSYS::curTime()-(int64_t)(3.6e9*cntr.restDtTm()))), true);
 		    }
 		    prm.at().vlAt("err").at().setS(_("10:Data not allow."),0,true);
 		    prm.at().isEVAL = true;
@@ -416,9 +439,11 @@ void *TMdContr::Task( void *icntr )
 	}catch(TError err)	{ mess_err(err.cat.c_str(),err.mess.c_str()); }
 
 	//> Calc acquisition process time
+	t_prev = t_cnt;
 	cntr.tmGath = TSYS::curTime()-t_cnt;
+	cntr.call_st = false;
 
-	TSYS::taskSleep((long long)(1e9*cntr.period()));
+	TSYS::taskSleep(cntr.period(), (cntr.period()?0:TSYS::cron(cntr.cron())));
     }
 
 
@@ -456,6 +481,9 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
     if(opt->name() == "info")
     {
 	TController::cntrCmdProc(opt);
+	ctrRemoveNode(opt,"/cntr/cfg/PERIOD");
+        ctrMkNode("fld",opt,-1,"/cntr/cfg/SCHEDULE",cfg("SCHEDULE").fld().descr(),RWRWR_,"root",SDAQ_ID,4,
+            "tp","str","dest","sel_ed","sel_list",TMess::labSecCRONsel(),"help",TMess::labSecCRON());
 	ctrMkNode("fld",opt,-1,"/cntr/cfg/STATIONS",cfg("STATIONS").fld().descr(),RWRWR_,"root",SDAQ_ID,4,"tp","str","cols","100","rows","4",
 	    "help",_("Remote OpenSCADA stations' identifiers list used into it controller."));
 	ctrMkNode("fld",opt,-1,"/cntr/cfg/CNTRPRM",cfg("CNTRPRM").fld().descr(),RWRWR_,"root",SDAQ_ID,4,"tp","str","cols","100","rows","4",
@@ -635,7 +663,7 @@ void TMdPrm::vlArchMake( TVal &val )
 {
     if( val.arch().freeStat() ) return;
     val.arch().at().setSrcMode(TVArchive::PassiveAttr,val.arch().at().srcData());
-    val.arch().at().setPeriod((long long)(1e6*owner().period()));
+    val.arch().at().setPeriod(owner().period() ? owner().period()/1000 : 1000000);
     val.arch().at().setHardGrid( true );
     val.arch().at().setHighResTm( true );
 }
@@ -664,7 +692,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/prm/st/id",_("Id"),R_R_R_,"root",SDAQ_ID,1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/prm/st/nm",_("Name"),R_R_R_,"root",SDAQ_ID,1,"tp","str");
 	    }
-	    XMLNode *cfgN = ctrMkNode("area",opt,-1,"/prm/cfg",_("Config"));
+	    XMLNode *cfgN = ctrMkNode("area",opt,-1,"/prm/cfg",_("Configuration"));
 	    if(cfgN)
 	    {
 		//>> Get remote parameter's config section
@@ -680,7 +708,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 		if(req.childSize())
 		{
 		    *cfgN = *req.childGet(0);
-		    cfgN->setAttr("dscr",_("Remote station config"));
+		    cfgN->setAttr("dscr",_("Remote station configuration"));
 		}
 	    }
 	}
