@@ -19,6 +19,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include <string.h>
 #include <limits.h>
 #include <math.h>
 
@@ -35,14 +36,13 @@ Func *JavaLikeCalc::p_fnc;
 //*************************************************
 //* Func: Function                                *
 //*************************************************
-Func::Func( const char *iid, const char *name ) :
+Func::Func( const string &iid, const string &name ) :
     TConfig(&mod->elFnc()), TFunction(iid,SDAQ_ID),
-    mName(cfg("NAME").getSd()), mDescr(cfg("DESCR").getSd()), max_calc_tm(cfg("MAXCALCTM").getId()),
-    prg_src(cfg("FORMULA").getSd()), parse_res(mod->parseRes())
+    mName(cfg("NAME").getSd()), mDescr(cfg("DESCR").getSd()), prg_src(cfg("FORMULA").getSd()),
+    max_calc_tm(cfg("MAXCALCTM").getId()), parse_res(mod->parseRes())
 {
     cfg("ID").setS(id());
-    mName = name;
-    if(!mName.size()) mName = id();
+    mName = name.empty() ? id() : name;
 }
 
 Func::~Func( )
@@ -78,13 +78,14 @@ Lib &Func::owner( )
 
 string Func::name( )
 {
-    return mName.size()?mName:id();
+    string tNm = mName;
+    return tNm.size() ? tNm : id();
 }
 
 TCntrNode &Func::operator=( TCntrNode &node )
 {
     Func *src_n = dynamic_cast<Func*>(&node);
-    if( !src_n ) return *this;
+    if(!src_n) return *this;
 
     *(TConfig *)this = *(TConfig*)src_n;
     *(TFunction *)this = *(TFunction*)src_n;
@@ -92,7 +93,7 @@ TCntrNode &Func::operator=( TCntrNode &node )
     //> Set to DB
     cfg("ID").setS(mId);
 
-    if( src_n->startStat( ) && !startStat( ) )	setStart( true );
+    if(src_n->startStat() && !startStat()) setStart(true);
 
     return *this;
 }
@@ -219,8 +220,8 @@ void Func::saveIO( )
 
     //> Clear IO
     cfg.cfgViewAll(false);
-    for( int fld_cnt=0; SYS->db().at().dataSeek(io_bd,io_cfgpath,fld_cnt++,cfg ); )
-	if( ioId(cfg.cfg("ID").getS()) < 0 )
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(io_bd,io_cfgpath,fld_cnt++,cfg); )
+	if(ioId(cfg.cfg("ID").getS()) < 0)
 	{
 	    SYS->db().at().dataDel(io_bd,io_cfgpath,cfg,true);
 	    fld_cnt--;
@@ -247,17 +248,17 @@ void Func::delIO( )
 void Func::preIOCfgChange()
 {
     be_start = startStat();
-    if( be_start )
+    if(be_start)
     {
 	setStart(false);
-	if( mTVal ) { delete mTVal; mTVal = NULL; }
+	if(mTVal) { delete mTVal; mTVal = NULL; }
     }
     TFunction::preIOCfgChange();
 }
 
 void Func::postIOCfgChange()
 {
-    if( be_start ) setStart(true);
+    if(be_start) setStart(true);
     TFunction::postIOCfgChange();
 }
 
@@ -272,7 +273,7 @@ void Func::setStart( bool val )
     //> Stop calc
     else
     {
-	ResAlloc res(calc_res,true);
+	ResAlloc res(fRes(), true);
 	prg = "";
 	regClear();
 	regTmpClean( );
@@ -307,8 +308,8 @@ void Func::ioMove( int pos, int to )
 
 void Func::progCompile( )
 {
-    ResAlloc res(parse_res,true);
-    ResAlloc res1(calc_res,true);
+    ResAlloc res(parse_res, true);
+    ResAlloc res1(fRes(), true);
 
     //> Context clear for usings
     for(unsigned i = 0; i < used.size(); i++) used[i]->ctxClear();
@@ -316,20 +317,23 @@ void Func::progCompile( )
     p_fnc  = this;	//Parse func
     p_err  = "";	//Clear error messages
     la_pos = 0;		//LA position
-    prg    = "";	//Clear programm
+    sprg = prg_src;
+    prg.clear();	//Clear program
     regClear();		//Clear registers list
     regTmpClean( );	//Clear temporary registers list
     funcClear();	//Clear functions list
 
-    if( yyparse( ) )
+    if(yyparse())
     {
-	prg = "";
+	prg.clear();
+	sprg.clear();
 	regClear();
 	regTmpClean( );
 	funcClear();
 	run_st = false;
 	throw TError(nodePath().c_str(),"%s",p_err.c_str());
     }
+    sprg.clear();
     regTmpClean( );
 }
 
@@ -394,41 +398,16 @@ int Func::ioGet( const string &nm )
 {
     int rez = -1;
 
-    if(nm == "SYS")
-    {
-	rez = regNew();
-	Reg *rg = regAt(rez);
-	rg->setType(Reg::Obj);
-
-	//> Make code
-	uint16_t addr;
-	prg += (uint8_t)Reg::MviSysObject;
-	addr = rg->pos(); prg.append((char*)&addr,sizeof(uint16_t));
-	prg += (char)0;
-    }
-    else if(nm == "arguments")
-    {
-	rez = regNew();
-	Reg *rg = regAt(rez);
-	rg->setType(Reg::Obj);
-
-	//> Make code
-	uint16_t addr;
-	prg += (uint8_t)Reg::MviFuncArg;
-	addr = rg->pos(); prg.append((char*)&addr,sizeof(uint16_t));
-    }
-    //> Check IO
-    else
-	for( int i_io = 0; i_io < ioSize(); i_io++ )
-	    if( io(i_io)->id() == nm )
-	    {
-		rez = regNew(true);
-		Reg *rg = regAt(rez);
-		rg->setName(nm);
-		rg->setVar(i_io);
-		rg->setLock(true);
-		break;
-	    }
+    for(int i_io = 0; i_io < ioSize(); i_io++)
+	if(io(i_io)->id() == nm)
+	{
+	    rez = regNew(true);
+	    Reg *rg = regAt(rez);
+	    rg->setName(nm);
+	    rg->setVar(i_io);
+	    rg->setLock(true);
+	    break;
+	}
     return rez;
 }
 
@@ -458,12 +437,12 @@ void Func::regTmpClean( )
 
 Reg *Func::cdMvi( Reg *op, bool no_code )
 {
-    if( op->pos() >= 0 ) return op;	//Already load
-    int r_id = p_fnc->regNew( );
+    if(op->pos() >= 0) return op;	//Already load
+    int r_id = p_fnc->regNew();
     Reg *rez = regAt(r_id);
     *rez = *op;
     op->free();
-    if( no_code ) return rez;
+    if(no_code) return rez;
     uint16_t addr = rez->pos();
 
     switch(rez->type())
@@ -471,27 +450,49 @@ Reg *Func::cdMvi( Reg *op, bool no_code )
 	case Reg::Free: case Reg::Dynamic:
 	    throw TError(nodePath().c_str(),_("Variable '%s' is used but undefined"),rez->name().c_str());
 	case Reg::Bool:
-	    prg+=(uint8_t)Reg::MviB;
+	    prg += (uint8_t)Reg::MviB;
 	    prg.append((char *)&addr,sizeof(uint16_t));
-	    prg+=(uint8_t)rez->val().b_el;
+	    prg += (uint8_t)rez->val().b_el;
 	    break;
 	case Reg::Int:
-	    prg+=(uint8_t)Reg::MviI;
+	    prg += (uint8_t)Reg::MviI;
 	    prg.append((char *)&addr,sizeof(uint16_t));
 	    prg.append((char *)&rez->val().i_el,sizeof(int));
 	    break;
 	case Reg::Real:
-	    prg+=(uint8_t)Reg::MviR;
+	    prg += (uint8_t)Reg::MviR;
 	    prg.append((char *)&addr,sizeof(uint16_t));
 	    prg.append((char *)&rez->val().r_el,sizeof(double));
 	    break;
 	case Reg::String:
-	    if( rez->val().s_el->size() > 255 )
-		throw TError(nodePath().c_str(),_("String constant size is more 255 symbols."));
-	    prg+=(uint8_t)Reg::MviS;
+	{
+	    string sval = *rez->val().s_el;
+	    //if(sval.size() > 255) throw TError(nodePath().c_str(),_("String constant size is more 255 symbols."));
+	    prg += (uint8_t)Reg::MviS;
 	    prg.append((char *)&addr,sizeof(uint16_t));
-	    prg+=(uint8_t)rez->val().s_el->size();
-	    prg+= *rez->val().s_el;
+	    prg += (uint8_t)vmin(255,sval.size());
+	    prg += sval.substr(0,vmin(255,sval.size()));
+	    //> Load and append next parts for big string (>255)
+	    for(unsigned i_chunk = 1; i_chunk < (sval.size()/255+((sval.size()%255)?1:0)); i_chunk++)
+	    {
+		Reg *treg = regTmpNew();
+		*treg = sval.substr(i_chunk*255,vmin(255,(sval.size()-i_chunk*255)));
+		rez = cdBinaryOp(Reg::Add,rez,treg);
+	    }
+	    break;
+	}
+	case Reg::Obj:
+	    if(rez->name() == "SYS")
+	    {
+		prg += (uint8_t)Reg::MviSysObject;
+		prg.append((char*)&addr,sizeof(uint16_t));
+		prg += (char)0;
+	    }
+	    else if(rez->name() == "arguments")
+	    {
+		prg += (uint8_t)Reg::MviFuncArg;
+		prg.append((char*)&addr,sizeof(uint16_t));
+	    }
 	    break;
 	default: break;
     }
@@ -548,23 +549,23 @@ Reg *Func::cdMviRegExp( int p_cnt )
 
     Reg *rg_expr = NULL;
     Reg *rg_arg  = NULL;
+
     if(p_cnt == 2)
     {
-	rg_expr = cdMvi(f_prmst[1]);
-	rg_arg = cdMvi(f_prmst[0]);
+	rg_arg = cdMvi(f_prmst.front());	f_prmst.pop_front();
+	rg_expr = cdMvi(f_prmst.front());	f_prmst.pop_front();
     }
     else
     {
-	rg_expr = cdMvi(f_prmst[0]);
-        rg_arg = regAt(regNew());
-        rg_arg->setType(Reg::String);
+	rg_expr = cdMvi(f_prmst.front());	f_prmst.pop_front();
+	rg_arg = regTmpNew();
+	rg_arg->setType(Reg::String);
         rg_arg = cdMvi(rg_arg);
     }
     int p_expr = rg_expr->pos();
     int p_arg  = rg_arg->pos();
     rg_expr->free();
     rg_arg->free();
-    f_prmst.clear();
 
     //> Make result
     Reg *rez = regAt(regNew());
@@ -592,13 +593,13 @@ Reg *Func::cdTypeConv( Reg *op, Reg::Type tp, bool no_code )
 		    switch(rez->vType(this))
 		    {
 		        case Reg::Int:
-			    *rez = (char)((rez->val().i_el!=EVAL_INT) ? (bool)rez->val().i_el : EVAL_BOOL);
+			    *rez = (char)((rez->val().i_el!=EVAL_INT) ? (bool)rez->val().i_el : (char)EVAL_BOOL);
 			    break;
 			case Reg::Real:
-			    *rez = (char)((rez->val().r_el!=EVAL_REAL) ? (bool)rez->val().r_el : EVAL_BOOL);
+			    *rez = (char)((rez->val().r_el!=EVAL_REAL) ? (bool)rez->val().r_el : (char)EVAL_BOOL);
 			    break;
 			case Reg::String:
-			    *rez = (char)((*rez->val().s_el!=EVAL_STR) ? (bool)atoi(rez->val().s_el->c_str()) : EVAL_BOOL);
+			    *rez = (char)((*rez->val().s_el!=EVAL_STR) ? (bool)atoi(rez->val().s_el->c_str()) : (char)EVAL_BOOL);
 			    break;
 			default: break;
 		    }
@@ -607,13 +608,13 @@ Reg *Func::cdTypeConv( Reg *op, Reg::Type tp, bool no_code )
 		    switch(rez->vType(this))
 		    {
 			case Reg::Bool:
-			    *rez = (rez->val().b_el!=EVAL_BOOL) ? (int)rez->val().b_el : EVAL_INT;
+			    *rez = (rez->val().b_el != EVAL_BOOL) ? (int)rez->val().b_el : EVAL_INT;
 			    break;
 			case Reg::Real:
 			//    *rez = (int)rez->val().r_el;
 			    break;
 			case Reg::String:
-			    *rez = (*rez->val().s_el!=EVAL_STR) ? atoi(rez->val().s_el->c_str()) : EVAL_INT;
+			    *rez = (*rez->val().s_el != EVAL_STR) ? atoi(rez->val().s_el->c_str()) : EVAL_INT;
 			    break;
 			default: break;
 		    }
@@ -622,13 +623,13 @@ Reg *Func::cdTypeConv( Reg *op, Reg::Type tp, bool no_code )
 		    switch(rez->vType(this))
 		    {
 			case Reg::Bool:
-			    *rez = (rez->val().b_el!=EVAL_BOOL) ? (double)rez->val().b_el : EVAL_REAL;
+			    *rez = (rez->val().b_el != EVAL_BOOL) ? (double)rez->val().b_el : EVAL_REAL;
 			    break;
 			case Reg::Int:
-			    *rez = (rez->val().i_el!=EVAL_INT) ? (double)rez->val().i_el : EVAL_REAL;
+			    *rez = (rez->val().i_el != EVAL_INT) ? (double)rez->val().i_el : EVAL_REAL;
 			    break;
 			case Reg::String:
-			    *rez = (*rez->val().s_el!=EVAL_STR) ? atof(rez->val().s_el->c_str()) : EVAL_REAL;
+			    *rez = (*rez->val().s_el != EVAL_STR) ? atof(rez->val().s_el->c_str()) : EVAL_REAL;
 			    break;
 			default: break;
 		    }
@@ -637,13 +638,13 @@ Reg *Func::cdTypeConv( Reg *op, Reg::Type tp, bool no_code )
 		    switch(rez->vType(this))
 		    {
 			case Reg::Bool:
-			    *rez = (rez->val().b_el!=EVAL_BOOL) ? TSYS::int2str(rez->val().b_el) : EVAL_STR;
+			    *rez = (rez->val().b_el != EVAL_BOOL) ? TSYS::int2str(rez->val().b_el) : EVAL_STR;
 			    break;
 			case Reg::Int:
-			    *rez = (rez->val().i_el!=EVAL_INT) ? TSYS::int2str(rez->val().i_el) : EVAL_STR;
+			    *rez = (rez->val().i_el != EVAL_INT) ? TSYS::int2str(rez->val().i_el) : EVAL_STR;
 			    break;
 			case Reg::Real:
-			    *rez = (rez->val().r_el!=EVAL_REAL) ? TSYS::real2str(rez->val().r_el) : EVAL_STR;
+			    *rez = (rez->val().r_el != EVAL_REAL) ? TSYS::real2str(rez->val().r_el) : EVAL_STR;
 			    break;
 			default: break;
 		    }
@@ -814,6 +815,32 @@ Reg *Func::cdBinaryOp( Reg::Code cod, Reg *op1, Reg *op2 )
     return rez;
 }
 
+Reg *Func::cdCondBinaryOp( int p_cmd, Reg *op1, Reg *op2, int p_end )
+{
+    //> Make operation cod
+    p_end -= p_cmd;
+    //>> Prepare operands
+    op1 = cdMvi(op1);
+    Reg::Type op1_tp = op1->vType(this);
+    Reg::Type rez_tp = op1->objEl() ? Reg::Dynamic : op1_tp;
+    int op1_pos = op1->pos();
+    int op2_pos = op2->pos();
+    op1->free();
+    op2->free();
+    //>> Prepare rezult
+    Reg *rez = regAt(regNew());
+    rez->setType(rez_tp);
+
+    //> [CRRrrRRnn]
+    int a_sz = sizeof(uint16_t);
+    uint16_t addr = rez->pos();	prg.replace(p_cmd+1, sizeof(uint16_t), (char*)&addr, sizeof(uint16_t));
+    prg.replace(p_cmd+3, sizeof(uint16_t), (char*)&op1_pos, sizeof(uint16_t));
+    prg.replace(p_cmd+5, sizeof(uint16_t), (char*)&op2_pos, sizeof(uint16_t));
+    prg.replace(p_cmd+5+a_sz, a_sz, ((char *)&p_end), a_sz);
+
+    return rez;
+}
+
 Reg *Func::cdUnaryOp( Reg::Code cod, Reg *op )
 {
     //> Check allow the buildin calc and calc
@@ -883,7 +910,7 @@ Reg *Func::cdCond( Reg *cond, int p_cmd, int p_else, int p_end, Reg *thn, Reg *e
     int a_sz = sizeof(uint16_t);
     string cd_tmp;
 
-    //> Mvi cond register (insert to programm)
+    //> Mvi cond register (insert to program)
     cd_tmp = prg.substr(p_cmd);
     prg.erase(p_cmd);
     cond = cdMvi(cond);
@@ -893,21 +920,21 @@ Reg *Func::cdCond( Reg *cond, int p_cmd, int p_else, int p_end, Reg *thn, Reg *e
     prg += cd_tmp;
     uint16_t p_cond = cond->pos(); cond->free();
 
-    if( thn != NULL && els != NULL )
+    if(thn != NULL && els != NULL)
     {
-	//> Add Move command to "then" end (insert to programm)
+	//> Add Move command to "then" end (insert to program)
 	cd_tmp = prg.substr(p_else-1);	//-1 pass end command
 	prg.erase(p_else-1);
-	thn = cdMvi( thn );
-	rez = cdMove(NULL,thn);
+	thn = cdMvi(thn);
+	rez = cdMove(NULL, thn);
 	p_end += prg.size()-p_else+1;
 	p_else = prg.size()+1;
-	prg+=cd_tmp;
-	//> Add Move command to "else" end (insert to programm)
+	prg += cd_tmp;
+	//> Add Move command to "else" end (insert to program)
 	cd_tmp = prg.substr(p_end-1);   //-1 pass end command
 	prg.erase(p_end-1);
-	els = cdMvi( els );
-	cdMove(rez,els);
+	els = cdMvi(els);
+	cdMove(rez, els);
 	p_end = prg.size()+1;
 	prg += cd_tmp;
     }
@@ -917,9 +944,9 @@ Reg *Func::cdCond( Reg *cond, int p_cmd, int p_else, int p_end, Reg *thn, Reg *e
     p_end  -= p_cmd;
 
     //> [CRR00nn]
-    prg.replace(p_cmd+1,sizeof(uint16_t),(char*)&p_cond,sizeof(uint16_t));
-    prg.replace(p_cmd+3,a_sz,((char *)&p_else),a_sz);
-    prg.replace(p_cmd+3+a_sz,a_sz,((char *)&p_end),a_sz);
+    prg.replace(p_cmd+1, sizeof(uint16_t), (char*)&p_cond, sizeof(uint16_t));
+    prg.replace(p_cmd+3, a_sz, ((char *)&p_else), a_sz);
+    prg.replace(p_cmd+3+a_sz, a_sz, ((char *)&p_end), a_sz);
 
     return rez;
 }
@@ -930,7 +957,7 @@ void Func::cdCycle( int p_cmd, Reg *cond, int p_solve, int p_end, int p_postiter
     string cd_tmp;
     int p_body = (p_postiter?p_postiter:p_solve)-1;	//Include Reg::End command
 
-    //> Mvi cond register (insert to programm)
+    //> Mvi cond register (insert to program)
     cd_tmp = prg.substr(p_body);
     prg.erase(p_body);
     cond = cdMvi(cond);
@@ -961,7 +988,7 @@ void Func::cdCycleObj( int p_cmd, Reg *cond, int p_solve, int p_end, Reg *var )
     uint16_t p_var = var->pos();
     var->free();
 
-    //> Mvi cond register (insert to programm)
+    //> Mvi cond register (insert to program)
     string cd_tmp = prg.substr(p_body);
     prg.erase(p_body);
     cond = cdMvi(cond);
@@ -1105,7 +1132,7 @@ Reg *Func::cdProp( Reg *obj, const string &sprp, Reg *dprp )
 {
     uint16_t addr;
     Reg *ro = obj;
-    if( !ro->objEl() ) { ro = cdMove( NULL, cdMvi(ro), false ); ro->setObjEl(); }
+    if(!ro->objEl()) { ro = cdMove(NULL, cdMvi(ro), false); ro->setObjEl(); }
 
     if( !dprp )
     {
@@ -1161,20 +1188,24 @@ TVariant Func::oFuncCall( TVariant &vl, const string &prop, vector<TVariant> &pr
     {
 	switch(vl.type())
 	{
+	    case TVariant::Null:
+		if(prop == "isEVal")	return true;
+		return false;
 	    case TVariant::Object:	return vl.getO()->funcCall( prop, prms );
 	    case TVariant::Boolean:
 		// bool isEVal( ) - check value to "EVAL"
 		if( prop == "isEVal" )	return (vl.getB() == EVAL_BOOL);
 		// string toString( ) - performs the value as the string “true” or “false”
 		if( prop == "toString" )return string(vl.getB() ? "true" : "false");
-		throw TError(nodePath().c_str(),_("Boolean type have not function '%s' or not enough parameters for it."),prop.c_str());
+		return false;
+		//throw TError(nodePath().c_str(),_("Boolean type have not function '%s' or not enough parameters for it."),prop.c_str());
 	    case TVariant::Integer:
 		// bool isEVal( ) - check value to "EVAL"
 		if( prop == "isEVal" )	return (vl.getI() == EVAL_INT);
 	    case TVariant::Real:
 		// bool isEVal( ) - check value to "EVAL"
 		if( prop == "isEVal" )	return (vl.getR() == EVAL_REAL);
-		// string toExponential(int numbs) - return the string of the number, formatted in exponential notation, 
+		// string toExponential(int numbs = -1) - return the string of the number, formatted in exponential notation, 
 		//      and with the number of significant digits <numbs>
 		//  numbs - number of significant digits, if is missing the number of digits will have as much as needed
 		if( prop == "toExponential" )
@@ -1183,7 +1214,7 @@ TVariant Func::oFuncCall( TVariant &vl, const string &prop, vector<TVariant> &pr
 		    if( n < 0 )	return TSYS::strMess("%e",vl.getR());
 		    return TSYS::strMess("%.*e",n,vl.getR());
 		}
-		// string toFixed(int numbs, int len, bool sign) - return the string of the number, formatted in the notation of fixed-point,
+		// string toFixed(int numbs = 0, int len = 0, bool sign = false) - return the string of the number, formatted in the notation of fixed-point,
 		//      and with the number of significant digits after the decimal point <numbs>
 		//  numbs - the number of significant digits after the decimal point, if <numbs> is missing the number
 		//          of digits after the decimal point is equal to zero
@@ -1194,10 +1225,10 @@ TVariant Func::oFuncCall( TVariant &vl, const string &prop, vector<TVariant> &pr
 		    double val = vl.getR();
 		    bool sign = (val < 0 || (prms.size() >= 3 && prms[2].getB()));
 		    int n = prms.size() ? vmax(0,vmin(20,prms[0].getI())) : 0;
-		    int w = (prms.size() >= 2) ? vmin(100,prms[1].getI()) - (int)sign : 0;
+		    int w = (prms.size() >= 2) ? vmin(100,prms[1].getI()) : 0;
 		    return TSYS::strMess((sign?"%+0*.*f":"%0*.*f"),w,n,val);
 		}
-		// string toPrecision(int prec) - return the string of the formatted number with the number of significant digits <prec>
+		// string toPrecision(int prec = -1) - return the string of the formatted number with the number of significant digits <prec>
 		//  prec - number of significant digits
 		if(prop == "toPrecision")
 		{
@@ -1205,7 +1236,7 @@ TVariant Func::oFuncCall( TVariant &vl, const string &prop, vector<TVariant> &pr
 		    if(n < 0)	return TSYS::strMess("%g",vl.getR());
 		    return TSYS::strMess("%.*g",n,vl.getR());
 		}
-		// string toString(int base, int len, bool sign) - return the string of the formatted number of integer type for
+		// string toString(int base = 10, int len = -1, bool sign = false) - return the string of the formatted number of integer type for
 		//	minimum length <len> and strong sign present <sign>.
 		//  base - representation base (2-36);
 		//  len - minimum string number length (fill by 0);
@@ -1218,12 +1249,13 @@ TVariant Func::oFuncCall( TVariant &vl, const string &prop, vector<TVariant> &pr
 		    if(prms.size()) n = vmax(2,vmin(36,prms[0].getI()));
 		    if(prms.size() >= 2) w = vmin(100,prms[1].getI()) - (int)sign;
 		    string rez;
-		    for(unsigned c_vl = abs(val); c_vl || (w > 0 && (int)rez.size() < w); c_vl = c_vl/n)
+		    for(unsigned c_vl = abs(val); c_vl || rez.empty() || (w > 0 && (int)rez.size() < w); c_vl = c_vl/n)
 			rez += dsymb[c_vl%n];
 		    if(val < 0 || sign) rez += (val >= 0) ? "+" : "-";
 		    return TSYS::strEncode(rez,TSYS::Reverse);
 		}
-		throw TError(nodePath().c_str(),_("Integer or real type have not function '%s' or not enough parameters for it."),prop.c_str());
+		return false;
+		//throw TError(nodePath().c_str(),_("Integer or real type have not function '%s' or not enough parameters for it."),prop.c_str());
 	    case TVariant::String:
 		// bool isEVal( ) - check value to "EVAL"
 		if(prop == "isEVal")	return (vl.getS() == EVAL_STR);
@@ -1419,7 +1451,8 @@ TVariant Func::oFuncCall( TVariant &vl, const string &prop, vector<TVariant> &pr
 		if(prop == "path2sep")
 		    return TSYS::path2sepstr( vl.getS(), (prms.size() && prms[0].getS().size()) ? prms[0].getS()[0] : '.' );
 
-		throw TError(nodePath().c_str(),_("String type have not properties '%s' or not enough parameters for it."),prop.c_str());
+		return false;
+		//throw TError(nodePath().c_str(),_("String type have not properties '%s' or not enough parameters for it."),prop.c_str());
 	    default:	break;
 	}
 	return false;
@@ -1457,6 +1490,7 @@ TVariant Func::getVal( TValFunc *io, RegW &rg, bool fObj )
 		case TFld::Integer:	vl = rg.val().p_attr->at().getI();	break;
 		case TFld::Real:	vl = rg.val().p_attr->at().getR();	break;
 		case TFld::String:	vl = rg.val().p_attr->at().getS();	break;
+		default: break;
 	    }
 	    break;
 	case Reg::Obj:	vl = rg.val().o_el;	break;
@@ -1478,9 +1512,9 @@ string Func::getValS( TValFunc *io, RegW &rg )
     if(rg.propEmpty())
 	switch(rg.type())
 	{
-	    case Reg::Bool:	return (rg.val().b_el!=EVAL_BOOL) ? TSYS::int2str((bool)rg.val().b_el) : EVAL_STR;
-	    case Reg::Int:	return (rg.val().i_el!=EVAL_INT) ? TSYS::int2str(rg.val().i_el) : EVAL_STR;
-	    case Reg::Real:	return (rg.val().r_el!=EVAL_REAL) ? TSYS::real2str(rg.val().r_el) : EVAL_STR;
+	    case Reg::Bool:	return (rg.val().b_el != EVAL_BOOL) ? TSYS::int2str((bool)rg.val().b_el) : EVAL_STR;
+	    case Reg::Int:	return (rg.val().i_el != EVAL_INT) ? TSYS::int2str(rg.val().i_el) : EVAL_STR;
+	    case Reg::Real:	return (rg.val().r_el != EVAL_REAL) ? TSYS::real2str(rg.val().r_el) : EVAL_STR;
 	    case Reg::String:	return *rg.val().s_el;
 	    case Reg::Var:	return io->getS(rg.val().io);
 	    case Reg::PrmAttr:	return rg.val().p_attr->at().getS();
@@ -1497,10 +1531,10 @@ int Func::getValI( TValFunc *io, RegW &rg )
     if(rg.propEmpty())
 	switch(rg.type())
 	{
-	    case Reg::Bool:	return (rg.val().b_el!=EVAL_BOOL) ? (bool)rg.val().b_el : EVAL_INT;
+	    case Reg::Bool:	return (rg.val().b_el != EVAL_BOOL) ? (bool)rg.val().b_el : EVAL_INT;
 	    case Reg::Int:	return rg.val().i_el;
-	    case Reg::Real:	return (rg.val().r_el!=EVAL_REAL) ? (int)rg.val().r_el : EVAL_INT;
-	    case Reg::String:	return ((*rg.val().s_el)!=EVAL_STR) ? atoi(rg.val().s_el->c_str()) : EVAL_INT;
+	    case Reg::Real:	return (rg.val().r_el != EVAL_REAL) ? (int)rg.val().r_el : EVAL_INT;
+	    case Reg::String:	return ((*rg.val().s_el) != EVAL_STR) ? atoi(rg.val().s_el->c_str()) : EVAL_INT;
 	    case Reg::Var:	return io->getI(rg.val().io);
 	    case Reg::PrmAttr:	return rg.val().p_attr->at().getI();
 	    case Reg::Obj:	return 1;
@@ -1516,10 +1550,10 @@ double Func::getValR( TValFunc *io, RegW &rg )
     if( rg.propEmpty( ) )
 	switch( rg.type() )
 	{
-	    case Reg::Bool:	return (rg.val().b_el!=EVAL_BOOL) ? (bool)rg.val().b_el : EVAL_REAL;
-	    case Reg::Int:	return (rg.val().i_el!=EVAL_INT) ? rg.val().i_el : EVAL_REAL;
+	    case Reg::Bool:	return (rg.val().b_el != EVAL_BOOL) ? (bool)rg.val().b_el : EVAL_REAL;
+	    case Reg::Int:	return (rg.val().i_el != EVAL_INT) ? rg.val().i_el : EVAL_REAL;
 	    case Reg::Real:	return rg.val().r_el;
-	    case Reg::String:	return ((*rg.val().s_el)!=EVAL_STR) ? atof(rg.val().s_el->c_str()) : EVAL_REAL;
+	    case Reg::String:	return ((*rg.val().s_el) != EVAL_STR) ? atof(rg.val().s_el->c_str()) : EVAL_REAL;
 	    case Reg::Var:	return io->getR(rg.val().io);
 	    case Reg::PrmAttr:	return rg.val().p_attr->at().getR();
 	    case Reg::Obj:	return 1;
@@ -1536,9 +1570,9 @@ char Func::getValB( TValFunc *io, RegW &rg )
 	switch( rg.type() )
 	{
 	    case Reg::Bool:	return rg.val().b_el;
-	    case Reg::Int:	return (rg.val().i_el!=EVAL_INT) ? (bool)rg.val().i_el : EVAL_BOOL;
-	    case Reg::Real:	return (rg.val().r_el!=EVAL_REAL) ? (bool)rg.val().r_el : EVAL_BOOL;
-	    case Reg::String:	return ((*rg.val().s_el)!=EVAL_STR) ? (bool)atoi(rg.val().s_el->c_str()) : EVAL_BOOL;
+	    case Reg::Int:	return (rg.val().i_el != EVAL_INT) ? (bool)rg.val().i_el : EVAL_BOOL;
+	    case Reg::Real:	return (rg.val().r_el != EVAL_REAL) ? (bool)rg.val().r_el : EVAL_BOOL;
+	    case Reg::String:	return ((*rg.val().s_el) != EVAL_STR) ? (bool)atoi(rg.val().s_el->c_str()) : EVAL_BOOL;
 	    case Reg::Var:	return io->getB(rg.val().io);
 	    case Reg::PrmAttr:	return rg.val().p_attr->at().getB();
 	    case Reg::Obj:	return true;
@@ -1599,15 +1633,15 @@ void Func::setVal( TValFunc *io, RegW &rg, const TVariant &val )
 		    case TVariant::Real:	rg = val.getR();	break;
 		    case TVariant::String:	rg = val.getS();	break;
 		    case TVariant::Object:	rg = val.getO();	break;
-		    default:	break;
+		    default:			rg = (char)EVAL_BOOL;	break;
 		}
 		break;
 	}
-    else if( rg.type() == Reg::Obj )
+    else if(rg.type() == Reg::Obj)
     {
 	TVariant vl(rg.val().o_el);
-	for( int i_p = 0; i_p < rg.propSize( ); i_p++ )
-	    if( i_p < (rg.propSize( )-1) ) vl = vl.getO()->propGet(rg.propGet(i_p));
+	for(int i_p = 0; i_p < rg.propSize( ); i_p++)
+	    if(i_p < (rg.propSize( )-1)) vl = vl.getO()->propGet(rg.propGet(i_p));
 	    else vl.getO()->propSet(rg.propGet(i_p),val);
     }
 }
@@ -1674,22 +1708,27 @@ void Func::setValO( TValFunc *io, RegW &rg, TVarObj *val )
 
 void Func::calc( TValFunc *val )
 {
-    ResAlloc res(calc_res,false);
-    if(!startStat())	return;
+    ResAlloc res(fRes(), false);
+    if(!startStat()) return;
 
     //> Init list of registers
     RegW reg[mRegs.size()];
     for(unsigned i_rg = 0; i_rg < mRegs.size(); i_rg++)
-    {
-	reg[i_rg].setType(mRegs[i_rg]->type());
-	if(reg[i_rg].type() == Reg::Var)
-	    reg[i_rg].val().io = mRegs[i_rg]->val().io;
-	else if(reg[i_rg].type() == Reg::PrmAttr)
-	    *reg[i_rg].val().p_attr = *mRegs[i_rg]->val().p_attr;
-    }
+	switch(mRegs[i_rg]->type())
+	{
+	    case Reg::Var:
+		reg[i_rg].setType(Reg::Var);
+		reg[i_rg].val().io = mRegs[i_rg]->val().io;
+		break;
+	    case Reg::PrmAttr:
+		reg[i_rg].setType(Reg::PrmAttr);
+		*reg[i_rg].val().p_attr = *mRegs[i_rg]->val().p_attr;
+		break;
+	    default:	break;
+	}
 
     //> Exec calc
-    ExecData dt = { 1, 0, 0 };
+    ExecData dt = { 1, time(NULL), 0 };
     try{ exec(val,reg,(const uint8_t*)prg.c_str(),dt); }
     catch(TError err){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
     res.release();
@@ -1697,16 +1736,15 @@ void Func::calc( TValFunc *val )
 
 void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 {
-    while( !(dt.flg&0x01) )
+    while(!(dt.flg&0x01))
     {
 	//> Calc time control mechanism
-	if(!((dt.com_cnt++)%1000))
+	if(!((dt.com_cnt++)%10))
 	{
-	    if(!dt.start_tm)	dt.start_tm = time(NULL);
-	    else if(time(NULL) > dt.start_tm+max_calc_tm)
+	    if(time(NULL) > (dt.start_tm+max_calc_tm))
 	    {
-		mess_err(nodePath().c_str(),_("Timeouted function calc."));
-		dt.flg|=0x01;
+		mess_err(nodePath().c_str(),_("Timeouted function calculation"));
+		dt.flg |= 0x01;
 		return;
 	    }
 	}
@@ -1858,7 +1896,7 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 		    case Reg::Int:	reg[ptr->toR] = getValI(val,reg[ptr->fromR]);	break;
 		    case Reg::Real:	reg[ptr->toR] = getValR(val,reg[ptr->fromR]);	break;
 		    case Reg::String:	reg[ptr->toR] = getValS(val,reg[ptr->fromR]);	break;
-		    case Reg::Obj:	reg[ptr->toR] = getValO(val,reg[ptr->fromR]);	break;
+		    case Reg::Obj:	reg[ptr->toR] = getVal(val,reg[ptr->fromR]);	break;
 		    default:	break;
 		}
 		cprg += sizeof(SCode); break;
@@ -2019,6 +2057,21 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 		reg[ptr->rez] = (getValB(val,reg[ptr->a1])==1) || (getValB(val,reg[ptr->a2])==1);
 		cprg += sizeof(SCode); break;
 	    }
+	    case Reg::LCOr:
+	    {
+		struct SCode { uint8_t cod; uint16_t rez; uint16_t a1; uint16_t a2; uint16_t end; } __attribute__((packed));
+		const struct SCode *ptr = (const struct SCode *)cprg;
+#if OSC_DEBUG >= 5
+		printf("CODE: %d = %d c|| %d.\n",ptr->rez,ptr->a1,ptr->a2);
+#endif
+		if(getValB(val,reg[ptr->a1]) != true)
+		{
+		    exec(val,reg,cprg+sizeof(SCode),dt);
+		    reg[ptr->rez] = (getValB(val,reg[ptr->a2])==true);
+		}
+		else reg[ptr->rez] = true;
+		cprg += ptr->end; break;
+	    }
 	    case Reg::LAnd:
 	    {
 		struct SCode { uint8_t cod; uint16_t rez; uint16_t a1; uint16_t a2; } __attribute__((packed));
@@ -2029,6 +2082,21 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 		reg[ptr->rez] = (getValB(val,reg[ptr->a1])==1) && (getValB(val,reg[ptr->a2])==1);
 		cprg += sizeof(SCode); break;
 	    }
+	    case Reg::LCAnd:
+	    {
+		struct SCode { uint8_t cod; uint16_t rez; uint16_t a1; uint16_t a2; uint16_t end; } __attribute__((packed));
+		const struct SCode *ptr = (const struct SCode *)cprg;
+#if OSC_DEBUG >= 5
+		printf("CODE: %d = %d c&& %d.\n",ptr->rez,ptr->a1,ptr->a2);
+#endif
+		if(getValB(val,reg[ptr->a1]) == true)
+		{
+		    exec(val,reg,cprg+sizeof(SCode),dt);
+		    reg[ptr->rez] = (getValB(val,reg[ptr->a2])==true);
+		}
+		else reg[ptr->rez] = false;
+		cprg += ptr->end; break;
+	    }
 	    case Reg::LT:
 	    {
 		struct SCode { uint8_t cod; uint16_t rez; uint16_t a1; uint16_t a2; } __attribute__((packed));
@@ -2086,7 +2154,9 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 			    reg[ptr->rez] = getValS(val,reg[ptr->a1]) == getValS(val,reg[ptr->a2]);
 			    break;
 			default:
-			    throw TError(nodePath().c_str(),_("Not supported type for operation 'EQU'."));
+			    reg[ptr->rez] = false;
+			    break;
+			    //throw TError(nodePath().c_str(),_("Not supported type for operation 'EQU'."));
 		    }
 		else
 		{
@@ -2099,7 +2169,9 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 			case TVariant::String:
 			    reg[ptr->rez] = op1.getS() == getValS(val,reg[ptr->a2]); break;
 			default:
-			    throw TError(nodePath().c_str(),_("Not supported type for operation 'EQU'."));
+			    reg[ptr->rez] = false;
+			    break;
+			    //throw TError(nodePath().c_str(),_("Not supported type for operation 'EQU'."));
 		    }
 		}
 		cprg += sizeof(SCode); break;
@@ -2121,7 +2193,9 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 			    reg[ptr->rez] = getValS(val,reg[ptr->a1]) != getValS(val,reg[ptr->a2]);
 			    break;
 			default:
-			    throw TError(nodePath().c_str(),_("Not supported type for operation 'Add'."));
+			    reg[ptr->rez] = false;
+			    break;
+			    //throw TError(nodePath().c_str(),_("Not supported type for operation 'Add'."));
 		    }
 		else
 		{
@@ -2133,7 +2207,9 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 			case TVariant::String:
 			    reg[ptr->rez] = op1.getS() != getValS(val,reg[ptr->a2]); break;
 			default:
-			    throw TError(nodePath().c_str(),_("Not supported type for operation 'Add'."));
+			    reg[ptr->rez] = false;
+			    break;
+			    //throw TError(nodePath().c_str(),_("Not supported type for operation 'Add'."));
 		    }
 		}
 		cprg += sizeof(SCode); break;
@@ -2576,7 +2652,7 @@ void Func::exec( TValFunc *val, RegW *reg, const uint8_t *cprg, ExecData &dt )
 	    }
 	    default:
 		setStart(false);
-		throw TError(nodePath().c_str(),_("Operation %c(%xh) error. Function '%s' is stoped."),*cprg,*cprg,id().c_str());
+		throw TError(nodePath().c_str(),_("Operation %c(%xh) error. Function '%s' is stopped."),*cprg,*cprg,id().c_str());
 	}
     }
 }
@@ -2590,8 +2666,8 @@ void Func::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Function: ")+name(),owner().DB().empty()?R_R_R_:RWRWR_,"root",SDAQ_ID);
 	ctrMkNode("fld",opt,-1,"/func/cfg/name",_("Name"),owner().DB().empty()?R_R_R_:RWRWR_,"root",SDAQ_ID,2,"tp","str","len","50");
 	ctrMkNode("fld",opt,-1,"/func/cfg/descr",_("Description"),owner().DB().empty()?R_R_R_:RWRWR_,"root",SDAQ_ID,3,"tp","str","cols","100","rows","5");
-	ctrMkNode("fld",opt,-1,"/func/cfg/m_calc_tm",_("Maximum calc time (sec)"),RWRWR_,"root",SDAQ_ID,1,"tp","dec");
-	if(ctrMkNode("area",opt,-1,"/io",_("Programm")))
+	ctrMkNode("fld",opt,-1,"/func/cfg/m_calc_tm",_("Maximum calculate time (sec)"),RWRWR_,"root",SDAQ_ID,3,"tp","dec","min","0","max","3600");
+	if(ctrMkNode("area",opt,-1,"/io",_("Program")))
 	{
 	    if(ctrMkNode("table",opt,-1,"/io/io",_("IO"),RWRWR_,"root",SDAQ_ID,1,"s_com","add,del,ins,move"))
 	    {
@@ -2606,7 +2682,7 @@ void Func::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/io/io/4",_("Hide"),RWRWR_,"root",SDAQ_ID,1,"tp","bool");
 		ctrMkNode("list",opt,-1,"/io/io/5",_("Default"),RWRWR_,"root",SDAQ_ID,1,"tp","str");
 	    }
-	    ctrMkNode("fld",opt,-1,"/io/prog",_("Programm"),RWRW__,"root",SDAQ_ID,3,"tp","str","rows","10","SnthHgl","1");
+	    ctrMkNode("fld",opt,-1,"/io/prog",_("Program"),RWRW__,"root",SDAQ_ID,3,"tp","str","rows","10","SnthHgl","1");
 	}
 	return;
     }
@@ -2750,17 +2826,19 @@ Reg::Type Reg::vType( Func *fnc )
 		case TFld::Integer:	return Int;
 		case TFld::Real:	return Real;
 		case TFld::String:	return String;
+		default: break;
 	    }
-	default:	break;
+	default: break;
     }
     return type();
 }
 
 void Reg::free()
 {
-    if( lock() ) return;
+    if(lock()) return;
 
     setType(Free);
+    mNm.clear();
     mObjEl = mLock = false;
 }
 
@@ -2769,21 +2847,35 @@ void Reg::free()
 //*************************************************
 void RegW::operator=( TVarObj *ivar )
 {
-    setType(Reg::Obj);
+    setType(Reg::Free);
+    mTp = Reg::Obj;
     el.o_el = ivar;
     el.o_el->connect();
 }
 
+void RegW::operator=( const TVariant &ivar )
+{
+    switch(ivar.type())
+    {
+	case TVariant::Null:	*this = string(EVAL_STR);break;
+	case TVariant::Boolean:	*this = ivar.getB();	break;
+	case TVariant::Integer:	*this = ivar.getI();	break;
+	case TVariant::Real:	*this = ivar.getR();	break;
+	case TVariant::String:	*this = ivar.getS();	break;
+	case TVariant::Object:	*this = ivar.getO();	break;
+    }
+}
+
 void RegW::setType( Reg::Type tp )
 {
     mPrps.clear();
-    if( mTp == tp && mTp != Reg::Obj )    return;
+    if(mTp == tp) return;
     //Free old type
     switch(mTp)
     {
 	case Reg::String:	delete el.s_el;		break;
 	case Reg::Obj:
-	    if( el.o_el && !el.o_el->disconnect() ) delete el.o_el;
+	    if(el.o_el && !el.o_el->disconnect()) { delete el.o_el; el.o_el = NULL; }
 	    break;
 	case Reg::PrmAttr:	delete el.p_attr;	break;
 	default:	break;
@@ -2792,7 +2884,7 @@ void RegW::setType( Reg::Type tp )
     switch(tp)
     {
 	case Reg::String:	el.s_el = new string;	break;
-	case Reg::Obj:		el.o_el = NULL;	break;
+	case Reg::Obj:		el.o_el = new TVarObj(); el.o_el->connect(); break;
 	case Reg::PrmAttr:	el.p_attr = new AutoHD<TVal>;	break;
 	default:	break;
     }
@@ -2821,9 +2913,10 @@ Reg::Type RegW::vType( Func *fnc )
 		case TFld::Integer:	return Reg::Int;
 		case TFld::Real:	return Reg::Real;
 		case TFld::String:	return Reg::String;
+		default: break;
 	    }
 	    break;
-	default:	break;
+	default: break;
     }
     return type();
 }
