@@ -38,8 +38,8 @@
 #define MOD_TYPE	SDAQ_ID
 #define VER_TYPE	SDAQ_VER
 #define SUB_TYPE	"LIB"
-#define MOD_VER		"1.9.0"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"1.9.5"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Allow java-like based calculator and function's libraries engine. User can create and modify function and libraries.")
 #define LICENSE		"GPL2"
 //*************************************************
@@ -74,7 +74,7 @@ TipContr::TipContr( string src ) : TTipDAQ(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= src;
@@ -94,13 +94,13 @@ void TipContr::postEnable( int flag )
     //> Controller db structure
     fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"60","system") );
     fldAdd( new TFld("FUNC",_("Controller's function"),TFld::String,TFld::NoFlag,"40") );
-    fldAdd( new TFld("SCHEDULE",_("Calc schedule"),TFld::String,TFld::NoFlag,"100","1") );
-    fldAdd( new TFld("PRIOR",_("Calc task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
-    fldAdd( new TFld("ITER",_("Iteration number in single calc"),TFld::Integer,TFld::NoFlag,"2","1","1;99") );
+    fldAdd( new TFld("SCHEDULE",_("Calculation schedule"),TFld::String,TFld::NoFlag,"100","1") );
+    fldAdd( new TFld("PRIOR",_("Calculation task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
+    fldAdd( new TFld("ITER",_("Iteration number in single calculation"),TFld::Integer,TFld::NoFlag,"2","1","1;99") );
 
     //> Controller value db structure
     val_el.fldAdd( new TFld("ID",_("IO ID"),TFld::String,TCfg::Key,"20") );
-    val_el.fldAdd( new TFld("VAL",_("IO value"),TFld::String,TFld::NoFlag,"50") );
+    val_el.fldAdd( new TFld("VAL",_("IO value"),TFld::String,TFld::NoFlag,"10000") );
 
     //> Add parameter types
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
@@ -117,7 +117,7 @@ void TipContr::postEnable( int flag )
     fnc_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
     fnc_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
     fnc_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TCfg::TransltText,"300") );
-    fnc_el.fldAdd( new TFld("MAXCALCTM",_("Maximum calc time"),TFld::Integer,TFld::NoFlag,"3","10","0;999") );
+    fnc_el.fldAdd( new TFld("MAXCALCTM",_("Maximum calculate time (sec)"),TFld::Integer,TFld::NoFlag,"4","10","0;3600") );
     fnc_el.fldAdd( new TFld("FORMULA",_("Formula"),TFld::String,TCfg::TransltText,"10000") );
 
     //> Function's IO structure
@@ -187,50 +187,65 @@ void TipContr::compileFuncSynthHighl( const string &lang, XMLNode &shgl )
     {
 	shgl.childAdd("rule")->setAttr("expr","\"(\\\\\"|[^\"])*\"")->setAttr("color","darkgreen")->
 	     childAdd("rule")->setAttr("expr","\\\\([xX][a-zA-Z0-9]{2}|[0-7]{3}|.{1})")->setAttr("color","green")->setAttr("font_weight","1");
-	//shgl.childAdd("rule")->setAttr("expr","\".+?[^\\\\]\"")->setAttr("color","darkgreen");
-	//shgl.childAdd("blk")->setAttr("beg","\"")->setAttr("end","\"")->setAttr("color","darkgreen");
 	shgl.childAdd("blk")->setAttr("beg","/\\*")->setAttr("end","\\*/")->setAttr("color","gray")->setAttr("font_italic","1");
 	shgl.childAdd("rule")->setAttr("expr","//[^\n]*")->setAttr("color","gray")->setAttr("font_italic","1");
-	shgl.childAdd("rule")->setAttr("expr","\\b(if|else|for|while|in|using|new|var|break|continue|return|Array|Object)\\b")->setAttr("color","darkblue")->setAttr("font_weight","1");
+	shgl.childAdd("rule")->setAttr("expr","\\b(if|else|for|while|in|using|new|var|break|continue|return|Array|Object|RegExp)\\b")->setAttr("color","darkblue")->setAttr("font_weight","1");
 	shgl.childAdd("rule")->setAttr("expr","(\\?|\\:)")->setAttr("color","darkblue")->setAttr("font_weight","1");
 	shgl.childAdd("rule")->setAttr("expr","\\b(0[xX][0-9a-fA-F]*|[0-9]*\\.?[0-9]+|[0-9]*\\.?[0-9]+[eE][-+]?[0-9]*|true|false)\\b")->setAttr("color","darkorange");
-	shgl.childAdd("rule")->setAttr("expr","(\\=|\\!|\\+|\\-|\\>|\\<|\\*|\\/|\\||\\&)")->setAttr("color","darkblue")->setAttr("font_weight","1");
+	shgl.childAdd("rule")->setAttr("expr","(\\=|\\!|\\+|\\-|\\>|\\<|\\*|\\/|\\%|\\||\\&)")->setAttr("color","darkblue")->setAttr("font_weight","1");
 	shgl.childAdd("rule")->setAttr("expr","(\\;|\\,|\\{|\\}|\\[|\\]|\\(|\\))")->setAttr("color","blue");
     }
 }
 
-string TipContr::compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text, const string &usings )
+string TipContr::compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text, const string &usings, int maxCalcTm )
 {
-    if( lang != "JavaScript" )	throw TError(nodePath().c_str(),_("Compilation with the help of the program language %s is not supported."),lang.c_str());
-    if( !lbPresent("sys_compile") ) lbReg( new Lib("sys_compile","","") );
-    if( !lbAt("sys_compile").at().present(fnc_cfg.id()) ) lbAt("sys_compile").at().add(fnc_cfg.id().c_str(),"");
+    if(lang != "JavaScript") throw TError(nodePath().c_str(),_("Compilation with the help of the program language %s is not supported."),lang.c_str());
+    if(!lbPresent("sys_compile")) lbReg(new Lib("sys_compile","",""));
 
-    AutoHD<Func> func = lbAt("sys_compile").at().at(fnc_cfg.id());
+    //> Function id generation for "<auto>" or call nodePath() for it
+    string funcId = fnc_cfg.id();
+    if(funcId == "<auto>")
+	for(int aId = 1; lbAt("sys_compile").at().present(funcId); aId++)
+	    funcId = TSYS::strMess("Auto_%d",aId);
+    else funcId = fnc_cfg.nodePath('_',true);
 
-    bool isStart = func.at().startStat();
+    //> Connect or use allowed compiled function object
+    if(!lbAt("sys_compile").at().present(funcId)) lbAt("sys_compile").at().add(funcId.c_str(),"");
+    AutoHD<Func> func = lbAt("sys_compile").at().at(funcId);
+    func.at().setMaxCalcTm(maxCalcTm);
+
+    //> Try hot config fields change for work function
+    if(func.at().use() && func.at().startStat())
+	try
+	{
+    	    ((TFunction&)func.at()).operator=(fnc_cfg);
+	    if(prog_text == func.at().prog()) return func.at().nodePath(0,true);
+	}
+	catch(TError err)
+	{
+	    func.at().setStart(true);
+	    throw;
+	}
+    //> Standard compile
     try
     {
-	((TFunction&)func.at()).operator=(fnc_cfg);
-	if( func.at().startStat() && prog_text == func.at().prog() ) return func.at().nodePath();
-    }
-    catch(TError err) { if( isStart ) func.at().setStart(true); throw; }
-    func.at().setProg(prog_text.c_str());
-    try
-    {
-	if( func.at().startStat() ) func.at().setStart(false);
+	if(func.at().startStat()) func.at().setStart(false);
+	func.at().setProg(prog_text.c_str());
 	func.at().setUsings(usings);
+	((TFunction&)func.at()).operator=(fnc_cfg);
 	func.at().setStart(true);
     }
     catch(TError err)
     {
-	if( !func.at().use() )
+	if(!func.at().use())
 	{
 	    func.free();
-	    lbAt("sys_compile").at().del(fnc_cfg.id().c_str());
+	    lbAt("sys_compile").at().del(funcId.c_str());
 	}
 	throw TError(nodePath().c_str(),_("Compile error: %s\n"),err.mess.c_str());
     }
-    return func.at().nodePath();
+
+    return func.at().nodePath(0,true);
 }
 
 void TipContr::load_( )
@@ -246,23 +261,27 @@ void TipContr::load_( )
 	TConfig c_el(&elLib());
 	c_el.cfgViewAll(false);
 	vector<string> db_ls;
+	map<string, bool> itReg;
 
 	//>> Search into DB
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+libTable(),"",lib_cnt++,c_el); )
+	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+libTable(),nodePath()+"lib",lib_cnt++,c_el); )
 	    {
 		string l_id = c_el.cfg("ID").getS();
 		if(!lbPresent(l_id)) lbReg(new Lib(l_id.c_str(),"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]));
+		itReg[l_id] = true;
 	    }
 
-	//>> Search into config file
-	if(SYS->chkSelDB("<cfg>"))
-	    for( int lib_cnt = 0; SYS->db().at().dataSeek("",nodePath()+"lib/",lib_cnt++,c_el); )
-	    {
-		string l_id = c_el.cfg("ID").getS();
-		if(!lbPresent(l_id)) lbReg(new Lib(l_id.c_str(),"","*.*"));
-	    }
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            lbList(db_ls);
+            for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                if(itReg.find(db_ls[i_it]) == itReg.end() && SYS->chkSelDB(lbAt(db_ls[i_it]).at().DB()))
+                    lbUnreg(db_ls[i_it]);
+        }
     }catch(TError err)
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -343,12 +362,11 @@ BFunc *TipContr::bFuncGet( const char *nm )
 //* Contr: Controller object                      *
 //*************************************************
 Contr::Contr(string name_c, const string &daq_db, ::TElem *cfgelem) :
-    ::TController(name_c, daq_db, cfgelem), TValFunc(name_c.c_str(),NULL,false), prc_st(false), endrun_req(false),
+    ::TController(name_c, daq_db, cfgelem), TValFunc(name_c.c_str(),NULL,false), prc_st(false), call_st(false), endrun_req(false),
     mPrior(cfg("PRIOR").getId()), mIter(cfg("ITER").getId()), mSched(cfg("SCHEDULE").getSd()), mFnc(cfg("FUNC").getSd()),
-    id_freq(-1), id_start(-1), id_stop(-1)
+    id_freq(-1), id_start(-1), id_stop(-1), tm_calc(0)
 {
     cfg("PRM_BD").setS("JavaLikePrm_"+name_c);
-    setDimens(true);
 }
 
 Contr::~Contr()
@@ -375,13 +393,14 @@ void Contr::postDisable(int flag)
 
 string Contr::getStatus( )
 {
-    string val = TController::getStatus( );
+    string val = TController::getStatus();
 
-    if( startStat( ) && !redntUse( ) )
+    if(startStat() && !redntUse())
     {
-	if( period() ) val += TSYS::strMess(_("Call by period: %s. "),TSYS::time2str(1e-3*period()).c_str());
-	else val += TSYS::strMess(_("Call next by cron '%s'. "),TSYS::time2str(TSYS::cron(cron(),time(NULL)),"%d-%m-%Y %R").c_str());
-	val += TSYS::strMess(_("Spent time: %s."),TSYS::time2str(calcTm()).c_str());
+        if(call_st)	val += TSYS::strMess(_("Call now. "));
+	if(period())	val += TSYS::strMess(_("Call by period: %s. "),TSYS::time2str(1e-3*period()).c_str());
+	else val += TSYS::strMess(_("Call next by cron '%s'. "),TSYS::time2str(TSYS::cron(cron()),"%d-%m-%Y %R").c_str());
+	val += TSYS::strMess(_("Spent time: %s."),TSYS::time2str(tm_calc).c_str());
     }
 
     return val;
@@ -389,19 +408,19 @@ string Contr::getStatus( )
 
 void Contr::enable_( )
 {
-    if( !mod->lbPresent(TSYS::strSepParse(mFnc,0,'.')) )
+    if(!mod->lbPresent(TSYS::strSepParse(mFnc,0,'.')))
 	throw TError(nodePath().c_str(),_("Functions library '%s' is not present. Please, create functions library!"),TSYS::strSepParse(mFnc,0,'.').c_str());
-    if( !mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().present(TSYS::strSepParse(mFnc,1,'.')) )
+    if(!mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().present(TSYS::strSepParse(mFnc,1,'.')))
     {
-	mess_info(nodePath().c_str(),_("Create new function '%s'."),mFnc.c_str());
+	mess_info(nodePath().c_str(),_("Create new function '%s'."),mFnc.getVal().c_str());
 	mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().add(TSYS::strSepParse(mFnc,1,'.').c_str());
     }
-    setFunc( &mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().at(TSYS::strSepParse(mFnc,1,'.')).at() );
+    setFunc(&mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().at(TSYS::strSepParse(mFnc,1,'.')).at());
     try{ loadFunc( ); }
     catch(TError err)
     {
 	mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
-	mess_warning(nodePath().c_str(),_("Load function and its io error."));
+	mess_warning(nodePath().c_str(),_("Load function and its IO error."));
     }
 }
 
@@ -419,9 +438,9 @@ void Contr::load_( )
 
 void Contr::loadFunc( bool onlyVl )
 {
-    if( func() != NULL )
+    if(func() != NULL)
     {
-	if( !onlyVl ) ((Func *)func())->load();
+	if(!onlyVl) ((Func *)func())->load();
 
 	//> Creating special IO
 	if(func()->ioId("f_frq") < 0) func()->ioIns( new IO("f_frq",_("Function calculate frequency (Hz)"),IO::Real,Func::SysAttr,"1000",false),0);
@@ -434,10 +453,10 @@ void Contr::loadFunc( bool onlyVl )
 	string bd_tbl = TController::id()+"_val";
 	string bd = DB()+"."+bd_tbl;
 
-	for( int fld_cnt=0; SYS->db().at().dataSeek(bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg); )
+	for(int fld_cnt = 0; SYS->db().at().dataSeek(bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg); )
 	{
 	    int ioId = func()->ioId(cfg.cfg("ID").getS());
-	    if( ioId < 0 || func()->io(ioId)->flg()&Func::SysAttr ) continue;
+	    if(ioId < 0 || func()->io(ioId)->flg()&Func::SysAttr) continue;
 	    setS(ioId,cfg.cfg("VAL").getS());
 	}
     }
@@ -472,10 +491,10 @@ void Contr::save_( )
 
 	//> Clear VAL
 	cfg.cfgViewAll(false);
-	for( int fld_cnt=0; SYS->db().at().dataSeek(val_bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg); )
-	    if( ioId(cfg.cfg("ID").getS()) < 0 )
+	for(int fld_cnt = 0; SYS->db().at().dataSeek(val_bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg); )
+	    if(ioId(cfg.cfg("ID").getS()) < 0)
 	    {
-		SYS->db().at().dataDel(val_bd,mod->nodePath()+bd_tbl,cfg,true);
+		SYS->db().at().dataDel(val_bd, mod->nodePath()+bd_tbl, cfg, true);
 		fld_cnt--;
 	    }
     }
@@ -493,16 +512,16 @@ void Contr::start_( )
     if(id_this >= 0) setO(id_this,new TCntrNodeObj(AutoHD<TCntrNode>(this),"root"));
 
     //> Schedule process
-    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(long long)(1e9*atof(mSched.c_str()))) : 0;
+    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(int64_t)(1e9*atof(mSched.getVal().c_str()))) : 0;
 
     //> Start the request data task
-    if( !prc_st ) SYS->taskCreate( nodePath('.',true), mPrior, Contr::Task, this, &prc_st );
+    if(!prc_st) SYS->taskCreate(nodePath('.',true), mPrior, Contr::Task, this);
 }
 
 void Contr::stop_( )
 {
     //> Stop the request and calc data task
-    if( prc_st ) SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
+    if(prc_st) SYS->taskDestroy(nodePath('.',true), &endrun_req);
 }
 
 void *Contr::Task( void *icntr )
@@ -514,13 +533,16 @@ void *Contr::Task( void *icntr )
 
     bool is_start = true;
     bool is_stop  = false;
+    int64_t t_cnt, t_prev = TSYS::curTime();
 
     while(true)
     {
 	if(!cntr.redntUse())
 	{
+	    cntr.call_st = true;
+	    t_cnt = TSYS::curTime();
 	    //> Setting special IO
-	    if(cntr.id_freq >= 0) cntr.setR(cntr.id_freq, cntr.period()?(float)cntr.iterate()*1e9/(float)cntr.period():0);
+	    if(cntr.id_freq >= 0) cntr.setR(cntr.id_freq, cntr.period()?((float)cntr.iterate()*1e9/(float)cntr.period()):(-1e-6*(t_cnt-t_prev)));
 	    if(cntr.id_start >= 0) cntr.setB(cntr.id_start, is_start);
 	    if(cntr.id_stop >= 0) cntr.setB(cntr.id_stop, is_stop);
 
@@ -528,9 +550,12 @@ void *Contr::Task( void *icntr )
 		try { cntr.calc(); }
 		catch(TError err)
 		{
-		    mess_err(err.cat.c_str(),"%s",err.mess.c_str() ); 
-		    mess_err(cntr.nodePath().c_str(),_("Calc controller's function error."));
+		    mess_err(err.cat.c_str(),"%s",err.mess.c_str() );
+		    mess_err(cntr.nodePath().c_str(),_("Calculation controller's function error."));
 		}
+	    t_prev = t_cnt;
+            cntr.tm_calc = TSYS::curTime()-t_cnt;
+            cntr.call_st = false;
 	}
 
 	if(is_stop) break;
@@ -597,7 +622,7 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("fld",opt,-1,"/cntr/cfg/FUNC",cfg("FUNC").fld().descr(),RWRWR_,"root",SDAQ_ID,3,"tp","str","dest","sel_ed","select","/cntr/flst");
 	ctrMkNode("fld",opt,-1,"/cntr/cfg/SCHEDULE",cfg("SCHEDULE").fld().descr(),RWRWR_,"root",SDAQ_ID,4,
 	    "tp","str","dest","sel_ed","sel_list",TMess::labSecCRONsel(),"help",TMess::labSecCRON());
-	if(enableStat() && ctrMkNode("area",opt,-1,"/fnc",_("Calcing")))
+	if(enableStat() && ctrMkNode("area",opt,-1,"/fnc",_("Calculation")))
 	{
 	    if(ctrMkNode("table",opt,-1,"/fnc/io",_("Data"),RWRWR_,"root",SDAQ_ID,2,"s_com","add,del,ins,move","rows","15"))
 	    {
@@ -611,7 +636,7 @@ void Contr::cntrCmdProc( XMLNode *opt )
 		    "sel_list",_("Input;Output;Return"));
 		ctrMkNode("list",opt,-1,"/fnc/io/4",_("Value"),RWRWR_,"root",SDAQ_ID,1,"tp","str");
 	    }
-	    ctrMkNode("fld",opt,-1,"/fnc/prog",_("Programm"),RWRW__,"root",SDAQ_ID,3,"tp","str","rows","10","SnthHgl","1");
+	    ctrMkNode("fld",opt,-1,"/fnc/prog",_("Program"),RWRW__,"root",SDAQ_ID,3,"tp","str","rows","10","SnthHgl","1");
 	}
 	return;
     }
@@ -668,7 +693,7 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	    int row = atoi(opt->attr("row").c_str());
 	    if(func()->io(row)->flg()&Func::SysAttr)
 		throw TError(nodePath().c_str(),_("Deleting lock attribute in not allow."));
-	    ((Func *)func())->ioDel(row); 
+	    ((Func *)func())->ioDel(row);
 	    modif();
 	}
 	if(ctrChkNode(opt,"move",RWRWR_,"root",SDAQ_ID,SEC_WR))
@@ -846,6 +871,7 @@ void Prm::vlSet( TVal &val, const TVariant &pvl )
 		case TFld::Boolean:
 		    ((Contr &)owner()).setB(io_id,val.getB(0,true));
 		    break;
+		default: break;
 	    }
 	}
     }catch(TError err) { disable(); }
@@ -855,7 +881,7 @@ void Prm::vlGet( TVal &val )
 {
     if( val.name() == "err" )
     {
-	if( !owner().startStat() )	val.setS(_("2:Controller is stoped"),0,true);
+	if( !owner().startStat() )	val.setS(_("2:Controller is stopped"),0,true);
 	else if( !enableStat() )	val.setS(_("1:Parameter is disabled"),0,true);
 	else val.setS("0",0,true);
 	return;
@@ -877,6 +903,7 @@ void Prm::vlGet( TVal &val )
 		    val.setR(enableStat()?owner().getR(io_id):EVAL_REAL,0,true);break;
 		case TFld::Boolean:
 		    val.setB(enableStat()?owner().getB(io_id):EVAL_BOOL,0,true);break;
+		default: break;
 	    }
         }
     }catch(TError err) { disable(); }
@@ -900,7 +927,7 @@ void Prm::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("fld",opt,-1,"/prm/cfg/FLD",cfg("FLD").fld().descr(),RWRWR_,"root",SDAQ_ID,2,"SnthHgl","1",
 	    "help",_("Attributes configuration list. List must be written by lines in format: [<io>:<aid>:<anm>]\n"
 	    "Where:\n"
-	    "  io - calc function's IO;\n"
+	    "  io - calculation function's IO;\n"
 	    "  aid - created attribute identifier;\n"
 	    "  anm - created attribute name.\n"
 	    "If 'aid' or 'anm' are not set they will be generated from selected function's IO."));
