@@ -45,7 +45,7 @@ TProt::TProt(string name) : TProtocol(PRT_ID), mPrtLen(0)
     mType	= PRT_TYPE;
     mName	= PRT_NAME;
     mVers	= PRT_MVER;
-    mAutor	= PRT_AUTORS;
+    mAuthor	= PRT_AUTHORS;
     mDescr	= PRT_DESCR;
     mLicense	= PRT_LICENSE;
     mSource	= name;
@@ -62,8 +62,8 @@ TProt::TProt(string name) : TProtocol(PRT_ID), mPrtLen(0)
     mNodeEl.fldAdd(new TFld("PRT",_("Protocol"),TFld::String,TFld::Selected,"5","*","RTU;ASCII;TCP;*",_("RTU;ASCII;TCP/IP;All")));
     mNodeEl.fldAdd(new TFld("MODE",_("Mode"),TFld::Integer,TFld::Selected,"1","0","0;1;2",_("Data;Gateway node;Gateway net")));
     //>> For "Data" mode
-    mNodeEl.fldAdd(new TFld("DT_PER",_("Calc data period (s)"),TFld::Real,0,"5.3","1","0.001;99"));
-    mNodeEl.fldAdd(new TFld("DT_PROG",_("Programm"),TFld::String,TCfg::TransltText,"10000"));
+    mNodeEl.fldAdd(new TFld("DT_PER",_("Calculate data period (s)"),TFld::Real,0,"5.3","1","0.001;99"));
+    mNodeEl.fldAdd(new TFld("DT_PROG",_("Program"),TFld::String,TCfg::TransltText,"10000"));
     //>> For "Gateway" mode
     mNodeEl.fldAdd(new TFld("TO_TR",_("To transport"),TFld::String,0,"20"));
     mNodeEl.fldAdd(new TFld("TO_PRT",_("To protocol"),TFld::String,TFld::Selected,"5","RTU","RTU;ASCII;TCP",_("RTU;ASCII;TCP/IP")));
@@ -86,7 +86,6 @@ TProt::~TProt( )
 
 void TProt::nAdd(const string &iid, const string &db)
 {
-    if(chldPresent(mNode,iid)) return;
     chldAdd(mNode, new Node(iid,db,&nodeEl()));
 }
 
@@ -101,23 +100,27 @@ void TProt::load_( )
 	TConfig g_cfg(&nodeEl());
 	g_cfg.cfgViewAll(false);
 	vector<string> db_ls;
+	map<string, bool> itReg;
 
 	//>>> Search into DB
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+modId()+"_node","",fld_cnt++,g_cfg); )
+	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+modId()+"_node",nodePath()+modId()+"_node",fld_cnt++,g_cfg); )
 	    {
 		string id = g_cfg.cfg("ID").getS();
 		if(!nPresent(id)) nAdd(id,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+		itReg[id] = true;
 	    }
 
-	    //>>> Search into config file
-	if(SYS->chkSelDB("<cfg>"))
-	    for(int fld_cnt=0; SYS->db().at().dataSeek("",nodePath()+modId()+"_node",fld_cnt++,g_cfg); )
-	    {
-		string id = g_cfg.cfg("ID").getS();
-		if(!nPresent(id)) nAdd(id,"*.*");
-	    }
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            nList(db_ls);
+            for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                if(itReg.find(db_ls[i_it]) == itReg.end() && SYS->chkSelDB(nAt(db_ls[i_it]).at().DB()))
+                    nDel(db_ls[i_it]);
+        }
     }catch(TError err)
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -296,6 +299,7 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 		while(rez.size() < (resp_sz+6))
 		{
 		    resp_len = tro.messIO(NULL, 0, buf, sizeof(buf), reqTm, true);
+		    if(!resp_len) throw TError(nodePath().c_str(),_("Not full respond"));
 		    rez.append(buf, resp_len);
 		}
 		pdu = rez.substr(7);
@@ -316,10 +320,10 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 		int resp_len = tro.messIO(mbap.data(), mbap.size(), buf, sizeof(buf), reqTm, true);
 		rez.assign(buf, resp_len);
 		//> Wait tail
-		while(true)
+		while(resp_len)
 		{
 		    try{ resp_len = tro.messIO(NULL, 0, buf, sizeof(buf), 0, true); } catch(TError err){ break; }
-		    rez.append( buf, resp_len );
+		    rez.append(buf, resp_len);
 		}
 
 		if(rez.size() < 2) { err = _("13:Error respond: Too short."); continue; }
@@ -336,7 +340,7 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 	    mbap += (uint8_t)node;		//Unit identifier
 	    mbap += pdu;
 	    mbap += LRC(mbap);
-	    mbap = ":"+DataToASCII(mbap)+"\r\n";
+	    mbap = ":"+DataToASCII(mbap)+"\x0D\x0A";
 
 	    //> Send request
 	    for(int i_tr = 0; i_tr < reqTry; i_tr++)
@@ -344,13 +348,13 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 		int resp_len = tro.messIO(mbap.data(), mbap.size(), buf, sizeof(buf), reqTm, true);
 		rez.assign(buf, resp_len);
 		//> Wait tail
-		while(rez.size() < 3 || rez.substr(rez.size()-2,2) != "\r\n")
+		while(resp_len && (rez.size() < 3 || rez.substr(rez.size()-2,2) != "\x0D\x0A"))
 		{
 		    try{ resp_len = tro.messIO(NULL, 0, buf, sizeof(buf), 0, true); } catch(TError err){ break; }
 		    rez.append(buf, resp_len);
 		}
 
-		if(rez.size() < 3 || rez[0] != ':' || rez.substr(rez.size()-2,2) != "\r\n")
+		if(rez.size() < 3 || rez[0] != ':' || rez.substr(rez.size()-2,2) != "\x0D\x0A")
 		{ err = _("13:Error respond: Error format."); continue; }
 		string rezEnc = ASCIIToData(rez.substr(1,rez.size()-3));
 		if(LRC(rezEnc.substr(0,rezEnc.size()-1)) != (uint8_t)rezEnc[rezEnc.size()-1])
@@ -375,7 +379,7 @@ void TProt::outMess( XMLNode &io, TTransportOut &tro )
 		    case 0x4: err = _("4:Server failure.");				break;
 		    case 0x5: err = _("5:Request requires too long time for execute.");	break;
 		    case 0x6: err = _("6:Server is busy.");				break;
-		    case 0x7: err = _("7:Programm function is error. By request functions 13 or 14.");	break;
+		    case 0x7: err = _("7:Program function is error. By request functions 13 or 14.");	break;
 		    case 0xA: case 0xB: err = _("10:Gateway problem.");			break;
 		    default: err = TSYS::strMess(_("12:%02X:Unknown error."),(unsigned char)(pdu[1]));	break;
 		}
@@ -494,7 +498,7 @@ bool TProtIn::mess( const string &ireqst, string &answer, const string &sender )
 
 retry:
     //>> ASCII check
-    if(reqst.size() > 3 && reqst[0] == ':' && reqst.substr(reqst.size()-2,2) == "\r\n")
+    if(reqst.size() > 3 && reqst[0] == ':' && reqst.substr(reqst.size()-2,2) == "\x0D\x0A")
     {
 	prt = "ASCII";
 	string req = modPrt->ASCIIToData(reqst.substr(1,reqst.size()-3));
@@ -568,7 +572,7 @@ retry:
 	answer += (uint8_t)node;		//Unit identifier
 	answer += pdu;
 	answer += modPrt->LRC(answer);
-	answer = ":"+modPrt->DataToASCII(answer)+"\r\n";
+	answer = ":"+modPrt->DataToASCII(answer)+"\x0D\x0A";
     }
 
     if(owner().prtLen( ) && prt.size() && answer.size())
@@ -648,7 +652,11 @@ void Node::postDisable( int flag )
 
 TProt &Node::owner( )		{ return *(TProt*)nodePrev(); }
 
-string Node::name( )		{ return mName.size() ? mName : id(); }
+string Node::name( )
+{
+    string tNm = mName;
+    return tNm.size() ? tNm : id();
+}
 
 string Node::tbl( )		{ return owner().modId()+"_node"; }
 
@@ -780,10 +788,10 @@ void Node::save_( )
     }
     //> Clear IO
     cfg.cfgViewAll(false);
-    for( int fld_cnt=0; SYS->db().at().dataSeek(fullDB()+"_io",owner().nodePath()+tbl()+"_io",fld_cnt++,cfg); )
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(fullDB()+"_io",owner().nodePath()+tbl()+"_io",fld_cnt++,cfg); )
     {
-	string sio = cfg.cfg("ID").getS( );
-	if( ioId(sio) < 0 || io(ioId(sio))->flg()&Node::LockAttr )
+	string sio = cfg.cfg("ID").getS();
+	if(ioId(sio) < 0 || io(ioId(sio))->flg()&Node::LockAttr)
 	{
 	    SYS->db().at().dataDel(fullDB()+"_io",owner().nodePath()+tbl()+"_io",cfg,true);
 	    fld_cnt--;
@@ -812,7 +820,7 @@ void Node::setEnable( bool vl )
 	    else
 	    {
 		string mWorkProg = SYS->daq().at().at(TSYS::strSepParse(progLang(),0,'.')).at().compileFunc(TSYS::strSepParse(progLang(),1,'.'),*this,prog());
-		data->val.setFunc(&((AutoHD<TFunction>)SYS->nodeAt(mWorkProg,1)).at());
+		data->val.setFunc(&((AutoHD<TFunction>)SYS->nodeAt(mWorkProg)).at());
 	    }
 	}catch( TError err )
 	{ mess_err(nodePath().c_str(),_("Compile function by language '%s' error: %s"),progLang().c_str(),err.mess.c_str()); throw; }
@@ -850,16 +858,16 @@ void Node::setEnable( bool vl )
 	}
 
 	//>> Start task
-	SYS->taskCreate( nodePath('.',true), 0, Task, this, &prcSt );
+	SYS->taskCreate(nodePath('.',true), 0, Task, this);
     }
     //> Disable node
-    if( !vl )
+    if(!vl)
     {
 	//> Stop the calc data task
-	if( prcSt ) SYS->taskDestroy( nodePath('.',true), &prcSt, &endrunRun );
+	if(prcSt) SYS->taskDestroy(nodePath('.',true), &endrunRun);
 
 	//> Data structure delete
-	if( data ) { delete data; data = NULL; }
+	if(data) { delete data; data = NULL; }
     }
 
     mEn = vl;
@@ -906,7 +914,7 @@ bool Node::req( const string &itr, const string &iprt, unsigned char inode, stri
 	    {
 		int c_sz = 0;
 		if( pdu.size() == 5 ) c_sz = ((unsigned short)(pdu[3]<<8)|(unsigned char)pdu[4]);
-		if( c_sz < 1 || c_sz > 2000 ) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if( c_sz < 1 || c_sz > 2000 ) { pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
 		int c_addr = ((unsigned short)(pdu[1]<<8)|(unsigned char)pdu[2]);
 		pdu.assign(1,pdu[0]);
 		pdu += (char)(c_sz/8+((c_sz%8)?1:0));
@@ -927,7 +935,7 @@ bool Node::req( const string &itr, const string &iprt, unsigned char inode, stri
 	    {
 		int r_sz = 0;
 		if( pdu.size() == 5 ) r_sz = ((unsigned short)(pdu[3]<<8)|(unsigned char)pdu[4]);
-		if( r_sz < 1 || r_sz > 125 ) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if( r_sz < 1 || r_sz > 125 ) { pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
 		int r_addr = ((unsigned short)(pdu[1]<<8)|(unsigned char)pdu[2]);
 		pdu.assign(1,pdu[0]);
 		pdu += (char)(r_sz*2);
@@ -948,7 +956,7 @@ bool Node::req( const string &itr, const string &iprt, unsigned char inode, stri
 	    }
 	    case 0x05:	//Write single coil
 	    {
-		if(pdu.size() != 5) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if(pdu.size() != 5) { pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
 		int c_addr = ((unsigned short)(pdu[1]<<8)|(unsigned char)pdu[2]);
 
 		map<int,int>::iterator ic = data->coil.find(-c_addr);
@@ -966,7 +974,7 @@ bool Node::req( const string &itr, const string &iprt, unsigned char inode, stri
 	    }
 	    case 0x06:	//Write single register
 	    {
-		if(pdu.size() != 5) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if(pdu.size() != 5) { pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
 		int r_addr = ((unsigned short)(pdu[1]<<8)|(unsigned char)pdu[2]);
 
 		map<int,int>::iterator ir = data->reg.find(-r_addr);
@@ -985,11 +993,11 @@ bool Node::req( const string &itr, const string &iprt, unsigned char inode, stri
 	    }
 	    case 0x0F:	//Write multiple coils
 	    {
-		if(pdu.size() < 6) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if(pdu.size() < 6) { pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
 		int c_aSt = ((unsigned short)(pdu[1]<<8)|(unsigned char)pdu[2]);
 		int c_aCnt = ((unsigned short)(pdu[3]<<8)|(unsigned char)pdu[4]);
 		int bCnt = (unsigned char)pdu[5];
-		if((int)pdu.size() != (6+bCnt) || bCnt < (c_aCnt/8)) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if((int)pdu.size() != (6+bCnt) || bCnt < (c_aCnt/8)) { pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
 		bool noWrReg = false;
 		for(int i_c = 0; i_c < c_aCnt; i_c++)
 		{
@@ -1010,11 +1018,13 @@ bool Node::req( const string &itr, const string &iprt, unsigned char inode, stri
 	    }
 	    case 0x10:	//Write multiple register
 	    {
-		if(pdu.size() < 6) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if(pdu.size() < 6) { pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
 		int r_aSt = ((unsigned short)(pdu[1]<<8)|(unsigned char)pdu[2]);
 		int r_aCnt = ((unsigned short)(pdu[3]<<8)|(unsigned char)pdu[4]);
 		int bCnt = (unsigned char)pdu[5];
-		if((int)pdu.size() != (6+bCnt) || bCnt < (r_aCnt*2)) { pdu.assign(1,pdu[0]|0x80); pdu += 0x1; return true; }
+		if((int)pdu.size() != (6+bCnt) || bCnt < (r_aCnt*2) || r_aCnt > 123)
+		{ pdu.assign(1,pdu[0]|0x80); pdu += 0x3; return true; }
+		ResAlloc res(data->val.func()->fRes(), true);
 		bool noWrReg = false;
 		for(int i_r = 0; i_r < r_aCnt; i_r++)
 		{
@@ -1080,7 +1090,7 @@ void *Node::Task( void *ind )
     {
 	if(SYS->daq().at().subStartStat())
 	{
-	    long long t_cnt = TSYS::curTime();
+	    int64_t t_cnt = TSYS::curTime();
 
 	    //> Setting special IO
 	    if(ioFrq >= 0) nd.data->val.setR(ioFrq,(float)1/nd.period());
@@ -1134,7 +1144,7 @@ void *Node::Task( void *ind )
 	    catch(TError err)
 	    {
 		mess_err(err.cat.c_str(),"%s",err.mess.c_str() );
-		mess_err(nd.nodePath().c_str(),_("Calc node's function error."));
+		mess_err(nd.nodePath().c_str(),_("Calculate node's function error."));
 	    }
 
 	    //> Calc acquisition process time
@@ -1142,7 +1152,7 @@ void *Node::Task( void *ind )
 	}
 
 	if(isStop) break;
-	TSYS::taskSleep((long long)(1e9*nd.period()));
+	TSYS::taskSleep((int64_t)(1e9*nd.period()));
 	if(nd.endrunRun) isStop = true;
 	isStart = false;
 	nd.modif();
@@ -1170,7 +1180,7 @@ void Node::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/nd/st/db",_("DB"),RWRWR_,"root",SPRT_ID,4,
 		    "tp","str","dest","select","select","/db/list","help",TMess::labDB());
 	    }
-	    if(ctrMkNode("area",opt,-1,"/nd/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/nd/cfg",_("Configuration")))
 	    {
 		TConfig::cntrCmdMake(opt,"/nd/cfg",0,"root",SPRT_ID,RWRWR_);
 		//>> Append configuration properties
@@ -1194,8 +1204,8 @@ void Node::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/dt/io/lnk",_("Link"),RWRWR_,"root",SPRT_ID,1,"tp","bool");
 		ctrMkNode("list",opt,-1,"/dt/io/vl",_("Value"),RWRWR_,"root",SPRT_ID,1,"tp","str");
 	    }
-	    ctrMkNode("fld",opt,-1,"/dt/progLang",_("Programm language"),RWRWR_,"root",SPRT_ID,3,"tp","str","dest","sel_ed","select","/dt/plang_ls");
-	    ctrMkNode("fld",opt,-1,"/dt/prog",_("Programm"),RWRWR_,"root",SPRT_ID,3,"tp","str","rows","10","SnthHgl","1");
+	    ctrMkNode("fld",opt,-1,"/dt/progLang",_("Program language"),RWRWR_,"root",SPRT_ID,3,"tp","str","dest","sel_ed","select","/dt/plang_ls");
+	    ctrMkNode("fld",opt,-1,"/dt/prog",_("Program"),RWRWR_,"root",SPRT_ID,3,"tp","str","rows","10","SnthHgl","1");
 	}
 	if(mode() == 0 && ctrMkNode("area",opt,-1,"/lnk",_("Links")))
 	    for(int i_io = 0; i_io < ioSize(); i_io++)
