@@ -48,7 +48,7 @@ TTpContr::TTpContr( string name ) : TTipDAQ(DAQ_ID)
     mName	= DAQ_NAME;
     mType	= DAQ_TYPE;
     mVers	= DAQ_MVER;
-    mAutor	= DAQ_AUTOR;
+    mAuthor	= DAQ_AUTOR;
     mDescr	= DAQ_DESCR;
     mLicense	= DAQ_LICENSE;
     mSource	= name;
@@ -75,8 +75,8 @@ void TTpContr::postEnable( int flag )
     TTipDAQ::postEnable(flag);
 
     //> Controler's bd structure
-    fldAdd( new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30","") );
-    fldAdd( new TFld("SCHEDULE",_("Calc schedule"),TFld::String,TFld::NoFlag,"100","1") );
+    fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"30","") );
+    fldAdd( new TFld("SCHEDULE",_("Acquisition schedule"),TFld::String,TFld::NoFlag,"100","1") );
     fldAdd( new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99") );
     fldAdd( new TFld("SYNCPER",_("Sync inter remote station period (s)"),TFld::Real,TFld::NoFlag,"6.2","60","0;1000") );
     fldAdd( new TFld("ADDR",_("Transport address"),TFld::String,TFld::NoFlag,"30","") );
@@ -105,7 +105,7 @@ TMdContr::TMdContr(string name_c, const string &daq_db, ::TElem *cfgelem) :
     mPrior(cfg("PRIOR").getId()), mSync(cfg("SYNCPER").getRd()), mSched(cfg("SCHEDULE").getSd()),
     mAddr(cfg("ADDR").getSd()), mEndPoint(cfg("EndPoint").getSd()), mSecPolicy(cfg("SecPolicy").getSd()),
     mSecMessMode(cfg("SecMessMode").getId()), mPAttrLim(cfg("AttrsLimit").getId()),
-    prc_st(false), endrun_req(false), mPCfgCh(false), mBrwsVar(_("Root folder (84)")), tm_gath(0), tmDelay(0), servSt(0)
+    prc_st(false), call_st(false), endrun_req(false), mPCfgCh(false), mBrwsVar(_("Root folder (84)")), tm_gath(0), tmDelay(0), servSt(0)
 {
     cfg("PRM_BD").setS("OPC_UA_Prm_"+name_c);
 }
@@ -123,7 +123,7 @@ string TMdContr::getStatus( )
 {
     string rez = TController::getStatus( );
 
-    if(startStat( ) && !redntUse())
+    if(startStat() && !redntUse())
     {
 	if(tmDelay > -1)
 	{
@@ -132,10 +132,11 @@ string TMdContr::getStatus( )
 	}
 	else
 	{
-	    if( period() ) rez += TSYS::strMess(_("Call by period: %s. "),TSYS::time2str(1e-3*period()).c_str());
-	    else rez += TSYS::strMess(_("Call next by cron '%s'. "),TSYS::time2str(TSYS::cron(cron(),time(NULL)),"%d-%m-%Y %R").c_str());
+	    if(call_st)	rez += TSYS::strMess(_("Call now. "));
+	    if(period())rez += TSYS::strMess(_("Call by period: %s. "),TSYS::time2str(1e-3*period()).c_str());
+	    else rez += TSYS::strMess(_("Call next by cron '%s'. "),TSYS::time2str(TSYS::cron(cron()),"%d-%m-%Y %R").c_str());
 	    rez += TSYS::strMess(_("Spent time: %s. Requests %.6g."),TSYS::time2str(tm_gath).c_str(),-tmDelay);
-	    if( servSt ) rez.replace(0,1,TSYS::strMess("0x%x",servSt));
+	    if(servSt) rez.replace(0,1,TSYS::strMess("0x%x",servSt));
 	}
     }
     return rez;
@@ -300,18 +301,18 @@ void TMdContr::start_( )
     catch( TError err ){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 
     //> Schedule process
-    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(long long)(1e9*atof(mSched.c_str()))) : 0;
+    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(int64_t)(1e9*atof(mSched.getVal().c_str()))) : 0;
 
     tmDelay = 0;
 
     //> Start the gathering data task
-    if( !prc_st ) SYS->taskCreate( nodePath('.',true), mPrior, TMdContr::Task, this, &prc_st );
+    if(!prc_st) SYS->taskCreate(nodePath('.',true), mPrior, TMdContr::Task, this);
 }
 
 void TMdContr::stop_( )
 {
     //> Stop the request and calc data task
-    SYS->taskDestroy( nodePath('.',true), &prc_st, &endrun_req );
+    SYS->taskDestroy(nodePath('.',true), &endrun_req);
 }
 
 void TMdContr::prmEn( const string &id, bool val )
@@ -343,10 +344,11 @@ void *TMdContr::Task( void *icntr )
     {
 	for(unsigned int it_cnt = cntr.p_hd.size(); !cntr.endrun_req; it_cnt++)
 	{
-	    if(cntr.redntUse())	{ sleep(1); continue; }
-	    if(cntr.tmDelay > 0){ sleep(1); cntr.tmDelay = vmax(0,cntr.tmDelay-1); continue; }
+	    if(cntr.redntUse())	{ usleep(1000000); continue; }
+	    if(cntr.tmDelay > 0){ usleep(1000000); cntr.tmDelay = vmax(0,cntr.tmDelay-1); continue; }
 
-	    long long t_cnt = TSYS::curTime();
+	    int64_t t_cnt = TSYS::curTime();
+	    cntr.call_st = true;
 	    unsigned int div = cntr.period() ? (unsigned int)(cntr.mSync/(1e-9*cntr.period())) : 0;
 
 	    ResAlloc res(cntr.en_res,false);
@@ -405,6 +407,7 @@ void *TMdContr::Task( void *icntr )
 
 	    firstCall = false;
 	    cntr.tm_gath = TSYS::curTime()-t_cnt;
+	    cntr.call_st = false;
 
 	    TSYS::taskSleep(cntr.period(),cntr.period()?0:TSYS::cron(cntr.cron()));
 	}
@@ -537,9 +540,9 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
 		    char cRW = atoi(nAVl.c_str());
 		    nAVl = "";
 		    if(cRW & TProt::ACS_Read)		nAVl += _("Readable, ");
-		    if(cRW & TProt::ACS_Write)		nAVl += _("Writible, ");
+		    if(cRW & TProt::ACS_Write)		nAVl += _("Writable, ");
 		    if(cRW & TProt::ACS_HistRead)	nAVl += _("History readable, ");
-		    if(cRW & TProt::ACS_HistWrite)	nAVl += _("History writible, ");
+		    if(cRW & TProt::ACS_HistWrite)	nAVl += _("History writable, ");
 		    if(cRW & TProt::ACS_SemChange)	nAVl += _("Semantic change, ");
 		    break;
 		}
@@ -549,9 +552,9 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
 		    char cRW = atoi(nAVl.c_str());
 		    nAVl = "";
 		    if(cRW & TProt::ACS_Read)		nAVl += _("Readable, ");
-		    if(cRW & TProt::ACS_Write)		nAVl += _("Writible, ");
+		    if(cRW & TProt::ACS_Write)		nAVl += _("Writable, ");
 		    if(cRW & TProt::ACS_HistRead)	nAVl += _("History readable, ");
-		    if(cRW & TProt::ACS_HistWrite)	nAVl += _("History writible, ");
+		    if(cRW & TProt::ACS_HistWrite)	nAVl += _("History writable, ");
 		    break;
 		}
 		case TProt::AId_MinimumSamplingInterval:nANm = _("MinimumSamplingInterval");	break;
@@ -761,7 +764,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	TParamContr::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/prm/cfg/ND_LS",cfg("ND_LS").fld().descr(),RWRWR_,"root",SDAQ_ID,3,"rows","8","SnthHgl","1",
 	    "help",_("Variables and it containers (Objects) list. All variables will put into the parameter attributes list.\n"
-		"Variables writed by separated lines into format: [ns:id].\n"
+		"Variables wrote by separated lines into format: [ns:id].\n"
 		"Where:\n"
 		"  ns - names scope, number, zero value can missed;\n"
 		"  id - node identifier, number, string, bytes string and GUID.\n"
@@ -791,7 +794,7 @@ void TMdPrm::vlGet( TVal &val )
     if(!enableStat() || !owner().startStat())
     {
 	if(!enableStat())		val.setS(_("1:Parameter is disabled."),0,true);
-	else if(!owner().startStat())	val.setS(_("2:Acquisition is stoped."),0,true);
+	else if(!owner().startStat())	val.setS(_("2:Acquisition is stopped."),0,true);
 	return;
     }
     if(owner().redntUse()) return;
