@@ -43,9 +43,9 @@
 #define MOD_NAME	_("SNMP client")
 #define MOD_TYPE	SDAQ_ID
 #define VER_TYPE	SDAQ_VER
-#define MOD_VER		"0.6.0"
-#define AUTORS		_("Roman Savochenko")
-#define DESCRIPTION	_("Allow realising of SNMP client service.")
+#define MOD_VER		"0.6.1"
+#define AUTHORS		_("Roman Savochenko")
+#define DESCRIPTION	_("Allow realization of SNMP client service.")
 #define LICENSE		"GPL2"
 //*************************************************
 
@@ -78,7 +78,7 @@ TTpContr::TTpContr( string name ) : TTipDAQ(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -104,7 +104,7 @@ void TTpContr::postEnable(int flag)
 
     //> Controler's bd structure
     fldAdd(new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30",""));
-    fldAdd(new TFld("SCHEDULE",_("Calc schedule"),TFld::String,TFld::NoFlag,"100","1"));
+    fldAdd(new TFld("SCHEDULE",_("Acquisition schedule"),TFld::String,TFld::NoFlag,"100","1"));
     fldAdd(new TFld("PRIOR",_("Gather task priority"),TFld::Integer,TFld::NoFlag,"2","0","-1;99"));
     fldAdd(new TFld("ADDR",_("Remote host address"),TFld::String,TFld::NoFlag,"30","localhost"));
     fldAdd(new TFld("RETR",_("Retries"),TFld::Integer,TFld::NoFlag,"1","1","0;10"));
@@ -112,7 +112,7 @@ void TTpContr::postEnable(int flag)
     fldAdd(new TFld("VER",_("SNMP version"),TFld::String,TFld::Selected,"2","1","1;2c;2u;3","SNMPv1;SNMPv2c;SNMPv2u;SNMPv3"));
     fldAdd(new TFld("COMM",_("Server community/user"),TFld::String,TFld::NoFlag,"20","public"));
     fldAdd(new TFld("V3",_("V3 parameters"),TFld::String,TFld::NoFlag,"50","authNoPriv:MD5::DES:"));
-    fldAdd(new TFld("PATTR_LIM",_("Param's attributes limit"),TFld::Integer,TFld::NoFlag,"3","100","10;10000"));
+    fldAdd(new TFld("PATTR_LIM",_("Parameter's attributes limit"),TFld::Integer,TFld::NoFlag,"3","100","10;10000"));
 
     //> Parameter type bd structure
     int t_prm = tpParmAdd("std", "PRM_BD", _("Standard"));
@@ -131,7 +131,7 @@ TMdContr::TMdContr(string name_c, const string &daq_db, ::TElem *cfgelem) :
     ::TController(name_c,daq_db,cfgelem),
     m_prior(cfg("PRIOR").getId()), m_pattr_lim(cfg("PATTR_LIM").getId()), m_retr(cfg("RETR").getId()), m_tm(cfg("TM").getId()),
     mSched(cfg("SCHEDULE").getSd()), m_addr(cfg("ADDR").getSd()), m_ver(cfg("VER").getSd()), m_comm(cfg("COMM").getSd()),
-    m_V3(cfg("V3").getSd()), prc_st(false), endrun_req(false), tm_gath(0)
+    m_V3(cfg("V3").getSd()), prc_st(false), call_st(false), endrun_req(false), tm_gath(0)
 {
     cfg("PRM_BD").setS("SNMPPrm_"+name_c);
 }
@@ -149,8 +149,9 @@ string TMdContr::getStatus( )
 	if(!acq_err.getVal().empty())	rez = acq_err.getVal();
 	else
 	{
-	    if(period()) rez += TSYS::strMess(_("Call by period: %s. "),TSYS::time2str(1e-3*period()).c_str());
-            else rez += TSYS::strMess(_("Call next by cron '%s'. "),TSYS::time2str(TSYS::cron(cron(),time(NULL)),"%d-%m-%Y %R").c_str());
+	    if(call_st)	rez += TSYS::strMess(_("Call now. "));
+	    if(period())rez += TSYS::strMess(_("Call by period: %s. "),TSYS::time2str(1e-3*period()).c_str());
+            else rez += TSYS::strMess(_("Call next by cron '%s'. "),TSYS::time2str(TSYS::cron(cron()),"%d-%m-%Y %R").c_str());
             rez += TSYS::strMess(_("Spent time: %s."),TSYS::time2str(tm_gath).c_str());
         }
     }
@@ -220,15 +221,15 @@ TParamContr *TMdContr::ParamAttach(const string &name, int type)
 void TMdContr::start_( )
 {
     //> Schedule process
-    mPer = TSYS::strSepParse(mSched,1,' ').empty() ? vmax(0,(long long)(1e9*atof(mSched.c_str()))) : 0;
+    mPer = TSYS::strSepParse(mSched, 1, ' ').empty() ? vmax(0,(int64_t)(1e9*atof(mSched.getVal().c_str()))) : 0;
 
     //> Session init
     snmp_sess_init(&session);
     session.version = SNMP_VERSION_1;
-    if(m_ver == "1")		session.version = SNMP_VERSION_1;
-    else if(m_ver == "2c")	session.version = SNMP_VERSION_2c;
-    else if(m_ver == "2u")	session.version = SNMP_VERSION_2u;
-    else if(m_ver == "3")	session.version = SNMP_VERSION_3;
+    if(m_ver.getVal() == "1")		session.version = SNMP_VERSION_1;
+    else if(m_ver.getVal() == "2c")	session.version = SNMP_VERSION_2c;
+    else if(m_ver.getVal() == "2u")	session.version = SNMP_VERSION_2u;
+    else if(m_ver.getVal() == "3")	session.version = SNMP_VERSION_3;
     w_addr = TSYS::strParse(m_addr, 0, ":");
     session.peername = (char *)w_addr.c_str();
     session.retries = m_retr;
@@ -271,12 +272,14 @@ void TMdContr::start_( )
 
 	if(session.securityLevel == SNMP_SEC_LEVEL_AUTHPRIV)
 	{
+#ifdef USM_PRIV_PROTO_AES_LEN
 	    if(secPrivProto() == "AES")
 	    {
 		session.securityPrivProto = usmAESPrivProtocol;
 		session.securityPrivProtoLen = sizeof(usmAESPrivProtocol)/sizeof(oid);
 	    }
 	    else
+#endif
 	    {
 		session.securityPrivProto = usmDESPrivProtocol;
 		session.securityPrivProtoLen = sizeof(usmDESPrivProtocol)/sizeof(oid);
@@ -291,13 +294,13 @@ void TMdContr::start_( )
     }
 
     //> Start the gathering data task
-    if(!prc_st) SYS->taskCreate(nodePath('.',true), m_prior, TMdContr::Task, this, &prc_st);
+    if(!prc_st) SYS->taskCreate(nodePath('.',true), m_prior, TMdContr::Task, this);
 }
 
 void TMdContr::stop_( )
 {
     //> Stop the request and calc data task
-    if(prc_st) SYS->taskDestroy(nodePath('.',true), &prc_st, &endrun_req);
+    if(prc_st) SYS->taskDestroy(nodePath('.',true), &endrun_req);
 }
 
 void TMdContr::prmEn(const string &id, bool val)
@@ -333,7 +336,8 @@ void *TMdContr::Task(void *icntr)
 
     while(!cntr.endrun_req)
     {
-	long long t_cnt = TSYS::curTime();
+	cntr.call_st = true;
+	int64_t t_cnt = TSYS::curTime();
 
 	//>> Update controller's data
 	daqerr.clear();
@@ -476,6 +480,7 @@ void *TMdContr::Task(void *icntr)
 	    catch(TError err) { daqerr = err.mess; }
 	cntr.en_res.resRelease();
 	cntr.tm_gath = TSYS::curTime()-t_cnt;
+	cntr.call_st = false;
 
 	cntr.acq_err.setVal(daqerr);
 
@@ -519,7 +524,7 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
         ctrMkNode("fld",opt,-1,"/cntr/cfg/COMM",cfg("COMM").fld().descr(),RWRWR_,"root",SDAQ_ID,2,"tp","str",
     	    "help",_("Community group or user."));
 	ctrRemoveNode(opt,"/cntr/cfg/V3");
-	if(m_ver == "3")
+	if(m_ver.getVal() == "3")
 	{
     	    ctrMkNode("fld",opt,-1,"/cntr/cfg/SecLev",_("Security level"),RWRWR_,"root",SDAQ_ID,5,"tp","str","idm","1","dest","select",
         	"sel_id","noAurhNoPriv;authNoPriv;authPriv","sel_list",_("No auth/No privacy;Auth/No privacy;Auth/Privacy"));
@@ -530,7 +535,12 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
 	    }
 	    if(secLev() == "authPriv")
 	    {
-		ctrMkNode("fld",opt,-1,"/cntr/cfg/PrivProto",_("Privacy"),RWRWR_,"root",SDAQ_ID,3,"tp","str","dest","select","sel_list","DES;AES");
+#ifdef USM_PRIV_PROTO_AES_LEN
+		const char *prtLs = "DES;AES";
+#else
+		const char *prtLs = "DES";
+#endif
+		ctrMkNode("fld",opt,-1,"/cntr/cfg/PrivProto",_("Privacy"),RWRWR_,"root",SDAQ_ID,3,"tp","str","dest","select","sel_list",prtLs);
 		ctrMkNode("fld",opt,-1,"/cntr/cfg/PrivPass","",RWRWR_,"root",SDAQ_ID,1,"tp","str");
 	    }
 	}
@@ -598,6 +608,11 @@ void TMdPrm::enable( )
 
     owner().prmEn(id(), true);
 
+    //> Remove all OID's attributes
+    for(unsigned i_p = 0; i_p < p_el.fldSize(); )
+	try{ p_el.fldDel(i_p); }
+        catch(TError err) { i_p++; }
+
     parseOIDList(m_oid);
 }
 
@@ -660,12 +675,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     }
 
     //> Process command to page
-    if(a_path == "/prm/cfg/OID_LS" && ctrChkNode(opt,"set",RWRWR_,"root",SDAQ_ID,SEC_WR))
-    {
-	if(enableStat()) throw TError(nodePath().c_str(),_("Parameter is enabled."));
-	parseOIDList(opt->text());
-    }
-    else TParamContr::cntrCmdProc(opt);
+    TParamContr::cntrCmdProc(opt);
 }
 
 void TMdPrm::vlSet( TVal &valo, const TVariant &pvl )
