@@ -19,11 +19,13 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include <string.h>
 #include <getopt.h>
 #include <signal.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
+#include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <arpa/inet.h>
 #include <openssl/rand.h>
@@ -40,8 +42,8 @@
 #define MOD_NAME	_("SSL")
 #define MOD_TYPE	STR_ID
 #define VER_TYPE	STR_VER
-#define MOD_VER		"0.9.6"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"1.0.0"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Allow security socket layer based transport. Used OpenSSL and supported SSLv2, SSLv3 and TLSv1.")
 #define LICENSE		"GPL2"
 //************************************************
@@ -76,7 +78,7 @@ TTransSock::TTransSock( string name ) : TTipTransport(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -115,8 +117,8 @@ void TTransSock::postEnable( int flag )
 
     if( flag&TCntrNode::NodeConnect )
     {
-	owner().inEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
-	owner().outEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
+	owner().inEl().fldAdd( new TFld("A_PRMS",_("Addition parameters"),TFld::String,TFld::FullText,"10000") );
+	owner().outEl().fldAdd( new TFld("A_PRMS",_("Addition parameters"),TFld::String,TFld::FullText,"10000") );
     }
 }
 
@@ -243,7 +245,7 @@ void TSocketIn::start()
     connNumb = clsConnByLim = 0;
 
     //> Wait connection main task start
-    SYS->taskCreate( nodePath('.',true), taskPrior(), Task, this, &run_st );
+    SYS->taskCreate(nodePath('.',true), taskPrior(), Task, this);
 }
 
 void TSocketIn::stop()
@@ -256,7 +258,7 @@ void TSocketIn::stop()
     connNumb = clsConnByLim = 0;
 
     //> Wait connection main task stop
-    SYS->taskDestroy( nodePath('.',true), &run_st, &endrun );
+    SYS->taskDestroy(nodePath('.',true), &endrun);
 }
 
 void *TSocketIn::Task( void *sock_in )
@@ -285,9 +287,12 @@ void *TSocketIn::Task( void *sock_in )
     SSL_METHOD *meth;
 #endif
     meth = SSLv23_server_method();
-    if(ssl_method == "SSLv2")		meth = SSLv2_server_method();
-    else if(ssl_method == "SSLv3")	meth = SSLv3_server_method();
-    else if(ssl_method == "TLSv1")	meth = TLSv1_server_method();
+
+#ifndef OPENSSL_NO_SSL2
+    if(ssl_method == "SSLv2")	meth = SSLv2_server_method();
+#endif
+    if(ssl_method == "SSLv3")	meth = SSLv3_server_method();
+    if(ssl_method == "TLSv1")	meth = TLSv1_server_method();
 
     try
     {
@@ -301,7 +306,7 @@ void *TSocketIn::Task( void *sock_in )
 	//> Write certificate and private key to temorary file
 	cfile = tmpnam(err);
 	int icfile = open(cfile.c_str(),O_EXCL|O_CREAT|O_WRONLY,0644);
-	if(icfile < 0) throw TError(s.nodePath().c_str(),_("Open temporaty file '%s' error: '%s'"),cfile.c_str(),strerror(errno));
+	if(icfile < 0) throw TError(s.nodePath().c_str(),_("Open temporary file '%s' error: '%s'"),cfile.c_str(),strerror(errno));
 	write(icfile,s.certKey().data(),s.certKey().size());
 	close(icfile);
 
@@ -353,13 +358,18 @@ void *TSocketIn::Task( void *sock_in )
 	//> Select mode
 	struct  timeval tv;
 	fd_set  rd_fd;
-	while( !s.endrun )
+	while(!s.endrun)
         {
 	    tv.tv_sec  = 0; tv.tv_usec = STD_WAIT_DELAY*1000;
 	    FD_ZERO(&rd_fd); FD_SET(BIO_get_fd(abio,NULL),&rd_fd);
 
 	    int kz = select(BIO_get_fd(abio,NULL)+1,&rd_fd,NULL,NULL,&tv);
-	    if( kz == 0 || (kz == -1 && errno == EINTR) || kz < 0 || !FD_ISSET(BIO_get_fd(abio,NULL),&rd_fd) ) continue;
+	    if(kz < 0 && errno != EINTR)
+    	    {
+        	mess_err(s.nodePath().c_str(),_("Close input transport by error: %s"),strerror(errno));
+        	break;
+    	    }
+	    if(kz <= 0 || !FD_ISSET(BIO_get_fd(abio,NULL),&rd_fd)) continue;
 
 	    if( BIO_do_accept(abio) <= 0 )
 	    {
@@ -370,14 +380,14 @@ void *TSocketIn::Task( void *sock_in )
 
 	    BIO *cbio = BIO_pop(abio);
 
-	    if( s.maxFork() <= s.opConnCnt() )	{ s.clsConnByLim++; /*BIO_reset(cbio);*/ close(BIO_get_fd(cbio,NULL)); BIO_free(cbio); }
+	    if(s.maxFork() <= s.opConnCnt())	{ s.clsConnByLim++; /*BIO_reset(cbio);*/ close(BIO_get_fd(cbio,NULL)); BIO_free(cbio); }
 	    //> Make client's socket thread
 	    else
 	    {
 		SSockIn *sin = new SSockIn(&s,cbio);
 		try
 		{
-		    SYS->taskCreate( s.nodePath('.',true)+"."+TSYS::int2str(s.opConnCnt()), s.taskPrior(), ClTask, sin, NULL, 0, &pthr_attr );
+		    SYS->taskCreate(s.nodePath('.',true)+"."+TSYS::int2str(BIO_get_fd(cbio,NULL)), s.taskPrior(), ClTask, sin, 5, &pthr_attr);
 		    s.connNumb++;
 		}catch(TError err)
 		{
@@ -417,10 +427,10 @@ void *TSocketIn::ClTask( void *s_inf )
     AutoHD<TProtocolIn> prot_in;
     SSL		*ssl;
 
-    int cSock = s.s->clientReg( pthread_self() );
+    int cSock = s.s->clientReg(pthread_self());
 
 #if OSC_DEBUG >= 3
-    mess_debug(s.s->nodePath().c_str(),_("Socket has been connected by <%s>!"),s.sender.c_str() );
+    mess_debug(s.s->nodePath().c_str(),_("Socket has been connected by '%s'!"),s.sender.c_str() );
 #endif
 
     if(BIO_do_handshake(s.bio) <= 0)
@@ -436,7 +446,7 @@ void *TSocketIn::ClTask( void *s_inf )
 	    }
 	    BIO_flush(s.bio);
 	    delete (SSockIn*)s_inf;
-	    pthread_exit(NULL);
+	    return NULL;
 	}
     }
 
@@ -461,21 +471,26 @@ void *TSocketIn::ClTask( void *s_inf )
 	}
 
 	rez = BIO_read(s.bio,buf,sizeof(buf));
-	if( rez == 0 )	break;		//Connection closed by client
+	if(rez <= 0)	break;		//Connection closed by client
 #if OSC_DEBUG >= 4
-        mess_debug(s.s->nodePath().c_str(),_("The message is received with the size <%d>."),rez);
+        mess_debug(s.s->nodePath().c_str(),_("The message is received with the size '%d'."),rez);
 #endif
 	req.assign(buf,rez);
+	s.s->sock_res.resRequestW();
 	s.s->trIn += rez;
+	s.s->sock_res.resRelease();
 
 	s.s->messPut(cSock,req,answ,s.sender,prot_in);
 	if(answ.size())
 	{
 #if OSC_DEBUG >= 4
-            mess_debug(s.s->nodePath().c_str(),_("The message is replied with the size <%d>."),answ.size());
+            mess_debug(s.s->nodePath().c_str(),_("The message is replied with the size '%d'."),answ.size());
 #endif
-	    do { rez=BIO_write(s.bio,answ.data(),answ.size()); } while( rez < 0 && SSL_get_error(ssl,rez) == SSL_ERROR_WANT_WRITE );
-	    s.s->trOut += answ.size();
+	    do { rez = BIO_write(s.bio,answ.data(),answ.size()); }
+	    while(rez < 0 && SSL_get_error(ssl,rez) == SSL_ERROR_WANT_WRITE);
+	    s.s->sock_res.resRequestW();
+	    s.s->trOut += vmax(0, rez);
+	    s.s->sock_res.resRelease();
 	    answ = "";
 	    cnt++;
 	    tm = time(NULL);
@@ -507,7 +522,7 @@ void *TSocketIn::ClTask( void *s_inf )
 
     delete (SSockIn*)s_inf;
 
-    pthread_exit(NULL);
+    return NULL;
 }
 
 void TSocketIn::messPut( int sock, string &request, string &answer, string sender, AutoHD<TProtocolIn> &prot_in )
@@ -720,10 +735,13 @@ void TSocketOut::start()
 #else
     SSL_METHOD *meth;
 #endif
+
     meth = SSLv23_client_method();
-    if( ssl_method == "SSLv2" )		meth = SSLv2_client_method();
-    else if( ssl_method == "SSLv3" )	meth = SSLv3_client_method();
-    else if( ssl_method == "TLSv1" )	meth = TLSv1_client_method();
+#ifndef OPENSSL_NO_SSL2
+    if(ssl_method == "SSLv2")	meth = SSLv2_client_method();
+#endif
+    if(ssl_method == "SSLv3")	meth = SSLv3_client_method();
+    if(ssl_method == "TLSv1")	meth = TLSv1_client_method();
 
     try
     {
@@ -743,7 +761,7 @@ void TSocketOut::start()
 	    //>> Write certificate and private key to temorary file
 	    cfile = tmpnam(err);
 	    int icfile = open(cfile.c_str(),O_EXCL|O_CREAT|O_WRONLY,0644);
-	    if( icfile < 0 ) throw TError(nodePath().c_str(),_("Open temporaty file '%s' error: '%s'"),cfile.c_str(),strerror(errno));
+	    if( icfile < 0 ) throw TError(nodePath().c_str(),_("Open temporary file '%s' error: '%s'"),cfile.c_str(),strerror(errno));
 	    write(icfile,certKey().data(),certKey().size());
 	    close(icfile);
 
@@ -835,7 +853,7 @@ repeate:
 	//>> Input buffer clear
 	while(BIO_read(conn,err,sizeof(err)) > 0) ;
 	//>> Write request
-	do { ret=BIO_write(conn,obuf,len_ob); } while(ret < 0 && SSL_get_error(ssl,ret) == SSL_ERROR_WANT_WRITE);
+	do { ret = BIO_write(conn,obuf,len_ob); } while(ret < 0 && SSL_get_error(ssl,ret) == SSL_ERROR_WANT_WRITE);
 	if(ret <= 0) { res.release(); stop(); start(); res.request(true); goto repeate; }
 
 	if(!time) time = mTmCon;
@@ -846,7 +864,7 @@ repeate:
 
     trOut += ret;
 #if OSC_DEBUG >= 4
-    if( ret > 0 ) mess_debug(nodePath().c_str(),_("The message is sent with the size <%d>."),ret);
+    if( ret > 0 ) mess_debug(nodePath().c_str(),_("The message is sent with the size '%d'."),ret);
 #endif
 
     //> Read reply
@@ -887,10 +905,10 @@ repeate:
     }
 
 #if OSC_DEBUG >= 4
-    if( ret > 0 ) mess_debug(nodePath().c_str(),_("The message is received with the size <%d>."),ret);
+    if(ret > 0) mess_debug(nodePath().c_str(),_("The message is received with the size '%d'."),ret);
 #endif
 
-    return ret;
+    return vmax(0,ret);
 }
 
 void TSocketOut::cntrCmdProc( XMLNode *opt )
