@@ -43,8 +43,8 @@
 #define MOD_NAME	_("Serial interfaces")
 #define MOD_TYPE	STR_ID
 #define VER_TYPE	STR_VER
-#define MOD_VER		"0.7.2"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"0.7.3"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Allow serial based interfaces. Used for data exchanging through serial interfaces like RS232, RS485, GSM and other.")
 #define LICENSE		"GPL2"
 //************************************************
@@ -79,7 +79,7 @@ TTr::TTr( string name ) : TTipTransport(MOD_ID), prcSt(false)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -107,8 +107,8 @@ void TTr::postEnable( int flag )
 
     if( flag&TCntrNode::NodeConnect )
     {
-	owner().inEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
-	owner().outEl().fldAdd( new TFld("A_PRMS",_("Addon parameters"),TFld::String,TFld::FullText,"10000") );
+	owner().inEl().fldAdd( new TFld("A_PRMS",_("Addition parameters"),TFld::String,TFld::FullText,"10000") );
+	owner().outEl().fldAdd( new TFld("A_PRMS",_("Addition parameters"),TFld::String,TFld::FullText,"10000") );
     }
 }
 
@@ -185,7 +185,7 @@ void TTr::Task( union sigval obj )
 
 void TTr::writeLine( int fd, const string &ln )
 {
-    string obuf = ln+"\r\n";
+    string obuf = ln+"\x0D\x0A";
     for(unsigned wOff = 0, kz = 0; wOff != obuf.size(); wOff += kz)
 	if((kz=write(fd,obuf.data()+wOff,obuf.size()-wOff)) <= 0)
 	    throw TError(mod->nodePath().c_str(),_("Write line error."));
@@ -422,7 +422,7 @@ void TTrIn::connect( )
 		}
 		TTr::writeLine(fd,mdmInitStr1());
 		if(TTr::expect(fd,mdmInitResp(),mdmTm()).empty())
-		    throw TError(nodePath().c_str(),_("No response to init request '%s'."),mdmInitStr1().c_str());
+		    throw TError(nodePath().c_str(),_("No response to initial request '%s'."),mdmInitStr1().c_str());
 		usleep((int)(mdmPostInit()*1e6));
 	    }
 	    //>> Send init 2 string
@@ -436,7 +436,7 @@ void TTrIn::connect( )
 		}
 		TTr::writeLine(fd,mdmInitStr2());
 		if(TTr::expect(fd,mdmInitResp(),mdmTm()).empty())
-		    throw TError(nodePath().c_str(),_("No response to init request '%s'."),mdmInitStr2().c_str());
+		    throw TError(nodePath().c_str(),_("No response to initial request '%s'."),mdmInitStr2().c_str());
 		usleep((int)(mdmPostInit()*1e6));
 	    }
 	}
@@ -453,12 +453,13 @@ void TTrIn::start()
     if( run_st ) return;
 
     //> Status clear
-    trIn = trOut = tmMax = 0;
+    trIn = trOut = 0;
+    tmMax = 0;
 
     connect( );
 
     //> Start listen task
-    SYS->taskCreate( nodePath('.',true), 0, Task, this, &run_st );
+    SYS->taskCreate(nodePath('.',true), 0, Task, this);
 }
 
 void TTrIn::stop()
@@ -470,11 +471,12 @@ void TTrIn::stop()
     mMdmMode = mMdmDataMode = false;
 
     //> Status clear
-    trIn = trOut = tmMax = 0;
+    trIn = trOut = 0;
+    tmMax = 0;
 
-    SYS->taskDestroy( nodePath('.',true), &run_st, &endrun );
+    SYS->taskDestroy(nodePath('.',true), &endrun);
 
-    if( fd >= 0 ) close(fd);
+    if(fd >= 0) close(fd);
     fd = -1;
 }
 
@@ -487,52 +489,52 @@ void *TTrIn::Task( void *tr_in )
 
     AutoHD<TProtocolIn> prot_in;
     struct timeval tv;
-    int r_len = 0;
+    ssize_t r_len = 0;
     string req, answ;
     char buf[1000];
     fd_set fdset;
 
     double wCharTm = atof(TSYS::strSepParse(tr->timings(),0,':').c_str());
     int wFrTm = 1000*atoi(TSYS::strSepParse(tr->timings(),1,':').c_str());
-    long long stFrTm = 0, tmW = 0, tmTmp1;
+    int64_t stFrTm = 0, tmW = 0, tmTmp1;
 
-    fcntl( tr->fd, F_SETFL, 0 );
+    fcntl(tr->fd, F_SETFL, 0);
 
-    while( !tr->endrun )
+    while(!tr->endrun)
     {
 	//>> Char timeout
 	while(true)
 	{
 	    tv.tv_sec = 0; tv.tv_usec = (int)(1.5e3*wCharTm);
-	    FD_ZERO( &fdset ); FD_SET( tr->fd, &fdset );
+	    FD_ZERO(&fdset); FD_SET(tr->fd, &fdset);
 
-	    if( select( tr->fd+1, &fdset, NULL, NULL, &tv ) <= 0 )
+	    if(select(tr->fd+1, &fdset, NULL, NULL, &tv) <= 0)
 	    {
-		if( tr->endrun || !req.empty() )	break;
+		if(tr->endrun || !req.empty())	break;
 		continue;
 	    }
-	    r_len = read( tr->fd, buf, sizeof(buf));
-	    if( r_len <= 0 ) break;
+	    r_len = read(tr->fd, buf, sizeof(buf));
+	    if(r_len <= 0) break;
 
 	    //>> Requests statistic
 	    tmTmp1 = TSYS::curTime();
-	    if( req.empty() ) stFrTm = tmW = tmTmp1;
-	    if( tmW ) tr->tmMax = vmax(tr->tmMax,1e-3*(tmTmp1-tmW));
+	    if(req.empty()) stFrTm = tmW = tmTmp1;
+	    if(tmW) tr->tmMax = vmax(tr->tmMax,1e-3*(tmTmp1-tmW));
 	    tmW = tmTmp1;
 
-	    req += string(buf,r_len);
-	    if( (TSYS::curTime()-stFrTm) > wFrTm )	break;
+	    req += string(buf, r_len);
+	    if((TSYS::curTime()-stFrTm) > wFrTm) break;
 	}
-	if( tr->endrun || req.empty() )
+	if(tr->endrun || req.empty())
 	{
-	    if( r_len == 0 )
+	    if(r_len == 0)
 	    {
 		close(tr->fd); tr->fd = -1;
-		if( tr->mMdmMode && tr->mMdmDataMode )
+		if(tr->mMdmMode && tr->mMdmDataMode)
 		{
 		    //>> Reconnect try after hung up by remote agent
 		    mod->devUnLock(tr->mDevPort);
-		    sleep(10);
+		    usleep(10000000);
 		    try{ tr->connect(); } catch(TError err) { break; }
 		    continue;
 		}
@@ -543,19 +545,19 @@ void *TTrIn::Task( void *tr_in )
 	tr->trIn += req.size();
 
 #if OSC_DEBUG >= 5
-	mess_debug( nodePath().c_str(), _("Serial received message <%d>."), req.size() );
+	mess_debug(nodePath().c_str(), _("Serial received message '%d'."), req.size());
 #endif
 
 	//> Check for device lock and RING request from modem
-	if( tr->mMdmMode && !tr->mMdmDataMode )
+	if(tr->mMdmMode && !tr->mMdmDataMode)
 	{
-	    if( mod->devLock(tr->mDevPort,true) ) continue;
-	    if( req.find(tr->mdmRingReq()) != string::npos )
+	    if(mod->devLock(tr->mDevPort,true)) continue;
+	    if(req.find(tr->mdmRingReq()) != string::npos)
 	    {
-		if( !mod->devLock(tr->mDevPort) ) continue;
+		if(!mod->devLock(tr->mDevPort)) continue;
 		//>> Send ring answer
 		TTr::writeLine(tr->fd,tr->mdmRingAnswer());
-		if( TTr::expect(tr->fd,tr->mdmRingAnswerResp(),tr->mdmTm()).empty() ) continue;
+		if(TTr::expect(tr->fd,tr->mdmRingAnswerResp(),tr->mdmTm()).empty()) continue;
 		tr->mMdmDataMode = true;
 
 		req = "";
@@ -565,14 +567,14 @@ void *TTrIn::Task( void *tr_in )
 	//> Send message to protocol
 	try
 	{
-	    if( prot_in.freeStat() )
+	    if(prot_in.freeStat())
 	    {
 		AutoHD<TProtocol> proto = SYS->protocol().at().modAt(tr->protocol());
 		string n_pr = tr->id()+TSYS::int2str(tr->fd);
-		if( !proto.at().openStat(n_pr) ) proto.at().open( n_pr, tr->workId() );
-		prot_in = proto.at().at( n_pr );
+		if(!proto.at().openStat(n_pr)) proto.at().open(n_pr, tr->workId());
+		prot_in = proto.at().at(n_pr);
 	    }
-	    prot_in.at().mess(req,answ,"");
+	    prot_in.at().mess(req, answ, "");
 	}catch(TError err)
 	{
 	    mess_err(tr->nodePath().c_str(),"%s",err.mess.c_str() );
@@ -580,15 +582,16 @@ void *TTrIn::Task( void *tr_in )
 	}
 
 	//> Send respond
-	if( answ.size() )
+	if(answ.size())
 	{
 #if OSC_DEBUG >= 5
-	    mess_debug(nodePath().c_str(), _("Serial replied message <%d>."), answ.size());
+	    mess_debug(nodePath().c_str(), _("Serial replied message '%d'."), answ.size());
 #endif
-	    for(unsigned wOff = 0, wL = 1; wOff != answ.size() && wL > 0; wOff += wL)
+	    ssize_t wL = 1;
+	    for(unsigned wOff = 0; wOff != answ.size() && wL > 0; wOff += wL)
 	    {
 		wL = write(tr->fd, answ.data()+wOff, answ.size()-wOff);
-		tr->trOut += wL;
+		tr->trOut += vmax(0,wL);
 	    }
 	    answ = "";
 	}
@@ -629,11 +632,11 @@ void TTrIn::cntrCmdProc( XMLNode *opt )
 	if(atoi(TSYS::strParse(addr(),4,":").c_str()) && ctrMkNode("area",opt,-1,"/mod",_("Modem"),R_R_R_,"root",STR_ID))
 	{
 	    ctrMkNode("fld",opt,-1,"/mod/tm",_("Timeout (sec)"),RWRWR_,"root",STR_ID,1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/mod/preInitDl",_("Pre-init delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
-	    ctrMkNode("fld",opt,-1,"/mod/postInitDl",_("Post-init delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/mod/preInitDl",_("Pre-initial delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/mod/postInitDl",_("Post-initial delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
 	    ctrMkNode("fld",opt,-1,"/mod/initStr1",_("Initialization string 1"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/initStr2",_("Initialization string 2"),RWRWR_,"root",STR_ID,1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/mod/initResp",_("Init response"),RWRWR_,"root",STR_ID,1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/mod/initResp",_("Initial response"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/ringReq",_("Ring request"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/ringAnswer",_("Ring answer"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/ringAnswerResp",_("Ring answer response"),RWRWR_,"root",STR_ID,1,"tp","str");
@@ -898,8 +901,8 @@ void TTrOut::start( )
 	    }
 
 	//>> Set port's data
-	tcflush( fd, TCIOFLUSH );
-	tcsetattr( fd, TCSANOW, &tio );
+	tcflush(fd, TCIOFLUSH);
+	tcsetattr(fd, TCSANOW, &tio);
 
 	//> Modem connection establish
 	string telNumb = TSYS::strNoSpace(TSYS::strSepParse(addr(),4,':'));
@@ -920,7 +923,7 @@ void TTrOut::start( )
 		}
 		TTr::writeLine(fd,mdmInitStr1());
 		if(TTr::expect(fd,mdmInitResp(),mdmTm()).empty())
-		    throw TError(nodePath().c_str(),_("No response to init request '%s'."),mdmInitStr1().c_str());
+		    throw TError(nodePath().c_str(),_("No response to initial request '%s'."),mdmInitStr1().c_str());
 		usleep((int)(mdmPostInit()*1e6));
 	    }
 	    //>> Send init 2 string
@@ -934,7 +937,7 @@ void TTrOut::start( )
 		}
 		TTr::writeLine(fd,mdmInitStr2());
 		if(TTr::expect(fd,mdmInitResp(),mdmTm()).empty())
-		    throw TError(nodePath().c_str(),_("No response to init request '%s'."),mdmInitStr2().c_str());
+		    throw TError(nodePath().c_str(),_("No response to initial request '%s'."),mdmInitStr2().c_str());
 		usleep((int)(mdmPostInit()*1e6));
 	    }
 	    //>> Dial number and connection wait
@@ -1001,56 +1004,57 @@ void TTrOut::check( )
 
 int TTrOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, int time, bool noRes )
 {
-    int blen = 0, off = 0, kz;
+    ssize_t blen = 0;
+    int off = 0, kz;
 
-    if( !noRes ) ResAlloc res( nodeRes(), true );
+    if(!noRes) ResAlloc res(nodeRes(), true);
 
-    if( !run_st ) throw TError(nodePath().c_str(),_("Transport is not started!"));
+    if(!run_st) throw TError(nodePath().c_str(),_("Transport is not started!"));
 
     int wReqTm = atoi(TSYS::strSepParse(timings(),0,':',&off).c_str());
     wReqTm = time ? time : wReqTm;
     double wCharTm = atof(TSYS::strSepParse(timings(),0,':',&off).c_str());
 
-    long long tmW = TSYS::curTime();
+    int64_t tmW = TSYS::curTime();
 
     //> Write request
-    if( obuf && len_ob > 0 )
+    if(obuf && len_ob > 0)
     {
-	tcflush( fd, TCIOFLUSH );
-	if( (tmW-mLstReqTm) < (4000*wCharTm) ) kz = usleep( (int)((4000*wCharTm)-(tmW-mLstReqTm)) );
-	for( int wOff = 0, kz = 0; wOff != len_ob; wOff += kz )
+	tcflush(fd, TCIOFLUSH);
+	if((tmW-mLstReqTm) < (4000*wCharTm)) kz = usleep((int)((4000*wCharTm)-(tmW-mLstReqTm)));
+	for(int wOff = 0, kz = 0; wOff != len_ob; wOff += kz)
 	{
 	    kz = write(fd,obuf+wOff,len_ob-wOff);
-	    if( kz <= 0 ) { mLstReqTm = TSYS::curTime(); stop(); throw TError(nodePath().c_str(),_("Writing request error.")); }
+	    if(kz <= 0) { mLstReqTm = TSYS::curTime(); stop(); throw TError(nodePath().c_str(),_("Writing request error.")); }
+	    else trOut += kz;
 	}
-	trOut += len_ob;
     }
 
     //> Read reply
-    if( ibuf != NULL && len_ib > 0 )
+    if(ibuf != NULL && len_ib > 0)
     {
 	fd_set rd_fd;
 	struct timeval tv;
 
 	do
 	{
-	    if( obuf && len_ob > 0 ) { tv.tv_sec  = wReqTm/1000; tv.tv_usec = 1000*(wReqTm%1000); }
+	    if(obuf && len_ob > 0) { tv.tv_sec  = wReqTm/1000; tv.tv_usec = 1000*(wReqTm%1000); }
 	    else { tv.tv_sec = (int)(1.5e-3*wCharTm); tv.tv_usec = (int)(1.5e3*wCharTm)%1000000; }
 	    FD_ZERO(&rd_fd); FD_SET(fd,&rd_fd);
 	    kz = select(fd+1,&rd_fd,NULL,NULL,&tv);
 	}
-	while( kz == -1 && errno == EINTR );
-	if( kz == 0 )	{ mLstReqTm = TSYS::curTime(); throw TError(nodePath().c_str(),_("Timeouted!")); }
-	else if( kz < 0){ mLstReqTm = TSYS::curTime(); throw TError(nodePath().c_str(),_("Serial error!")); }
-	else if( FD_ISSET(fd, &rd_fd) )
+	while(kz == -1 && errno == EINTR);
+	if(kz == 0)	{ mLstReqTm = TSYS::curTime(); throw TError(nodePath().c_str(),_("Timeouted!")); }
+	else if(kz < 0)	{ mLstReqTm = TSYS::curTime(); throw TError(nodePath().c_str(),_("Serial error!")); }
+	else if(FD_ISSET(fd, &rd_fd))
 	{
-	    blen = read(fd,ibuf,len_ib);
-	    trIn += blen;
+	    blen = read(fd, ibuf, len_ib);
+	    trIn += vmax(0, blen);
 	}
     }
     mLstReqTm = TSYS::curTime();
 
-    return blen;
+    return vmax(0,blen);
 }
 
 void TTrOut::cntrCmdProc( XMLNode *opt )
@@ -1075,11 +1079,11 @@ void TTrOut::cntrCmdProc( XMLNode *opt )
 	{
 	    ctrMkNode("fld",opt,-1,"/mod/tm",_("Timeout (sec)"),RWRWR_,"root",STR_ID,1,"tp","dec");
 	    ctrMkNode("fld",opt,-1,"/mod/lifeTm",_("Life time (sec)"),RWRWR_,"root",STR_ID,1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/mod/preInitDl",_("Pre-init delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
-	    ctrMkNode("fld",opt,-1,"/mod/postInitDl",_("Post-init delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/mod/preInitDl",_("Pre-initial delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/mod/postInitDl",_("Post-initial delay (sec)"),RWRWR_,"root",STR_ID,1,"tp","real");
 	    ctrMkNode("fld",opt,-1,"/mod/initStr1",_("Initialization string 1"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/initStr2",_("Initialization string 2"),RWRWR_,"root",STR_ID,1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/mod/initResp",_("Init response"),RWRWR_,"root",STR_ID,1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/mod/initResp",_("Initial response"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/dialStr",_("Dial string"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/cnctResp",_("Connect response"),RWRWR_,"root",STR_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/mod/busyResp",_("Busy response"),RWRWR_,"root",STR_ID,1,"tp","str");
