@@ -37,6 +37,7 @@
 #include <QToolTip>
 #include <QStatusBar>
 #include <QMenu>
+#include <QPlastiqueStyle>
 
 #include <tsys.h>
 
@@ -196,35 +197,41 @@ void LineEdit::changed( )
 
 void LineEdit::setValue(const QString &txt)
 {
-    if( ed_fld ) ed_fld->blockSignals(true);
+    if(ed_fld) ed_fld->blockSignals(true);
     switch(type())
     {
 	case Text:
+	    if(txt == ((QLineEdit*)ed_fld)->text()) break;
 	    ((QLineEdit*)ed_fld)->setText(txt);
 	    ((QLineEdit*)ed_fld)->setCursorPosition(0);
 	    break;
 	case Integer:
+	    if(txt.toInt() == ((QSpinBox*)ed_fld)->value()) break;
 	    ((QSpinBox*)ed_fld)->setValue(txt.toInt());
 	    break;
 	case Real:
+	    if(txt.toDouble() == ((QDoubleSpinBox*)ed_fld)->value()) break;
 	    ((QDoubleSpinBox*)ed_fld)->setValue(txt.toDouble());
 	    break;
 	case Time:
+	    if(QTime().addSecs(txt.toInt()) == ((QTimeEdit*)ed_fld)->time()) break;
 	    ((QTimeEdit*)ed_fld)->setTime(QTime().addSecs(txt.toInt()));
 	    break;
 	case Date: case DateTime:
+	    if(QDateTime::fromTime_t(txt.toInt()) == ((QDateTimeEdit*)ed_fld)->dateTime()) break;
 	    ((QDateTimeEdit*)ed_fld)->setDateTime(QDateTime::fromTime_t(txt.toInt()));
 	    break;
 	case Combo:
-	    if( ((QComboBox*)ed_fld)->findText(txt) < 0 ) ((QComboBox*)ed_fld)->addItem(txt);
+	    if(txt == ((QComboBox*)ed_fld)->currentText()) break;
+	    if(((QComboBox*)ed_fld)->findText(txt) < 0) ((QComboBox*)ed_fld)->addItem(txt);
 	    ((QComboBox*)ed_fld)->setEditText(txt);
 	    break;
     }
-    if( ed_fld ) ed_fld->blockSignals(false);
+    if(ed_fld) ed_fld->blockSignals(false);
 
     m_val = txt;
 
-    if( bt_fld ) viewApplyBt(false);
+    if(bt_fld) viewApplyBt(false);
 }
 
 void LineEdit::setCfg(const QString &cfg)
@@ -347,73 +354,94 @@ void SyntxHighl::setSnthHgl(XMLNode nd)
     rehighlight();
 }
 
-void SyntxHighl::rule(XMLNode *rl, const QString &text, QTextCharFormat defForm, int off)
+void SyntxHighl::rule(XMLNode *irl, const QString &text, int off, char lev)
 {
-    if(text.isEmpty() || rl->name() != "rule") return;
+    XMLNode *rl;
+    vector<int> rul_pos(irl->childSize(),-1);
+    int minPos = -1, minRule, endIndex, startBlk, sizeBlk;
     QTextCharFormat kForm;
-    kForm.setForeground(QColor(rl->attr("color").c_str()));
-    kForm.setFontWeight(atoi(rl->attr("font_weight").c_str()) ? QFont::Bold : QFont::Normal);
-    kForm.setFontItalic(atoi(rl->attr("font_italic").c_str()));
-    QRegExp expr(rl->attr("expr").c_str());
+    QRegExp expr;
 
-    for(int index = 0; true; index+=expr.matchedLength())
+    if(lev > 3) return;
+
+    //> Init previous block continue
+    int curBlk = (currentBlockState()>>(lev*8))&0xFF;
+
+    //> Stream process by rules
+    for(int i_t = 0; i_t < text.length(); )
     {
-	if((index=expr.indexIn(text,index)) < 0 || expr.matchedLength() <= 0) break;
-	if(format(index+off)!=defForm) continue;
-	setFormat(index+off, expr.matchedLength(), kForm);
-	for(unsigned i_ch = 0; i_ch < rl->childSize(); i_ch++)
-	    rule(rl->childGet(i_ch),text.mid(index,expr.matchedLength()),kForm,index+off);
+	if(curBlk && !i_t) { minRule = curBlk-1; minPos = 0; }
+	else minRule = -1;
+
+	for(int i_ch = 0; i_t != minPos && i_ch < (int)irl->childSize(); i_ch++)
+	{
+	    if(!(minPos < i_t || rul_pos[i_ch] < i_t || rul_pos[i_ch] < minPos)) continue;
+	    if(rul_pos[i_ch] >= i_t && rul_pos[i_ch] < minPos)	{ minPos = rul_pos[i_ch]; minRule = i_ch; continue; }
+	    if(rul_pos[i_ch] == i_t && rul_pos[i_ch] == minPos)	{ minRule = i_ch; break; }
+
+	    //> Call rule
+	    rl = irl->childGet(i_ch);
+	    if(rl->name() == "rule")	expr.setPattern(rl->attr("expr").c_str());
+	    else if(rl->name() == "blk")expr.setPattern(rl->attr("beg").c_str());
+	    else continue;
+	    rul_pos[i_ch] = expr.indexIn(text,i_t);
+	    if(expr.matchedLength() <= 0) continue;
+	    if(rul_pos[i_ch] < 0) rul_pos[i_ch] = text.length();
+	    if(minPos < i_t || rul_pos[i_ch] < minPos) { minPos = rul_pos[i_ch]; minRule = i_ch; }
+	}
+	if(minRule < 0)	break;
+
+	//> Process minimal rule
+	rl = irl->childGet(minRule);
+        kForm.setForeground(QColor(rl->attr("color").c_str()));
+        kForm.setFontWeight(atoi(rl->attr("font_weight").c_str()) ? QFont::Bold : QFont::Normal);
+        kForm.setFontItalic(atoi(rl->attr("font_italic").c_str()));
+
+	if(rl->name() == "rule")
+	{
+	    expr.setPattern(rl->attr("expr").c_str());
+	    if(expr.indexIn(text,i_t) != rul_pos[minRule]) break;
+	    setFormat(rul_pos[minRule]+off, expr.matchedLength(), kForm);
+            //> Call include rules
+            if(rl->childSize()) rule(rl, text.mid(rul_pos[minRule],expr.matchedLength()), rul_pos[minRule]+off, lev+1);
+	    i_t = rul_pos[minRule]+expr.matchedLength();
+	}
+	else if(rl->name() == "blk")
+	{
+	    if(curBlk) rul_pos[minRule] = curBlk = startBlk = 0;
+	    else
+	    {
+		expr.setPattern(rl->attr("beg").c_str());
+		if(expr.indexIn(text,i_t) != rul_pos[minRule]) break;
+		startBlk = rul_pos[minRule]+expr.matchedLength();
+	    }
+	    QRegExp eExpr(rl->attr("end").c_str());
+	    endIndex = eExpr.indexIn(text, startBlk);
+            if(endIndex == -1 || eExpr.matchedLength() <= 0)
+            {
+        	setFormat(rul_pos[minRule]+off, (text.length()-rul_pos[minRule]), kForm);
+                sizeBlk = text.length()-startBlk;
+                i_t = text.length();
+            }
+            else
+            {
+                setFormat(rul_pos[minRule]+off, (endIndex-rul_pos[minRule]+eExpr.matchedLength()), kForm);
+                sizeBlk = endIndex-startBlk;
+                i_t = endIndex + eExpr.matchedLength();
+            }
+            //> Call include rules
+            if(rl->childSize()) rule(rl, text.mid(startBlk,sizeBlk), startBlk+off, lev+1);
+            if(endIndex == -1 || eExpr.matchedLength() <= 0)
+		setCurrentBlockState(((minRule+1)<<(lev*8))|currentBlockState());
+	    else setCurrentBlockState(currentBlockState()& ~(0xFFFFFFFF<<(lev*8)));
+	}
     }
 }
 
 void SyntxHighl::highlightBlock(const QString &text)
 {
-    QTextCharFormat kForm, defkForm;
-    if(text.length()) defkForm = format(0);
-    setCurrentBlockState(previousBlockState());
-
-    for(int i_ch = 0; i_ch < (int)rules.childSize(); i_ch++)
-    {
-	XMLNode *rl = rules.childGet(i_ch);
-	kForm.setForeground(QColor(rl->attr("color").c_str()));
-	kForm.setFontWeight(atoi(rl->attr("font_weight").c_str()) ? QFont::Bold : QFont::Normal);
-	kForm.setFontItalic(atoi(rl->attr("font_italic").c_str()));
-
-	if(rl->name() == "rule") rule(rl,text,defkForm);
-	else if(rl->name() == "blk" && (currentBlockState() == -1 || currentBlockState() == i_ch))
-	{
-	    QRegExp bExpr(rl->attr("beg").c_str());
-	    QRegExp eExpr(rl->attr("end").c_str());
-	    for(int stIndex = 0, endIndex = 0, startBlk = 0, sizeBlk = 0; true; )
-	    {
-		if(currentBlockState() == -1)
-		{
-		    if((stIndex=bExpr.indexIn(text,stIndex)) == -1 || bExpr.matchedLength() <= 0) break;
-		    if(format(stIndex)!=defkForm) { stIndex += bExpr.matchedLength(); continue; }
-		    setCurrentBlockState(i_ch);
-		    startBlk = stIndex+bExpr.matchedLength();
-		}
-		endIndex = eExpr.indexIn(text, stIndex);
-		if(endIndex == -1 || eExpr.matchedLength() <= 0)
-		{
-		    setFormat(stIndex, (text.length()-stIndex), kForm);
-		    sizeBlk = text.length()-startBlk;
-		}
-		else
-		{
-		    setCurrentBlockState(-1);
-		    setFormat(stIndex, (endIndex-stIndex+eExpr.matchedLength()), kForm);
-		    sizeBlk = endIndex-startBlk;
-		}
-		//> Call include rules
-		for(unsigned i_ch1 = 0; i_ch1 < rl->childSize(); i_ch1++)
-		    rule(rl->childGet(i_ch1),text.mid(startBlk,sizeBlk),kForm,startBlk);
-
-		if(endIndex == -1 || eExpr.matchedLength() <= 0) break;
-		stIndex = endIndex+eExpr.matchedLength();
-	    }
-	}
-    }
+    setCurrentBlockState((previousBlockState()<0)?0:previousBlockState());
+    rule(&rules,text);
 }
 
 //*************************************************
@@ -428,6 +456,7 @@ TextEdit::TextEdit( QWidget *parent, const char *name, bool prev_dis ) :
     box->setSpacing(0);
 
     ed_fld = new QTextEdit(this);
+    ed_fld->setStyle(new QPlastiqueStyle());	//> Force style set for resize allow everywhere
     ed_fld->setContextMenuPolicy(Qt::CustomContextMenu);
     ed_fld->setTabStopWidth(20);
     ed_fld->setAcceptRichText(false);
@@ -483,10 +512,10 @@ bool TextEdit::hasFocus( ) const
 
 void TextEdit::setText(const QString &text)
 {
-    isInit=true;
-    ed_fld->setPlainText(text);
+    isInit = true;
+    if(text != ed_fld->toPlainText()) ed_fld->setPlainText(text);
     ed_fld->document()->setModified(false);
-    isInit=false;
+    isInit = false;
     changed();
 }
 
@@ -605,7 +634,7 @@ void TextEdit::find( )
 //************************************************
 CfgTable::CfgTable( QWidget *parent ) : QTableWidget(parent)
 {
-
+    setStyle(new QPlastiqueStyle());	//> Force style set for resize allow everywhere
 }
 
 bool CfgTable::event( QEvent *e )
@@ -630,8 +659,8 @@ InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
 	const QString &ndlg, int with_id, int with_nm, QDialogButtonBox::StandardButtons buttons ) :
 		QDialog(parent), mId(NULL), mName(NULL)
 {
+    setMaximumSize(800,600);
     setWindowTitle(ndlg);
-    setMinimumSize( QSize( 120, 150 ) );
     setWindowIcon(icon);
     setSizeGripEnabled(true);
 
@@ -675,11 +704,12 @@ InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
 	    ed_lay->addWidget( mName, 4, 1 );
 	}
     }
+
+    ed_lay->addItem(new QSpacerItem(10, 0, QSizePolicy::Minimum, QSizePolicy::Expanding),100,0,1,-1);
+
     dlg_lay->addItem(ed_lay);
 
     //- Qk and Cancel buttons -
-    dlg_lay->addItem( new QSpacerItem( 10, 0, QSizePolicy::Minimum, QSizePolicy::Expanding ) );
-
     QFrame *sep = new QFrame(this);
     sep->setFrameShape( QFrame::HLine );
     sep->setFrameShadow( QFrame::Raised );
@@ -740,7 +770,9 @@ void InputDlg::setMess( const QString &val )
 
 void InputDlg::showEvent( QShowEvent * event )
 {
+    QSize src = size();
     adjustSize();
+    resize(size().expandedTo(src));
 }
 
 //*****************************************************
@@ -871,7 +903,9 @@ void DlgUser::finish( int result )
 
 void DlgUser::showEvent( QShowEvent * event )
 {
+    QSize src = size();
     adjustSize();
+    resize(size().expandedTo(src));
 }
 
 //*********************************************
@@ -954,6 +988,15 @@ QWidget *TableDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem
     QVariant val_user = index.data(Qt::UserRole);
 
     if(val_user.isValid()) w_del = new QComboBox(parent);
+    else if(value.type() == QVariant::String)
+    {
+        w_del = new QTextEdit(parent);
+        ((QTextEdit*)w_del)->setTabStopWidth(40);
+        ((QTextEdit*)w_del)->setLineWrapMode(QTextEdit::NoWrap);
+        ((QTextEdit*)w_del)->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+        ((QTextEdit*)w_del)->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+        //((QTextEdit*)w_del)->resize(50,50);
+    }
     else
     {
 	QItemEditorFactory factory;
@@ -968,23 +1011,19 @@ void TableDelegate::setEditorData( QWidget *editor, const QModelIndex &index ) c
     QVariant value = index.data(Qt::DisplayRole);
     QVariant val_user = index.data(Qt::UserRole);
 
-    if( dynamic_cast<QComboBox*>(editor) )
+    if(dynamic_cast<QComboBox*>(editor))
     {
 	QComboBox *comb = dynamic_cast<QComboBox*>(editor);
-	if(value.type() == QVariant::Bool)	comb->setCurrentIndex(value.toBool());
+	if(value.type() == QVariant::Bool) comb->setCurrentIndex(value.toBool());
 	else if(val_user.isValid())
 	{
 	    comb->addItems(val_user.toStringList());
 	    comb->setCurrentIndex(comb->findText(value.toString()));
 	}
-	return;
-    }
-    if( dynamic_cast<QLineEdit*>(editor) )
-    {
-	QLineEdit *led = dynamic_cast<QLineEdit*>(editor);
-	led->setText(value.toString());
-	return;
     }
+    else if(dynamic_cast<QTextEdit*>(editor))	((QTextEdit*)editor)->setPlainText(value.toString());
+    else if(dynamic_cast<QLineEdit*>(editor))	((QLineEdit*)editor)->setText(value.toString());
+    else QItemDelegate::setEditorData(editor, index);
 }
 
 void TableDelegate::setModelData( QWidget *editor, QAbstractItemModel *model, const QModelIndex &index ) const
@@ -996,14 +1035,10 @@ void TableDelegate::setModelData( QWidget *editor, QAbstractItemModel *model, co
 	if(!val_user.isValid())
 	    model->setData(index,(bool)comb->currentIndex(),Qt::EditRole);
 	else model->setData(index,comb->currentText(),Qt::EditRole);
-	return;
-    }
-    if(dynamic_cast<QLineEdit*>(editor))
-    {
-	QLineEdit *led = dynamic_cast<QLineEdit*>(editor);
-	model->setData(index,led->text(),Qt::EditRole);
-	return;
     }
+    else if(dynamic_cast<QTextEdit*>(editor))	model->setData(index, ((QTextEdit*)editor)->toPlainText(),Qt::EditRole);
+    else if(dynamic_cast<QLineEdit*>(editor))	model->setData(index, ((QLineEdit*)editor)->text(),Qt::EditRole);
+    else QItemDelegate::setModelData(editor, model, index);
 }
 
 void TableDelegate::updateEditorGeometry( QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex & ) const
@@ -1031,6 +1066,26 @@ bool TableDelegate::eventFilter( QObject *object, QEvent *event )
 		    return false;
 	    }
     }
+    else if(dynamic_cast<QTextEdit*>(object))
+    {
+        QTextEdit *ted = dynamic_cast<QTextEdit*>(object);
+        if(event->type() == QEvent::KeyPress)
+            switch(static_cast<QKeyEvent *>(event)->key())
+            {
+                case Qt::Key_Enter:
+                case Qt::Key_Return:
+                    if( QApplication::keyboardModifiers()&Qt::ControlModifier )
+                    {
+                        emit commitData(ted);
+                        emit closeEditor(ted, QAbstractItemDelegate::SubmitModelCache);
+                        return true;
+                    }
+                    else return false;
+                case Qt::Key_Escape:
+                    emit closeEditor(ted, QAbstractItemDelegate::RevertModelCache);
+                    return true;
+            }
+    }
 
     return QItemDelegate::eventFilter(object,event);
 }
