@@ -47,8 +47,8 @@
 #define MOD_NAME	_("QT GUI starter")
 #define MOD_TYPE	SUI_ID
 #define VER_TYPE	SUI_VER
-#define MOD_VER		"1.6.1"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"1.6.2"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Allow QT GUI starter. It is single for all QT GUI modules!")
 #define LICENSE		"GPL2"
 //*************************************************
@@ -83,7 +83,7 @@ TUIMod::TUIMod( string name ) : TUI(MOD_ID), demon_mode(false), end_run(false),
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -154,7 +154,7 @@ void TUIMod::postEnable( int flag )
 	{
 	    end_run = false;
 
-	    SYS->taskCreate(nodePath('.',true), 0, Task, this, &run_st);
+	    SYS->taskCreate(nodePath('.',true), 0, Task, this);
 	}
     }
 }
@@ -162,7 +162,7 @@ void TUIMod::postEnable( int flag )
 void TUIMod::postDisable( int flag )
 {
     if(run_st)
-	try { SYS->taskDestroy(nodePath('.',true), &run_st, &end_run); }
+	try { SYS->taskDestroy(nodePath('.',true), &end_run); }
 	catch(TError err){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
@@ -194,7 +194,7 @@ void TUIMod::load_( )
 	}
     } while(next_opt != -1);
 
-    //> Load parameters from config file
+    //> Load parameters from config-file
     start_mod = TBDS::genDBGet(nodePath()+"StartMod",start_mod);
 }
 
@@ -231,7 +231,7 @@ string TUIMod::optDescr( )
 
     snprintf(buf,sizeof(buf),_(
 	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n"
+	"---------- Parameters of the module section '%s' in config-file ----------\n"
 	"StartMod  <moduls>    Start modules list (sep - ';').\n\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
 
@@ -262,16 +262,19 @@ void *TUIMod::Task( void * )
     if(!ico_t.load(TUIS::icoPath(SYS->id()+"_splash").c_str()))	ico_t.load(":/images/splash.png");
     QSplashScreen *splash = new QSplashScreen(QPixmap::fromImage(ico_t));
     splash->show();
+    QFont wFnt = splash->font();
+    wFnt.setPixelSize(10);
+    splash->setFont(wFnt);
 
     while(!mod->startCom( ) && !mod->endRun())
     {
 	SYS->archive().at().messGet(st_time, time(NULL), recs, "", TMess::Debug, BUF_ARCH_NM);
 	QString mess;
-	for(int i_m = recs.size()-1; i_m >= 0 && i_m > ((int)recs.size()-7); i_m--)
+	for(int i_m = recs.size()-1; i_m >= 0 && i_m > ((int)recs.size()-10); i_m--)
 	    mess += QString("\n%1: %2").arg(recs[i_m].categ.c_str()).arg(recs[i_m].mess.c_str());
 	splash->showMessage(mess,Qt::AlignBottom|Qt::AlignLeft);
 	QtApp->processEvents();
-	sleep(1);
+	usleep(1000000);
     }
     delete splash;
 
@@ -305,16 +308,18 @@ void *TUIMod::Task( void * )
     if(!ico_t.load(TUIS::icoPath(SYS->id()+"_splash_exit").c_str()))	ico_t.load(":/images/splash.png");
     splash = new QSplashScreen(QPixmap::fromImage(ico_t));
     splash->show();
+    splash->setFont(wFnt);
+
     st_time = time(NULL);
     while(!mod->endRun())
     {
 	SYS->archive().at().messGet( st_time, time(NULL), recs, "", TMess::Debug, BUF_ARCH_NM );
 	QString mess;
-	for(int i_m = recs.size()-1; i_m >= 0 && i_m > ((int)recs.size()-7); i_m--)
+	for(int i_m = recs.size()-1; i_m >= 0 && i_m > ((int)recs.size()-10); i_m--)
 	    mess+=QString("\n%1: %2").arg(recs[i_m].categ.c_str()).arg(recs[i_m].mess.c_str());
 	splash->showMessage(mess,Qt::AlignBottom|Qt::AlignLeft);
 	QtApp->processEvents();
-	sleep(1);
+	usleep(1000000);
     }
     delete splash;
 
