@@ -91,7 +91,7 @@ void WidgetLib::postEnable( int flag )
 
 void WidgetLib::preDisable( int flag )
 {
-    if( enable() )  setEnable(false);
+    if(enable()) setEnable(false);
 }
 
 void WidgetLib::postDisable( int flag )
@@ -122,13 +122,15 @@ void WidgetLib::postDisable( int flag )
 
 string WidgetLib::name( )
 {
-    return (m_name.size())?m_name:mId;
+    string tNm = m_name;
+    return tNm.size() ? tNm : mId;
 }
 
 void WidgetLib::setFullDB( const string &it )
 {
-    work_lib_db = TSYS::strSepParse(it,0,'.')+"."+TSYS::strSepParse(it,1,'.');
-    m_dbt = TSYS::strSepParse(it,2,'.');
+    size_t dpos = it.rfind(".");
+    work_lib_db = (dpos!=string::npos) ? it.substr(0,dpos) : "";
+    m_dbt = (dpos!=string::npos) ? it.substr(dpos+1) : "";
     modifG();
 }
 
@@ -141,12 +143,24 @@ void WidgetLib::load_( )
     SYS->db().at().dataGet(DB()+"."+mod->wlbTable(),mod->nodePath()+"LIB/",*this);
 
     //> Create new widgets
+    map<string, bool>   itReg;
     TConfig c_el(&mod->elWdg());
     c_el.cfgViewAll(false);
-    for( int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),mod->nodePath()+tbl(), fld_cnt++,c_el); )
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),mod->nodePath()+tbl(),fld_cnt++,c_el); )
     {
 	string f_id = c_el.cfg("ID").getS();
-	if( !present(f_id) )	add(f_id,"","");
+	if(!present(f_id)) add(f_id,"","");
+	itReg[f_id] = true;
+    }
+
+    //>>> Check for remove items removed from DB
+    if(!SYS->selDB().empty())
+    {
+	vector<string> it_ls;
+        list(it_ls);
+        for(unsigned i_it = 0; i_it < it_ls.size(); i_it++)
+    	    if(itReg.find(it_ls[i_it]) == itReg.end())
+                del(it_ls[i_it]);
     }
 
     mOldDB = TBDS::realDBName(DB());
@@ -194,12 +208,12 @@ void WidgetLib::mimeDataList( vector<string> &list, const string &idb )
 {
     string wtbl = tbl()+"_mime";
     string wdb  = idb.empty() ? DB() : idb;
-    TConfig c_el( &mod->elWdgData() );
-    c_el.cfgViewAll( false );
+    TConfig c_el(&mod->elWdgData());
+    c_el.cfgViewAll(false);
 
-    list.clear( );
-    for( int fld_cnt = 0; SYS->db().at().dataSeek(wdb+"."+wtbl,mod->nodePath()+wtbl,fld_cnt,c_el); fld_cnt++ )
-	list.push_back( c_el.cfg("ID").getS() );
+    list.clear();
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(wdb+"."+wtbl,mod->nodePath()+wtbl,fld_cnt,c_el); fld_cnt++)
+	list.push_back(c_el.cfg("ID").getS());
 }
 
 bool WidgetLib::mimeDataGet( const string &iid, string &mimeType, string *mimeData, const string &idb )
@@ -290,7 +304,7 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
     if(opt->name() == "info")
     {
 	TCntrNode::cntrCmdProc(opt);
-	ctrMkNode("oscada_cntr",opt,-1,"/",_("Widget's library: ")+id(),RWRWR_,"root",SUI_ID);
+	ctrMkNode("oscada_cntr",opt,-1,"/",_("Widgets library: ")+id(),RWRWR_,"root",SUI_ID);
 	if(ico().size()) ctrMkNode("img",opt,-1,"/ico","",R_R_R_);
 	if(ctrMkNode("branches",opt,-1,"/br","",R_R_R_))
 	    ctrMkNode("grp",opt,-1,"/br/wdg_",_("Widget"),RWRWR_,"root",SUI_ID,2,"idm","1","idSz","30");
@@ -302,7 +316,7 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/obj/st/db",_("Library DB"),RWRWR_,"root",SUI_ID,4,"tp","str","dest","sel_ed","select","/db/tblList",
 		    "help",_("DB address in format [<DB module>.<DB name>.<Table name>].\nFor use main work DB set '*.*'."));
 	    }
-	    if(ctrMkNode("area",opt,-1,"/obj/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/obj/cfg",_("Configuration")))
 	    {
 		ctrMkNode("fld",opt,-1,"/obj/cfg/id",_("Id"),R_R_R_,"root",SUI_ID,1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/obj/cfg/name",_("Name"),RWRWR_,"root",SUI_ID,2,"tp","str","len","50");
@@ -487,8 +501,8 @@ string LWidget::path( )
 
 string LWidget::ico( )
 {
-    if( m_ico.size() )          return m_ico;
-    if( !parent().freeStat() )  return parent().at().ico();
+    if(m_ico.size() )          return m_ico;
+    if(!parent().freeStat() )  return parent().at().ico();
     return "";
 }
 
@@ -554,7 +568,7 @@ void LWidget::setCalcPer( int vl )
 
 void LWidget::setParentNm( const string &isw )
 {
-    if( enable() && mParent != isw ) setEnable(false);
+    if(enable() && mParent.getVal() != isw) setEnable(false);
     mParent = isw;
     modif();
 }
@@ -570,12 +584,13 @@ void LWidget::load_( )
 
     //> Inherit modify attributes
     vector<string> als;
-    attrList( als );
+    attrList(als);
+    string tAttrs = m_attrs;
     for(unsigned i_a = 0; i_a < als.size(); i_a++)
     {
 	if(!attrPresent(als[i_a])) continue;
 	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if(attr.at().modif() && m_attrs.find(als[i_a]+";") == string::npos)
+	if(attr.at().modif() && tAttrs.find(als[i_a]+";") == string::npos)
 	{
 	    attr.at().setModif(0);
 	    inheritAttr(als[i_a]);
@@ -583,7 +598,7 @@ void LWidget::load_( )
     }
 
     //> Load generic attributes
-    mod->attrsLoad( *this, db+"."+tbl, cfg("DBV").getI(), id(), "", m_attrs, true );
+    mod->attrsLoad( *this, db+"."+tbl, id(), "", tAttrs, true );
 
     //> Load all other attributes
     loadIO();
@@ -591,30 +606,42 @@ void LWidget::load_( )
 
 void LWidget::loadIO( )
 {
-    if( !enable() ) return;
+    if(!enable()) return;
 
     //> Load widget's work attributes
-    mod->attrsLoad( *this, ownerLib().DB()+"."+ownerLib().tbl(), cfg("DBV").getI(), id(), "", m_attrs );
+    mod->attrsLoad(*this, ownerLib().DB()+"."+ownerLib().tbl(), id(), "", m_attrs);
 
     //> Load cotainer widgets
-    if( !isContainer() ) return;
+    if(!isContainer()) return;
+    map<string, bool>   itReg;
     TConfig c_el(&mod->elInclWdg());
     string db  = ownerLib().DB();
     string tbl = ownerLib().tbl()+"_incl";
     c_el.cfg("IDW").setS(id(),true);
-    for( int fld_cnt=0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
     {
 	string sid  = c_el.cfg("ID").getS();
-	if( c_el.cfg("PARENT").getS() == "<deleted>" )
+	if(c_el.cfg("PARENT").getS() == "<deleted>")
 	{
-	    if( wdgPresent(sid) )	wdgDel(sid);
+	    if(wdgPresent(sid))	wdgDel(sid);
 	    continue;
 	}
-	if( !wdgPresent(sid) )
+	if(!wdgPresent(sid))
 	    try{ wdgAdd(sid,"",""); }
 	    catch(TError err){ mess_err(err.cat.c_str(),err.mess.c_str()); }
 
 	wdgAt(sid).at().load();
+	itReg[sid] = true;
+    }
+
+    //>>> Check for remove items removed from DB
+    if(!SYS->selDB().empty())
+    {
+	vector<string> it_ls;
+        wdgList(it_ls);
+        for(unsigned i_it = 0; i_it < it_ls.size(); i_it++)
+            if(itReg.find(it_ls[i_it]) == itReg.end())
+                wdgDel(it_ls[i_it]);
     }
 }
 
@@ -623,10 +650,8 @@ void LWidget::save_( )
     string db  = ownerLib().DB();
     string tbl = ownerLib().tbl();
 
-    cfg("DBV").setI(2);
-
     //> Save generic attributes
-    m_attrs = mod->attrsSave( *this, db+"."+tbl, cfg("DBV").getI(), id(), "", true );
+    m_attrs = mod->attrsSave( *this, db+"."+tbl, id(), "", true );
 
     //> Save generic widget's data
     SYS->db().at().dataSet( db+"."+tbl, mod->nodePath()+tbl, *this );
@@ -640,7 +665,7 @@ void LWidget::saveIO( )
     if( !enable() ) return;
 
     //> Save widget's attributes
-    mod->attrsSave( *this, ownerLib().DB()+"."+ownerLib().tbl(), cfg("DBV").getI(), id(), "" );
+    mod->attrsSave( *this, ownerLib().DB()+"."+ownerLib().tbl(), id(), "" );
 }
 
 void LWidget::wClear( )
@@ -724,7 +749,7 @@ void LWidget::cntrCmdProc( XMLNode *opt )
 //* CWidget: Container stored widget             *
 //************************************************
 CWidget::CWidget( const string &iid, const string &isrcwdg ) :
-        Widget(iid), TConfig(&mod->elInclWdg()), delMark(false),
+        Widget(iid), TConfig(&mod->elInclWdg()),
         mParent(cfg("PARENT").getSd()), m_attrs(cfg("ATTRS").getSd())
 {
     cfg("ID").setS(id());
@@ -757,20 +782,48 @@ void CWidget::postEnable( int flag )
 
 void CWidget::preDisable( int flag )
 {
-    if( flag )	delMark = !((flag>>8)&0x10) && !parent().freeStat() && parent().at().isLink();
+    if(flag)	ChldResrv = !((flag>>8)&0x10) && !parent().freeStat() && parent().at().isLink();
 
-    Widget::preDisable( flag );
+    Widget::preDisable(flag);
+}
+
+void CWidget::postDisable( int flag )
+{
+    if(flag)
+    {
+	string db  = ownerLWdg().ownerLib().DB();
+	string tbl = ownerLWdg().ownerLib().tbl();
+
+	//>> Remove from library table
+	if(ChldResrv)
+	{
+	    mParent = "<deleted>";
+	    SYS->db().at().dataSet(db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this);
+	}
+	else SYS->db().at().dataDel(db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this, true);
+
+
+	//>> Remove widget's work and users IO from library IO table
+	string tAttrs = m_attrs;
+
+	TConfig c_el( &mod->elWdgIO() );
+	c_el.cfg("IDW").setS( ownerLWdg().id(), true ); c_el.cfg("IDC").setS( id(), true );
+	SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el );
+	c_el.setElem(&mod->elWdgUIO());
+	c_el.cfg("IDW").setS( ownerLWdg().id(), true ); c_el.cfg("IDC").setS( id(), true );
+	SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el );
+    }
 }
 
 string CWidget::ico( )
 {
-    if( !parent().freeStat() )  return parent().at().ico();
+    if(!parent().freeStat()) return parent().at().ico();
     return "";
 }
 
 void CWidget::setParentNm( const string &isw )
 {
-    if( enable() && mParent != isw ) setEnable(false);
+    if(enable() && mParent.getVal() != isw) setEnable(false);
     mParent = isw;
     modif();
 }
@@ -786,7 +839,7 @@ void CWidget::setEnable( bool val )
 	for(unsigned i_h = 0; i_h < ownerLWdg().herit().size(); i_h++)
 	    if(!ownerLWdg().herit()[i_h].at().wdgAt(id()).at().enable() && ownerLWdg().herit()[i_h].at().wdgPresent(id()))
 		try { ownerLWdg().herit()[i_h].at().wdgAt(id()).at().setEnable(true); }
-		catch(...) { mess_err(nodePath().c_str(),_("Inheriting widget <%s> enable error"),id().c_str()); }
+		catch(...) { mess_err(nodePath().c_str(),_("Inheriting widget '%s' enable error."),id().c_str()); }
 }
 
 string CWidget::calcId( )
@@ -825,11 +878,12 @@ void CWidget::load_( )
     //> Inherit modify attributes
     vector<string> als;
     attrList(als);
+    string tAttrs = m_attrs;
     for(unsigned i_a = 0; i_a < als.size(); i_a++)
     {
 	if(!attrPresent(als[i_a])) continue;
 	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if(attr.at().modif() && m_attrs.find(als[i_a]+";") == string::npos)
+	if(attr.at().modif() && tAttrs.find(als[i_a]+";") == string::npos)
 	{
 	    attr.at().setModif(0);
 	    inheritAttr(als[i_a]);
@@ -837,7 +891,7 @@ void CWidget::load_( )
     }
 
     //> Load generic attributes
-    mod->attrsLoad( *this, db+"."+ownerLWdg().ownerLib().tbl(), cfg("DBV").getI(), ownerLWdg().id(), id(), m_attrs, true );
+    mod->attrsLoad( *this, db+"."+ownerLWdg().ownerLib().tbl(), ownerLWdg().id(), id(), tAttrs, true );
 
     //> Load all other attributes
     loadIO();
@@ -848,7 +902,7 @@ void CWidget::loadIO( )
     if( !enable() ) return;
 
     //> Load widget's work attributes
-    mod->attrsLoad( *this, ownerLWdg().ownerLib().DB()+"."+ownerLWdg().ownerLib().tbl(), cfg("DBV").getI(), ownerLWdg().id(), id(), m_attrs );
+    mod->attrsLoad( *this, ownerLWdg().ownerLib().DB()+"."+ownerLWdg().ownerLib().tbl(), ownerLWdg().id(), id(), m_attrs );
 }
 
 void CWidget::save_( )
@@ -856,63 +910,21 @@ void CWidget::save_( )
     string db  = ownerLWdg().ownerLib().DB();
     string tbl = ownerLWdg().ownerLib().tbl();
 
-    //> Delete from DB
-    if(nodeMode() == TCntrNode::Disable)
-    {
-	//>> Remove from library table
-	if(delMark)
-	{
-	    mParent = "<deleted>";
-	    SYS->db().at().dataSet(db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this);
-	}
-	else SYS->db().at().dataDel(db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this, true);
+    //> Save generic attributes
+    m_attrs = mod->attrsSave( *this, db+"."+tbl, ownerLWdg().id(), id(), true );
 
-	//>> Remove widget's work and users IO from library IO table
-	if(cfg("DBV").getI() == 1)
-	{
-	    TConfig c_el( &mod->elWdgIO() );
-	    c_el.cfg("IDW").setS( ownerLWdg().id(), true );
-	    for(unsigned i_a = 0; i_a < m_attrs.size(); i_a++)
-	    {
-		c_el.cfg("ID").setS(id()+"/"+m_attrs[i_a],true);
-		SYS->db().at().dataDel(db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el);
-	    }
-	    c_el.setElem(&mod->elWdgUIO());
-	    c_el.cfg("IDW").setS(ownerLWdg().id(), true);
-	    for(int io_cnt = 0; SYS->db().at().dataSeek( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", io_cnt++, c_el ); )
-		if(c_el.cfg("ID").getS().find(id()+"/") == 0)
-		{ SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el, true ); io_cnt--; }
-	}
-	if( cfg("DBV").getI() == 2 )
-	{
-	    TConfig c_el( &mod->elWdgIO() );
-	    c_el.cfg("IDW").setS( ownerLWdg().id(), true ); c_el.cfg("IDC").setS( id(), true );
-	    SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el );
-	    c_el.setElem(&mod->elWdgUIO());
-	    c_el.cfg("IDW").setS( ownerLWdg().id(), true ); c_el.cfg("IDC").setS( id(), true );
-	    SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el );
-	}
-    }
-    //> Save widget's data
-    else
-    {
-	cfg("DBV").setI(2);
-	//> Save generic attributes
-	m_attrs = mod->attrsSave( *this, db+"."+tbl, cfg("DBV").getI(), ownerLWdg().id(), id(), true );
+    //> Save generic widget's data
+    SYS->db().at().dataSet(db+"."+tbl+"_incl",mod->nodePath()+tbl+"_incl",*this);
 
-	//> Save generic widget's data
-	SYS->db().at().dataSet(db+"."+tbl+"_incl",mod->nodePath()+tbl+"_incl",*this);
-
-	//>> Save widget's attributes
-	saveIO();
-    }
+    //>> Save widget's attributes
+    saveIO();
 }
 
 void CWidget::saveIO( )
 {
-    if( !enable() ) return;
+    if(!enable()) return;
 
-    mod->attrsSave( *this, ownerLWdg().ownerLib().DB()+"."+ownerLWdg().ownerLib().tbl(), cfg("DBV").getI(), ownerLWdg().id(), id() );
+    mod->attrsSave(*this, ownerLWdg().ownerLib().DB()+"."+ownerLWdg().ownerLib().tbl(), ownerLWdg().id(), id());
 }
 
 void CWidget::wClear( )
