@@ -37,8 +37,8 @@
 #define MOD_TYPE	SUI_ID
 #define VER_TYPE	SUI_VER
 #define MOD_SUBTYPE	"VCAEngine"
-#define MOD_VER		"1.1.0"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"1.2.0"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Generic visual control area engine.")
 #define LICENSE		"GPL2"
 //*************************************************
@@ -49,14 +49,14 @@ extern "C"
 {
     TModule::SAt module( int n_mod )
     {
-	if( n_mod==0 )	return TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE);
+	if(n_mod == 0)	return TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE);
 	return TModule::SAt("");
     }
 
     TModule *attach( const TModule::SAt &AtMod, const string &source )
     {
-	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) )
-	    return new VCA::Engine( source );
+	if(AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE))
+	    return new VCA::Engine(source);
 	return NULL;
     }
 }
@@ -66,7 +66,7 @@ using namespace VCA;
 //************************************************
 //* Engine                                       *
 //************************************************
-Engine::Engine( string name ) : TUI(MOD_ID), mFrcClr(false),
+Engine::Engine( string name ) : TUI(MOD_ID),
     mSynthCom("echo \"%t\" | ru_tts | sox -t raw -s -b 8 -r 10k -c 1 -v 0.8 - -t ogg -"), mSynthCode("KOI8-R")
 {
     mod		= this;
@@ -74,7 +74,7 @@ Engine::Engine( string name ) : TUI(MOD_ID), mFrcClr(false),
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -102,23 +102,11 @@ string Engine::modInfo( const string &name )
     else return TModule::modInfo( name);
 }
 
-string Engine::optDescr( )
-{
-    char buf[STR_BUF_LEN];
-
-    snprintf(buf,sizeof(buf),_(
-	"======================= The module <%s:%s> options =======================\n"
-	"    --VCADBClearForce            Force clear VCA DB from data of API 1.\n\n"),
-	    MOD_TYPE,MOD_ID,nodePath().c_str());
-
-    return buf;
-}
-
 void Engine::postEnable( int flag )
 {
-    TModule::postEnable( flag );
+    TModule::postEnable(flag);
 
-    if( !(flag&TCntrNode::NodeConnect) ) return;
+    if(!(flag&TCntrNode::NodeConnect)) return;
 
     //> Make lib's DB structure
     lbwdg_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
@@ -137,21 +125,19 @@ void Engine::postEnable( int flag )
     wdg_el.fldAdd( new TFld("ICO",_("Icon"),TFld::String,TFld::NoFlag,"10000") );
     wdg_el.fldAdd( new TFld("PARENT",_("Parent widget"),TFld::String,TFld::NoFlag,"200") );
     wdg_el.fldAdd( new TFld("PROC",_("Procedure text and language"),TFld::String,TCfg::TransltText,"10000") );
-    wdg_el.fldAdd( new TFld("PROC_PER",_("Procedure calc period"),TFld::Integer,TFld::NoFlag,"5","-1") );
+    wdg_el.fldAdd( new TFld("PROC_PER",_("Procedure calculate period"),TFld::Integer,TFld::NoFlag,"5","-1") );
     wdg_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
-    wdg_el.fldAdd( new TFld("DBV",_("DB version"),TFld::Integer,TFld::NoFlag,"1","1") );
 
     //> Make include widgets' DB structure
     inclwdg_el.fldAdd( new TFld("IDW",_("IDW"),TFld::String,TCfg::Key,"100") );
     inclwdg_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
     inclwdg_el.fldAdd( new TFld("PARENT",_("Parent widget"),TFld::String,TFld::NoFlag,"200") );
     inclwdg_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
-    inclwdg_el.fldAdd( new TFld("DBV",_("DB version"),TFld::Integer,TFld::NoFlag,"1","1") );
 
     //> Make widget's IO DB structure
     wdgio_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"100") );
-    wdgio_el.fldAdd( new TFld("IDC",_("Child ID"),TFld::String,TCfg::Key,"30") );
     wdgio_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"61") );
+    wdgio_el.fldAdd( new TFld("IDC",_("Child ID"),TFld::String,TCfg::Key,"30") );
     wdgio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TCfg::TransltText,"100000") );
     wdgio_el.fldAdd( new TFld("SELF_FLG",_("Attribute self flags"),TFld::Integer,TFld::NoFlag,"5") );
     wdgio_el.fldAdd( new TFld("CFG_TMPL",_("Configuration template"),TFld::String,TCfg::TransltText,"30") );
@@ -159,8 +145,8 @@ void Engine::postEnable( int flag )
 
     //> Make widget's user IO DB structure
     wdguio_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"100") );
-    wdguio_el.fldAdd( new TFld("IDC",_("Child ID"),TFld::String,TCfg::Key,"30") );
     wdguio_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"61") );
+    wdguio_el.fldAdd( new TFld("IDC",_("Child ID"),TFld::String,TCfg::Key,"30") );
     wdguio_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"100") );
     wdguio_el.fldAdd( new TFld("IO_TYPE",_("Attribute generic flags and type"),TFld::Integer,TFld::NoFlag,"10") );
     wdguio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TCfg::TransltText,"100000") );
@@ -176,8 +162,8 @@ void Engine::postEnable( int flag )
     prj_el.fldAdd( new TFld("ICO",_("Icon"),TFld::String,TFld::NoFlag,"10000") );
     prj_el.fldAdd( new TFld("USER",_("User"),TFld::String,TFld::NoFlag,"20","root") );
     prj_el.fldAdd( new TFld("GRP",_("Group"),TFld::String,TFld::NoFlag,"20","UI") );
-    prj_el.fldAdd( new TFld("PERMIT",_("Permision"),TFld::Integer,TFld::OctDec,"3","436") );
-    prj_el.fldAdd( new TFld("PER",_("Calc period (ms)"),TFld::Integer,TFld::NoFlag,"4","100") );
+    prj_el.fldAdd( new TFld("PERMIT",_("Permission"),TFld::Integer,TFld::OctDec,"3","436") );
+    prj_el.fldAdd( new TFld("PER",_("Calculate period (ms)"),TFld::Integer,TFld::NoFlag,"4","100") );
     prj_el.fldAdd( new TFld("STYLE",_("Work style"),TFld::Integer,TFld::NoFlag,"2","-1") );
     prj_el.fldAdd( new TFld("FLGS",_("Flags"),TFld::Integer,TFld::NoFlag,"4") );
 
@@ -187,10 +173,9 @@ void Engine::postEnable( int flag )
     page_el.fldAdd( new TFld("ICO",_("Icon"),TFld::String,TFld::NoFlag,"10000") );
     page_el.fldAdd( new TFld("PARENT",_("Parent widget"),TFld::String,TFld::NoFlag,"200") );
     page_el.fldAdd( new TFld("PROC",_("Procedure text and language"),TFld::String,TCfg::TransltText,"10000") );
-    page_el.fldAdd( new TFld("PROC_PER",_("Procedure calc period"),TFld::Integer,TFld::NoFlag,"5","-1") );
+    page_el.fldAdd( new TFld("PROC_PER",_("Procedure calculate period"),TFld::Integer,TFld::NoFlag,"5","-1") );
     page_el.fldAdd( new TFld("FLGS",_("Flags"),TFld::Integer,TFld::NoFlag,"1","0") );
     page_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
-    page_el.fldAdd( new TFld("DBV",_("DB version"),TFld::Integer,TFld::NoFlag,"1","1") );
 
     //> Make sessions' IO values of projects DB structure
     prj_ses_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"200") );
@@ -255,44 +240,24 @@ void Engine::preDisable( int flag )
     for(unsigned l_id = 0; l_id < ls.size(); l_id++)
 	wlbAt(ls[l_id]).at().setEnable(false);
 
-    TModule::preDisable( flag );
+    TModule::preDisable(flag);
 }
 
 void Engine::load_( )
 {
     mess_info(nodePath().c_str(),_("Load module."));
 
-    //> Load parameters from command line
-    int next_opt;
-    const char *short_opt="h";
-    struct option long_opt[] =
-    {
-	{"help"    ,0,NULL,'h'},
-	{"VCADBClearForce" ,0,NULL,'f'},
-	{NULL      ,0,NULL,0  }
-    };
-
-    optind=opterr=0;
-    do
-    {
-	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
-	switch(next_opt)
-	{
-	    case 'h': fprintf(stdout,"%s",optDescr().c_str()); break;
-	    case 'f': mFrcClr = true; break;
-	    case -1 : break;
-	}
-    } while(next_opt != -1);
-
-    //> Load parameters from config file and DB
+    //> Load parameters from config-file and DB
     setSynthCom( TBDS::genDBGet(nodePath()+"SynthCom",synthCom()) );
     setSynthCode( TBDS::genDBGet(nodePath()+"SynthCode",synthCode()) );
 
 #if OSC_DEBUG >= 3
-    long long w_tm = TSYS::curTime();
+    int64_t w_tm = TSYS::curTime();
 #endif
 
-    //>> Load widget's libraries
+    map<string, bool>	itReg;
+
+    //>> Load widgets libraries
     try
     {
 	//>>> Search and create new libraries
@@ -302,20 +267,23 @@ void Engine::load_( )
 
 	//>>>> Search into DB
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+wlbTable(),"",lib_cnt++,c_el); )
+	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+wlbTable(),nodePath()+"LIB",lib_cnt++,c_el); )
 	    {
 		string l_id = c_el.cfg("ID").getS();
 		if(!wlbPresent(l_id)) wlbAdd(l_id,"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+		itReg[l_id] = true;
 	    }
 
-	//>>>> Search into config file
-	if(SYS->chkSelDB("<cfg>"))
-	    for(int lib_cnt = 0; SYS->db().at().dataSeek("",nodePath()+"LIB/",lib_cnt++,c_el); )
-	    {
-		string l_id = c_el.cfg("ID").getS();
-		if(!wlbPresent(l_id))	wlbAdd(l_id,"","*.*");
-	    }
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            wlbList(db_ls);
+            for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                if(itReg.find(db_ls[i_it]) == itReg.end() && SYS->chkSelDB(wlbAt(db_ls[i_it]).at().DB()))
+                    wlbDel(db_ls[i_it]);
+        }
 
 #if OSC_DEBUG >= 3
 	w_tm = TSYS::curTime();
@@ -334,7 +302,7 @@ void Engine::load_( )
     }catch( TError err )
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-	mess_err(nodePath().c_str(),_("Load widget's libraries error."));
+	mess_err(nodePath().c_str(),_("Load widgets libraries error."));
     }
 
     //>> Load projects
@@ -344,23 +312,27 @@ void Engine::load_( )
 	TConfig c_el(&elProject());
 	c_el.cfgViewAll(false);
 	vector<string> db_ls;
+	itReg.clear();
 
 	//>>>> Search into DB
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+prjTable(),"",lib_cnt++,c_el); )
+	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+prjTable(),nodePath()+"PRJ",lib_cnt++,c_el); )
 	    {
 		string prj_id = c_el.cfg("ID").getS();
 		if(!prjPresent(prj_id))	prjAdd(prj_id,"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+		itReg[prj_id] = true;
 	    }
 
-	//>>>> Search into config file
-	if(SYS->chkSelDB("<cfg>"))
-	    for(int el_cnt = 0; SYS->db().at().dataSeek("",nodePath()+"PRJ/",el_cnt++,c_el); )
-	    {
-		string prj_id = c_el.cfg("ID").getS();
-		if(!prjPresent(prj_id))	prjAdd(prj_id,"","*.*");
-	    }
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            prjList(db_ls);
+            for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                if(itReg.find(db_ls[i_it]) == itReg.end() && SYS->chkSelDB(prjAt(db_ls[i_it]).at().DB()))
+                    prjDel(db_ls[i_it]);
+        }
 
 #if OSC_DEBUG >= 3
 	w_tm = TSYS::curTime();
@@ -430,6 +402,7 @@ void Engine::load_( )
 	    }catch(...){ }
 	}
     }catch(...){ }
+    res.release();
 
     modifGClr( );
 }
@@ -568,16 +541,14 @@ string Engine::callSynth( const string &itxt )
     return TSYS::strEncode( rez, TSYS::base64 );
 }
 
-void Engine::attrsLoad( Widget &w, const string &fullDB, int vDB, const string &idw, const string &idc, const string &attrs, bool ldGen )
+void Engine::attrsLoad( Widget &w, const string &fullDB, const string &idw, const string &idc, const string &attrs, bool ldGen )
 {
     string wdb = fullDB+"_io";
     string tbl = TSYS::strSepParse(wdb,2,';');
 
-    if( vDB < 1 || vDB > 2 ) return;
-
     TConfig c_el(&elWdgIO());
     c_el.cfg("IDW").setS(idw);
-    if( vDB == 2 ) c_el.cfg("IDC").setS(idc);
+    c_el.cfg("IDC").setS(idc);
     string tstr;
 
     for( int off = 0; !(tstr = TSYS::strSepParse(attrs,0,';',&off)).empty(); )
@@ -589,10 +560,12 @@ void Engine::attrsLoad( Widget &w, const string &fullDB, int vDB, const string &
 		(!ldGen && (attr.at().flgGlob()&Attr::Generic || (!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser))) )
 	    continue;
 
-	if( vDB == 1 ) c_el.cfg("ID").setS( idc.empty() ? tstr : idc+"/"+tstr );
-	else if( vDB == 2 ) c_el.cfg("ID").setS(tstr);
+	c_el.cfg("ID").setS(tstr);
         c_el.cfg("IO_VAL").setNoTransl(!(attr.at().type() == TFld::String &&
 		!(attr.at().flgGlob()&(TFld::NoStrTransl|Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address))));
+	c_el.cfg("CFG_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
+		!(attr.at().flgGlob()&(TFld::NoStrTransl|Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address)) &&
+		(attr.at().flgSelf()&(Attr::CfgConst|Attr::CfgLnkIn))) );
 
 	if( !SYS->db().at().dataGet(wdb,nodePath()+tbl,c_el) ) continue;
 
@@ -609,26 +582,17 @@ void Engine::attrsLoad( Widget &w, const string &fullDB, int vDB, const string &
     tbl = TSYS::strSepParse(wdb,2,';');
     c_el.setElem(&elWdgUIO());
     c_el.cfg("IDW").setS(idw,true);
-    if( vDB == 2 ) c_el.cfg("IDC").setS(idc,true);
+    c_el.cfg("IDC").setS(idc,true);
 
-    for( int fld_cnt = 0; SYS->db().at().dataSeek(wdb,nodePath()+tbl,fld_cnt++,c_el); )
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(wdb,nodePath()+tbl,fld_cnt++,c_el); )
     {
 	string sid = c_el.cfg("ID").getS();
 	unsigned flg = c_el.cfg("IO_TYPE").getI();
 
-	if( vDB == 1 )
-	{
-	    if( idc.empty() && !TSYS::pathLev(sid,1).empty() ) continue;
-	    if( !idc.empty() )
-	    {
-		if( TSYS::pathLev(sid,0) == idc && !TSYS::pathLev(sid,1).empty() ) sid = TSYS::pathLev(sid,1);
-		else continue;
-	    }
-	}
-	else if( vDB == 2 && !TSYS::pathLev(sid,1).empty() ) continue;
+	if(!TSYS::pathLev(sid,1).empty()) continue;
 
-	if( !w.attrPresent(sid) )
-	    w.attrAdd( new TFld(sid.c_str(),c_el.cfg("NAME").getS().c_str(),(TFld::Type)(flg&0x0f),flg>>4) );
+	if(!w.attrPresent(sid))
+	    w.attrAdd(new TFld(sid.c_str(),c_el.cfg("NAME").getS().c_str(),(TFld::Type)(flg&0x0f),flg>>4));
 	AutoHD<Attr> attr = w.attrAt(sid);
 	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
 	attr.at().setS(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),0,'|'));
@@ -640,19 +604,17 @@ void Engine::attrsLoad( Widget &w, const string &fullDB, int vDB, const string &
     }
 }
 
-string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string &idw, const string &idc, bool ldGen )
+string Engine::attrsSave( Widget &w, const string &fullDB, const string &idw, const string &idc, bool ldGen )
 {
     string tbl = TSYS::strSepParse(fullDB,2,';');
     string m_attrs = "";
     vector<string> als;
 
-    if( vDB < 1 || vDB > 2 ) return m_attrs;
-
     w.attrList( als );
     TConfig c_el(&mod->elWdgIO()); c_el.cfg("IDW").setS(idw,true);
-    if( vDB == 2 ) c_el.cfg("IDC").setS(idc,true);
+    c_el.cfg("IDC").setS(idc,true);
     TConfig c_elu(&mod->elWdgUIO()); c_elu.cfg("IDW").setS(idw,true);
-    if( vDB == 2 ) c_elu.cfg("IDC").setS(idc,true);
+    c_elu.cfg("IDC").setS(idc,true);
     for(unsigned i_a = 0; i_a < als.size(); i_a++)
     {
 	AutoHD<Attr> attr = w.attrAt(als[i_a]);
@@ -663,8 +625,7 @@ string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string
 	//> Main attributes store
 	if(attr.at().flgSelf()&Attr::IsInher || !(attr.at().flgGlob()&Attr::IsUser))
 	{
-	    if( vDB == 1 ) c_el.cfg("ID").setS( idc.empty() ? als[i_a] : idc+"/"+als[i_a] );
-	    if( vDB == 2 ) c_el.cfg("ID").setS( als[i_a] );
+	    c_el.cfg("ID").setS( als[i_a] );
 	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
 		    !(attr.at().flgGlob()&(TFld::NoStrTransl|Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address))) );
 	    c_el.cfg("IO_VAL").setS(attr.at().getS());
@@ -672,15 +633,14 @@ string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string
 	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
 	    c_el.cfg("CFG_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
 		    !(attr.at().flgGlob()&(TFld::NoStrTransl|Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address)) &&
-		    (attr.at().flgSelf()&Attr::CfgConst || (attr.at().flgSelf()&Attr::CfgLnkIn && attr.at().cfgVal().substr(0,4) == "val:"))) );
+		    (attr.at().flgSelf()&(Attr::CfgConst|Attr::CfgLnkIn))) );
 	    c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
 	    SYS->db().at().dataSet(fullDB+"_io",nodePath()+tbl+"_io",c_el);
 	}
 	//> User attributes store
 	else if( !ldGen )
 	{
-	    if( vDB == 1 ) c_elu.cfg("ID").setS( idc.empty() ? als[i_a] : idc+"/"+als[i_a] );
-	    if( vDB == 2 ) c_elu.cfg("ID").setS( als[i_a] );
+	    c_elu.cfg("ID").setS( als[i_a] );
 	    c_elu.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
 		    !(attr.at().flgGlob()&(TFld::NoStrTransl|Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address))) );
 	    c_elu.cfg("IO_VAL").setS(attr.at().getS()+"|"+
@@ -692,7 +652,7 @@ string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string
 	    c_elu.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
 	    c_elu.cfg("CFG_VAL").setNoTransl( !(attr.at().type() == TFld::String &&
 		    !(attr.at().flgGlob()&(TFld::NoStrTransl|Attr::Image|Attr::DateTime|Attr::Color|Attr::Font|Attr::Address)) &&
-		    (attr.at().flgSelf()&Attr::CfgConst || (attr.at().flgSelf()&Attr::CfgLnkIn && attr.at().cfgVal().substr(0,4) == "val:"))) );
+		    (attr.at().flgSelf()&(Attr::CfgConst|Attr::CfgLnkIn))) );
 	    c_elu.cfg("CFG_VAL").setS(attr.at().cfgVal());
 	    SYS->db().at().dataSet(fullDB+"_uio",nodePath()+tbl+"_uio",c_elu);
 	}
@@ -702,10 +662,10 @@ string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string
     {
 	//> Clear no present IO for main io table
 	c_el.cfgViewAll(false);
-	for( int fld_cnt=0; vDB == 2 && SYS->db().at().dataSeek(fullDB+"_io",nodePath()+tbl+"_io",fld_cnt++,c_el); )
+	for(int fld_cnt = 0; SYS->db().at().dataSeek(fullDB+"_io",nodePath()+tbl+"_io",fld_cnt++,c_el); )
 	{
 	    string sid = c_el.cfg("ID").getS();
-	    if( w.attrPresent(sid) || (idc.empty() && !TSYS::pathLev(sid,1).empty() && !forceDBClear()) ) continue;
+	    if(w.attrPresent(sid) || (idc.empty() && !TSYS::pathLev(sid,1).empty())) continue;
 
 	    SYS->db().at().dataDel(fullDB+"_io",nodePath()+tbl+"_io",c_el,true);
 	    fld_cnt--;
@@ -713,12 +673,10 @@ string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string
 
 	//> Clear no present IO for user io table
 	c_elu.cfgViewAll(false);
-	for( int fld_cnt=0; SYS->db().at().dataSeek(fullDB+"_uio",nodePath()+tbl+"_uio",fld_cnt++,c_elu); )
+	for(int fld_cnt = 0; SYS->db().at().dataSeek(fullDB+"_uio",nodePath()+tbl+"_uio",fld_cnt++,c_elu); )
 	{
 	    string sid = c_elu.cfg("ID").getS();
-	    if( vDB == 1 && idc.empty() && (!TSYS::pathLev(sid,1).empty() || w.attrPresent(sid)) ) continue;
-	    if( vDB == 1 && !idc.empty() && (TSYS::pathLev(sid,0) != idc || w.attrPresent(TSYS::pathLev(sid,1))) ) continue;
-	    if( vDB == 2 && (w.attrPresent(sid) || (idc.empty() && !TSYS::pathLev(sid,1).empty() && !forceDBClear())) ) continue;
+	    if(w.attrPresent(sid) || (idc.empty() && !TSYS::pathLev(sid,1).empty())) continue;
 
 	    SYS->db().at().dataDel(fullDB+"_uio",nodePath()+tbl+"_uio",c_elu,true);
 	    fld_cnt--;
@@ -728,6 +686,16 @@ string Engine::attrsSave( Widget &w, const string &fullDB, int vDB, const string
     return m_attrs;
 }
 
+void Engine::perSYSCall( unsigned int cnt )
+{
+    //> Check for stop and remove lost and not used already project's sessions
+    vector<string> sls;
+    sesList(sls);
+    for(unsigned i_s = 0; i_s < sls.size(); i_s++)
+	if(!sesAt(sls[i_s]).at().backgrnd() && (time(NULL)-sesAt(sls[i_s]).at().reqTm()) > 30*60)
+	    sesDel(sls[i_s]);
+}
+
 void Engine::cntrCmdProc( XMLNode *opt )
 {
     string a_path = opt->attr("path");
@@ -755,7 +723,7 @@ void Engine::cntrCmdProc( XMLNode *opt )
 	    //>> User permission check
 	    AutoHD<Project> wprj = (!sess.empty()) ? sesAt(sess).at().parent() : prjAt(prj);
 	    if(!SYS->security().at().access(opt->attr("user"),SEC_RD,wprj.at().owner(),wprj.at().grp(),wprj.at().permit()))
-		throw TError(nodePath().c_str(),_("Connection to session is not permited for '%s'."),opt->attr("user").c_str());
+		throw TError(nodePath().c_str(),_("Connection to session is not permitted for '%s'."),opt->attr("user").c_str());
 	    //>> Connect to present session
 	    if(!sess.empty())	sesAt(sess).at().connect();
 	    //>> Create session
@@ -832,13 +800,13 @@ void Engine::cntrCmdProc( XMLNode *opt )
     {
 	TUI::cntrCmdProc(opt);
 	ctrMkNode("grp",opt,-1,"/br/prj_",_("Project"),RWRWR_,"root",SUI_ID,2,"idm","1","idSz","30");
-	ctrMkNode("grp",opt,-1,"/br/wlb_",_("Widget's library"),RWRWR_,"root",SUI_ID,2,"idm","1","idSz","30");
+	ctrMkNode("grp",opt,-1,"/br/wlb_",_("Widgets library"),RWRWR_,"root",SUI_ID,2,"idm","1","idSz","30");
 	ctrMkNode("grp",opt,-1,"/br/ses_",_("Session"),RWRWR_,"root",SUI_ID);
 	ctrMkNode("grp",opt,-1,"/br/vca",_("Functions"),R_R_R_,"root",SUI_ID,1,"idm","1");
 	if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Configuration"),R_R_R_,"root",SUI_ID))
 	{
 	    ctrMkNode("list",opt,-1,"/prm/cfg/prj",_("Project"),RWRWR_,"root",SUI_ID,5,"tp","br","idm","1","s_com","add,del","br_pref","prj_","idSz","30");
-	    ctrMkNode("list",opt,-1,"/prm/cfg/wlb",_("Widget's libraries"),RWRWR_,"root",SUI_ID,5,"tp","br","idm","1","s_com","add,del","br_pref","wlb_","idSz","30");
+	    ctrMkNode("list",opt,-1,"/prm/cfg/wlb",_("Widgets libraries"),RWRWR_,"root",SUI_ID,5,"tp","br","idm","1","s_com","add,del","br_pref","wlb_","idSz","30");
 	}
 	if(ctrMkNode("area",opt,1,"/ses",_("Sessions"),R_R_R_,"root",SUI_ID))
 	{
