@@ -88,9 +88,9 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     if(!ico_t.load(TUIS::icoPath("help").c_str())) ico_t.load(":/images/help.png");
     QAction *actAbout = new QAction(QPixmap::fromImage(ico_t),_("&About"),this);
     actAbout->setShortcut(Qt::Key_F1);
-    actAbout->setToolTip(_("Programm and OpenSCADA information"));
-    actAbout->setWhatsThis(_("The button for display the programm and OpenSCADA information"));
-    actAbout->setStatusTip(_("Press to display the programm and OpenSCADA information."));
+    actAbout->setToolTip(_("Program and OpenSCADA information"));
+    actAbout->setWhatsThis(_("The button for display the program and OpenSCADA information"));
+    actAbout->setStatusTip(_("Press to display the program and OpenSCADA information."));
     connect(actAbout, SIGNAL(triggered()), this, SLOT(about()));
     //>>> About Qt
     QAction *actQtAbout = new QAction(_("About &Qt"),this);
@@ -128,7 +128,7 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     if(!ico_t.load(TUIS::icoPath("vision_prj_run").c_str())) ico_t.load(":/images/prj_run.png");
     actPrjRun = new QAction(QPixmap::fromImage(ico_t),_("Run project execution"),this);
     actPrjRun->setToolTip(_("Run project execution for selected item"));
-    actPrjRun->setWhatsThis(_("The button for runing project execution for selected item"));
+    actPrjRun->setWhatsThis(_("The button for running project execution for selected item"));
     actPrjRun->setStatusTip(_("Press to run project execution for selected item."));
     actPrjRun->setEnabled(false);
     connect(actPrjRun, SIGNAL(triggered()), this, SLOT(prjRun()));
@@ -190,6 +190,24 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     actVisItClear->setStatusTip(_("Press to go to visual item changes clear."));
     actVisItClear->setEnabled(false);
     connect(actVisItClear, SIGNAL(triggered()), this, SLOT(visualItClear()));
+    //>>> UnDo visual item changes
+    if(!ico_t.load(TUIS::icoPath("undo").c_str())) ico_t.load(":/images/undo.png");
+    actVisItUnDo = new QAction(QPixmap::fromImage(ico_t),_("Visual item changes UnDo"),this);
+    actVisItUnDo->setObjectName("undo");
+    actVisItUnDo->setToolTip(_("Make visual item changes UnDo:"));
+    actVisItUnDo->setWhatsThis(_("The button for making visual item changes UnDo"));
+    actVisItUnDo->setStatusTip(_("Press to make visual item changes UnDo."));
+    actVisItUnDo->setShortcut(QKeySequence("Ctrl+Z"));
+    actVisItUnDo->setEnabled(false);
+    //>>> ReDo visual item changes
+    if(!ico_t.load(TUIS::icoPath("redo").c_str())) ico_t.load(":/images/redo.png");
+    actVisItReDo = new QAction(QPixmap::fromImage(ico_t),_("Visual item changes ReDo"),this);
+    actVisItReDo->setObjectName("redo");
+    actVisItReDo->setToolTip(_("Make visual item changes ReDo:"));
+    actVisItReDo->setWhatsThis(_("The button for making visual item changes ReDo"));
+    actVisItReDo->setStatusTip(_("Press to make visual item changes ReDo."));
+    actVisItReDo->setShortcut(QKeySequence("Ctrl+Y"));
+    actVisItReDo->setEnabled(false);
     //>>> Cut visual item
     if(!ico_t.load(TUIS::icoPath("editcut").c_str())) ico_t.load(":/images/editcut.png");
     actVisItCut = new QAction(QPixmap::fromImage(ico_t),_("Visual item cut"),this);
@@ -336,7 +354,7 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     actElFigArc->setEnabled(false);
     //>>> Add Besie curve
     if(!ico_t.load(TUIS::icoPath("vision_elfig_besie").c_str())) ico_t.load(":/images/elfig_besie.png");
-    actElFigBesie = new QAction(QPixmap::fromImage(ico_t),_("Add besier curve"),this);
+    actElFigBesie = new QAction(QPixmap::fromImage(ico_t),_("Add Besier curve"),this);
     actElFigBesie->setCheckable(true);
     actElFigBesie->setObjectName("besier");
     actElFigBesie->setToolTip(_("Add Besier curve to elementary figure"));
@@ -408,6 +426,9 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     mn_file->addAction(actClose);
     mn_file->addAction(actQuit);
     mn_edit = menuBar()->addMenu(_("&Edit"));
+    mn_edit->addAction(actVisItUnDo);
+    mn_edit->addAction(actVisItReDo);
+    mn_edit->addSeparator();
     mn_edit->addAction(actVisItCut);
     mn_edit->addAction(actVisItCopy);
     mn_edit->addAction(actVisItPaste);
@@ -471,6 +492,9 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     visItToolBar->addAction(actVisItProp);
     visItToolBar->addAction(actVisItEdit);
     visItToolBar->addSeparator();
+    visItToolBar->addAction(actVisItUnDo);
+    visItToolBar->addAction(actVisItReDo);
+    visItToolBar->addSeparator();
     visItToolBar->addAction(actVisItCut);
     visItToolBar->addAction(actVisItCopy);
     visItToolBar->addAction(actVisItPaste);
@@ -507,7 +531,7 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     mn_view->addSeparator();
     //> Init status bar
     mWUser = new UserStBar( open_user.c_str(), user_pass.c_str(), VCAstat.c_str(), this);
-    mWUser->setWhatsThis(_("This label displays curent user."));
+    mWUser->setWhatsThis(_("This label displays current user."));
     mWUser->setToolTip(_("Field for display the current user."));
     mWUser->setStatusTip(_("Double click to change user."));
     statusBar()->insertPermanentWidget(0,mWUser);
@@ -583,11 +607,11 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     string st = TSYS::strDecode(mod->uiPropGet("devWinState",user()),TSYS::base64);
     restoreState(QByteArray(st.data(),st.size()));
     //Restore ToolBars icons size
-    for( int i_ch = 0; i_ch < children().size(); i_ch++ )
+    for(int i_ch = 0; i_ch < children().size(); i_ch++)
     {
-	if( !qobject_cast<QToolBar*>(children()[i_ch]) ) continue;
+	if(!qobject_cast<QToolBar*>(children()[i_ch])) continue;
 	int icSz = atoi(mod->uiPropGet((children()[i_ch]->objectName()+"_icSz").toAscii().data(),user()).c_str());
-	if( icSz ) ((QToolBar*)children()[i_ch])->setIconSize(QSize(icSz,icSz));
+	if(icSz) ((QToolBar*)children()[i_ch])->setIconSize(QSize(icSz,icSz));
     }
 
     //Hide specific tools
@@ -595,7 +619,7 @@ VisDevelop::VisDevelop( const string &open_user, const string &user_pass, const
     elFigTool->setVisible(false);
 
     //mWStat->setText(host.st_nm.c_str());
-    statusBar()->showMessage(_("Ready"), 2000 );
+    statusBar()->showMessage(_("Ready"), 2000);
 }
 
 VisDevelop::~VisDevelop()
@@ -611,8 +635,8 @@ VisDevelop::~VisDevelop()
     work_wdgTimer->stop();
 
     //> Other data clean
-    if( prjLibPropDlg )	delete prjLibPropDlg;
-    if( visItPropDlg )	delete visItPropDlg;
+    if(prjLibPropDlg)	delete prjLibPropDlg;
+    if(visItPropDlg)	delete visItPropDlg;
 
     mod->unregWin(this);
 }
@@ -675,11 +699,11 @@ void VisDevelop::closeEvent( QCloseEvent* ce )
 QMenu *VisDevelop::createPopupMenu( )
 {
     QMenu *mn = QMainWindow::createPopupMenu( );
-    if( !mn ) return mn;
+    if(!mn) return mn;
 
     //> Check for widget under cursor
     QWidget *ucw = childAt(mapFromGlobal(QCursor::pos()));
-    if( qobject_cast<QToolBar*>(ucw) && !mn->children().isEmpty() )
+    if(qobject_cast<QToolBar*>(ucw) && !mn->children().isEmpty())
     {
 	QAction *first = mn->actions().isEmpty() ? NULL : mn->actions()[0];
 	QMenu *iSz = new QMenu(_("Icons size"));
@@ -716,11 +740,11 @@ QMenu *VisDevelop::createPopupMenu( )
 
 void VisDevelop::setToolIconSize( )
 {
-    if( !sender() ) return;
+    if(!sender()) return;
 
     QToolBar *tb = qobject_cast<QToolBar*>((QToolBar*)TSYS::str2addr(sender()->property("toolAddr").toString().toAscii().data()));
     int icSz = atoi(sender()->objectName().toAscii().data());
-    if( tb )
+    if(tb)
     {
 	tb->setIconSize(QSize(icSz,icSz));
 	mod->uiPropSet((tb->objectName()+"_icSz").toAscii().data(),TSYS::int2str(icSz),user());
@@ -769,18 +793,18 @@ bool VisDevelop::exitModifChk( )
 
 void VisDevelop::endRunChk( )
 {
-    if( mod->endRun() )	close();
+    if(mod->endRun())	close();
 }
 
 void VisDevelop::quitSt()
 {
-    if( exitModifChk( ) ) SYS->stop( );
+    if(exitModifChk())	SYS->stop( );
 }
 
 void VisDevelop::about()
 {
     QMessageBox::about(this,windowTitle(),
-	    QString(_("%1 v%2.\n%3\nAutor: %4\nDevelopers: %5\nLicense: %6\n\n%7 v%8.\n%9\nLicense: %10\nAuthor: %11\nWeb site: %12")).
+	    QString(_("%1 v%2.\n%3\nAuthor: %4\nDevelopers: %5\nLicense: %6\n\n%7 v%8.\n%9\nLicense: %10\nAuthor: %11\nWeb site: %12")).
 		arg(mod->modInfo("Name").c_str()).arg(mod->modInfo("Version").c_str()).arg(mod->modInfo("Description").c_str()).
 		arg(mod->modInfo("Author").c_str()).arg(mod->modInfo(_("Developers")).c_str()).arg(mod->modInfo("License").c_str()).
 		arg(PACKAGE_NAME).arg(VERSION).arg(_(PACKAGE_DESCR)).arg(PACKAGE_LICENSE).arg(_(PACKAGE_AUTHOR)).arg(PACKAGE_SITE));
@@ -798,16 +822,16 @@ void VisDevelop::enterWhatsThis()
 
 void VisDevelop::selectItem( const string &item, bool force )
 {
-    if( winClose )	return;
+    if(winClose) return;
 
     work_wdg_new = item;
-    if( force )	applyWorkWdg( );
-    else work_wdgTimer->start( );
+    if(force)	applyWorkWdg();
+    else work_wdgTimer->start();
 }
 
 void VisDevelop::applyWorkWdg( )
 {
-    if( winClose )      return;
+    if(winClose) return;
 
     modifyToolUpdate(work_wdg_new);
 
@@ -816,7 +840,7 @@ void VisDevelop::applyWorkWdg( )
     lnkInsp->setWdg(work_wdg_new);
 
     //> Update actions
-    if( work_wdg == work_wdg_new )	return;
+    if(work_wdg == work_wdg_new) return;
     work_wdg = work_wdg_new;
 
     string cur_wdg = TSYS::strSepParse(work_wdg,0,';');	//Get first select element
@@ -833,13 +857,14 @@ void VisDevelop::applyWorkWdg( )
     //> Set visual item's actions
     actVisItAdd->setEnabled(isProj || (isLib&&sel3.empty()));
     //>> Process add actions of visual items
-    for( int i_a = 0; i_a < actGrpWdgAdd->actions().size(); i_a++ )
+    for(int i_a = 0; i_a < actGrpWdgAdd->actions().size(); i_a++)
 	actGrpWdgAdd->actions().at(i_a)->setEnabled(isProj || (isLib&&sel3.empty()));
     //> Process visual item actions
     actVisItDel->setEnabled(isProj || isLib);
     actVisItProp->setEnabled(isProj || isLib);
     actVisItEdit->setEnabled((isProj || isLib) && sel2.size());
     actVisItClear->setEnabled((isProj || isLib) && sel2.size());
+
     //> Edit actions update
     editToolUpdate( );
 }
@@ -851,10 +876,10 @@ void VisDevelop::modifyToolUpdate( const string &wdgs )
     //> Request modify flag for select widgets
     string cur_wdg;
     XMLNode req("modify");
-    for( int i_off = 0; (cur_wdg=TSYS::strSepParse(wdgs,0,';',&i_off)).size(); )
+    for(int i_off = 0; (cur_wdg=TSYS::strSepParse(wdgs,0,';',&i_off)).size(); )
     {
 	req.setAttr("path",cur_wdg+"/%2fobj");
-	if( !cntrIfCmd(req) && atoi(req.text().c_str()) )
+	if(!cntrIfCmd(req) && atoi(req.text().c_str()))
 	{
 	    actDBLoad->setEnabled(true);
 	    actDBSave->setEnabled(true);
@@ -864,14 +889,14 @@ void VisDevelop::modifyToolUpdate( const string &wdgs )
     //> Request global VCA modify
     mStModify->setText(" ");
     req.setAttr("path","/%2fobj");
-    if( !cntrIfCmd(req) && atoi(req.text().c_str()) )
+    if(!cntrIfCmd(req) && atoi(req.text().c_str()))
 	mStModify->setText("*");
 }
 
 void VisDevelop::updateMenuWindow( )
 {
     mn_window->clear();
-    //- Add actions to menu -
+    //> Add actions to menu
     mn_window->addAction(actWinClose);
     mn_window->addAction(actWinCloseAll);
     mn_window->addSeparator();
@@ -882,7 +907,7 @@ void VisDevelop::updateMenuWindow( )
     mn_window->addAction(actWinPrevious);
 
     QList<QWidget *> windows = work_space->windowList();
-    //- Enable action state -
+    //> Enable action state
     QWidget *act_win = work_space->activeWindow();
     actWinClose->setEnabled(act_win);
     actWinCloseAll->setEnabled(!windows.isEmpty());
@@ -891,8 +916,8 @@ void VisDevelop::updateMenuWindow( )
     actWinNext->setEnabled(act_win&&windows.size()>1);
     actWinPrevious->setEnabled(act_win&&windows.size()>1);
 
-    //- Add windows list -
-    if( !windows.isEmpty() )	mn_window->addSeparator();
+    //> Add windows list
+    if(!windows.isEmpty()) mn_window->addSeparator();
     for(int i_w = 0; i_w < windows.size(); ++i_w)
     {
 	QWidget *child = windows.at(i_w);
@@ -907,11 +932,11 @@ void VisDevelop::updateMenuWindow( )
 void VisDevelop::itDBLoad( )
 {
     string own_wdg = work_wdg;
-    if( own_wdg.empty() ) return;
+    if(own_wdg.empty()) return;
 
     //> Request to confirm
     InputDlg dlg(this,actDBLoad->icon(),
-	    QString(_("Are you sure of loading visual items '%1' from DB?")).arg(own_wdg.c_str()),
+	    QString(_("Are you sure of loading visual items '%1' from DB?")).arg(QString(own_wdg.c_str()).replace(";","; ")),
 	    _("Load visual item's data from DB"),false,false);
     if( dlg.exec() == QDialog::Accepted )
     {
@@ -932,17 +957,17 @@ void VisDevelop::itDBSave( )
 {
     string own_wdg = work_wdg;
 
-    if( sender() == mStModify )
+    if(sender() == mStModify)
     {
-	if( mStModify->text() != "*" ) return;
+	if(mStModify->text() != "*") return;
 	own_wdg = "/";
     }
     if( !own_wdg.empty() )
     {
 	//> Request to confirm
 	InputDlg dlg(this,actDBSave->icon(),
-		(own_wdg == "/" ? QString(_("Are you sure of saving all modifings to DB?")) :
-		QString(_("Are you sure of saving visual items '%1' to DB?")).arg(own_wdg.c_str())),
+		(own_wdg == "/" ? QString(_("Are you sure of saving all modifications to DB?")) :
+		QString(_("Are you sure of saving visual items '%1' to DB?")).arg(QString(own_wdg.c_str()).replace(";","; "))),
 		_("Save visual item's data to DB"),false,false);
 	if( dlg.exec() == QDialog::Accepted )
 	{
@@ -964,7 +989,7 @@ void VisDevelop::prjRun( )
 {
     string own_wdg = TSYS::strSepParse(work_wdg,0,';');
 
-    VisRun *sess = new VisRun( own_wdg, user(), password(), VCAStation() );
+    VisRun *sess = new VisRun(own_wdg, user(), password(), VCAStation());
     sess->show();
     sess->raise();
     sess->activateWindow();
@@ -985,7 +1010,7 @@ void VisDevelop::prjNew( )
 	    mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	else
 	{
-	    //-- Set enable for item container --
+	    //>> Set enable for item container
 	    req.clear()->setName("set")->
 			 setAttr("path",string("/prj_")+dlg.id().toAscii().data()+"/%2fobj%2fst%2fen")->
 			 setText("1");
@@ -1010,7 +1035,7 @@ void VisDevelop::libNew( )
 	    mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	else
 	{
-	    //-- Set enable for item container --
+	    //>> Set enable for item container
 	    req.clear()->setName("set")->
 			 setAttr("path",string("/wlb_")+dlg.id().toAscii().data()+"/%2fobj%2fst%2fen")->
 			 setText("1");
@@ -1020,38 +1045,39 @@ void VisDevelop::libNew( )
     }
 }
 
-void VisDevelop::visualItAdd( QAction *cact, const QPointF &pnt, const string &iWid, const string &iWnm )
+void VisDevelop::visualItAdd( QAction *cact, const QPointF &pnt, const string &iWid, const string &iWnm, const string &iOwn, bool chNoWr )
 {
     XMLNode req("get");
     //QAction *cact = (QAction *)sender();
-    string own_wdg = TSYS::strSepParse(work_wdg,0,';');
+    string own_wdg = iOwn.empty() ? TSYS::strSepParse(work_wdg,0,';') : iOwn;
     string par_nm = cact->objectName().toAscii().data();
 
-    if( work_space->activeWindow() && !wdgTree->hasFocus() && !prjTree->hasFocus() && pnt.isNull() &&
-	    !((DevelWdgView*)((QScrollArea*)work_space->activeWindow())->widget())->edit()  )
+    if(work_space->activeWindow() && !wdgTree->hasFocus() && !prjTree->hasFocus() && pnt.isNull() &&
+	    !((DevelWdgView*)((QScrollArea*)work_space->activeWindow())->widget())->edit())
 	return;
 
     //> Count level
     int p_el_cnt = 0;
-    for( int i_off = 0; TSYS::pathLev(own_wdg,0,true,&i_off).size(); p_el_cnt++ ) ;
+    for(int i_off = 0; TSYS::pathLev(own_wdg,0,true,&i_off).size(); p_el_cnt++) ;
     string sid1 = TSYS::pathLev(own_wdg,0);
 
     //> Make request id and name dialog
     InputDlg dlg(this,cact->icon(),
 	    _("Enter new widget's/page's identifier and name."),_("Create widget/page"),true,true);
     dlg.setIdLen(30);
-    if( p_el_cnt > 1 && iWid.empty() )
+
+    if(p_el_cnt > 1 && iWid.empty())
     {
 	//>> New include item id generator
 	//>>> Present include widgets list request
-	if( sid1.substr(0,4) == "prj_" && pnt.isNull() ) req.setAttr("path",own_wdg+"/%2fpage%2fpage");
+	if(sid1.substr(0,4) == "prj_" && pnt.isNull()) req.setAttr("path",own_wdg+"/%2fpage%2fpage");
 	else req.setAttr("path",own_wdg+"/%2finclwdg%2fwdg");
-	if( cntrIfCmd(req) ) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
+	if(cntrIfCmd(req)) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	else
 	{
 	    //>>> Get parent widget id
 	    string base_nm = "item";
-	    if( !par_nm.empty() )	base_nm = TSYS::pathLev(par_nm,1,true).substr(4);
+	    if(!par_nm.empty())	base_nm = TSYS::pathLev(par_nm,1,true).substr(4);
 	    unsigned i_c = 1, i_w = 0;
 	    while(i_w < req.childSize())
 		if(req.childGet(i_w)->attr("id") == base_nm+TSYS::int2str(i_c))	{ i_w = 0; i_c++; }
@@ -1061,7 +1087,7 @@ void VisDevelop::visualItAdd( QAction *cact, const QPointF &pnt, const string &i
     }
 
     //>> Execute dialog
-    if( !iWid.empty() || dlg.exec() == QDialog::Accepted )
+    if(!iWid.empty() || dlg.exec() == QDialog::Accepted)
     {
 	string w_id = iWid.empty() ? dlg.id().toAscii().data() : iWid;
 	string w_nm = iWid.empty() ? dlg.name().toAscii().data() : iWnm;
@@ -1069,34 +1095,35 @@ void VisDevelop::visualItAdd( QAction *cact, const QPointF &pnt, const string &i
 	//>> Check for widget's library
 	req.clear()->setName("add");
 	string new_wdg;
-	if( sid1.substr(0,4) == "wlb_" )
+	if(sid1.substr(0,4) == "wlb_")
 	{
-	    if( p_el_cnt == 1 )
+	    if(p_el_cnt == 1)
 		req.setAttr("path",own_wdg+"/%2fwdg%2fwdg")->setAttr("id",w_id)->setText(w_nm);
 	    else req.setAttr("path",own_wdg+"/%2finclwdg%2fwdg")->setAttr("id",w_id)->setText(w_nm);
-	    new_wdg=own_wdg+"/wdg_"+w_id;
+	    new_wdg = own_wdg+"/wdg_"+w_id;
 	}
-	else if( sid1.substr(0,4) == "prj_" )
+	else if(sid1.substr(0,4) == "prj_")
 	{
-	    if( pnt.isNull() )
+	    if(pnt.isNull())
 	    {
 		req.setAttr("path",own_wdg+"/%2fpage%2fpage")->setAttr("id",w_id)->setText(w_nm);
-		new_wdg=own_wdg+"/pg_"+w_id;
+		new_wdg = own_wdg+"/pg_"+w_id;
 	    }
 	    else
 	    {
 		req.setAttr("path",own_wdg+"/%2finclwdg%2fwdg")->setAttr("id",w_id)->setText(w_nm);
-		new_wdg=own_wdg+"/wdg_"+w_id;
+		new_wdg = own_wdg+"/wdg_"+w_id;
 	    }
 	}
+
 	//>> Create widget
 	int err = cntrIfCmd(req);
-	if( err ) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
+	if(err) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	else
 	{
 	    //>>> Set some parameters
 	    req.clear()->setName("set");
-	    if( !par_nm.empty() )
+	    if(!par_nm.empty())
 	    {
 		//>>>> Set parent widget name and enable item
 		req.setAttr("path",new_wdg+"/%2fwdg%2fst%2fparent")->setText(par_nm);
@@ -1105,37 +1132,47 @@ void VisDevelop::visualItAdd( QAction *cact, const QPointF &pnt, const string &i
 		err = cntrIfCmd(req);
 	    }
 	    //>>>> Set geometry for include widget
-	    if( !err && !pnt.isNull() )
+	    if(!err && !pnt.isNull())
 	    {
 		req.setAttr("path",new_wdg+"/%2fattr%2fgeomX")->setText(TSYS::real2str(pnt.x()));
 		err = cntrIfCmd(req);
 		req.setAttr("path",new_wdg+"/%2fattr%2fgeomY")->setText(TSYS::real2str(pnt.y()));
 		err = cntrIfCmd(req);
+
+		//> Send change request to opened for edit widget
+		if(!chNoWr)
+		{
+            	    DevelWdgView *dw = work_space->findChild<DevelWdgView*>(own_wdg.c_str());
+            	    if(dw) dw->chRecord(*XMLNode("chldAdd").setAttr("path",new_wdg)->setAttr("id",w_id)->setAttr("name",w_nm)->setAttr("parent",par_nm)->
+            			setAttr("x",TSYS::real2str(pnt.x()))->setAttr("y",TSYS::real2str(pnt.y())));
+            	}
 	    }
-	    if( err ) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
+	    if(err) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	    emit modifiedItem(new_wdg);
 	}
     }
     cact->setChecked(false);
 }
 
-void VisDevelop::visualItDel( const string &itms )
+void VisDevelop::visualItDel( const string &itms, bool chNoWr )
 {
     string work_wdg_loc = itms.empty() ? work_wdg : itms;
     string del_wdg, lst_wdg;
+    DevelWdgView *dw = NULL;
+    XMLNode chCtx("chldDel");
 
-    //- Request to confirm -
-    if( itms.empty() )
+    //> Request to confirm
+    if(itms.empty())
     {
 	InputDlg dlg(this,actVisItDel->icon(),
-		QString(_("Are you sure of deleting visual items: '%1'?")).arg(work_wdg_loc.c_str()),
+		QString(_("Are you sure of deleting visual items: '%1'?")).arg(QString(work_wdg_loc.c_str()).replace(";","; ")),
 		_("Delete visual items"),false,false);
-	if( dlg.exec() != QDialog::Accepted )	return;
+	if(dlg.exec() != QDialog::Accepted) return;
     }
 
-    for( int w_off = 0; (del_wdg=TSYS::strSepParse(work_wdg_loc,0,';',&w_off)).size(); )
+    for(int w_off = 0; (del_wdg=TSYS::strSepParse(work_wdg_loc,0,';',&w_off)).size(); )
     {
-	//- Get owner object path and deleted item identifier -
+	//> Get owner object path and deleted item identifier
 	string it_own, it_id;
 	int p_el_cnt = 0;
 	string it_tmp = TSYS::pathLev(del_wdg,p_el_cnt++);
@@ -1144,44 +1181,52 @@ void VisDevelop::visualItDel( const string &itms )
 	    it_own= it_own+(it_id.empty() ? "" : ("/"+it_id));
 	    it_id = it_tmp;
 	}
-	while( (it_tmp=TSYS::pathLev(del_wdg,p_el_cnt++)).size() );
+	while((it_tmp=TSYS::pathLev(del_wdg,p_el_cnt++)).size());
 	p_el_cnt--;
 
 	XMLNode req("del");
 	string sid1 = TSYS::pathLev(it_own,0);
 	//Check for widget's library
-	if( sid1.empty() )
+	if(sid1.empty())
 	{
-	    if( it_id.substr(0,4) == "wlb_" )
+	    if(it_id.substr(0,4) == "wlb_")
 		req.setAttr("path","/%2fprm%2fcfg%2fwlb")->setAttr("id",it_id.substr(4));
-	    else if( it_id.substr(0,4) == "prj_" )
+	    else if(it_id.substr(0,4) == "prj_")
 		req.setAttr("path","/%2fprm%2fcfg%2fprj")->setAttr("id",it_id.substr(4));
 	}
-	else if( sid1.substr(0,4) == "wlb_" )
+	else if(sid1.substr(0,4) == "wlb_")
 	{
-	    if( p_el_cnt <= 2 )
+	    if(p_el_cnt <= 2)
 		req.setAttr("path",it_own+"/%2fwdg%2fwdg")->setAttr("id",it_id.substr(4));
 	    else req.setAttr("path",it_own+"/%2finclwdg%2fwdg")->setAttr("id",it_id.substr(4));
 	}
-	else if( sid1.substr(0,4) == "prj_" )
+	else if(sid1.substr(0,4) == "prj_")
 	{
-	    if( p_el_cnt <= 2 )
+	    if(p_el_cnt <= 2)
 		req.setAttr("path",it_own+"/%2fpage%2fpage")->setAttr("id",it_id.substr(3));
-	    else if( it_id.substr(0,3) == "pg_" )
+	    else if(it_id.substr(0,3) == "pg_")
 		req.setAttr("path",it_own+"/%2fpage%2fpage")->setAttr("id",it_id.substr(3));
 	    else req.setAttr("path",it_own+"/%2finclwdg%2fwdg")->setAttr("id",it_id.substr(4));
 	}
-	if( cntrIfCmd(req) )
+
+	//> Save removed widget context for change record
+	if(!chNoWr && (dw=work_space->findChild<DevelWdgView*>(del_wdg.c_str())))
+	    dw->chLoadCtx(chCtx, "parent;");
+
+	//> Send remove context
+	if(cntrIfCmd(req))
 	    mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
-	else if( p_el_cnt < 3 )	emit modifiedItem(del_wdg);
+	else if(p_el_cnt < 3)	emit modifiedItem(del_wdg);
 	else
 	{
-	    if( !lst_wdg.empty() && lst_wdg != it_own )
-		emit modifiedItem(lst_wdg);
+	    if(!lst_wdg.empty() && lst_wdg != it_own)	emit modifiedItem(lst_wdg);
 	    lst_wdg = it_own;
+
+	    //> Send change request to opened for edit widget
+	    if(dw) dw->chRecord(chCtx);
 	}
     }
-    if( !lst_wdg.empty() )	emit modifiedItem(lst_wdg);
+    if(!lst_wdg.empty()) emit modifiedItem(lst_wdg);
 }
 
 void VisDevelop::visualItProp( )
@@ -1236,13 +1281,13 @@ void VisDevelop::visualItEdit( )
 #if QT_VERSION >= 0x040400
 	scrl->setAlignment(Qt::AlignCenter);
 #endif
-	//QPalette plt = scrl->palette();
-        //plt.setBrush(QPalette::Window,QBrush("grey",Qt::CrossPattern));
-	//scrl->setPalette(plt);
-	scrl->setBackgroundRole(QPalette::Dark);
+	QPalette plt = scrl->palette();
+        plt.setBrush(QPalette::Window,QBrush("grey",Qt::Dense7Pattern));
+	scrl->setPalette(plt);
+	//scrl->setBackgroundRole(QPalette::Dark);
 	scrl->setAttribute(Qt::WA_DeleteOnClose);
 	scrl->setWindowTitle(w_title);
-	//- Set window icon -
+	//> Set window icon
 	XMLNode req("get");
 	req.setAttr("path",ed_wdg+"/%2fico");
 	if( !cntrIfCmd(req) )
@@ -1252,14 +1297,16 @@ void VisDevelop::visualItEdit( )
 	    if( ico_t.loadFromData((const uchar*)simg.c_str(),simg.size()) )
 		scrl->setWindowIcon(QPixmap::fromImage(ico_t));
 	}
-	//- Make and place view widget -
+	//> Make and place view widget
 	DevelWdgView *vw = new DevelWdgView(ed_wdg,0,this);
 	vw->load("");
 	connect(vw, SIGNAL(selected(const string&)), this, SLOT(selectItem(const string&)));
 	connect(vw, SIGNAL(apply(const string&)), this, SIGNAL(modifiedItem(const string&)));
 	connect(this, SIGNAL(modifiedItem(const string&)), vw, SLOT(load(const string &)));
 
-	scrl->setWidget( vw );
+	scrl->installEventFilter(vw);
+
+	scrl->setWidget(vw);
 	scrl->resize(vmax(300,vmin(950,vw->size().width()+10)),vmax(200,vmin(650,vw->size().height()+10)));
 	work_space->addWindow(scrl);
 	scrl->show();
@@ -1276,8 +1323,8 @@ void VisDevelop::visualItClear( const string &el_wa )
 	work_wdg_loc = work_wdg;
 
 	InputDlg dlg(this,actVisItClear->icon(),
-		QString(_("Are you sure of clear all changes for visual items: '%1'? "
-			  "All changes will be lost and variables are returning to default values or will be inherited!")).arg(work_wdg_loc.c_str()),
+		QString(_("Are you sure of clear all changes for visual items: '%1'?\n"
+			  "All changes will be lost and variables are returning to default values or will be inherited!")).arg(QString(work_wdg_loc.c_str()).replace(";","; ")),
 		_("Visual items' changes clear"),false,false);
 	if( dlg.exec() != QDialog::Accepted )	return;
     }
@@ -1305,190 +1352,211 @@ void VisDevelop::visualItClear( const string &el_wa )
 
 void VisDevelop::visualItCut( )
 {
-    if( !actVisItCut->property("wdgAddr").toString().isEmpty() )   return;
+    if(!actVisItCut->property("wdgAddr").toString().isEmpty())	return;
     copy_buf = "1"+work_wdg;
     editToolUpdate();
 }
 
 void VisDevelop::visualItCopy( )
 {
-    if( !actVisItCopy->property("wdgAddr").toString().isEmpty() )   return;
+    if(!actVisItCopy->property("wdgAddr").toString().isEmpty())	return;
     copy_buf = "0"+work_wdg;
     editToolUpdate();
 }
 
-void VisDevelop::visualItPaste( )
+void VisDevelop::visualItPaste( const string &wsrc, const string &wdst, const string &wnm, bool chNoWr )
 {
-    if( !actVisItPaste->property("wdgAddr").toString().isEmpty() )   return;
+    if(!actVisItPaste->property("wdgAddr").toString().isEmpty())return;
     string copy_buf_el, del_els, last_del;
-    string work_wdg_work = work_wdg;
+    string copy_buf_w = wsrc.empty() ? copy_buf : wsrc;
+    string work_wdg_w = wdst.empty() ? work_wdg : wdst;
     vector<string> copy_els;
     QCheckBox *wInher = NULL;
 
     InputDlg dlg(this,actVisItPaste->icon(),"",_("Visual items move or copy"),true,true);
     dlg.setIdLen(30);
-    for( int w_off = 0; (copy_buf_el=TSYS::strSepParse(copy_buf.substr(1),0,';',&w_off)).size(); )
+    for(int w_off = 0; (copy_buf_el=TSYS::strSepParse(copy_buf_w.substr(1),0,';',&w_off)).size(); )
     {
 	string s_elp, d_elp, s_el, d_el, t_el, t1_el;
 	//> Destination elements calc
 	int n_del = 0;
-	for( int off = 0; !(t_el=TSYS::pathLev(work_wdg_work,0,true,&off)).empty(); n_del++ )
-	{ if( n_del ) d_elp += ("/"+d_el); d_el = t_el; }
+	for(int off = 0; !(t_el=TSYS::pathLev(work_wdg_w,0,true,&off)).empty(); n_del++)
+	{ if(n_del) d_elp += ("/"+d_el); d_el = t_el; }
 	//> Src elements calc
 	int n_sel = 0;
-	for( int off = 0; !(t_el=TSYS::pathLev(copy_buf_el,0,true,&off)).empty(); n_sel++ )
-	{ if( n_sel ) s_elp += ("/"+s_el); s_el = t_el; }
+	for(int off = 0; !(t_el=TSYS::pathLev(copy_buf_el,0,true,&off)).empty(); n_sel++)
+	{ if(n_sel) s_elp += ("/"+s_el); s_el = t_el; }
 
 	//> Copy visual item
 	XMLNode req("get");
 	//>> Project copy
-	if( s_el.substr(0,4)=="prj_" )
+	if(s_el.substr(0,4) == "prj_")
 	{
-	    t_el = (QString((copy_buf[0] == '1') ? _("Project '%1' move.\n") : _("Project '%1' copy.\n"))+
+	    t_el = (QString((copy_buf_w[0] == '1') ? _("Project '%1' move.\n") : _("Project '%1' copy.\n"))+
 		_("Enter new project's identifier and name.")).arg(s_el.substr(4).c_str()).toAscii().data();
 	    req.setAttr("path","/%2fprm%2fcfg%2fprj");
 	    d_el = "prj_";
 	    t1_el = s_el.substr(4);
 	}
 	//>> Widget's library copy
-	else if( s_el.substr(0,4)=="wlb_" )
+	else if(s_el.substr(0,4) == "wlb_")
 	{
-	    t_el = (QString((copy_buf[0] == '1') ? _("Widget's library '%1' move.\n") : _("Widget's library '%1' copy.\n"))+
+	    t_el = (QString((copy_buf_w[0] == '1') ? _("Widget's library '%1' move.\n") : _("Widget's library '%1' copy.\n"))+
 		_("Enter new widget's library identifier and name.")).arg(s_el.substr(4).c_str()).toAscii().data();
 	    req.setAttr("path","/%2fprm%2fcfg%2fwlb");
 	    d_el = "wlb_";
 	    t1_el = s_el.substr(4);
 	}
 	//>> Page copy
-	else if( s_el.substr(0,3)=="pg_" && (d_el.substr(0,4)=="prj_" || d_el.substr(0,3)=="pg_" || d_el.substr(0,4)=="wlb_") )
+	else if(s_el.substr(0,3) == "pg_" && (d_el.substr(0,4) == "prj_" || d_el.substr(0,3) == "pg_" || d_el.substr(0,4) == "wlb_"))
 	{
-	    t_el = (QString((copy_buf[0] == '1') ? _("Move page '%1' to '%2'.\n") : _("Copy page '%1' to '%2'.\n"))+
-		_("Enter new widget's/page's identifier and name.")).arg(copy_buf_el.c_str()).arg(work_wdg_work.c_str()).toAscii().data();
-	    if( d_el.substr(0,4)=="wlb_" ) 	req.setAttr("path",work_wdg_work+"/%2fwdg%2fwdg");
-	    else req.setAttr("path",work_wdg_work+"/%2fpage%2fpage");
+	    t_el = (QString((copy_buf_w[0] == '1') ? _("Move page '%1' to '%2'.\n") : _("Copy page '%1' to '%2'.\n"))+
+		_("Enter new widget's/page's identifier and name.")).arg(copy_buf_el.c_str()).arg(work_wdg_w.c_str()).toAscii().data();
+	    if(d_el.substr(0,4)=="wlb_") req.setAttr("path",work_wdg_w+"/%2fwdg%2fwdg");
+	    else req.setAttr("path",work_wdg_w+"/%2fpage%2fpage");
 	    d_elp += ("/"+d_el);
 	    d_el = (d_el.substr(0,4)=="wlb_") ? "wdg_" : "pg_";
 	    t1_el = s_el.substr(3);
 	}
 	//>> Widget copy
-	else if( s_el.substr(0,4)=="wdg_" && (d_el.substr(0,3)=="pg_" || d_el.substr(0,4)=="wlb_" || (TSYS::pathLev(d_elp,0).substr(0,4)=="wlb_" && n_del==2)) )
+	else if(s_el.substr(0,4) == "wdg_" && (d_el.substr(0,3) == "pg_" || d_el.substr(0,4) == "wlb_" ||
+	    (TSYS::pathLev(d_elp,0).substr(0,4) == "wlb_" && n_del == 2)))
 	{
-	    t_el = (QString((copy_buf[0] == '1') ? _("Move widget '%1' to '%2'.\n") : _("Copy widget '%1' to '%2'.\n"))+
-		_("Enter new widget's identifier and name.")).arg(copy_buf_el.c_str()).arg(work_wdg_work.c_str()).toAscii().data();
-	    if( d_el.substr(0,4)=="wlb_" )	req.setAttr("path",work_wdg_work+"/%2fwdg%2fwdg");
-	    else req.setAttr("path",work_wdg_work+"/%2finclwdg%2fwdg");
+	    t_el = (QString((copy_buf_w[0] == '1') ? _("Move widget '%1' to '%2'.\n") : _("Copy widget '%1' to '%2'.\n"))+
+		_("Enter new widget's identifier and name.")).arg(copy_buf_el.c_str()).arg(work_wdg_w.c_str()).toAscii().data();
+	    if(d_el.substr(0,4)=="wlb_") req.setAttr("path",work_wdg_w+"/%2fwdg%2fwdg");
+	    else req.setAttr("path",work_wdg_w+"/%2finclwdg%2fwdg");
 	    d_elp += ("/"+d_el);
 	    d_el = "wdg_";
 	    t1_el = s_el.substr(4);
 	}
+	//>> Direct widget to widget copy
+	else if(s_el.substr(0,4) == "wdg_" && d_el.substr(0,4) == "wdg_")
+	{
+	    t_el = (QString((copy_buf_w[0] == '1') ? _("Move widget '%1' to '%2'.\n") : _("Copy widget '%1' to '%2'.\n"))+
+		_("Enter new widget's identifier and name.")).arg(copy_buf_el.c_str()).arg(work_wdg_w.c_str()).toAscii().data();
+	    req.setAttr("path",work_wdg_w+"/%2finclwdg%2fwdg");
+	    t1_el = d_el.substr(4);
+	    d_el = "wdg_";
+	}
 	//>> Copy scheme error
 	else
 	{
 	    mod->postMess(mod->nodePath().c_str(),QString(_("Copy/move scheme from '%1' to '%2' is not supported.")).
-		arg(copy_buf_el.c_str()).arg(work_wdg_work.c_str()),TVision::Error,this);
+		arg(copy_buf_el.c_str()).arg(work_wdg_w.c_str()),TVision::Error,this);
 	    return;
 	}
 	//>> Prepare new widget identifier
 	//>>> Remove digits from end of new identifier
-	unsigned i_w = 0;
-	if( cntrIfCmd(req) ) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
-	else
+	if(wdst.empty())
 	{
-	    for(i_w = 0; i_w < req.childSize(); i_w++)
-		if(req.childGet(i_w)->attr("id") == t1_el)
-		    break;
-	    if(i_w < req.childSize())
+	    if(cntrIfCmd(req)) mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
+	    else
 	    {
-		int no_numb = t1_el.size()-1;
-		while(no_numb >= 0 && t1_el[no_numb] >= '0' && t1_el[no_numb] <= '9') no_numb--;
-		if(no_numb >= 0) t1_el = t1_el.substr(0,no_numb+1);
-		//>>> New identifier generator
-		unsigned i_c = 1, i_w = 0;
-		while(i_w < req.childSize())
-		    if(req.childGet(i_w)->attr("id") == t1_el+TSYS::int2str(i_c)) { i_w = 0; i_c++; }
-		    else i_w++;
-		t1_el += TSYS::int2str(i_c);
+		unsigned i_w = 0;
+		for(i_w = 0; i_w < req.childSize(); i_w++)
+		    if(req.childGet(i_w)->attr("id") == t1_el)
+			break;
+		if(i_w < req.childSize())
+		{
+		    int no_numb = t1_el.size()-1;
+		    while(no_numb >= 0 && t1_el[no_numb] >= '0' && t1_el[no_numb] <= '9') no_numb--;
+		    if(no_numb >= 0) t1_el = t1_el.substr(0,no_numb+1);
+		    //>>> New identifier generator
+		    unsigned i_c = 1, i_w = 0;
+		    while(i_w < req.childSize())
+			if(req.childGet(i_w)->attr("id") == t1_el+TSYS::int2str(i_c)) { i_w = 0; i_c++; }
+			else i_w++;
+		    t1_el += TSYS::int2str(i_c);
+		}
 	    }
 	}
 	//>> Make request dialog
 	dlg.setMess(t_el.c_str());
 	dlg.setId(t1_el.c_str());
 	//>> Add Link flag for copy operation
-	if( copy_buf[0] != '1' && d_el.substr(0,4) != "prj_" && d_el.substr(0,4) != "wlb_" )
+	if(copy_buf_w[0] != '1' && d_el.substr(0,4) != "prj_" && d_el.substr(0,4) != "wlb_")
 	{
-	    dlg.edLay()->addWidget( new QLabel(_("Inherit:"),&dlg), 2, 0 );
+	    dlg.edLay()->addWidget(new QLabel(_("Inherit:"),&dlg), 2, 0);
 	    wInher = new QCheckBox(&dlg);
-	    dlg.edLay()->addWidget( wInher, 2, 1 );
+	    dlg.edLay()->addWidget(wInher, 2, 1);
 	}
-	if( /*i_w >= req.childSize() ||*/ dlg.exec() == QDialog::Accepted )
+	if(!wsrc.empty() || dlg.exec() == QDialog::Accepted)
 	{
 	    d_el += dlg.id().toAscii().data();
-	    string it_nm = dlg.name().toAscii().data();
+	    string it_nm = wnm.empty() ? dlg.name().toAscii().data() : wnm;
 
 	    //>>> Make link
-	    if( wInher && wInher->isChecked() )
+	    if(wInher && wInher->isChecked())
 	    {
 		QAction addAct(NULL);
 		addAct.setObjectName((s_elp+"/"+s_el).c_str());
-		work_wdg = d_elp;
-		visualItAdd( &addAct, QPointF(), dlg.id().toAscii().data(), it_nm );
+		visualItAdd(&addAct, QPointF(), dlg.id().toAscii().data(), it_nm, d_elp);
 	    }
 	    //>>> Make copy
 	    else
 	    {
 		req.clear()->setName("set")->
-		    setAttr( "path", "/%2fprm%2fcfg%2fcp%2fcp" )->
-		    setAttr( "src", s_elp+"/"+s_el )->
-		    setAttr( "dst", d_elp+"/"+d_el );
-		if( cntrIfCmd(req) )
+		    setAttr("path", "/%2fprm%2fcfg%2fcp%2fcp")->
+		    setAttr("src", s_elp+"/"+s_el)->
+		    setAttr("dst", d_elp+"/"+d_el);
+		if(cntrIfCmd(req))
 		    mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 		else
 		{
-		    if( it_nm.size() )
+		    if(it_nm.size())
 		    {
 			req.clear()->setName("set")->setText(it_nm);
-			if( d_el.substr(0,4) == "prj_" || d_el.substr(0,4) == "wlb_" )
+			if(d_el.substr(0,4) == "prj_" || d_el.substr(0,4) == "wlb_")
 			    req.setAttr("path",d_elp+"/"+d_el+"/%2fobj%2fcfg%2fname");
 			else req.setAttr("path",d_elp+"/"+d_el+"/%2fwdg%2fcfg%2fname");
 			cntrIfCmd(req);
 		    }
-		    if( n_del < 2 )	copy_els.push_back(d_elp+"/"+d_el);
+		    if(n_del < 2) copy_els.push_back(d_elp+"/"+d_el);
 		    else
 		    {
-			if( !last_del.empty() && last_del != d_elp )
+			if(!last_del.empty() && last_del != d_elp)
 			    copy_els.push_back(last_del);
 			last_del = d_elp;
 		    }
 		    del_els += copy_buf_el+";";
+
+		    //> Send change request to opened for edit widget
+            	    if(!chNoWr)
+            	    {
+                	DevelWdgView *dw = work_space->findChild<DevelWdgView*>(d_elp.c_str());
+                	if(dw) dw->chRecord(*XMLNode("chldPaste").
+                		setAttr("src",s_elp+"/"+s_el)->setAttr("dst",d_elp+"/"+d_el)->setAttr("name",it_nm));
+            	    }
 		}
 	    }
 	}
     }
-    if( !last_del.empty() )	copy_els.push_back(last_del);
+    if(!last_del.empty())	copy_els.push_back(last_del);
 
-    //- Send created widgets events -
+    //> Send events to created widgets
     for(unsigned i_e = 0; i_e < copy_els.size(); i_e++)
 	emit modifiedItem(copy_els[i_e]);
-    //- Remove source widget -
-    if(copy_buf[0] == '1') 	{ visualItDel(del_els); copy_buf = "0"; }
+    //> Remove source widget
+    if(copy_buf_w[0] == '1') 	{ visualItDel(del_els); if(wsrc.empty()) copy_buf = "0"; }
 }
 
 void VisDevelop::editToolUpdate( )
 {
-    if( !actVisItCopy->property("wdgAddr").toString().isEmpty() )   return;
+    if(!actVisItCopy->property("wdgAddr").toString().isEmpty()) return;
     actVisItCut->setEnabled(!work_wdg.empty());
     actVisItCopy->setEnabled(!work_wdg.empty());
-    //- Src and destination elements calc -
+    //> Src and destination elements calc
     string s_elp, d_elp, s_el, d_el, t_el;
     int n_sel = 0;
     int n_del = 0;
-    for( int off = 0; !(t_el=TSYS::pathLev(copy_buf.substr(1),0,true,&off)).empty(); n_sel++ )
+    for(int off = 0; !(t_el=TSYS::pathLev(copy_buf.substr(1),0,true,&off)).empty(); n_sel++)
     { s_elp += ("/"+s_el); s_el = t_el; }
-    for( int off = 0; !(t_el=TSYS::pathLev(work_wdg,0,true,&off)).empty(); n_del++ )
+    for(int off = 0; !(t_el=TSYS::pathLev(work_wdg,0,true,&off)).empty(); n_del++)
     { d_elp += ("/"+d_el); d_el = t_el; }
-    if( (s_el.substr(0,4)=="prj_" || s_el.substr(0,4)=="wlb_") ||										//Project and library copy
+    if((s_el.substr(0,4)=="prj_" || s_el.substr(0,4)=="wlb_") ||										//Project and library copy
 	    (s_el.substr(0,3)=="pg_" && (d_el.substr(0,4)=="prj_" || d_el.substr(0,3)=="pg_" || d_el.substr(0,4)=="wlb_")) ||			//Page copy
-	    (s_el.substr(0,4)=="wdg_" && (d_el.substr(0,3)=="pg_" || d_el.substr(0,4)=="wlb_" || (TSYS::pathLev(d_elp,0).substr(0,4)=="wlb_" && n_del==2))) )	//Widget copy
+	    (s_el.substr(0,4)=="wdg_" && (d_el.substr(0,3)=="pg_" || d_el.substr(0,4)=="wlb_" || (TSYS::pathLev(d_elp,0).substr(0,4)=="wlb_" && n_del==2))))	//Widget copy
 	actVisItPaste->setEnabled(true);
     else actVisItPaste->setEnabled(false);
 }
