@@ -26,6 +26,7 @@
 #include <linux/input.h>
 
 #include <QApplication>
+#include <QLocale>
 #include <QDesktopWidget>
 #include <QMenu>
 #include <QTimer>
@@ -57,9 +58,9 @@
 
 using namespace VISION;
 
-VisRun::VisRun( const string &prj_it, const string &open_user, const string &user_pass, const string &VCAstat, bool crSessForce, QWidget *parent ) :
-    QMainWindow(parent), winClose(false), master_pg(NULL), mPeriod(1000), wPrcCnt(0), reqtm(1), x_scale(1.0), y_scale(1.0), mAlrmSt(0xFFFFFF),
-    isConErr(false)
+VisRun::VisRun( const string &iprj_it, const string &open_user, const string &user_pass, const string &VCAstat, bool icrSessForce, QWidget *parent ) :
+    QMainWindow(parent), winClose(false), crSessForce(icrSessForce), keepAspectRatio(false), prj_it(iprj_it), master_pg(NULL), mPeriod(1000),
+    wPrcCnt(0), reqtm(1), expDiagCnt(1), expDocCnt(1), x_scale(1.0), y_scale(1.0), mAlrmSt(0xFFFFFF), isConErr(false)
 {
     QImage ico_t;
 
@@ -156,9 +157,9 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
     if(!ico_t.load(TUIS::icoPath("help").c_str())) ico_t.load(":/images/help.png");
     QAction *actAbout = new QAction(QPixmap::fromImage(ico_t),_("&About"),this);
     actAbout->setShortcut(Qt::Key_F1);
-    actAbout->setToolTip(_("Programm and OpenSCADA information"));
-    actAbout->setWhatsThis(_("The button for display the programm and OpenSCADA information"));
-    actAbout->setStatusTip(_("Press for display the programm and OpenSCADA information."));
+    actAbout->setToolTip(_("Program and OpenSCADA information"));
+    actAbout->setWhatsThis(_("The button for display the program and OpenSCADA information"));
+    actAbout->setStatusTip(_("Press for display the program and OpenSCADA information."));
     connect(actAbout, SIGNAL(triggered()), this, SLOT(about()));
     //>>> About Qt
     QAction *actQtAbout = new QAction(_("About &Qt"),this);
@@ -175,36 +176,36 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
     //connect(actWhatIs, SIGNAL(triggered()), this, SLOT(enterWhatsThis()));
 
     //>> Alarms actions
-    //>>> Alarm level display button and full alarms quitance
+    //>>> Alarm level display button and full alarms quittance
     if(!ico_t.load(TUIS::icoPath("alarmLev").c_str())) ico_t.load(":/images/alarmLev.png");
     actAlrmLev = new QAction( QPixmap::fromImage(ico_t), _("Alarm level"), this );
     actAlrmLev->setObjectName("alarmLev");
     actAlrmLev->setToolTip(_("Alarm level"));
-    actAlrmLev->setWhatsThis(_("The button for all alarms quitance"));
-    actAlrmLev->setStatusTip(_("Press for all alarms quitance."));
+    actAlrmLev->setWhatsThis(_("The button for all alarms quittance"));
+    actAlrmLev->setStatusTip(_("Press for all alarms quittance."));
     //>>> Alarm by Light
     if(!ico_t.load(TUIS::icoPath("alarmLight").c_str())) ico_t.load(":/images/alarmLight.png");
     actAlrmLight = new QAction( QPixmap::fromImage(ico_t), _("Blink alarm"), this );
     actAlrmLight->setObjectName("alarmLight");
     actAlrmLight->setToolTip(_("Blink alarm"));
-    actAlrmLight->setWhatsThis(_("The button for all blink alarms quitance"));
-    actAlrmLight->setStatusTip(_("Press for all blink alarms quitance."));
+    actAlrmLight->setWhatsThis(_("The button for all blink alarms quittance"));
+    actAlrmLight->setStatusTip(_("Press for all blink alarms quittance."));
     actAlrmLight->setVisible(false);
     //>>> Alarm by mono sound (PC speaker)
     if(!ico_t.load(TUIS::icoPath("alarmAlarm").c_str())) ico_t.load(":/images/alarmAlarm.png");
     actAlrmAlarm = new QAction( QPixmap::fromImage(ico_t), _("Speaker alarm"), this );
     actAlrmAlarm->setObjectName("alarmAlarm");
     actAlrmAlarm->setToolTip(_("PC speaker alarm"));
-    actAlrmAlarm->setWhatsThis(_("The button for all PC speaker alarms quitance"));
-    actAlrmAlarm->setStatusTip(_("Press for all PC speaker alarms quitance."));
+    actAlrmAlarm->setWhatsThis(_("The button for all PC speaker alarms quittance"));
+    actAlrmAlarm->setStatusTip(_("Press for all PC speaker alarms quittance."));
     actAlrmAlarm->setVisible(false);
     //>>> Alarm by sound or synthesis of speech
     if(!ico_t.load(TUIS::icoPath("alarmSound").c_str())) ico_t.load(":/images/alarmSound.png");
     actAlrmSound = new QAction( QPixmap::fromImage(ico_t), _("Sound/speech alarm"), this );
     actAlrmSound->setObjectName("alarmSound");
     actAlrmSound->setToolTip(_("Sound or speech alarm"));
-    actAlrmSound->setWhatsThis(_("The button for all sound or speech alarms quitance"));
-    actAlrmSound->setStatusTip(_("Press for all sound or speech alarms quitance."));
+    actAlrmSound->setWhatsThis(_("The button for all sound or speech alarms quittance"));
+    actAlrmSound->setStatusTip(_("Press for all sound or speech alarms quittance."));
     actAlrmSound->setVisible(false);
 
     //> Create menu
@@ -244,10 +245,10 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
     mWTime = new QLabel( this );
     mWTime->setVisible(false);
     mWTime->setAlignment(Qt::AlignCenter);
-    mWTime->setWhatsThis(_("This label displays curent system's time."));
+    mWTime->setWhatsThis(_("This label displays current system's time."));
     statusBar()->insertPermanentWidget(0,mWTime);
     mWUser = new UserStBar( open_user.c_str(), user_pass.c_str(), VCAstat.c_str(), this );
-    mWUser->setWhatsThis(_("This label displays curent user."));
+    mWUser->setWhatsThis(_("This label displays current user."));
     mWUser->setToolTip(_("Field for display of the current user."));
     mWUser->setStatusTip(_("Double click to change user."));
     connect( mWUser, SIGNAL(userChanged(const QString&,const QString&)), this, SLOT(userChanged(const QString&,const QString&)) );
@@ -267,8 +268,9 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
 
     //> Init scroller
     QScrollArea *scrl = new QScrollArea;
-    scrl->setFocusPolicy( Qt::NoFocus );
-    setCentralWidget( scrl );
+    scrl->setFocusPolicy(Qt::NoFocus);
+    scrl->setAlignment(Qt::AlignCenter);
+    setCentralWidget(scrl);
 
     //> Create timers
     //>> End run timer
@@ -285,13 +287,13 @@ VisRun::VisRun( const string &prj_it, const string &open_user, const string &use
 
     menuBar()->setVisible(SYS->security().at().access(user(),SEC_WR,"root","root",RWRWR_));
 
-    resize( 600, 400 );
+    resize(600, 400);
 
     //> Init session
     initSess(prj_it,crSessForce);
 
     //mWStat->setText(host.st_nm.c_str());
-    statusBar()->showMessage(_("Ready"), 2000 );
+    statusBar()->showMessage(_("Ready"), 2000);
 
     alarmSet(0);
 }
@@ -337,7 +339,7 @@ int VisRun::cntrIfCmd( XMLNode &node, bool glob )
     if(rez == 10 && master_pg && !isConErr && !master_pg->findChild<QLabel*>("==ConnError=="))
     {
 	//> Create error message
-	conErr = new QLabel(QString(_("Connection to visualisation server '%1' error: %2")).
+	conErr = new QLabel(QString(_("Connection to visualization server '%1' error: %2")).
 				arg(VCAStation().c_str()).arg(node.text().c_str()),master_pg);
 	conErr->setObjectName("==ConnError==");
 	conErr->setAlignment(Qt::AlignCenter);
@@ -374,33 +376,35 @@ void VisRun::closeEvent( QCloseEvent* ce )
 
 void VisRun::resizeEvent( QResizeEvent *ev )
 {
-    if( ev && ev->oldSize().isValid() && master_pg )
+    if(ev && ev->oldSize().isValid() && master_pg)
     {
 	float x_scale_old = x_scale;
 	float y_scale_old = y_scale;
-	if( windowState() == Qt::WindowMaximized || windowState() == Qt::WindowFullScreen )
+	if(windowState() == Qt::WindowMaximized || windowState() == Qt::WindowFullScreen)
 	{
 	    x_scale *= (float)((QScrollArea*)centralWidget())->maximumViewportSize().width()/(float)master_pg->size().width();
 	    y_scale *= (float)((QScrollArea*)centralWidget())->maximumViewportSize().height()/(float)master_pg->size().height();
+	    //> Proportional scale
+	    if(keepAspectRatio) x_scale = y_scale = vmin(x_scale,y_scale);
 	}else x_scale = y_scale = 1.0;
-	if( x_scale_old != x_scale || y_scale_old != y_scale )	fullUpdatePgs();
+	if(x_scale_old != x_scale || y_scale_old != y_scale)	fullUpdatePgs();
     }
     mWTime->setVisible(windowState()==Qt::WindowFullScreen);
 }
 
 void VisRun::endRunChk( )
 {
-    if( mod->endRun() ) close();
+    if(mod->endRun()) close();
 }
 
-void VisRun::quitSt()
+void VisRun::quitSt( )
 {
     SYS->stop();
 }
 
 void VisRun::print( )
 {
-    if( master_pg ) printPg( master_pg->id() );
+    if(master_pg) printPg(master_pg->id());
 }
 
 void VisRun::printPg( const string &ipg )
@@ -408,10 +412,10 @@ void VisRun::printPg( const string &ipg )
     RunPageView *rpg;
     string pg = ipg;
 
-    if( pgList.empty() )	{ QMessageBox::warning(this,_("Print page"),_("No one page for print is present!")); return; }
+    if(pgList.empty())	{ QMessageBox::warning(this,_("Print page"),_("No one page for print is present!")); return; }
 
-    if( pg.empty() && pgList.size() == 1 )	pg = pgList[0];
-    if( pg.empty() && pgList.size() > 1 )
+    if(pg.empty() && pgList.size() == 1)	pg = pgList[0];
+    if(pg.empty() && pgList.size() > 1)
     {
 	//> Make select page dialog
 	QImage ico_t;
@@ -489,7 +493,7 @@ void VisRun::printDiag( const string &idg )
 	    //> Make select diagrams dialog
 	    QImage ico_t;
 	    if(!ico_t.load(TUIS::icoPath("print").c_str())) ico_t.load(":/images/print.png");
-	    InputDlg sdlg(this, QPixmap::fromImage(ico_t), _("Select diagramm for print."), _("Diagram print."), false, false);
+	    InputDlg sdlg(this, QPixmap::fromImage(ico_t), _("Select diagram for print."), _("Diagram print."), false, false);
 	    sdlg.edLay()->addWidget( new QLabel(_("Diagrams:"),&sdlg), 2, 0 );
 	    QComboBox *spg = new QComboBox(&sdlg);
 	    sdlg.edLay()->addWidget( spg, 2, 1 );
@@ -596,7 +600,7 @@ void VisRun::printDoc( const string &idoc )
 
 void VisRun::exportDef( )
 {
-    if( master_pg ) exportPg( master_pg->id() );
+    if(master_pg) exportPg(master_pg->id());
 }
 
 void VisRun::exportPg( const string &ipg )
@@ -604,10 +608,10 @@ void VisRun::exportPg( const string &ipg )
     RunPageView *rpg;
     string pg = ipg;
 
-    if( pgList.empty() )	{ QMessageBox::warning(this,_("Export page"),_("No one page for export is present!")); return; }
+    if(pgList.empty())	{ QMessageBox::warning(this,_("Export page"),_("No one page for export is present!")); return; }
 
-    if( pg.empty() && pgList.size() == 1 )	pg = pgList[0];
-    if( pg.empty() && pgList.size() > 1 )
+    if(pg.empty() && pgList.size() == 1)	pg = pgList[0];
+    if(pg.empty() && pgList.size() > 1)
     {
 	//> Make select page dialog
 	QImage ico_t;
@@ -619,21 +623,20 @@ void VisRun::exportPg( const string &ipg )
 	for(unsigned i_p = 0; i_p < pgList.size(); i_p++)
 	    if((rpg=findOpenPage(pgList[i_p])))
 		spg->addItem((rpg->name()+" ("+pgList[i_p]+")").c_str(),pgList[i_p].c_str());
-	if( sdlg.exec() != QDialog::Accepted )	return;
+	if(sdlg.exec() != QDialog::Accepted)	return;
 	pg = spg->itemData(spg->currentIndex()).toString().toAscii().data();
     }
 
     //> Find need page
     rpg = master_pg;
-    if( rpg->id() != pg )	rpg = findOpenPage(pg);
-    if( !rpg ) return;
+    if(rpg->id() != pg)	rpg = findOpenPage(pg);
+    if(!rpg) return;
 
     QPixmap img = QPixmap::grabWidget(rpg);
 
     //> Call save file dialog -
-    QString fileName = QFileDialog::getSaveFileName(this,_("Save page's image"),
-	(TSYS::path2sepstr(rpg->name())+".png").c_str(), _("Images (*.png *.xpm *.jpg)"));
-    if( !fileName.isEmpty() && !img.save(fileName) )
+    QString fileName = QFileDialog::getSaveFileName(this,_("Save page's image"),(rpg->name()+".png").c_str(), _("Images (*.png *.xpm *.jpg)"));
+    if(!fileName.isEmpty() && !img.save(fileName))
 	mod->postMess(mod->nodePath().c_str(),QString(_("Save to file '%1' is error.")).arg(fileName),TVision::Error,this);
 }
 
@@ -651,34 +654,119 @@ void VisRun::exportDiag( const string &idg )
 	for(unsigned i_p = 0; i_p < pgList.size(); i_p++)
 	    if((rpg=findOpenPage(pgList[i_p])))
 		rpg->shapeList("Diagram",lst);
-	if( lst.empty() )	{ QMessageBox::warning(this,_("Export diagram"),_("No one diagram is present!")); return; }
-	if( lst.size() == 1 )	dg = lst[0];
+	if(lst.empty())	{ QMessageBox::warning(this,_("Export diagram"),_("No one diagram is present!")); return; }
+	if(lst.size() == 1) dg = lst[0];
 	else
 	{
 	    //> Make select diagrams dialog
 	    QImage ico_t;
 	    if(!ico_t.load(TUIS::icoPath("print").c_str())) ico_t.load(":/images/export.png");
-	    InputDlg sdlg( this, QPixmap::fromImage(ico_t), _("Select diagramm for export."), _("Diagram export."), false, false );
-	    sdlg.edLay()->addWidget( new QLabel(_("Diagrams:"),&sdlg), 2, 0 );
+	    InputDlg sdlg(this, QPixmap::fromImage(ico_t), _("Select diagram for export."), _("Diagram export."), false, false);
+	    sdlg.edLay()->addWidget(new QLabel(_("Diagrams:"),&sdlg), 2, 0);
 	    QComboBox *spg = new QComboBox(&sdlg);
-	    sdlg.edLay()->addWidget( spg, 2, 1 );
+	    sdlg.edLay()->addWidget(spg, 2, 1);
 	    for(unsigned i_l = 0; i_l < lst.size(); i_l++)
 		if((rwdg=findOpenWidget(lst[i_l])))
 		    spg->addItem((rwdg->name()+" ("+lst[i_l]+")").c_str(),lst[i_l].c_str());
-	    if( sdlg.exec() != QDialog::Accepted )	return;
+	    if(sdlg.exec() != QDialog::Accepted) return;
 	    dg = spg->itemData(spg->currentIndex()).toString().toAscii().data();
 	}
     }
 
-    if( !(rwdg=findOpenWidget(dg)) )	return;
+    if(!(rwdg=findOpenWidget(dg))) return;
 
     QPixmap img = QPixmap::grabWidget(rwdg);
 
     //> Call save file dialog
-    QString fileName = QFileDialog::getSaveFileName(this,_("Save diagram's image"),
-	(TSYS::path2sepstr(rwdg->name())+".png").c_str(), _("Images (*.png *.xpm *.jpg)"));
-    if( !fileName.isEmpty() && !img.save(fileName) )
-	mod->postMess(mod->nodePath().c_str(),QString(_("Save to file '%1' is error.")).arg(fileName),TVision::Error,this);
+    QString fileName = QFileDialog::getSaveFileName(this, _("Save diagram"), QString(_("Trend %1.png")).arg(expDiagCnt++),
+	_("Images (*.png *.xpm *.jpg);;CSV file (*.csv)"));
+    if(!fileName.isEmpty())
+    {
+	//>> Export to CSV
+	if(fileName.indexOf(QRegExp("\\.csv$")) != -1)
+	{
+	    //>>> Open destination file
+	    int fd = ::open(fileName.toAscii().data(), O_WRONLY|O_CREAT|O_TRUNC, 0644);
+	    if(fd < 0)
+	    {
+		mod->postMess(mod->nodePath().c_str(),QString(_("Save to file '%1' is error.")).arg(fileName),TVision::Error,this);
+		return;
+	    }
+
+	    ShapeDiagram::ShpDt *dgDt = (ShapeDiagram::ShpDt*)rwdg->shpData;
+	    string CSVr;
+	    //>>> Trend type process
+	    if(dgDt->type == 0)
+	    {
+		int firstPrm = -1, vPos = 0;
+		//>>> Prepare header
+		CSVr += _("\"Date and time\";\"us\"");
+		for(unsigned i_p = 0; i_p < dgDt->prms.size(); i_p++)
+    		    if(dgDt->prms[i_p].val().size() && dgDt->prms[i_p].color().isValid())
+    		    {
+			CSVr += ";\""+TSYS::path2sepstr(dgDt->prms[i_p].addr())+"\"";
+			if(firstPrm < 0) firstPrm = i_p;
+		    }
+		CSVr += "\x0D\x0A";
+		if(firstPrm < 0) return;
+		//>>> Place data
+		deque<ShapeDiagram::TrendObj::SHg> &baseVls = dgDt->prms[firstPrm].val();
+		for(unsigned i_v = 0; i_v < baseVls.size(); i_v++)
+		{
+		    CSVr += TSYS::time2str(baseVls[i_v].tm/1000000,"\"%d/%m/%Y %H:%M:%S\"")+";"+TSYS::int2str(baseVls[i_v].tm%1000000);
+		    for(unsigned i_p = 0; i_p < dgDt->prms.size(); i_p++)
+		    {
+			ShapeDiagram::TrendObj &cPrm = dgDt->prms[i_p];
+    			if(cPrm.val().size() && cPrm.color().isValid())
+    			{
+			    vPos = cPrm.val(baseVls[i_v].tm);
+			    CSVr = CSVr + ";"+((vPos < (int)cPrm.val().size())?QLocale().toString(cPrm.val()[vPos].val).toAscii().data():"");
+			}
+		    }
+		    CSVr += "\x0D\x0A";
+		}
+	    }
+	    //>>> Frequency spectrum type
+	    else if(dgDt->type == 1)
+	    {
+#if HAVE_FFTW3_H
+		//>>> Prepare header
+		CSVr += _("\"Frequency (Hz)\"");
+		for(unsigned i_p = 0; i_p < dgDt->prms.size(); i_p++)
+    		    if(dgDt->prms[i_p].fftN && dgDt->prms[i_p].color().isValid())
+			CSVr += ";\""+TSYS::path2sepstr(dgDt->prms[i_p].addr())+"\"";
+		CSVr += "\x0D\x0A";
+		//>>> Place data
+		int fftN = rwdg->size().width();		//Samples number
+		double fftBeg = 1/dgDt->tSize;			//Minimum frequency or maximum period time (s)
+		double fftEnd = (double)fftN*fftBeg/2;		//Maximum frequency or minimum period time (s)
+		for(double i_frq = fftBeg; i_frq <= fftEnd; i_frq += fftBeg)
+		{
+		    CSVr += QLocale().toString(i_frq).toStdString();
+		    for(unsigned i_p = 0; i_p < dgDt->prms.size(); i_p++)
+		    {
+			ShapeDiagram::TrendObj &cPrm = dgDt->prms[i_p];
+    			if(cPrm.fftN && cPrm.color().isValid())
+    			{
+        		    int vpos = (int)((i_frq*cPrm.fftN)/(fftBeg*fftN));
+        		    double val = EVAL_REAL;
+        		    if(vpos >= 1 && vpos < (cPrm.fftN/2+1))
+				val = cPrm.fftOut[0][0]/cPrm.fftN + pow(pow(cPrm.fftOut[vpos][0],2)+pow(cPrm.fftOut[vpos][1],2),0.5)/(cPrm.fftN/2+1);
+			    CSVr += ";"+QLocale().toString(val).toStdString();
+			}
+		    }
+		    CSVr += "\x0D\x0A";
+		}
+#endif
+	    }
+	    //>>> Save to file
+	    ::write(fd,CSVr.data(),CSVr.size());
+	    ::close(fd);
+	}
+	//>> Export to image
+	else if(!img.save(fileName))
+	    mod->postMess(mod->nodePath().c_str(),QString(_("Save to file '%1' is error.")).arg(fileName),TVision::Error,this);
+    }
 }
 
 void VisRun::exportDoc( const string &idoc )
@@ -695,48 +783,114 @@ void VisRun::exportDoc( const string &idoc )
 	for(unsigned i_p = 0; i_p < pgList.size(); i_p++)
 	    if((rpg=findOpenPage(pgList[i_p])))
 		rpg->shapeList("Document",lst);
-	if( lst.empty() )	{ QMessageBox::warning(this,_("Export document"),_("No one document is present!")); return; }
-	if( lst.size() == 1 )	doc = lst[0];
+	if(lst.empty())	{ QMessageBox::warning(this,_("Export document"),_("No one document is present!")); return; }
+	if(lst.size() == 1) doc = lst[0];
 	else
 	{
 	    //> Make select diagrams dialog
 	    QImage ico_t;
 	    if(!ico_t.load(TUIS::icoPath("print").c_str())) ico_t.load(":/images/export.png");
-	    InputDlg sdlg( this, QPixmap::fromImage(ico_t), _("Select document for export."), _("Document export."), false, false );
-	    sdlg.edLay()->addWidget( new QLabel(_("Document:"),&sdlg), 2, 0 );
+	    InputDlg sdlg(this, QPixmap::fromImage(ico_t), _("Select document for export."), _("Document export."), false, false);
+	    sdlg.edLay()->addWidget(new QLabel(_("Document:"),&sdlg), 2, 0);
 	    QComboBox *spg = new QComboBox(&sdlg);
 	    sdlg.edLay()->addWidget( spg, 2, 1 );
 	    for(unsigned i_l = 0; i_l < lst.size(); i_l++)
 		if((rwdg=findOpenWidget(lst[i_l])))
 		    spg->addItem((rwdg->name()+" ("+lst[i_l]+")").c_str(),lst[i_l].c_str());
-	    if( sdlg.exec() != QDialog::Accepted )	return;
+	    if(sdlg.exec() != QDialog::Accepted) return;
 	    doc = spg->itemData(spg->currentIndex()).toString().toAscii().data();
 	}
     }
 
-    if( !(rwdg=findOpenWidget(doc)) )	return;
+    if(!(rwdg=findOpenWidget(doc))) return;
 
     //> Call save file dialog -
-    QString fileName = QFileDialog::getSaveFileName(this,_("Save document"),(TSYS::path2sepstr(rwdg->name())+".html").c_str(), _("HTML (*.html)"));
-    if( !fileName.isEmpty() )
+    QString fileName = QFileDialog::getSaveFileName(this, _("Save document"), QString(_("Document %1.html")).arg(expDocCnt++),
+	_("XHTML (*.html);;CSV file (*.csv)"));
+    if(!fileName.isEmpty())
     {
-	int fd = ::open( fileName.toAscii().data(), O_WRONLY|O_CREAT|O_TRUNC, 0644 );
-	if( fd < 0 )
+	int fd = ::open(fileName.toAscii().data(), O_WRONLY|O_CREAT|O_TRUNC, 0644);
+	if(fd < 0)
 	{
 	    mod->postMess(mod->nodePath().c_str(),QString(_("Save to file '%1' is error.")).arg(fileName),TVision::Error,this);
 	    return;
 	}
-	string rez = "<?xml version='1.0' ?>\n"
-	    "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN'\n"
-	    "'DTD/xhtml1-transitional.dtd'>\n"
-	    "<html xmlns='http://www.w3.org/1999/xhtml'>\n"
-	    "<head>\n"
-	    "  <meta http-equiv='Content-Type' content='text/html; charset="+Mess->charset()+"'/>\n"
-	    "  <style type='text/css'>\n"+((ShapeDocument::ShpDt*)rwdg->shpData)->style+"</style>\n"
-	    "</head>\n"+
-	    ((ShapeDocument::ShpDt*)rwdg->shpData)->doc+
-	    "</html>";
-	::write(fd,rez.data(),rez.size());
+	string rez;
+	//>> Export to CSV
+	if(fileName.indexOf(QRegExp("\\.csv$")) != -1)
+	{
+	    //>>> Parse document
+	    XMLNode docTree;
+	    docTree.load(((ShapeDocument::ShpDt*)rwdg->shpData)->doc, true);
+	    XMLNode *curNode = &docTree;
+	    vector<unsigned> treeStk;
+	    treeStk.push_back(0);
+	    while(!(!curNode->parent() && treeStk.back() >= curNode->childSize()))
+	    {
+		if(treeStk.back() < curNode->childSize())
+		{
+		    curNode = curNode->childGet(treeStk.back());
+		    treeStk.push_back(0);
+		    //>>> Check for marked table and process it
+		    if(strcasecmp(curNode->name().c_str(),"table") == 0 && atoi(curNode->attr("export").c_str()))
+		    {
+			map<int,int>	rowSpn;
+			XMLNode *tblN = NULL, *tblRow;
+			string val;
+			for(int i_st = 0; i_st < 4; i_st++)
+			{
+			    switch(i_st)
+			    {
+				case 0:	tblN = curNode->childGet("thead", 0, true);	break;
+				case 1: tblN = curNode->childGet("tbody", 0, true);	break;
+				case 2: tblN = curNode->childGet("tfoot", 0, true);	break;
+				case 3: tblN = curNode;					break;
+				default: tblN = NULL;
+			    }
+			    if(!tblN)	continue;
+			    //>>> Rows process
+			    for(unsigned i_n = 0; i_n < tblN->childSize(); i_n++)
+			    {
+				if(strcasecmp(tblN->childGet(i_n)->name().c_str(),"tr") != 0)	continue;
+				tblRow = tblN->childGet(i_n);
+				for(unsigned i_c = 0, i_cl = 0; i_c < tblRow->childSize(); i_c++)
+				{
+				    if(!(strcasecmp(tblRow->childGet(i_c)->name().c_str(),"th") == 0 || strcasecmp(tblRow->childGet(i_c)->name().c_str(),"td") == 0))
+					continue;
+				    while(rowSpn[i_cl] > 1) { rez += ";"; rowSpn[i_cl]--; i_cl++; }
+				    rowSpn[i_cl] = atoi(tblRow->childGet(i_c)->attr("rowspan",false).c_str());
+				    val = tblRow->childGet(i_c)->text(true,true);
+				    for(size_t i_sz = 0; (i_sz=val.find("\"",i_sz)) != string::npos; i_sz += 2) val.replace(i_sz,1,2,'"');
+				    rez += "\""+TSYS::strNoSpace(val)+"\";";
+				    //>>>> Colspan process
+				    int colSpan = atoi(tblRow->childGet(i_c)->attr("colspan",false).c_str());
+				    for(int i_cs = 1; i_cs < colSpan; i_cs++) rez += ";";
+				    i_cl++;
+				}
+				rez += "\x0D\x0A";
+			    }
+			}
+			rez += "\x0D\x0A";
+		    }
+		    else continue;
+		}
+		curNode = curNode->parent();
+		treeStk.pop_back();
+		treeStk.back()++;
+	    }
+	}
+	//>> Export to XHTML
+	else rez = "<?xml version='1.0' ?>\n"
+		"<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN'\n"
+		"'DTD/xhtml1-transitional.dtd'>\n"
+		"<html xmlns='http://www.w3.org/1999/xhtml'>\n"
+		"<head>\n"
+		"  <meta http-equiv='Content-Type' content='text/html; charset="+Mess->charset()+"'/>\n"
+		"  <style type='text/css'>\n"+((ShapeDocument::ShpDt*)rwdg->shpData)->style+"</style>\n"
+		"</head>\n"+((ShapeDocument::ShpDt*)rwdg->shpData)->doc+"</html>";
+
+	if(rez.empty())	mod->postMess(mod->nodePath().c_str(),QString(_("No data for export.")),TVision::Error,this);
+	else ::write(fd,rez.data(),rez.size());
 	::close(fd);
     }
 }
@@ -744,7 +898,7 @@ void VisRun::exportDoc( const string &idoc )
 void VisRun::about()
 {
     QMessageBox::about(this,windowTitle(),
-	QString(_("%1 v%2.\n%3\nAutor: %4\nDevelopers: %5\nLicense: %6\n\n%7 v%8.\n%9\nLicense: %10\nAuthor: %11\nWeb site: %12")).
+	QString(_("%1 v%2.\n%3\nAuthor: %4\nDevelopers: %5\nLicense: %6\n\n%7 v%8.\n%9\nLicense: %10\nAuthor: %11\nWeb site: %12")).
 	arg(mod->modInfo("Name").c_str()).arg(mod->modInfo("Version").c_str()).arg(mod->modInfo("Description").c_str()).
 	arg(mod->modInfo("Author").c_str()).arg(mod->modInfo(_("Developers")).c_str()).arg(mod->modInfo("License").c_str()).
 	arg(PACKAGE_NAME).arg(VERSION).arg(_(PACKAGE_DESCR)).arg(PACKAGE_LICENSE).arg(_(PACKAGE_AUTHOR)).arg(PACKAGE_SITE));
@@ -752,7 +906,7 @@ void VisRun::about()
 
 void VisRun::userChanged( const QString &oldUser, const QString &oldPass )
 {
-    //> Try second connect to session for permition check
+    //> Try second connect to session for permission check
     XMLNode req("connect");
     req.setAttr("path","/%2fserv%2fsess")->setAttr("sess",workSess());
     if( cntrIfCmd(req) )
@@ -776,6 +930,8 @@ void VisRun::userChanged( const QString &oldUser, const QString &oldPass )
 	{
 	    x_scale *= (float)((QScrollArea*)centralWidget())->maximumViewportSize().width()/(float)master_pg->size().width();
 	    y_scale *= (float)((QScrollArea*)centralWidget())->maximumViewportSize().height()/(float)master_pg->size().height();
+	    //> Proportional scale
+	    if(keepAspectRatio) x_scale = y_scale = vmin(x_scale,y_scale);
 	}
 	fullUpdatePgs();
     }
@@ -801,7 +957,7 @@ void VisRun::aboutQt()
 
 void VisRun::fullScreen( bool vl )
 {
-    if( vl ) setWindowState(Qt::WindowFullScreen);
+    if(vl) setWindowState(Qt::WindowFullScreen);
     else setWindowState(Qt::WindowNoState);
 }
 
@@ -814,21 +970,21 @@ void VisRun::alarmAct( QAction *alrm )
 {
     if(alrm == NULL) return;
 
-    int quitance = 0;
+    int quittance = 0;
     string qwdg;
-    if( alrm->objectName() == "alarmLev" )		quitance = 0xFF;
-    else if( alrm->objectName() == "alarmLight" )	quitance = 0x01;
-    else if( alrm->objectName() == "alarmAlarm" )	quitance = 0x02;
-    else if( alrm->objectName() == "alarmSound" )
+    if(alrm->objectName() == "alarmLev")	quittance = 0xFF;
+    else if(alrm->objectName() == "alarmLight")	quittance = 0x01;
+    else if(alrm->objectName() == "alarmAlarm")	quittance = 0x02;
+    else if(alrm->objectName() == "alarmSound")
     {
-	quitance = 0x04;
+	quittance = 0x04;
 	qwdg = alrmPlay->widget( );
     }
     else return;
 
     XMLNode req("quittance");
     req.setAttr("path","/ses_"+work_sess+"/%2fserv%2falarm")->
-	setAttr("tmpl",TSYS::uint2str(quitance))->
+	setAttr("tmpl",TSYS::uint2str(quittance))->
 	setAttr("wdg",qwdg);
     cntrIfCmd(req);
 
@@ -913,9 +1069,9 @@ void VisRun::initSess( const string &prj_it, bool crSessForce )
     }
 
     req.clear()->setName("connect")->setAttr("path","/%2fserv%2fsess");
-    if( work_sess.empty() ) req.setAttr("prj",src_prj);
+    if(work_sess.empty()) req.setAttr("prj",src_prj);
     else req.setAttr("sess",work_sess);
-    if( cntrIfCmd(req) )
+    if(cntrIfCmd(req))
     {
 	mod->postMess(req.attr("mcat").c_str(),req.text().c_str(),TVision::Error,this);
 	close();
@@ -928,7 +1084,7 @@ void VisRun::initSess( const string &prj_it, bool crSessForce )
     //>> Get project's name
     req.clear()->setName("get")->setAttr("path","/prj_"+src_prj+"/%2fobj%2fcfg%2fname");
     if( !cntrIfCmd(req) )	setWindowTitle(req.text().c_str());
-    else setWindowTitle(QString(_("Runing project: %1")).arg(src_prj.c_str()));
+    else setWindowTitle(QString(_("Running project: %1")).arg(src_prj.c_str()));
 
     //> Set project's icon to window
     req.clear()->setAttr("path","/ses_"+work_sess+"/%2fico");
@@ -959,11 +1115,12 @@ void VisRun::initSess( const string &prj_it, bool crSessForce )
     }
 
     //> Get project's flags
-    req.clear()->setName("get")->setAttr("path","/prj_"+src_prj+"/%2fobj%2fcfg%2frunWin");
-    if( !cntrIfCmd(req) )
+    req.clear()->setName("get")->setAttr("path","/prj_"+src_prj+"/%2fobj%2fcfg%2fflgs");
+    if(!cntrIfCmd(req))
     {
-	if( atoi(req.text().c_str())&0x01 )	setWindowState( Qt::WindowMaximized );
-	else if( atoi(req.text().c_str())&0x02 )actFullScr->setChecked( true );
+	if(atoi(req.text().c_str())&0x01)	setWindowState(Qt::WindowMaximized);
+	else if(atoi(req.text().c_str())&0x02)	actFullScr->setChecked(true);
+	keepAspectRatio = atoi(req.text().c_str())&0x04;
     }
 
     //> Get open pages list
@@ -1015,11 +1172,15 @@ void VisRun::callPage( const string& pg_it, bool updWdg )
     string stmp;
 
     //> Scan and update opened page
-    if( master_pg )
+    if(master_pg)
     {
 	RunPageView *pg = master_pg->findOpenPage(pg_it);
-	if( pg && updWdg ) pg->update(false);
-	if( pg ) return;
+	if(pg)
+	{
+	    if(!(wPrcCnt%(5000/vmin(5000,period()))))	pg->update(false, NULL, true);
+	    else if(updWdg) pg->update(false);
+	    return;
+	}
     }
 
     // Get group and parent page
@@ -1039,12 +1200,12 @@ void VisRun::callPage( const string& pg_it, bool updWdg )
 	//>> Create widget view
 	master_pg = new RunPageView(pg_it,this,centralWidget());
 	master_pg->load("");
-	master_pg->setFocusPolicy( Qt::StrongFocus );
+	master_pg->setFocusPolicy(Qt::StrongFocus);
 	((QScrollArea *)centralWidget())->setWidget( master_pg );
-	if( !(windowState()&(Qt::WindowFullScreen|Qt::WindowMaximized)) )
+	if(!(windowState()&(Qt::WindowFullScreen|Qt::WindowMaximized)))
 	{
 	    QRect ws = QApplication::desktop()->availableGeometry(this);
-	    resize( vmin(master_pg->size().width()+10,ws.width()-10), vmin(master_pg->size().height()+55,ws.height()-10) );
+	    resize(vmin(master_pg->size().width()+10,ws.width()-10), vmin(master_pg->size().height()+55,ws.height()-10));
 	}
 	else x_scale = y_scale = 1.0;
     }
@@ -1222,10 +1383,12 @@ void VisRun::cacheResSet( const string &res, const string &val )
 
 void VisRun::updatePage( )
 {
-    if( winClose ) return;
+    if(winClose) return;
+
+    int rez;
 
 #if OSC_DEBUG >= 3
-    long long t_cnt = TSYS::curTime();
+    int64_t t_cnt = TSYS::curTime();
 #endif
 
     //> Pages update
@@ -1233,7 +1396,7 @@ void VisRun::updatePage( )
     req.setAttr("tm",TSYS::uint2str(reqtm))->
 	setAttr("path","/ses_"+work_sess+"/%2fserv%2fpg");
 
-    if( !cntrIfCmd(req) )
+    if(!(rez=cntrIfCmd(req)))
     {
 	//>> Check for delete pages
 	RunPageView *pg;
@@ -1260,10 +1423,19 @@ void VisRun::updatePage( )
 	    callPage(req.childGet(i_ch)->text(),atoi(req.childGet(i_ch)->attr("updWdg").c_str()));
 	}
     }
+    //>> Restore closed session of used project.
+    else if(rez == 2)
+    {
+	mess_warning(mod->nodePath().c_str(),_("Session creation restore for '%s'."),prj_it.c_str());
+	updateTimer->stop();
+	initSess(prj_it,crSessForce);
+	return;
+    }
+
     reqtm = strtoul(req.attr("tm").c_str(),NULL,10);
 
-    //> Alarms update (one seconds update)
-    if(wPrcCnt%(500/vmin(500,period())) == 0)
+    //> Alarms update (0.5 second update)
+    if((wPrcCnt%(500/vmin(500,period()))) == 0)
     {
 	//>> Get alarm status
 	unsigned wAlrmSt = alarmSt( );
@@ -1325,10 +1497,13 @@ void VisRun::updatePage( )
     {
 	float xSc = (float)((QScrollArea*)centralWidget())->maximumViewportSize().width()/(float)master_pg->size().width();
 	float ySc = (float)((QScrollArea*)centralWidget())->maximumViewportSize().height()/(float)master_pg->size().height();
-	if( xSc < 1 || ySc < 1 )
+	if(xSc < 1 || ySc < 1)
 	{
 	    x_scale *= xSc;
 	    y_scale *= ySc;
+	    //> Proportional scale
+	    if(keepAspectRatio) x_scale = y_scale = vmin(x_scale,y_scale);
+
 	    fullUpdatePgs();
 	}
     }
