@@ -25,6 +25,7 @@
 #include <QApplication>
 #include <QPainter>
 #include <QComboBox>
+#include <QStatusBar>
 
 #include <tsys.h>
 
@@ -42,10 +43,10 @@ RunWdgView::RunWdgView( const string &iwid, int ilevel, VisRun *mainWind, QWidge
     WdgView(iwid,ilevel,(QMainWindow*)mainWind,parent,f), mPermCntr(false), mPermView(true)
 {
     size_t endElSt = iwid.rfind("/");
-    if( endElSt == string::npos ) return;
+    if(endElSt == string::npos) return;
     string lstEl = iwid.substr(endElSt+1);
-    if( lstEl.size() > 4 && lstEl.substr(0,4) == "wdg_" ) setObjectName(lstEl.substr(4).c_str());
-    if( lstEl.size() > 3 && lstEl.substr(0,3) == "pg_" )  setObjectName(lstEl.substr(3).c_str());
+    if(lstEl.size() > 4 && lstEl.substr(0,4) == "wdg_") setObjectName(lstEl.substr(4).c_str());
+    if(lstEl.size() > 3 && lstEl.substr(0,3) == "pg_")  setObjectName(lstEl.substr(3).c_str());
 }
 
 RunWdgView::~RunWdgView( )
@@ -55,10 +56,8 @@ RunWdgView::~RunWdgView( )
 
 string RunWdgView::name( )
 {
-    XMLNode req("get");
-    req.setAttr("path",id()+"/%2fattr%2fname");
-    if( !cntrIfCmd(req) )	return req.text();
-    return "";
+    if(!windowTitle().isEmpty()) return windowTitle().toStdString();
+    return mainWin()->wAttrGet(id(),"name");
 }
 
 string RunWdgView::user( )
@@ -97,64 +96,65 @@ WdgView *RunWdgView::newWdgItem( const string &iwid )
     return new RunWdgView(iwid,wLevel()+1,mainWin(),this);
 }
 
-void RunWdgView::update( bool full, XMLNode *aBr )
+void RunWdgView::update( bool full, XMLNode *aBr, bool FullTree )
 {
     bool reqBrCr = false;
-    if( !aBr )
+    if(!aBr)
     {
 	aBr = new XMLNode("get");
-	aBr->setAttr("path",id()+"/%2fserv%2fattrBr")->setAttr("tm",TSYS::uint2str(full?0:mainWin()->reqTm()));
+	aBr->setAttr("path",id()+"/%2fserv%2fattrBr")->
+	    setAttr("tm",TSYS::uint2str(full?0:mainWin()->reqTm()))->setAttr("FullTree",FullTree?"1":"0");
 	cntrIfCmd(*aBr);
 	reqBrCr = true;
     }
 
-    if( full )	setAllAttrLoad(true);
+    if(full)	setAllAttrLoad(true);
     for(unsigned i_el = 0; i_el < aBr->childSize(); i_el++)
 	if(aBr->childGet(i_el)->name() == "el")
 	    attrSet("",aBr->childGet(i_el)->text(),atoi(aBr->childGet(i_el)->attr("p").c_str()));
-    if( full )
+    if(full)
     {
 	setAllAttrLoad(false);
-
 	attrSet("","load",-1);
+    }
 
-	//> Delete child widgets
+    //> Delete child widgets check
+    if(full || FullTree)
 	for(int i_c = 0, i_l = 0; i_c < children().size(); i_c++)
 	{
-	    if( !qobject_cast<RunWdgView*>(children().at(i_c)) || qobject_cast<RunPageView*>(children().at(i_c)) ) continue;
-	    for( i_l = 0; i_l < (int)aBr->childSize(); i_l++ )
-		if( aBr->childGet(i_l)->name() == "w" &&
-			((WdgView*)children().at(i_c))->id() == (id()+"/wdg_"+aBr->childGet(i_l)->attr("id")) )
+	    if(!qobject_cast<RunWdgView*>(children().at(i_c)) || qobject_cast<RunPageView*>(children().at(i_c))) continue;
+	    for(i_l = 0; i_l < (int)aBr->childSize(); i_l++)
+		if(aBr->childGet(i_l)->name() == "w" &&
+			((WdgView*)children().at(i_c))->id() == (id()+"/wdg_"+aBr->childGet(i_l)->attr("id")))
 		    break;
-	    if( i_l >= (int)aBr->childSize() ) children().at(i_c)->deleteLater();
+	    if(i_l >= (int)aBr->childSize()) children().at(i_c)->deleteLater();
 	}
-    }
 
     //> Create new child widget
     for(int i_l = 0, i_c = 0; i_l < (int)aBr->childSize(); i_l++)
     {
-	if( aBr->childGet(i_l)->name() != "w" ) continue;
+	if(aBr->childGet(i_l)->name() != "w") continue;
 
-	for( i_c = 0; i_c < children().size(); i_c++ )
-	    if( qobject_cast<RunWdgView*>(children().at(i_c)) && !qobject_cast<RunPageView*>(children().at(i_c)) &&
-		    ((RunWdgView*)children().at(i_c))->id() == (id()+"/wdg_"+aBr->childGet(i_l)->attr("id")) )
+	for(i_c = 0; i_c < children().size(); i_c++)
+	    if(qobject_cast<RunWdgView*>(children().at(i_c)) && !qobject_cast<RunPageView*>(children().at(i_c)) &&
+		    ((RunWdgView*)children().at(i_c))->id() == (id()+"/wdg_"+aBr->childGet(i_l)->attr("id")))
 	    {
-		((RunWdgView*)children().at(i_c))->update(full,aBr->childGet(i_l));
+		((RunWdgView*)children().at(i_c))->update(full,aBr->childGet(i_l),FullTree);
 		break;
 	    }
-	if( i_c < children().size() ) continue;
+	if(i_c < children().size()) continue;
 	WdgView *nwdg = newWdgItem(id()+"/wdg_"+aBr->childGet(i_l)->attr("id"));
 	nwdg->show();
 	nwdg->load("");
     }
 
-    if( full )
+    if(full)
     {
 	orderUpdate();
 	QWidget::update();
     }
 
-    if( reqBrCr ) delete aBr;
+    if(reqBrCr) delete aBr;
 }
 
 void RunWdgView::shapeList( const string &snm, vector<string> &ls )
@@ -186,38 +186,55 @@ void RunWdgView::orderUpdate( )
 
     //> Update tab order
     RunWdgView *prev_aw = NULL;
-    for( int i_c = 0; i_c < children().size(); i_c++ )
+    for(int i_c = 0; i_c < children().size(); i_c++)
     {
 	RunWdgView *cw = qobject_cast<RunWdgView*>(children().at(i_c));
-	if( !cw || !(mod->getFocusedWdg(cw)->focusPolicy()&Qt::TabFocus) ) continue;
-	if( prev_aw )	QWidget::setTabOrder( mod->getFocusedWdg(prev_aw), mod->getFocusedWdg(cw) );
+	if(!cw || !(mod->getFocusedWdg(cw)->focusPolicy()&Qt::TabFocus)) continue;
+	if(prev_aw) QWidget::setTabOrder(mod->getFocusedWdg(prev_aw), mod->getFocusedWdg(cw));
 	prev_aw = cw;
     }
 }
 
 bool RunWdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
 {
-    bool rez = WdgView::attrSet( attr, val, uiPrmPos );
+    bool rez = WdgView::attrSet(attr, val, uiPrmPos);
 
     switch(uiPrmPos)
     {
 	case -2:	//focus
-	    if( (bool)atoi(val.c_str()) == hasFocus() )      break;
-	    if( (bool)atoi(val.c_str()) ) setFocus(Qt::OtherFocusReason);
+	    if((bool)atoi(val.c_str()) == hasFocus())	break;
+	    if((bool)atoi(val.c_str())) setFocus(Qt::OtherFocusReason);
 	    return true;
 	case -3:	//perm
-	    setPermCntr( atoi(val.c_str())&SEC_WR );
-	    setPermView( atoi(val.c_str())&SEC_RD );
+	    setPermCntr(atoi(val.c_str())&SEC_WR);
+	    setPermView(atoi(val.c_str())&SEC_RD);
+	    return true;
+	case -4:        //page name
+            setWindowTitle(val.c_str());
+            break;
+	case 3:		//pgOpenSrc
+	    setProperty("pgOpenSrc",val.c_str());
 	    return true;
 	case 4:		//pgGrp
 	    setProperty("pgGrp",val.c_str());
 	    return true;
-	case 3:		//pgOpenSrc
-	    setProperty("pgOpenSrc",val.c_str());
-	    return true;
+	case 5:         //en
+            setProperty("isVisible", atoi(val.c_str()) && (permView() || dynamic_cast<RunPageView*>(this)));
+            return true;
 	case 6:		//active
 	    setProperty("active",(bool)atoi(val.c_str()));
 	    return true;
+	case 11:	//geomZ
+	    if(!allAttrLoad() && !dynamic_cast<RunPageView*>(this))
+	    {
+		RunWdgView *wdg = qobject_cast<RunWdgView*>(parentWidget());
+		if(wdg) { wdg->orderUpdate(); wdg->QWidget::update(); }
+	    }
+	    return true;
+	case 16:	//tipStatus
+	    if(val.size() && mainWin()->masterPg() == this)
+		mainWin()->statusBar()->showMessage(val.c_str(), 10000);
+	    return true;
 	case 17:	//contextMenu
 	    setProperty("contextMenu",val.c_str());
 	    return true;
@@ -228,10 +245,10 @@ bool RunWdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
 
 string RunWdgView::resGet( const string &res )
 {
-    if( res.empty() )	return "";
-    string ret = mainWin( )->cacheResGet(res);
-    if( ret.empty() && !(ret=WdgView::resGet(res)).empty() )
-	mainWin( )->cacheResSet(res,ret);
+    if(res.empty())	return "";
+    string ret = mainWin()->cacheResGet(res);
+    if(ret.empty() && !(ret=WdgView::resGet(res)).empty())
+	mainWin()->cacheResSet(res,ret);
 
     return ret;
 }
@@ -239,12 +256,12 @@ string RunWdgView::resGet( const string &res )
 bool RunWdgView::event( QEvent *event )
 {
     //> Force event's process
-    switch( event->type() )
+    switch(event->type())
     {
 	case QEvent::Paint:
-	    if( permView() )	break;
+	    if(permView())	break;
 	    //> Paint message about access denied
-	    if( dynamic_cast<RunPageView*>(this) )
+	    if(dynamic_cast<RunPageView*>(this))
 	    {
 		QPainter pnt(this);
 		//>> Fill page and draw border
@@ -262,10 +279,10 @@ bool RunWdgView::event( QEvent *event )
 		    cfnt.setPointSize( 16 );
 		    pnt.setFont( cfnt );
 		}
-		pnt.drawText(rect(),QString(_("Page: '%1'.\nView access is not permited.")).arg(id().c_str()),to);
+		pnt.drawText(rect(),QString(_("Page: '%1'.\nView access is not permitted.")).arg(id().c_str()),to);
 	    }
 	    return true;
-	case QEvent::MouseButtonPress:
+	case QEvent::MouseButtonRelease:
 	    if(((QMouseEvent*)event)->button() == Qt::RightButton && !property("contextMenu").toString().isEmpty() && property("active").toBool() && permCntr())
 	    {
 		QAction *actTmp;
@@ -439,17 +456,17 @@ bool RunWdgView::event( QEvent *event )
     }
 
     //> Try put mouse event to next level widget into same container
-    if( !qobject_cast<RunPageView*>(this) && 
-	(event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease || event->type() == QEvent::MouseButtonDblClick) )
+    if(!qobject_cast<RunPageView*>(this) &&
+	(event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease || event->type() == QEvent::MouseButtonDblClick))
     {
 	bool isOk = false;
 	QPoint curp = parentWidget()->mapFromGlobal(cursor().pos());
-	for( int i_c = parentWidget()->children().size()-1; i_c >= 0; i_c-- )
+	for(int i_c = parentWidget()->children().size()-1; i_c >= 0; i_c--)
 	{
 	    RunWdgView *wdg = qobject_cast<RunWdgView*>(parentWidget()->children().at(i_c));
-	    if( !wdg ) continue;
-	    if( wdg == this ) isOk = true;
-	    else if( isOk && wdg->geometry().contains(curp) )	return QApplication::sendEvent(wdg,event);
+	    if(!wdg) continue;
+	    if(wdg == this) isOk = true;
+	    else if(isOk && wdg->geometry().contains(curp)) return QApplication::sendEvent(wdg,event);
 	}
     }
 
@@ -492,27 +509,27 @@ RunPageView *RunPageView::findOpenPage( const string &ipg )
     RunPageView *pg;
 
     //> Self check
-    if( id() == ipg ) return this;
+    if(id() == ipg) return this;
 
     //> Check to included widgets
-    for( int i_ch = 0; i_ch < children().size(); i_ch++ )
+    for(int i_ch = 0; i_ch < children().size(); i_ch++)
     {
-	if( qobject_cast<RunPageView*>(children().at(i_ch)) )
+	if(qobject_cast<RunPageView*>(children().at(i_ch)))
 	{
 	    pg = ((RunPageView*)children().at(i_ch))->findOpenPage(ipg);
-	    if( pg ) return pg;
+	    if(pg) return pg;
 	    continue;
 	}
-	if( !qobject_cast<RunWdgView*>(children().at(i_ch)) )	continue;
+	if(!qobject_cast<RunWdgView*>(children().at(i_ch)))	continue;
 	RunWdgView *rwdg = (RunWdgView*)children().at(i_ch);
-	if( rwdg->root() == "Box" )
+	if(rwdg->property("isVisible").toBool() && rwdg->root() == "Box")
 	{
-	    if( rwdg->pgOpenSrc() == ipg && !rwdg->property("inclPg").toString().isEmpty() )
+	    if(rwdg->pgOpenSrc() == ipg && !rwdg->property("inclPg").toString().isEmpty())
 		return (RunPageView*)TSYS::str2addr(rwdg->property("inclPg").toString().toAscii().data());
-	    if( ((ShapeBox::ShpDt*)rwdg->shpData)->inclWidget )
+	    if(((ShapeBox::ShpDt*)rwdg->shpData)->inclWidget)
 	    {
 		pg = ((ShapeBox::ShpDt*)rwdg->shpData)->inclWidget->findOpenPage(ipg);
-		if( pg ) return pg;
+		if(pg) return pg;
 	    }
 	}
     }
@@ -523,16 +540,16 @@ RunPageView *RunPageView::findOpenPage( const string &ipg )
 bool RunPageView::callPage( const string &pg_it, const string &pgGrp, const string &pgSrc )
 {
     //> Check for set include page
-    for( int i_ch = 0; i_ch < children().size(); i_ch++ )
-	if( !pgGrp.empty() && !qobject_cast<RunPageView*>(children().at(i_ch)) &&
-		((RunWdgView*)children().at(i_ch))->root() == "Box" )
+    for(int i_ch = 0; i_ch < children().size(); i_ch++)
+	if(!pgGrp.empty() && !qobject_cast<RunPageView*>(children().at(i_ch)) &&
+		((RunWdgView*)children().at(i_ch))->property("isVisible").toBool() && ((RunWdgView*)children().at(i_ch))->root() == "Box")
 	{
-	    if( ((RunWdgView*)children().at(i_ch))->pgGrp() == pgGrp )
+	    if(((RunWdgView*)children().at(i_ch))->pgGrp() == pgGrp)
 	    {
 		string pg_it_prev = ((RunWdgView*)children().at(i_ch))->pgOpenSrc();
-		if( pg_it != pg_it_prev )
+		if(pg_it != pg_it_prev)
 		{
-		    if( !pg_it_prev.empty() )
+		    if(!pg_it_prev.empty())
 		    {
 			XMLNode req("close");
 			req.setAttr("path","/ses_"+mainWin()->workSess()+"/%2fserv%2fpg")->setAttr("pg",pg_it_prev);
@@ -542,19 +559,19 @@ bool RunPageView::callPage( const string &pg_it, const string &pgGrp, const stri
 		}
 		return true;
 	    }
-	    if( ((ShapeBox::ShpDt*)((RunWdgView*)children().at(i_ch))->shpData)->inclWidget &&
-		    ((ShapeBox::ShpDt*)((RunWdgView*)children().at(i_ch))->shpData)->inclWidget->callPage(pg_it,pgGrp,pgSrc) )
+	    if(((ShapeBox::ShpDt*)((RunWdgView*)children().at(i_ch))->shpData)->inclWidget &&
+		    ((ShapeBox::ShpDt*)((RunWdgView*)children().at(i_ch))->shpData)->inclWidget->callPage(pg_it,pgGrp,pgSrc))
 		return true;
         }
     //> Put checking to self include pages
-    for( int i_ch = 0; i_ch < children().size(); i_ch++ )
-	if( qobject_cast<RunPageView*>(children().at(i_ch)) &&
+    for(int i_ch = 0; i_ch < children().size(); i_ch++)
+	if(qobject_cast<RunPageView*>(children().at(i_ch)) &&
 		((RunPageView *)children().at(i_ch))->callPage(pg_it,pgGrp,pgSrc))
 	    return true;
     //> Check for open child page or for unknown and empty source pages open as master page child windows
-    if( (pgGrp.empty() && pgSrc == id()) || this == mainWin()->master_pg )
+    if((pgGrp.empty() && pgSrc == id()) || this == mainWin()->master_pg)
     {
-	RunPageView *pg = new RunPageView(pg_it,mainWin(),this);
+	RunPageView *pg = new RunPageView(pg_it, mainWin(), this);
 	pg->setAttribute( Qt::WA_DeleteOnClose );
 	pg->setWindowFlags( Qt::Tool );
 	pg->load("");
@@ -562,13 +579,14 @@ bool RunPageView::callPage( const string &pg_it, const string &pgGrp, const stri
 			  mapToGlobal(pos()).y()+sizeF().height()/2-pg->sizeF().height()/2));
 	pg->setMinimumSize(pg->frameGeometry().size());
 	pg->setMaximumSize(pg->frameGeometry().size());
-	pg->setWindowState( pg->windowState() | Qt::WindowActive );
+	pg->setWindowState(pg->windowState() | Qt::WindowActive);
 
 	//>> Get page name
-	XMLNode req("get");
+	/*XMLNode req("get");
 	req.setAttr("path",pg->id()+"/%2fwdg%2fcfg%2fname");
 	if( !mainWin()->cntrIfCmd(req) ) pg->setWindowTitle(req.text().c_str());
-	else pg->setWindowTitle(mainWin()->windowTitle());
+	else pg->setWindowTitle(mainWin()->windowTitle());*/
+	if(pg->windowTitle().isEmpty()) pg->setWindowTitle(mainWin()->windowTitle());
 
 	return true;
     }
@@ -584,13 +602,13 @@ void RunPageView::closeEvent( QCloseEvent *event )
     mainWin()->cntrIfCmd(req);
 
     //> Close included pages
-    for( int i_ch = 0; i_ch < children().size(); i_ch++ )
-	if( !qobject_cast<RunPageView*>(children().at(i_ch)) && ((RunWdgView *)children().at(i_ch))->root() == "Box" &&
-		!((RunWdgView*)children().at(i_ch))->pgOpenSrc().empty() )
+    /*for(int i_ch = 0; i_ch < children().size(); i_ch++)
+	if(!qobject_cast<RunPageView*>(children().at(i_ch)) && ((RunWdgView *)children().at(i_ch))->root() == "Box" &&
+		!((RunWdgView*)children().at(i_ch))->pgOpenSrc().empty())
 	{
 	    req.setAttr("path","/ses_"+mainWin()->workSess()+"/%2fserv%2fpg")->setAttr("pg",((RunWdgView*)children().at(i_ch))->pgOpenSrc());
 	    mainWin()->cntrIfCmd(req);
-	}
+	}*/
 }
 
 //*********************************************
