@@ -57,10 +57,10 @@ using namespace VISION;
 InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess, const QString &ndlg, bool with_id, bool with_nm ) :
 	QDialog(parent), m_id(NULL), m_name(NULL)
 {
+    setMaximumSize(800,600);
     setWindowTitle(ndlg);
-    setMinimumSize( QSize( 120, 150 ) );
     setWindowIcon(icon);
-    setSizeGripEnabled(true);
+    //setSizeGripEnabled(true);
 
     QVBoxLayout *dlg_lay = new QVBoxLayout(this);
     dlg_lay->setMargin(10);
@@ -162,7 +162,9 @@ void InputDlg::setIdLen( int len )
 
 void InputDlg::showEvent( QShowEvent * event )
 {
+    QSize src = size();
     adjustSize();
+    resize(size().expandedTo(src));
 }
 
 //*************************************************
@@ -247,7 +249,9 @@ void DlgUser::finish( int result )
 
 void DlgUser::showEvent( QShowEvent * event )
 {
+    QSize src = size();
     adjustSize();
+    resize(size().expandedTo(src));
 }
 
 //*********************************************
@@ -362,7 +366,9 @@ void FontDlg::cfgChange()
 
 void FontDlg::showEvent( QShowEvent * event )
 {
+    QSize src = size();
     adjustSize();
+    resize(size().expandedTo(src));
 }
 
 //*********************************************
@@ -412,13 +418,16 @@ bool UserStBar::userSel()
 //* QTimeEdit, QDateEdit and QDateTimeEdit.                                                   *
 //*********************************************************************************************
 LineEdit::LineEdit( QWidget *parent, LType tp, bool prev_dis, bool resApply ) :
-    QWidget(parent), m_tp((LineEdit::LType)-1), mPrev(!prev_dis), applyReserve(resApply),
+    QWidget(parent), m_tp((LineEdit::LType)-1), mPrev(!prev_dis), applyReserve(resApply), mIsEdited(false),
     ed_fld(NULL), bt_fld(NULL), bt_tm(NULL)
 {
     QHBoxLayout *box = new QHBoxLayout(this);
     box->setMargin(0);
     box->setSpacing(0);
 
+    bt_tm = new QTimer(this);
+    connect(bt_tm, SIGNAL(timeout()), this, SLOT(cancelSlot()));
+
     setType(tp);
 }
 
@@ -433,20 +442,18 @@ void LineEdit::viewApplyBt( bool view )
 	bt_fld->setIconSize( QSize(12,12) );
 	bt_fld->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed) );
 	bt_fld->setMaximumWidth( 15 );
-	connect( bt_fld, SIGNAL( clicked() ), this, SLOT( applySlot() ) );
+	connect(bt_fld, SIGNAL(clicked()), this, SLOT(applySlot()));
 	layout()->addWidget( bt_fld );
-
-	bt_tm = new QTimer(this);
-	connect( bt_tm, SIGNAL( timeout() ), this, SLOT( cancelSlot() ) );
     }
     if( !view && bt_fld )
     {
-	bt_tm->stop(); bt_tm->deleteLater(); bt_tm = NULL;
+	bt_tm->stop(); //bt_tm->deleteLater(); bt_tm = NULL;
 	bt_fld->deleteLater(); bt_fld = NULL;
+	mIsEdited = false;
     }
 }
 
-bool LineEdit::isEdited( )	{ return bt_fld; }
+bool LineEdit::isEdited( )	{ return mIsEdited; }
 
 void LineEdit::setType( LType tp )
 {
@@ -515,20 +522,24 @@ void LineEdit::setType( LType tp )
 void LineEdit::changed( )
 {
     //> Enable apply
-    if( mPrev && !bt_fld )	viewApplyBt(true);
-    if( bt_tm ) bt_tm->start(5000);
+    if(mPrev && !bt_fld) viewApplyBt(true);
+    bt_tm->start(mPrev ? 5000 : 500);
+    mIsEdited = true;
 
     emit valChanged(value());
 }
 
 void LineEdit::setValue( const QString &txt )
 {
-    if( ed_fld ) ed_fld->blockSignals(true);
+    if(ed_fld) ed_fld->blockSignals(true);
     switch(type())
     {
 	case Text:
-	    ((QLineEdit*)ed_fld)->setText(txt);
-	    ((QLineEdit*)ed_fld)->setCursorPosition(0);
+	    if(txt != value())
+	    {
+		((QLineEdit*)ed_fld)->setText(txt);
+		((QLineEdit*)ed_fld)->setCursorPosition(0);
+	    }
 	    break;
 	case Integer:
 	    ((QSpinBox*)ed_fld)->setValue(txt.toInt());
@@ -548,11 +559,11 @@ void LineEdit::setValue( const QString &txt )
 	    ((QComboBox*)ed_fld)->setEditText(txt);
 	    break;
     }
-    if( ed_fld ) ed_fld->blockSignals(false);
+    if(ed_fld) ed_fld->blockSignals(false);
 
     m_val = txt;
 
-    if( bt_fld ) viewApplyBt(false);
+    if(bt_fld) viewApplyBt(false);
 }
 
 void LineEdit::setCfg(const QString &cfg)
@@ -645,8 +656,13 @@ void LineEdit::applySlot( )
 
 void LineEdit::cancelSlot( )
 {
-    setValue(m_val);
-    emit cancel();
+    mIsEdited = false;
+    if(mPrev)
+    {
+	setValue(m_val);
+	emit cancel();
+    }
+    else applySlot();
 }
 
 bool LineEdit::event( QEvent * e )
@@ -683,72 +699,94 @@ void SyntxHighl::setSnthHgl(XMLNode nd)
     rehighlight();
 }
 
-void SyntxHighl::rule(XMLNode *rl, const QString &text, QTextCharFormat defForm, int off)
+void SyntxHighl::rule(XMLNode *irl, const QString &text, int off, char lev)
 {
-    if(text.isEmpty() || rl->name() != "rule") return;
+    XMLNode *rl;
+    vector<int> rul_pos(irl->childSize(),-1);
+    int minPos = -1, minRule, endIndex, startBlk, sizeBlk;
     QTextCharFormat kForm;
-    kForm.setForeground(QColor(rl->attr("color").c_str()));
-    kForm.setFontWeight(atoi(rl->attr("font_weight").c_str()) ? QFont::Bold : QFont::Normal);
-    kForm.setFontItalic(atoi(rl->attr("font_italic").c_str()));
-    QRegExp expr(rl->attr("expr").c_str());
-    for(int index = 0; true; index+=expr.matchedLength())
+    QRegExp expr;
+
+    if(lev > 3) return;
+
+    //> Init previous block continue
+    int curBlk = (currentBlockState()>>(lev*8))&0xFF;
+
+    //> Stream process by rules
+    for(int i_t = 0; i_t < text.length(); )
     {
-	if((index=expr.indexIn(text,index)) < 0 || expr.matchedLength() <= 0) break;
-	if(format(index+off)!=defForm) continue;
-	setFormat(index+off, expr.matchedLength(), kForm);
-	for(unsigned i_ch = 0; i_ch < rl->childSize(); i_ch++)
-	    rule(rl->childGet(i_ch),text.mid(index,expr.matchedLength()),kForm,index+off);
+        if(curBlk && !i_t) { minRule = curBlk-1; minPos = 0; }
+        else minRule = -1;
+
+        for(int i_ch = 0; i_t != minPos && i_ch < (int)irl->childSize(); i_ch++)
+        {
+            if(!(minPos < i_t || rul_pos[i_ch] < i_t || rul_pos[i_ch] < minPos)) continue;
+            if(rul_pos[i_ch] >= i_t && rul_pos[i_ch] < minPos)	{ minPos = rul_pos[i_ch]; minRule = i_ch; continue; }
+            if(rul_pos[i_ch] == i_t && rul_pos[i_ch] == minPos) { minRule = i_ch; break; }
+
+            //> Call rule
+            rl = irl->childGet(i_ch);
+            if(rl->name() == "rule")    expr.setPattern(rl->attr("expr").c_str());
+            else if(rl->name() == "blk")expr.setPattern(rl->attr("beg").c_str());
+            else continue;
+            rul_pos[i_ch] = expr.indexIn(text,i_t);
+            if(expr.matchedLength() <= 0) continue;
+            if(rul_pos[i_ch] < 0) rul_pos[i_ch] = text.length();
+            if(minPos < i_t || rul_pos[i_ch] < minPos) { minPos = rul_pos[i_ch]; minRule = i_ch; }
+        }
+        if(minRule < 0) break;
+
+        //> Process minimal rule
+        rl = irl->childGet(minRule);
+        kForm.setForeground(QColor(rl->attr("color").c_str()));
+        kForm.setFontWeight(atoi(rl->attr("font_weight").c_str()) ? QFont::Bold : QFont::Normal);
+        kForm.setFontItalic(atoi(rl->attr("font_italic").c_str()));
+
+        if(rl->name() == "rule")
+        {
+            expr.setPattern(rl->attr("expr").c_str());
+            if(expr.indexIn(text,i_t) != rul_pos[minRule]) break;
+            setFormat(rul_pos[minRule]+off, expr.matchedLength(), kForm);
+            //> Call include rules
+            if(rl->childSize()) rule(rl, text.mid(rul_pos[minRule],expr.matchedLength()), rul_pos[minRule]+off, lev+1);
+            i_t = rul_pos[minRule]+expr.matchedLength();
+        }
+        else if(rl->name() == "blk")
+        {
+            if(curBlk) rul_pos[minRule] = curBlk = startBlk = 0;
+            else
+            {
+                expr.setPattern(rl->attr("beg").c_str());
+                if(expr.indexIn(text,i_t) != rul_pos[minRule]) break;
+                startBlk = rul_pos[minRule]+expr.matchedLength();
+            }
+            QRegExp eExpr(rl->attr("end").c_str());
+            endIndex = eExpr.indexIn(text, startBlk);
+            if(endIndex == -1 || eExpr.matchedLength() <= 0)
+            {
+                setFormat(rul_pos[minRule]+off, (text.length()-rul_pos[minRule]), kForm);
+                sizeBlk = text.length()-startBlk;
+                i_t = text.length();
+            }
+            else
+            {
+                setFormat(rul_pos[minRule]+off, (endIndex-rul_pos[minRule]+eExpr.matchedLength()), kForm);
+                sizeBlk = endIndex-startBlk;
+                i_t = endIndex + eExpr.matchedLength();
+            }
+            //> Call include rules
+            if(rl->childSize()) rule(rl, text.mid(startBlk,sizeBlk), startBlk+off, lev+1);
+            if(endIndex == -1 || eExpr.matchedLength() <= 0)
+                setCurrentBlockState(((minRule+1)<<(lev*8))|currentBlockState());
+            else setCurrentBlockState(currentBlockState()& ~(0xFFFFFFFF<<(lev*8)));
+        }
     }
 }
 
 void SyntxHighl::highlightBlock(const QString &text)
 {
-    QTextCharFormat kForm, defkForm;
-    if(text.length()) defkForm = format(0);
-    setCurrentBlockState(previousBlockState());
-
-    for(int i_ch = 0; i_ch < (int)rules.childSize(); i_ch++)
-    {
-	XMLNode *rl = rules.childGet(i_ch);
-	kForm.setForeground(QColor(rl->attr("color").c_str()));
-	kForm.setFontWeight(atoi(rl->attr("font_weight").c_str()) ? QFont::Bold : QFont::Normal);
-	kForm.setFontItalic(atoi(rl->attr("font_italic").c_str()));
-
-	if(rl->name() == "rule") rule(rl,text,defkForm);
-	else if(rl->name() == "blk" && (currentBlockState() == -1 || currentBlockState() == i_ch))
-	{
-	    QRegExp bExpr(rl->attr("beg").c_str());
-	    QRegExp eExpr(rl->attr("end").c_str());
-	    for(int stIndex = 0, endIndex = 0, startBlk = 0, sizeBlk = 0; true; )
-	    {
-		if(currentBlockState() == -1)
-		{
-		    if((stIndex=bExpr.indexIn(text,stIndex)) == -1 || bExpr.matchedLength() <= 0) break;
-		    if(format(stIndex)!=defkForm) { stIndex += bExpr.matchedLength(); continue; }
-		    setCurrentBlockState(i_ch);
-		    startBlk = stIndex+bExpr.matchedLength();
-		}
-		endIndex = eExpr.indexIn(text, stIndex);
-		if(endIndex == -1 || eExpr.matchedLength() <= 0)
-		{
-		    setFormat(stIndex, (text.length()-stIndex), kForm);
-		    sizeBlk = text.length()-startBlk;
-		}
-		else
-		{
-		    setCurrentBlockState(-1);
-		    setFormat(stIndex, (endIndex-stIndex+eExpr.matchedLength()), kForm);
-		    sizeBlk = endIndex-startBlk;
-		}
-		//> Call include rules
-		for(unsigned i_ch1 = 0; i_ch1 < rl->childSize(); i_ch1++)
-		    rule(rl->childGet(i_ch1),text.mid(startBlk,sizeBlk),kForm,startBlk);
-
-		if(endIndex == -1 || eExpr.matchedLength() <= 0) break;
-		stIndex = endIndex+eExpr.matchedLength();
-	    }
-	}
-    }
+    setCurrentBlockState((previousBlockState()<0)?0:previousBlockState());
+    rule(&rules,text);
 }
 
 //*************************************************
@@ -806,6 +844,9 @@ TextEdit::TextEdit( QWidget *parent, bool prev_dis ) :
     QWidget *w = parentWidget();
     while(w && w->parentWidget() && (!dynamic_cast<QMainWindow *>(w) || !((QMainWindow*)w)->statusBar())) w = w->parentWidget();
     stWin = dynamic_cast<QMainWindow *>(w);
+
+    bt_tm = new QTimer(this);
+    connect(bt_tm, SIGNAL(timeout()), this, SLOT(applySlot()));
 }
 
 QString TextEdit::text()
@@ -813,18 +854,18 @@ QString TextEdit::text()
     return ed_fld->toPlainText();
 }
 
-void TextEdit::setText(const QString &text)
+void TextEdit::setText(const QString &itext)
 {
-    isInit=true;
-    ed_fld->setPlainText(text);
-    if( but_box && but_box->isEnabled() )
+    isInit = true;
+    if(itext != text()) ed_fld->setPlainText(itext);
+    if(but_box && but_box->isEnabled())
     {
 	but_box->setVisible(false);
 	but_box->setEnabled(false);
     }
-    isInit=false;
+    isInit = false;
 
-    m_text = text;
+    m_text = itext;
 }
 
 void TextEdit::setSnthHgl(XMLNode nd)
@@ -841,6 +882,9 @@ void TextEdit::changed()
 	but_box->setVisible(true);
 	but_box->setEnabled(true);
     }
+
+    if(!but_box) bt_tm->start(500);
+
     emit textChanged(text());
 }
 
@@ -1000,7 +1044,7 @@ WdgView *WdgView::newWdgItem( const string &iwid )
 bool WdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
 {
     //> Send value to model
-    if( !attr.empty() )
+    if(!attr.empty())
     {
 	XMLNode req("set");
 	req.setAttr("path",id()+"/%2fserv%2fattr");
@@ -1009,25 +1053,25 @@ bool WdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
     }
     bool up = false;
 
-    switch( uiPrmPos )
+    switch(uiPrmPos)
     {
 	case -1:	//load
 	    up = true;
 	    break;
 	case 0:	return false;
 	case 1:		//root
-	    if( shape && shape->id() == val )	break;
-	    if( shape ) shape->destroy(this);
+	    if(shape && shape->id() == val)	break;
+	    if(shape) shape->destroy(this);
 	    shape = mod->getWdgShape(val);
-	    if( shape ) shape->init(this);
+	    if(shape) shape->init(this);
 	    break;
 	case 7:		//geomX
-	    if( wLevel( ) == 0 )	break;
+	    if(wLevel() == 0)	break;
 	    mWPos = QPointF(((WdgView*)parentWidget())->xScale(true)*atof(val.c_str()),posF().y());
 	    up = true;
 	    break;
 	case 8:		//geomY
-	    if( wLevel( ) == 0 )	break;
+	    if(wLevel() == 0)	break;
 	    mWPos = QPointF(posF().x(),((WdgView*)parentWidget())->yScale(true)*atof(val.c_str()));
 	    up = true;
 	    break;
@@ -1040,7 +1084,7 @@ bool WdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
 	    up = true;
 	    break;
 	case 11:	//geomZ
-	    if(wLevel( )>0) z_coord = atoi(val.c_str());
+	    if(wLevel() > 0) z_coord = atoi(val.c_str());
 	    break;
 	case 13:	//geomXsc
 	    mWSize = QSizeF((atof(val.c_str())/x_scale)*sizeF().width(),sizeF().height());
@@ -1061,7 +1105,7 @@ bool WdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
     }
     if( up && !allAttrLoad( ) )
     {
-	if(wLevel( ) > 0) moveF(posF());
+	if(wLevel() > 0) moveF(posF());
 	resizeF(sizeF());
     }
 
@@ -1084,7 +1128,7 @@ string WdgView::resGet( const string &res )
 void WdgView::load( const string& item, bool isLoad, bool isInit, XMLNode *aBr )
 {
 #if OSC_DEBUG >= 3
-    long long t_cnt;
+    int64_t t_cnt;
     if( wLevel() == 0 ) t_cnt = TSYS::curTime();
 #endif
 
@@ -1105,10 +1149,10 @@ void WdgView::load( const string& item, bool isLoad, bool isInit, XMLNode *aBr )
 #endif
 	}
 
-	setAllAttrLoad( true );
-	if( item.empty() || item == id() )
+	setAllAttrLoad(true);
+	if(item.empty() || item == id())
 	    for(unsigned i_el = 0; i_el < aBr->childSize(); i_el++)
-		if( aBr->childGet(i_el)->name() == "el" )
+		if(aBr->childGet(i_el)->name() == "el")
 		    attrSet("",aBr->childGet(i_el)->text(),atoi(aBr->childGet(i_el)->attr("p").c_str()));
 	setAllAttrLoad( false );
 
