@@ -37,8 +37,8 @@
 #define MOD_TYPE	SUI_ID
 #define VER_TYPE	SUI_VER
 #define SUB_TYPE	"WWW"
-#define MOD_VER		"1.5.5"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"1.5.6"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Allow the WEB based OpenSCADA system configurator.")
 #define LICENSE		"GPL2"
 //*************************************************
@@ -73,7 +73,7 @@ TWEB::TWEB( string name ) : TUI(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -125,7 +125,7 @@ void TWEB::load_( )
 {
     //> Load parameters from command line
 
-    //> Load parameters from config file
+    //> Load parameters from config-file
 }
 
 void TWEB::modStart()
@@ -140,11 +140,11 @@ void TWEB::modStop()
 
 string TWEB::httpHead( const string &rcode, int cln, const string &cnt_tp, const string &addattr )
 {
-    return  "HTTP/1.0 "+rcode+"\r\n"
-	    "Server: "+PACKAGE_STRING+"\r\n"
-	    "Accept-Ranges: bytes\r\n"
-	    "Content-Length: "+TSYS::int2str(cln)+"\r\n"
-	    "Content-Type: "+cnt_tp+";charset="+Mess->charset()+"\r\n"+addattr+"\r\n";
+    return  "HTTP/1.0 "+rcode+"\x0D\x0A"
+	    "Server: "+PACKAGE_STRING+"\x0D\x0A"
+	    "Accept-Ranges: bytes\x0D\x0A"
+	    "Content-Length: "+TSYS::int2str(cln)+"\x0D\x0A"
+	    "Content-Type: "+cnt_tp+";charset="+Mess->charset()+"\x0D\x0A"+addattr+"\x0D\x0A";
 }
 
 string TWEB::pgHead( string head_els )
@@ -248,9 +248,9 @@ void TWEB::getAbout( SSess &ses )
 	"<tr><td class='content'>\n"
 	"<table border='0px' cellspacing='3px'>\n"
 	"<TR><TD style='color: blue;'>"+_("Name: ")+"</TD><TD>"+_(MOD_NAME)+"</TD></TR>\n"
-	"<TR><TD style='color: blue;'>"+_("Desription: ")+"</TD><TD>"+_(DESCRIPTION)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Description: ")+"</TD><TD>"+_(DESCRIPTION)+"</TD></TR>\n"
 	"<TR><TD style='color: blue;'>"+_("License: ")+"</TD><TD>"+_(LICENSE)+"</TD></TR>\n"
-	"<TR><TD style='color: blue;'>"+_("Author: ")+"</TD><TD>"+_(AUTORS)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Author: ")+"</TD><TD>"+_(AUTHORS)+"</TD></TR>\n"
 	"</table>\n"
 	"</TD></TR>\n</table><br/>\n"
 	"<table class='work'>\n"
@@ -276,8 +276,8 @@ void TWEB::getHead( SSess &ses )
 	"<a href='"+path+"?com=save' title='"+_("Save")+"'><img src='/"MOD_ID"/img_save' alt='"+_("Save")+"'/></a>\n"
 	"<img src='/"MOD_ID"/img_line'/>\n"
 	"<a href='/"MOD_ID"' title='"+_("Root page")+"'><img src='/"MOD_ID"/img_gohome' alt='"+_("Root page")+"'/></a>\n"
-	"<a href='"+path+"' title='"+_("Curent page")+"'><img src='/"MOD_ID"/img_reload' alt='" +_("Curent page")+"'/></a>\n"
-	"<a href='"+path.substr(0,path.rfind("/"))+"' title='"+_("Previous page")+"'><img src='/"MOD_ID"/img_up' alt='"+_("Previos page")+"'/></a>\n"
+	"<a href='"+path+"' title='"+_("Current page")+"'><img src='/"MOD_ID"/img_reload' alt='" +_("Curent page")+"'/></a>\n"
+	"<a href='"+path.substr(0,path.rfind("/"))+"' title='"+_("Previous page")+"'><img src='/"MOD_ID"/img_up' alt='"+_("Previous page")+"'/></a>\n"
 	"<img src='/"MOD_ID"/img_line'/>\n"
 	"<a href='/"MOD_ID"/about' title='"+_("About")+"'><img src='/"MOD_ID"/img_help' alt='"+_("About")+"'/></a>\n"
 	"</td>\n"
@@ -411,7 +411,7 @@ void TWEB::getCmd( SSess &ses, XMLNode &node, string a_path )
     }
     if(f_cfg > 0) ses.page = ses.page+")<br/>\n";
 
-    ses.page = ses.page+"</form>\n"; 
+    ses.page = ses.page+"</form>\n";
 }
 
 bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
@@ -883,11 +883,11 @@ int TWEB::postCmd( SSess &ses, XMLNode &node, string prs_path )
 
 	    string url = string("/")+MOD_ID+"/"+TSYS::strEncode(req.text(),TSYS::HttpURL);
 
-	    mess_info(nodePath().c_str(),_("%s| Go to link <%s>"),ses.user.c_str(),url.c_str());
+	    mess_info(nodePath().c_str(),_("%s| Go to link '%s'"),ses.user.c_str(),url.c_str());
 
 	    ses.page = ses.page + "<meta http-equiv='Refresh' content='0; url="+url+"'>\n";
 	    messPost( ses.page,nodePath(),"Go to <"+url+"> !",TWEB::Info);
-	    return 0x01|0x02;  //No error. That no draw curent page
+	    return 0x01|0x02;  //No error. That no draw current page
 	}
 
 	for(unsigned i_prm = 0; i_prm < node.childSize(); i_prm++)
@@ -980,7 +980,7 @@ int TWEB::postList( SSess &ses, XMLNode &node, string prs_path )
 
 	ses.page = ses.page + "<meta http-equiv='Refresh' content='0; url="+url+"'/>\n";
 	messPost( ses.page, nodePath(), "Go to <"+url+"> !",TWEB::Info);
-	return 0x01|0x02;  //No error. That no draw curent page
+	return 0x01|0x02;  //No error. That no draw current page
     }
 
     XMLNode n_el1; n_el1.setAttr("path",f_path)->setAttr("user",ses.user);
@@ -991,7 +991,7 @@ int TWEB::postList( SSess &ses, XMLNode &node, string prs_path )
 	if( ind_m ) n_el1.setAttr("id",i_el);
 	else n_el1.setText(l_el);
 
-	mess_info(nodePath().c_str(),_("%s| Delete <%s> element <%s:%s>."),
+	mess_info(nodePath().c_str(),_("%s| Delete '%s' element <%s:%s>."),
 	    ses.user.c_str(), f_path.c_str(), i_el.c_str(), l_el.c_str());
     }
     else if( l_com == "ins" )
@@ -1002,7 +1002,7 @@ int TWEB::postList( SSess &ses, XMLNode &node, string prs_path )
 	if( ind_m ) n_el1.setAttr("id",ener_id);
 	n_el1.setText(ener_f);
 
-	mess_info(nodePath().c_str(),_("%s| Insert <%s> element <%s:%s> to %s."),
+	mess_info(nodePath().c_str(),_("%s| Insert '%s' element <%s:%s> to %s."),
 	    ses.user.c_str(), f_path.c_str(), ener_id.c_str(), ener_f.c_str(), i_pos.c_str());
     }
     else if( l_com == "add" )
@@ -1011,7 +1011,7 @@ int TWEB::postList( SSess &ses, XMLNode &node, string prs_path )
 	if( ind_m ) n_el1.setAttr("id",ener_id);
 	n_el1.setText(ener_f);
 
-	mess_info(nodePath().c_str(),_("%s| Add <%s> element <%s:%s>."),
+	mess_info(nodePath().c_str(),_("%s| Add '%s' element <%s:%s>."),
 	    ses.user.c_str(), f_path.c_str(), ener_id.c_str(), ener_f.c_str());
     }
     else if( l_com == "edit" )
@@ -1022,7 +1022,7 @@ int TWEB::postList( SSess &ses, XMLNode &node, string prs_path )
 	if( ind_m ) n_el1.setAttr("id",ener_id);
 	n_el1.setText(ener_f);
 
-	mess_info(nodePath().c_str(),_("%s| Set <%s> element %s to <%s:%s>."),
+	mess_info(nodePath().c_str(),_("%s| Set '%s' element %s to <%s:%s>."),
 	    ses.user.c_str(), f_path.c_str(), i_pos.c_str(), ener_id.c_str(), ener_f.c_str());
     }
     else if( l_com == "up" || l_com == "down" )
@@ -1034,7 +1034,7 @@ int TWEB::postList( SSess &ses, XMLNode &node, string prs_path )
 	n_el1.setAttr("pos",i_pos);
 	n_el1.setAttr("to",i_pos_to);
 
-	mess_info(nodePath().c_str(),_("%s| Move <%s> from %s to %s."),
+	mess_info(nodePath().c_str(),_("%s| Move '%s' from %s to %s."),
 	    ses.user.c_str(), f_path.c_str(), i_pos.c_str(), i_pos_to.c_str());
     }
 
@@ -1112,7 +1112,7 @@ int TWEB::postTable( SSess &ses, XMLNode &node, string prs_path )
 			    //-- Get current column id --
 			    n_el1.setAttr("col",t_linf->attr("id"));
 			}
-			mess_info(nodePath().c_str(),_("%s| Set <%s> cell (<%s>:%s) to: %s."),
+			mess_info(nodePath().c_str(),_("%s| Set '%s' cell ('%s':%s) to: %s."),
 			    ses.user.c_str(), f_path.c_str(), row_addr.c_str(), t_linf->attr("id").c_str(), new_val.c_str());
 
 			if(cntrIfCmd(n_el1)) ses.mess.push_back(n_el1.text().c_str());
@@ -1123,7 +1123,7 @@ int TWEB::postTable( SSess &ses, XMLNode &node, string prs_path )
     else if( l_com == "add" )
     {
 	n_el1.setName("add");
-	mess_info(nodePath().c_str(),_("%s| Add <%s> record."), ses.user.c_str(), f_path.c_str() );
+	mess_info(nodePath().c_str(),_("%s| Add '%s' record."), ses.user.c_str(), f_path.c_str() );
 
 	if(cntrIfCmd(n_el1)) ses.mess.push_back(n_el1.text().c_str());
     }
@@ -1134,7 +1134,7 @@ int TWEB::postTable( SSess &ses, XMLNode &node, string prs_path )
 	    {
 		n_el1.setName("ins");
 		n_el1.setAttr("row",TSYS::int2str(i_rw+op_cnt));
-		mess_info(nodePath().c_str(),_("%s| Insert <%s> record %d."),
+		mess_info(nodePath().c_str(),_("%s| Insert '%s' record %d."),
 		    ses.user.c_str(), f_path.c_str(), i_rw+op_cnt );
 
 		if(cntrIfCmd(n_el1)) ses.mess.push_back(n_el1.text().c_str());
@@ -1158,7 +1158,7 @@ int TWEB::postTable( SSess &ses, XMLNode &node, string prs_path )
 			    if( dt_tbl.childGet(i_el)->attr("id") == key )
 			    { n_el1.setAttr("key_"+key,dt_tbl.childGet(i_el)->childGet(i_rw)->text()); break; }
 		}
-		mess_info(nodePath().c_str(),_("%s| Delete <%s> record %d."),
+		mess_info(nodePath().c_str(),_("%s| Delete '%s' record %d."),
 		    ses.user.c_str(), f_path.c_str(), i_rw-op_cnt );
 
 		if(cntrIfCmd(n_el1)) ses.mess.push_back(n_el1.text().c_str());
@@ -1174,7 +1174,7 @@ int TWEB::postTable( SSess &ses, XMLNode &node, string prs_path )
 		if( l_com == "down" )  r_new = i_rw+1;
 		n_el1.setName("move");
 		n_el1.setAttr("row",TSYS::int2str(i_rw))->setAttr("to",TSYS::int2str(r_new));
-		mess_info(nodePath().c_str(),_("%s| Move <%s> record from %d to %d."),
+		mess_info(nodePath().c_str(),_("%s| Move '%s' record from %d to %d."),
 		    ses.user.c_str(), f_path.c_str(), i_rw, r_new );
 
 		if(cntrIfCmd(n_el1)) ses.mess.push_back(n_el1.text().c_str());
@@ -1324,7 +1324,7 @@ SSess::SSess( const string &iurl, const string &isender, const string &iuser, ve
     //> Content parse
     string boundary;
     const char *c_bound = "boundary=";
-    const char *c_term = "\r\n";
+    const char *c_term = "\x0D\x0A";
     const char *c_end = "--";
     const char *c_fd = "Content-Disposition";
     const char *c_name = "name=\"";
