@@ -38,8 +38,8 @@
 #define MOD_TYPE	SUI_ID
 #define VER_TYPE	SUI_VER
 #define SUB_TYPE	"WWW"
-#define MOD_VER		"0.6.6"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"0.8.0"
+#define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Allow the dynamic WEB based OpenSCADA system configurator. Use XHTML, CSS and JavaScript technologies.")
 #define LICENSE		"GPL2"
 //*************************************************
@@ -77,7 +77,7 @@ TWEB::TWEB( string name ) : TUI(MOD_ID)
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -105,9 +105,14 @@ TWEB::TWEB( string name ) : TUI(MOD_ID)
 	_("Add row"), _("Insert row"), _("Delete row"), _("Up row"), _("Down row"),
 	_("No one editable container present."), _("Add item to node '%1'."), _("Item '%1' already present."),
 	_("You sure for delete node '%1'?"),
-	_("Copy is imposible."), _("Selected"), _("Move node '%1' to '%2'."), _("Copy node '%1' to '%2'."), _("Node '%1' already present. Continue?"),
+	_("Copy is impossible."), _("Selected"), _("Move node '%1' to '%2'."), _("Copy node '%1' to '%2'."), _("Node '%1' already present. Continue?"),
 	_("Element type:", "ID:"), _("Name:"), _("Ok"), _("Close"),
-	_("Page loaded.")
+	_("Page loaded."),
+	_("Node id and/or name select"),
+	_("Item name set"),
+	_("Image set"),
+	_("Add node"),
+	_("Move or copy node"),
     };
 #endif
 }
@@ -135,7 +140,7 @@ void TWEB::load_( )
 {
     //> Load parameters from command line
 
-    //> Load parameters from config file
+    //> Load parameters from config-file
 }
 
 void TWEB::modStart()
@@ -150,12 +155,12 @@ void TWEB::modStop()
 
 string TWEB::httpHead( const string &rcode, int cln, const string &cnt_tp, const string &addattr )
 {
-    return  "HTTP/1.0 "+rcode+"\r\n"
-	    "Server: "+PACKAGE_STRING+"\r\n"
-	    "Accept-Ranges: bytes\r\n"
-	    "Content-Length: "+TSYS::int2str(cln)+"\r\n"
-	    "Content-Type: "+cnt_tp+"; charset="+Mess->charset()+"\r\n"
-	    "Cache-Control: no-cache\r\n"+addattr+"\r\n";
+    return  "HTTP/1.0 "+rcode+"\x0D\x0A"
+	    "Server: "+PACKAGE_STRING+"\x0D\x0A"
+	    "Accept-Ranges: bytes\x0D\x0A"
+	    "Content-Length: "+TSYS::int2str(cln)+"\x0D\x0A"
+	    "Content-Type: "+cnt_tp+"; charset="+Mess->charset()+"\x0D\x0A"
+	    "Cache-Control: no-cache\x0D\x0A"+addattr+"\x0D\x0A";
 }
 
 string TWEB::pgHead( string head_els )
@@ -218,15 +223,17 @@ void TWEB::imgConvert(SSess &ses)
 	sim = dim;
     }
     //>> Check for disable icon make
-    if(sim && (prmEl = ses.prm.find("filtr")) != ses.prm.end() && prmEl->second == "gray")
+    if(sim && (prmEl = ses.prm.find("filtr")) != ses.prm.end() && (prmEl->second == "gray" || prmEl->second == "unact"))
     {
         gdImagePtr dim = gdImageCreateTrueColor(gdImageSX(sim),gdImageSY(sim));
 	gdImageAlphaBlending(dim,0);
+	bool isUnAct = (prmEl->second == "unact");
 	for(int i_y = 0; i_y < gdImageSY(sim); i_y++)
 	    for(int i_x = 0; i_x < gdImageSX(sim); i_x++)
 	    {
 		int c = gdImageGetPixel(sim,i_x,i_y);
 		int y = (int)(0.3*gdImageRed(sim,c)+0.59*gdImageGreen(sim,c)+0.11*gdImageBlue(sim,c));
+		if(isUnAct) y = 255-(255-y)/2;
 		c = (int)gdImageColorResolveAlpha(dim,y,y,y,gdImageAlpha(sim,c));
 		gdImageSetPixel(dim,i_x,i_y,c);
 	    }
@@ -267,7 +274,7 @@ void TWEB::HttpGet( const string &urli, string &page, const string &sender, vect
 	else if( zero_lev == "ico" || zero_lev.substr(0,4) == "img_" )
 	{
 	    string itp;
-	    ses.page=TUIS::icoGet( zero_lev=="ico"?"UI."MOD_ID:zero_lev.substr(4), &itp );
+	    ses.page = TUIS::icoGet(((zero_lev == "ico")?"UI."MOD_ID:zero_lev.substr(4)), &itp);
 	    imgConvert(ses);
 	    page = httpHead("200 OK",ses.page.size(),string("image/")+itp)+ses.page;
 	    return;
@@ -287,6 +294,11 @@ void TWEB::HttpGet( const string &urli, string &page, const string &sender, vect
 	    else if( wp_com.empty() )
 	    {
 		ses.page = trMessReplace(WebCfgDVCA_html);
+		//>> User replace
+		size_t userPos = ses.page.find("##USER##");
+		if(userPos != string::npos)
+		ses.page.replace(userPos,8,TSYS::strMess("<span style=\"color: %s;\">%s</span>",((user=="root")?"red":"green"),user.c_str()));
+
 		page = httpHead("200 OK",ses.page.size(),"text/html")+ses.page;
 		return;
 	    }
@@ -348,25 +360,9 @@ void TWEB::HttpGet( const string &urli, string &page, const string &sender, vect
 		page = mod->httpHead("200 OK",ses.page.size(),"text/xml")+ses.page;
 		return;
 	    }
-	    else if( wp_com == "info" )
+	    else if(wp_com == "info" || wp_com == "get" || wp_com == "modify")
 	    {
-		XMLNode req("info"); req.setAttr("path",ses.url);
-		mod->cntrIfCmd(req,ses.user);
-		ses.page = req.save();
-		page = mod->httpHead("200 OK",ses.page.size(),"text/xml")+ses.page;
-		return;
-	    }
-	    else if( wp_com == "modify" )
-	    {
-		XMLNode req("modify"); req.setAttr("path",ses.url+"/%2fobj");
-		mod->cntrIfCmd(req,ses.user);
-		ses.page = req.save();
-		page = mod->httpHead("200 OK",ses.page.size(),"text/xml")+ses.page;
-		return;
-	    }
-	    else if( wp_com == "get" )
-	    {
-		XMLNode req("get"); req.setAttr("path",ses.url);
+		XMLNode req(wp_com); req.setAttr("path",ses.url);
 		mod->cntrIfCmd(req,ses.user);
 		ses.page = req.save();
 		page = mod->httpHead("200 OK",ses.page.size(),"text/xml")+ses.page;
@@ -407,9 +403,9 @@ void TWEB::getAbout( SSess &ses )
 	"<TR><TD class='content'>\n"
 	"<table border='0px' cellspacing='3px'>\n"
 	"<TR><TD style='color: blue;'>"+_("Name: ")+"</TD><TD>"+_(MOD_NAME)+"</TD></TR>\n"
-	"<TR><TD style='color: blue;'>"+_("Desription: ")+"</TD><TD>"+_(DESCRIPTION)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Description: ")+"</TD><TD>"+_(DESCRIPTION)+"</TD></TR>\n"
 	"<TR><TD style='color: blue;'>"+_("License: ")+"</TD><TD>"+_(LICENSE)+"</TD></TR>\n"
-	"<TR><TD style='color: blue;'>"+_("Author: ")+"</TD><TD>"+_(AUTORS)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Author: ")+"</TD><TD>"+_(AUTHORS)+"</TD></TR>\n"
 	"</table>\n"
 	"</TD></TR>\n</table><br/>\n"
 	"<table class='work'>\n"
@@ -441,6 +437,14 @@ void TWEB::HttpPost( const string &url, string &page, const string &sender, vect
 	ses.page = req.save();
 	page = httpHead("200 OK",ses.page.size(),"text/xml")+ses.page;
     }
+    //> Full request to control interface
+    else if( wp_com == "req" )
+    {
+	XMLNode req(""); req.load(ses.content);
+	mod->cntrIfCmd(req,ses.user);
+	ses.page = req.save(XMLNode::XMLHeader);
+	page = httpHead("200 OK",ses.page.size(),"text/xml")+ses.page;
+    }
     else if( wp_com == "img" )
     {
 	if( (cntEl=ses.cnt.find("name")) != ses.cnt.end() && !ses.files[cntEl->second].empty() )
@@ -550,7 +554,7 @@ SSess::SSess( const string &iurl, const string &isender, const string &iuser, ve
     //> Content parse
     string boundary;
     const char *c_bound = "boundary=";
-    const char *c_term = "\r\n";
+    const char *c_term = "\x0D\x0A";
     const char *c_end = "--";
     const char *c_fd = "Content-Disposition";
     const char *c_name = "name=\"";
