@@ -24,6 +24,26 @@ var isNN = navigator.appName.indexOf('Netscape') != -1;
 var isIE = navigator.appName.indexOf('Microsoft') != -1;
 var isOpera = navigator.appName.indexOf('Opera') != -1;
 var isKonq = navigator.userAgent.indexOf('Konqueror') != -1;
+
+/***************************************************
+ * strEncode - String encoding.                    *
+ ***************************************************/
+function strEncode( vl, tp )
+{
+  var encRez = '';
+  if(!tp || tp == "html")
+    for(var i_enc = 0; i_enc < vl.length; i_enc++)
+      switch(vl.charAt(i_enc))
+      {
+        case '&': encRez += '&amp;'; break;
+        case '>': encRez += '&gt;'; break;
+        case '<': encRez += '&lt;'; break;
+        case '"': encRez += '&quot;'; break;
+        default:  encRez += vl.charAt(i_enc);
+      }
+  return encRez;
+}
+
 /***************************************************
  * pathLev - Path parsing function.                *
  ***************************************************/
@@ -33,9 +53,9 @@ function pathLev( path, level, scan )
   var an_dir = scan ? pathLev.off : 0;
   var t_lev = 0;
   var t_dir;
-  while( an_dir<path.length && path.charAt(an_dir)=='/' ) an_dir++;
-  if( an_dir >= path.length ) return '';
-  while( true )
+  while(an_dir < path.length && path.charAt(an_dir) == '/') an_dir++;
+  if(an_dir >= path.length) return '';
+  while(true)
   {
     t_dir = path.indexOf('/',an_dir);
     if( t_dir < 0 ) { pathLev.off=path.length; return (t_lev==level)?path.substr(an_dir):''; }
@@ -93,9 +113,12 @@ function posGetX(obj,noWScrl)
 {
   var posX = 0;
   for( ; obj && obj.nodeName != 'BODY' ; obj=obj.parentNode )
+  {
     posX += (obj.style.left?parseInt(obj.style.left):0)+
 	(obj.parentNode.style.borderLeftWidth?parseInt(obj.parentNode.style.borderLeftWidth):0)+
 	(obj.parentNode.style.marginLeft?parseInt(obj.parentNode.style.marginLeft):0);
+    if(obj.style.position == 'relative') posX += obj.offsetLeft;
+  }
   return posX+(!noWScrl?-window.pageXOffset:0);
 }
 /***************************************************
@@ -105,9 +128,12 @@ function posGetY(obj,noWScrl)
 {
   var posY = 0;
   for( ; obj && obj.nodeName != 'BODY' ; obj=obj.parentNode )
+  {
     posY += (obj.style.top?parseInt(obj.style.top):0)+
 	(obj.parentNode.style.borderTopWidth?parseInt(obj.parentNode.style.borderTopWidth):0)+
 	(obj.parentNode.style.marginTop?parseInt(obj.parentNode.style.marginTop):0);
+    if(obj.style.position == 'relative') posY += obj.offsetTop;
+  }
   return posY+(!noWScrl?-window.pageYOffset:0);
 }
 /***************************************************
@@ -291,6 +317,11 @@ function getColor( cStr )
  *****************************************************/
 function chkPattern( val, patt )
 {
+  //> Check by regular expression
+  if(patt.length > 2 && patt.charAt(0) == '/' && patt.charAt(patt.length-1) == '/')
+    return (new RegExp(patt.slice(1,patt.length-1),'')).test(val);
+
+  //> Check by simple pattern
   var mult_s = false;
   var v_cnt = 0, p_cnt = 0;
   var v_bck = -1, p_bck = -1;
@@ -319,6 +350,7 @@ function chkPattern( val, patt )
   }
   return false;
 }
+
 /***************************************************
  * setFocus - Command for set focus                *
  ***************************************************/
@@ -331,99 +363,193 @@ function setFocus( wdg, onlyClr )
   if( onlyClr ) return;
   attrs.focus = '1'; attrs.event = 'ws_FocusIn'; setWAttrs(masterPage.focusWdf,attrs);
 }
-/***************************************************
- * pwDescr - Page/widget descriptor object         *
- ***************************************************/
+
+/**********************************************************
+ * callPage - call page 'pgId' for open update and other. *
+ **********************************************************/
 function callPage( pgId, updWdg, pgGrp, pgOpenSrc )
 {
-  if( !pgId ) return true;
+  if(!pgId) return true;
   //> Check and update present page
-  if( this == masterPage )
+  if(this == masterPage)
   {
     var opPg = this.findOpenPage(pgId);
-    if( opPg && updWdg ) opPg.makeEl(servGet(pgId,'com=attrsBr&tm='+tmCnt));
-    if( opPg ) return true;
+    if(opPg)
+    {
+      if(!(prcCnt%5)) opPg.makeEl(servGet(pgId,'com=attrsBr&FullTree=1&tm='+tmCnt),false,false,true);
+      else if(updWdg) opPg.makeEl(servGet(pgId,'com=attrsBr&tm='+tmCnt));
+      return true;
+    }
   }
   //> Create or replace main page
-  if( !pgGrp ) pgGrp = getWAttr(pgId,'pgGrp');
-  if( !pgOpenSrc ) pgOpenSrc = getWAttr(pgId,'pgOpenSrc');
-  if( !this.addr.length || (this == masterPage && pgGrp == 'main') || pgGrp == this.attrs['pgGrp'] )
+  if(!pgGrp) pgGrp = getWAttr(pgId,'pgGrp');
+  if(!pgOpenSrc) pgOpenSrc = getWAttr(pgId,'pgOpenSrc');
+  if(this == masterPage && (!this.addr.length || pgGrp == 'main' || pgGrp == this.attrs['pgGrp']))
   {
+    if(this.addr.length)
+    {
+	servSet(this.addr,'com=pgClose','');
+	document.body.removeChild(this.window);
+    }
     this.addr  = pgId;
     this.place = document.createElement('div');
-    this.makeEl(servGet(pgId,'com=attrsBr'));
-    document.body.appendChild(this.place);
+    this.place.setAttribute('id','mainCntr');
+    this.makeEl(servGet(pgId,'com=attrsBr'),false,true);
+    var centerTag = document.createElement('center');
+    centerTag.appendChild(this.place);
+    document.body.appendChild(centerTag);
+    this.window = centerTag;
     //> Set project's icon and RunTime page title
     document.getElementsByTagName('link')[0].setAttribute('href',location.pathname+'?com=ico');
     return true;
   }
   //> Find for include page creation
-  for( var i in this.wdgs )
-    if( this.wdgs[i].attrs['root'] == 'Box' )
+  for(var i in this.wdgs)
+    if(this.wdgs[i].attrs['root'] == 'Box' && this.wdgs[i].isVisible)
     {
-      if( pgGrp == this.wdgs[i].attrs['pgGrp'] && pgId != this.wdgs[i].attrs['pgOpenSrc'] )
+      if(pgGrp == this.wdgs[i].attrs['pgGrp'] && pgId != this.wdgs[i].attrs['pgOpenSrc'])
       {
 	this.wdgs[i].attrs['pgOpenSrc'] = pgId;
 	this.wdgs[i].makeEl(null,true);
 	setWAttrs(this.wdgs[i].addr,'pgOpenSrc',pgId);
 	return true;
       }
-      if( this.wdgs[i].inclOpen && this.wdgs[i].pages[this.wdgs[i].inclOpen].callPage(pgId,updWdg,pgGrp,pgOpenSrc) ) return true;
+      if(this.wdgs[i].inclOpen && this.wdgs[i].pages[this.wdgs[i].inclOpen].callPage(pgId,updWdg,pgGrp,pgOpenSrc)) return true;
     }
   //> Put checking to child pages
-  for( var i in this.pages )
-    if( this.pages[i].callPage(pgId,updWdg,pgGrp,pgOpenSrc) ) return true;
+  for(var i in this.pages)
+    if(this.pages[i].callPage(pgId,updWdg,pgGrp,pgOpenSrc)) return true;
   //> Check for open child page or for unknown and empty source pages open as master page child windows
-  if( (!pgGrp.length && pgOpenSrc == this.addr) || this == masterPage )
+  if((!pgGrp.length && pgOpenSrc == this.addr) || this == masterPage)
   {
     var iPg = new pwDescr(pgId,true,this);
-    iPg.window = window.open('',pgId,'width=600,height=400,directories=no,menubar=no,toolbar=no,scrollbars=yes,dependent=yes,location=no,locationbar=no,status=no,statusbar=no,alwaysRaised=yes');
-    if( !iPg.window ) return true;
-    iPg.window.document.open( );
-    iPg.window.document.write("<html><body style='background-color: #E6E6E6;'><div id='main'/></body></html>\n");
-    iPg.window.document.close( );
-    var mainDiv = iPg.window.document.getElementById('main');
-    iPg.place = mainDiv;
-    this.pages[pgId] = iPg;
-    iPg.makeEl( servGet(pgId,'com=attrsBr') );
+    var attrBrVal = servGet(pgId,'com=attrsBr');
+
+    var winName = null;
+    var winWidth = 600;
+    var winHeight = 400;
+    for(var i_ch = 0; i_ch < attrBrVal.childNodes.length; i_ch++)
+      if(attrBrVal.childNodes[i_ch].nodeName != 'el' ) continue;
+      else if(attrBrVal.childNodes[i_ch].getAttribute('id') == 'name') winName = nodeText(attrBrVal.childNodes[i_ch]);
+      else if(attrBrVal.childNodes[i_ch].getAttribute('id') == 'geomW') winWidth = parseInt(nodeText(attrBrVal.childNodes[i_ch]));
+      else if(attrBrVal.childNodes[i_ch].getAttribute('id') == 'geomH') winHeight = parseInt(nodeText(attrBrVal.childNodes[i_ch]));
+
+    //> New external <div> window create
+    if(winWidth < parseInt(masterPage.attrs['geomW']) && winHeight < parseInt(masterPage.attrs['geomH']))
+    {
+	iPg.window = document.createElement('table');
+	iPg.window.setAttribute('cellpadding','2');
+	iPg.window.setAttribute('cellspacing','0');
+	iPg.window.setAttribute('border','0');
+	iPg.window.setAttribute('style','position: absolute; font-size: 12px; z-index: 9999; background-color: #8FA9FF; border: 1px solid darkblue; '+
+	    'left: '+((parseInt(masterPage.attrs['geomW'])-winWidth-5)/2)+'px; '+
+	    'top: '+((parseInt(masterPage.attrs['geomH'])-winHeight-18)/2)+'px; ');
+	var wRow = document.createElement('tr');
+	iPg.window.appendChild(wRow);
+	wRow.setAttribute('style','cursor: move;');
+	wRow.win = iPg.window;
+        wRow.onmousedown = function(e)
+        {
+	    if(!e) e = window.event;
+	    this.clX = e.clientX; this.clY = e.clientY;
+	    return false;
+	}
+        wRow.onmouseup = function(e)
+        {
+	    if(!e) e = window.event;
+	    this.clX = this.clY = null;
+	    return false;
+	}
+	wRow.onmouseout = wRow.onmouseup;
+	wRow.onmousemove = function(e)
+	{
+	    if(!e) e = window.event;
+	    if(this.clX != null)
+	    {
+		this.win.style.left = (parseInt(this.win.style.left)+e.clientX-this.clX)+"px";
+		this.win.style.top  = (parseInt(this.win.style.top)+e.clientY-this.clY)+"px";
+		this.clX = e.clientX; this.clY = e.clientY;
+	    }
+	}
+	var titleBlk = document.createElement('td');
+	wRow.appendChild(titleBlk);
+	titleBlk.setAttribute('style','max-width: '+(winWidth-7)+'px; width: '+(winWidth-7)+'px; overflow: hidden; white-space: nowrap;');
+	titleBlk.setAttribute('title',winName);
+	titleBlk.appendChild(document.createTextNode(winName));
+	var closeWin = document.createElement('td');
+	wRow.appendChild(closeWin);
+	closeWin.appendChild(document.createTextNode('X'));
+	closeWin.setAttribute('style','color: red; cursor: pointer;');
+	closeWin.onclick = function()
+	{ servSet(this.iPg.addr,'com=pgClose',''); document.getElementById('mainCntr').removeChild(this.iPg.window); delete this.iPg.parent.pages[this.iPg.addr]; }
+	closeWin.iPg = iPg;
+	wRow = document.createElement('tr');
+	iPg.window.appendChild(wRow);
+	var wCntCell = document.createElement('td');
+	wRow.appendChild(wCntCell);
+	wCntCell.setAttribute('colspan','2');
+	wCntCell.setAttribute('align','center');
+	iPg.place = document.createElement('div');
+	wCntCell.appendChild(iPg.place);
+	document.getElementById('mainCntr').appendChild(iPg.window);
+    }
+    //> New external window create
+    else
+    {
+	iPg.window = window.open('about:blank',pgId,'width='+(winWidth+20)+',height='+(winHeight+60)+',directories=no,menubar=no,toolbar=no,scrollbars=yes,dependent=yes,location=no,locationbar=no,status=no,statusbar=no,alwaysRaised=yes');
+	if(!iPg.window) return true;
+	iPg.window.document.open( );
+	iPg.window.document.write("<html><body style='background-color: #E6E6E6;'><center><div id='main'/></center></body></html>\n");
+	iPg.window.document.close( );
+	iPg.window.document.title = winName;
+	var mainDiv = iPg.window.document.getElementById('main');
+	iPg.place = mainDiv;
+	iPg.windowExt = true;
+    }
+
+    this.pages[iPg.addr] = iPg;
+    iPg.makeEl(attrBrVal,false,true);
+
     return true;
   }
 
   return false;
 }
+
 function findOpenPage( pgId )
 {
   var opPg;
-  if( pgId == this.addr ) return this;
+  if(pgId == this.addr) return this;
   //> Check from included widgets
-  for( var i in this.wdgs )
-    if( this.wdgs[i].attrs['root'] == 'Box' )
+  for(var i in this.wdgs)
+    if(this.wdgs[i].attrs['root'] == 'Box' && this.wdgs[i].isVisible)
     {
-      if( pgId == this.wdgs[i].attrs['pgOpenSrc'] ) return this.wdgs[i].pages[pgId];
-      if( this.wdgs[i].inclOpen )
+      if(pgId == this.wdgs[i].attrs['pgOpenSrc']) return this.wdgs[i].pages[pgId];
+      if(this.wdgs[i].inclOpen)
       {
 	opPg = this.wdgs[i].pages[this.wdgs[i].inclOpen].findOpenPage(pgId);
-	if( opPg ) return opPg;
+	if(opPg) return opPg;
       }
     }
   //> Put checking to child pages
-  for( var i in this.pages )
+  for(var i in this.pages)
   {
     opPg = this.pages[i].findOpenPage(pgId);
-    if( opPg ) return opPg;
+    if(opPg) return opPg;
   }
   return null;
 }
-function makeEl( pgBr, inclPg )
+
+function makeEl( pgBr, inclPg, full, FullTree )
 {
   var margBrdUpd = false; var newAttr = false;
   this.place.wdgLnk = this;
   if( !inclPg && pgBr )
     for( var j = 0; j < pgBr.childNodes.length; j++ )
     {
-      if( pgBr.childNodes[j].nodeName != 'el' ) continue;
+      if(pgBr.childNodes[j].nodeName != 'el') continue;
       var i = pgBr.childNodes[j].getAttribute('id');
-      if( (i == 'bordWidth' || i == 'geomMargin') && this.attrs[i] != nodeText(pgBr.childNodes[j]) ) margBrdUpd = true;
+      if((i == 'bordWidth' || i == 'geomMargin') && this.attrs[i] != nodeText(pgBr.childNodes[j])) margBrdUpd = true;
       this.attrs[i] = nodeText(pgBr.childNodes[j]);
       newAttr = true;
     }
@@ -431,44 +557,51 @@ function makeEl( pgBr, inclPg )
   {
   var elMargin = parseInt(this.attrs['geomMargin']);
   var elBorder = 0;
-  if( this.attrs['bordWidth'] ) elBorder=parseInt(this.attrs['bordWidth']);
+  if(this.attrs['bordWidth']) elBorder = parseInt(this.attrs['bordWidth']);
   var elStyle = '';
-  if( !(parseInt(this.attrs['en']) && (this.pg || parseInt(this.attrs['perm'])&SEC_RD)) ) elStyle+='visibility : hidden; ';
+  this.isVisible = true;
+  if(!(parseInt(this.attrs['en']) && (this.pg || parseInt(this.attrs['perm'])&SEC_RD)))
+  { elStyle += 'visibility : hidden; '; this.isVisible = false; }
   var geomX = parseFloat(this.attrs['geomX']);
   var geomY = parseFloat(this.attrs['geomY']);
-  if( this.pg ) geomX = geomY = 0;
+  if( this.pg ) { geomX = geomY = 0; elStyle += 'overflow: hidden; '; }
   else{ geomX *= this.parent.xScale(true); geomY *= this.parent.yScale(true); }
   if( this.parent && !(this.pg && this.parent.pg) )
   {
     geomX -= parseInt(this.parent.attrs['geomMargin'])+parseInt(this.parent.attrs['bordWidth']);
     geomY -= parseInt(this.parent.attrs['geomMargin'])+parseInt(this.parent.attrs['bordWidth']);
   }
-  elStyle+='position: absolute; left: '+realRound(geomX)+'px; top: '+realRound(geomY)+'px; ';
-  var geomW = parseFloat(this.attrs['geomW'])-2*(elMargin+elBorder);
-  var geomH = parseFloat(this.attrs['geomH'])-2*(elMargin+elBorder);
-
-  if( this.pg && this.parent && this.parent.inclOpen && this.parent.inclOpen == this.addr )
-  {
-    elStyle += 'overflow: auto; ';
-    geomW = parseFloat(this.parent.attrs['geomW'])-2*(elMargin+elBorder);
-    geomH = parseFloat(this.parent.attrs['geomH'])-2*(elMargin+elBorder);
-  }
-  else elStyle += 'overflow: hidden; ';
+  elStyle+='position: '+((this==masterPage || this.window)?'relative':'absolute')+'; left: '+realRound(geomX)+'px; top: '+realRound(geomY)+'px; ';
 
   var xSc = this.xScale(true);
   var ySc = this.yScale(true);
-  geomW = realRound(geomW*xSc); geomH = realRound(geomH*ySc);
+  var geomW = realRound(parseFloat(this.attrs['geomW'])*xSc);
+  var geomH = realRound(parseFloat(this.attrs['geomH'])*ySc);
+
+  if(this.pg && this.parent && this.parent.inclOpen && this.parent.inclOpen == this.addr)
+  {
+    var geomWpar = parseFloat(this.parent.attrs['geomW'])*this.parent.xScale(true);
+    var geomHpar = parseFloat(this.parent.attrs['geomH'])*this.parent.yScale(true);
+    this.parent.place.style.overflow = (geomW > geomWpar || geomH > geomHpar) ? 'scroll' : 'visible';
+    //elStyle += 'overflow: auto; ';
+    geomW = Math.max(geomW, geomWpar);
+    geomH = Math.max(geomH, geomHpar);
+  }
+  //else elStyle += 'overflow: hidden; ';
+
+  geomW -= 2*(elMargin+elBorder);
+  geomH -= 2*(elMargin+elBorder);
 
   this.mousedown = new Array();
   this.mouseup = new Array();
 
   //> Set included window geometry to widget size
 //  if( this == masterPage ) resizeTo(geomW,geomH);
-  if( this.pg && this.window )
+  if(this.pg && this.window && this.windowExt)
   {
-    var pgWin = this.window;
-    pgWin.document.body.style.backgroundColor = getColor(this.attrs['backColor']);
-    pgWin.resizeTo(geomW+20,geomH+40);
+    if(this.window.innerHeight)
+      this.window.resizeTo(geomW+(this.window.outerWidth-this.window.innerWidth)+20,geomH+(this.window.outerHeight-this.window.innerHeight)+20);
+    else this.window.resizeTo(geomW+20,geomH+40);
   }
 
   if( parseInt(this.attrs['focus']) ) setFocus(this.addr,true);
@@ -477,8 +610,8 @@ function makeEl( pgBr, inclPg )
   {
     if( this.pg )
     {
-      elStyle+='background-color: #818181; border: 1px solid black; color: red; overflow: auto; ';
-      this.place.innerHTML = "<div class='vertalign' style='width: "+(geomW-2)+"px; height: "+(geomH-2)+"px;'>Page: '"+this.addr+"'.<br/>View access is no permited.</div>";
+      elStyle+='background-color: #B0B0B0; border: 1px solid black; color: red; overflow: auto; ';
+      this.place.innerHTML = "<div class='vertalign' style='width: "+(geomW-2)+"px; height: "+(geomH-2)+"px;'>Page: '"+this.addr+"'.<br/>View access is no permitted.</div>";
     }
   }
   else if( this.attrs['root'] == 'ElFigure' )
@@ -523,12 +656,13 @@ function makeEl( pgBr, inclPg )
   }
   else if( this.attrs['root'] == 'Box' )
   {
-    if( this.attrs['backColor'] ) elStyle+='background-color: '+getColor(this.attrs['backColor'])+'; ';
-    if( this.attrs['backImg'] )   elStyle+='background-image: url(\'/'+MOD_ID+this.addr+'?com=res&val='+this.attrs['backImg']+'\'); ';
-     elStyle+='border-style: solid; border-width: '+this.attrs['bordWidth']+'px; ';
-     if( this.attrs['bordColor'] ) elStyle+='border-color: '+getColor(this.attrs['bordColor'])+'; ';
-     switch( parseInt(this.attrs['bordStyle']) )
-     {
+    if(this == masterPage && this.attrs['tipStatus'].length) setStatus(this.attrs['tipStatus'],10000);
+    if(this.attrs['backColor']) elStyle += 'background-color: '+getColor(this.attrs['backColor'])+'; ';
+    if(this.attrs['backImg'])   elStyle += 'background-image: url(\'/'+MOD_ID+this.addr+'?com=res&val='+this.attrs['backImg']+'\'); ';
+      elStyle+='border-style: solid; border-width: '+this.attrs['bordWidth']+'px; ';
+    if(this.attrs['bordColor']) elStyle += 'border-color: '+getColor(this.attrs['bordColor'])+'; ';
+    switch(parseInt(this.attrs['bordStyle']))
+    {
 	case 1: elStyle+='border-style: dotted; '; break;
 	case 2: elStyle+='border-style: dashed; '; break;
 	case 3: elStyle+='border-style: solid; ';  break;
@@ -538,9 +672,10 @@ function makeEl( pgBr, inclPg )
 	case 7: elStyle+='border-style: inset; ';  break;
 	case 8: elStyle+='border-style: outset; '; break;
     }
-    if( !this.pg && ((this.inclOpen && this.attrs['pgOpenSrc'] != this.inclOpen) || (!this.inclOpen && this.attrs['pgOpenSrc'].length)) )
+    if(!this.pg && ((this.inclOpen && this.attrs['pgOpenSrc'] != this.inclOpen) || (!this.inclOpen && this.attrs['pgOpenSrc'].length)))
     {
-      if( this.inclOpen )
+      //elStyle += 'overflow: auto; ';
+      if(this.inclOpen)
       {
 	servSet(this.inclOpen,'com=pgClose','');
 	pgCache[this.inclOpen] = this.pages[this.inclOpen];
@@ -566,7 +701,7 @@ function makeEl( pgBr, inclPg )
         {
 	  var iPg = new pwDescr(this.inclOpen,true,this);
 	  iPg.place = this.place.ownerDocument.createElement('div');
-	  iPg.makeEl( servGet(this.inclOpen,'com=attrsBr') );
+	  iPg.makeEl(servGet(this.inclOpen,'com=attrsBr'),false,true);
 	  this.pages[this.inclOpen] = iPg;
 	  this.place.appendChild(iPg.place);
         }
@@ -616,6 +751,7 @@ function makeEl( pgBr, inclPg )
 	  case 1:
 	    argCfg=this.attrs['arg'+i+'cfg'].split(';');
 	    if( argCfg[1] == 'g' )      argVal=parseFloat(this.attrs['arg'+i+'val']).toPrecision(argCfg[2]);
+	    else if( argCfg[1] == 'e' ) argVal=parseFloat(this.attrs['arg'+i+'val']).toExponential(argCfg[2]);
 	    else if( argCfg[1] == 'f' ) argVal=parseFloat(this.attrs['arg'+i+'val']).toFixed(argCfg[2]);
 	    else argVal=this.attrs['arg'+i+'val'];
 	    break;
@@ -643,17 +779,21 @@ function makeEl( pgBr, inclPg )
   }
   else if( this.attrs['root'] == 'Media' )
   {
-    if( this.attrs['backColor'] ) elStyle+='background-color: '+getColor(this.attrs['backColor'])+'; ';
-    if( this.attrs['backImg'] )
-      elStyle+='background-image: url(\'/'+MOD_ID+this.addr+'?com=res&val='+this.attrs['backImg']+'\'); ';
-    elStyle+='border-style: solid; border-width: '+this.attrs['bordWidth']+'px; ';
-    if( this.attrs['bordColor'] ) elStyle+='border-color: '+getColor(this.attrs['bordColor'])+'; ';
+    if(this.attrs['backColor']) elStyle += 'background-color: '+getColor(this.attrs['backColor'])+'; ';
+    if(this.attrs['backImg'])
+      elStyle += 'background-image: url(\'/'+MOD_ID+this.addr+'?com=res&val='+this.attrs['backImg']+'\'); ';
+    elStyle += 'border-style: solid; border-width: '+this.attrs['bordWidth']+'px; ';
+    if(this.attrs['bordColor']) elStyle += 'border-color: '+getColor(this.attrs['bordColor'])+'; ';
     while(this.place.childNodes.length) this.place.removeChild(this.place.childNodes[0]);
     var medObj = this.place.ownerDocument.createElement('img');
     medObj.wdgLnk = this;
     medObj.src = this.attrs['src'].length ? ('/'+MOD_ID+this.addr+'?com=res&val='+this.attrs['src']) : '';
     medObj.border = 0;
-    if( this.attrs['fit'] == 1 ) { medObj.width = geomW; medObj.height = geomH; }
+    if(this.attrs['fit'] == 1)
+    {
+	medObj.width = geomW; medObj.height = geomH;
+	if(this.attrs['src'].length) medObj.src += "&size="+geomH;
+    }
     else medObj.onload = function()
     {
 	var cWdth = this.width; var cHeight = this.height;
@@ -711,7 +851,7 @@ function makeEl( pgBr, inclPg )
 	this.place.appendChild(formObj);
 	var geomWint = geomW-4;
 	formObj.wdgLnk = this;
-	formObj.style.cssText = 'position: absolute; top: '+((geomH-20)/2)+'px; width: '+geomWint+'px; height: '+Math.min(geomH,16)+'px; border: 1px solid black; font: '+fontCfg+'; padding: 1px;';
+	formObj.style.cssText = 'position: absolute; left: 0px; top: '+((geomH-20)/2)+'px; width: '+geomWint+'px; height: '+Math.min(geomH,16)+'px; border: 1px solid black; font: '+fontCfg+'; padding: 1px;';
 	this.place.view = parseInt(this.attrs['view']);
 	this.place.cfg = this.attrs['cfg'];
 	if( parseInt(this.attrs['active']) && parseInt(this.attrs['perm'])&SEC_WR )
@@ -1042,10 +1182,11 @@ function makeEl( pgBr, inclPg )
 	  formObj = this.place.ownerDocument.createElement('div');
 	  formObj.className = 'vertalign';
 	  formObj.style.font = fontCfg;
-	  elStyle+='border-style: '+((this.place.checkable && parseInt(this.attrs['value']))?'inset':'outset')+'; cursor: pointer; border-width: 2px; ';
-	  if( this.attrs['colorText'] ) elStyle+='color: '+getColor(this.attrs['colorText'])+'; ';
-	  if( this.attrs['color'] ) elStyle+='background-color: '+getColor(this.attrs['color'])+'; ';
-	  else elStyle+='background-color: snow; ';
+	  elStyle += 'border-style: '+((this.place.checkable && parseInt(this.attrs['value']))?'inset; ':'outset; ')+
+				      (parseInt(this.attrs['active'])?'cursor: pointer; ':'')+'border-width: 2px; ';
+	  if(this.attrs['colorText']) elStyle += 'color: '+(parseInt(this.attrs['active']) ? getColor(this.attrs['colorText']) : 'silver')+'; ';
+	  if(parseInt(this.attrs['active']) && this.attrs['color']) elStyle += 'background-color: '+getColor(this.attrs['color'])+'; ';
+	  else elStyle += 'background-color: snow; ';
 	  if(parseInt(this.attrs['active']) && parseInt(this.attrs['perm'])&SEC_WR)
 	  {
 	    this.mouseup[this.mouseup.length] = function(e,el)
@@ -1077,17 +1218,29 @@ function makeEl( pgBr, inclPg )
 	    this.place.wdgLnk = this;
 	  }
 	  var txtVal1 = '';
-	  if( iconImg )
-	    txtVal1 += "<IMG src='/"+MOD_ID+this.addr+"?com=res&val="+this.attrs['img']+"' "+
-			"width='"+Math.min(geomW-8,geomH-8)+"' height='"+Math.min(geomW-8,geomH-8)+"' float='left'/>";
-	  if( this.attrs['name'].length )
+	  if(iconImg)
+	    txtVal1 += "<IMG src='/"+MOD_ID+this.addr+"?com=res&val="+this.attrs['img']+"&size="+Math.min(geomW-6,geomH-6)+
+				  (!parseInt(this.attrs['active'])?"&filtr=unact":"")+"' "+
+			"width='"+Math.min(geomW-6,geomH-6)+"' height='"+Math.min(geomW-6,geomH-6)+"' float='left' "+
+			(this.attrs['name'].length?"style='margin-right: 2px'":"")+"/>";
+	  if(this.attrs['name'].length)
 	  {
-	    for( var j = 0; j < this.attrs['name'].length; j++ )
-		if( this.attrs['name'].substr(j,2) == '\\n' ) { txtVal1+='<br />'; j++; }
-		else txtVal1+=this.attrs['name'][j];
+	    var begBlk = true;
+	    for(var j = 0; j < this.attrs['name'].length; j++)
+		if(this.attrs['name'].substr(j,2) == '\\n')
+		{
+		    if(!begBlk) { txtVal1 += "</span>"; begBlk = true; }
+		    txtVal1 += '<br />';
+		}
+		else
+		{
+		    if(begBlk) { txtVal1 += "<span>"; begBlk = false; }
+		    txtVal1 += strEncode(this.attrs['name'][j]);
+		}
 	  }
+	  if(!begBlk) txtVal1 += "</span>";
 	  formObj.innerHTML = txtVal1;
-	  geomW-=6; geomH-=6;
+	  geomW -= 4; geomH -= 4;
 	}
 	else
 	{
@@ -1110,7 +1263,7 @@ function makeEl( pgBr, inclPg )
 	var formObj = this.place.ownerDocument.createElement('select');
 	formObj.disabled = !( parseInt(this.attrs['active']) && parseInt(this.attrs['perm'])&SEC_WR );
 	formObj.tabIndex = parseInt(this.attrs['geomZ'])+1;
-	formObj.style.cssText = 'position: absolute; top: '+((elTp==4)?(geomH-20)/2:0)+'px; height: '+((elTp==4)?Math.min(geomH,20):(geomH-4))+'px; width: '+(geomW-4)+'px; border: 1px solid black; font: '+fontCfg+'; padding: 1px; ';
+	formObj.style.cssText = 'position: absolute; left: 0px; top: '+((elTp==4)?(geomH-20)/2:0)+'px; height: '+((elTp==4)?Math.min(geomH,20):(geomH-4))+'px; width: '+(geomW-4)+'px; border: 1px solid black; font: '+fontCfg+'; padding: 1px; ';
 	formObj.wdgLnk = this;
 //	if( elTp == 5 ) formObj.setAttribute('size',100);
 	if( elTp == 4 )
@@ -1183,8 +1336,8 @@ function makeEl( pgBr, inclPg )
     if( this.attrs['backColor'] ) elStyle += 'background-color: '+getColor(this.attrs['backColor'])+'; ';
     else elStyle+='background-color: white; ';
     if( this.attrs['backImg'] )   elStyle += 'background-image: url(\'/'+MOD_ID+this.addr+'?com=res&val='+this.attrs['backImg']+'\'); ';
-    elStyle += 'border: 1px solid black; overflow: auto; padding: 2px; ';
-    geomW -= 4; geomH -= 4;
+    elStyle += 'border: 1px solid black; overflow: auto; padding: 2px; text-align: left; ';
+    geomW -= 6; geomH -= 6;
 
     this.wFont = getFont(this.attrs['font'],Math.min(xSc,ySc));
 
@@ -1417,7 +1570,7 @@ function makeEl( pgBr, inclPg )
   elStyle+='width: '+geomW+'px; height: '+geomH+'px; z-index: '+this.attrs['geomZ']+'; margin: '+elMargin+'px; ';
   this.place.style.cssText = elStyle;
   }
-  if( margBrdUpd ) for( var i in this.wdgs ) this.wdgs[i].makeEl();
+  if(margBrdUpd) for(var i in this.wdgs) this.wdgs[i].makeEl();
   this.place.setAttribute('title',this.attrs['tipTool']);
   this.place.onmouseover = function() { if( this.wdgLnk.attrs['tipStatus'] ) setStatus(this.wdgLnk.attrs['tipStatus'],10000); };
 
@@ -1474,14 +1627,14 @@ function makeEl( pgBr, inclPg )
   else this.place.oncontextmenu = null;
 
   //> Common mouse events process
-  if( this.mousedown.length )
+  if(this.mousedown && this.mousedown.length )
     this.place.onmousedown = function(e)
     {
       for( var i_on = 0; i_on < this.wdgLnk.mousedown.length; i_on++ )
 	this.wdgLnk.mousedown[i_on](e,this);
       return true;
     }
-  if( this.mouseup.length )
+  if(this.mouseup && this.mouseup.length)
     this.place.onmouseup = function(e)
     {
       for( var i_on = 0; i_on < this.wdgLnk.mouseup.length; i_on++ )
@@ -1489,20 +1642,36 @@ function makeEl( pgBr, inclPg )
       return true;
     }
 
-  //> Child widgets process
-  if( pgBr && !inclPg && parseInt(this.attrs['perm'])&SEC_RD )
-    for( var j = 0; j < pgBr.childNodes.length; j++ )
+  //> Delete child widgets check
+  if(FullTree)
+    for(var i in this.wdgs)
     {
-      if( pgBr.childNodes[j].nodeName != 'w' ) continue;
+	var j;
+	for(j = 0; j < pgBr.childNodes.length; j++)
+	    if(pgBr.childNodes[j].nodeName == 'w' && pgBr.childNodes[j].getAttribute('id') == i)
+		break;
+	if(j >= pgBr.childNodes.length)
+	{
+	    this.wdgs[i].place.parentNode.removeChild(this.wdgs[i].place);
+	    delete this.wdgs[i];
+	}
+    }
+
+  //> Child widgets process
+  if(pgBr && !inclPg && parseInt(this.attrs['perm'])&SEC_RD)
+    for(var j = 0; j < pgBr.childNodes.length; j++)
+    {
+      if(pgBr.childNodes[j].nodeName != 'w') continue;
       var chEl = pgBr.childNodes[j].getAttribute('id');
-      if( this.wdgs[chEl] ) this.wdgs[chEl].makeEl(pgBr.childNodes[j]);
+      if(this.wdgs[chEl]) this.wdgs[chEl].makeEl(pgBr.childNodes[j], false, full, FullTree);
       else
       {
 	var wdgO = new pwDescr(this.addr+'/wdg_'+chEl,false,this);
 	wdgO.place = this.place.ownerDocument.createElement('div');
-	wdgO.makeEl(pgBr.childNodes[j]);
 	this.place.appendChild(wdgO.place);
 	this.wdgs[chEl] = wdgO;
+	wdgO.makeEl(full?pgBr.childNodes[j]:servGet(wdgO.addr,'com=attrsBr'));
+	//wdgO.makeEl(pgBr.childNodes[j]);		//!!!! Need full
       }
     }
 }
@@ -1513,13 +1682,13 @@ function perUpdtEn( en )
      if( en && this.isEnabled() && !perUpdtWdgs[this.addr] && parseInt(this.attrs['trcPer']) ) perUpdtWdgs[this.addr] = this;
      if( !en && perUpdtWdgs[this.addr] ) delete perUpdtWdgs[this.addr];
   }
-  else if( this.attrs['root'] == 'Document' || this.attrs['root'] == 'FormEl' )
+  else if(this.attrs['root'] == 'Document' || this.attrs['root'] == 'FormEl')
   { if(en) perUpdtWdgs[this.addr] = this; else delete perUpdtWdgs[this.addr]; }
   for( var i in this.wdgs ) this.wdgs[i].perUpdtEn(en);
 }
 function perUpdt( )
 {
-  if( this.attrs['root'] == 'FormEl' && this.place.childNodes[0].clearTm && (--this.place.childNodes[0].clearTm) <= 0 )
+  if( this.attrs['root'] == 'FormEl' && this.place.childNodes.length && this.place.childNodes[0].clearTm && (--this.place.childNodes[0].clearTm) <= 0 )
     this.place.childNodes[0].chEscape();
   else if( this.attrs['root'] == 'Diagram' && (this.updCntr % parseInt(this.attrs['trcPer'])) == 0 )
   {
@@ -1564,19 +1733,19 @@ function perUpdt( )
 }
 function xScale( full )
 {
-  var rez = parseFloat(this.attrs['geomXsc'])
-  if( !full ) return rez;
-  if( !this.pg ) return rez*this.parent.xScale(full);
-  if( this != masterPage ) return masterPage.xScale()*rez;
-  return rez;
+    var rez = parseFloat(this.attrs['geomXsc'])
+    if(!full) return rez;
+    if(!this.pg) return rez*this.parent.xScale(full);
+    // if(this != masterPage) return masterPage.xScale()*rez;
+    return rez;
 }
 function yScale( full )
 {
-  var rez = parseFloat(this.attrs['geomYsc'])
-  if( !full ) return rez;
-  if( !this.pg ) return rez*this.parent.yScale(full);
-  if( this != masterPage ) return masterPage.yScale()*rez;
-  return rez;
+    var rez = parseFloat(this.attrs['geomYsc'])
+    if(!full) return rez;
+    if(!this.pg) return rez*this.parent.yScale(full);
+    // if(this != masterPage) return masterPage.yScale()*rez;
+    return rez;
 }
 function isEnabled( )
 {
@@ -1584,6 +1753,10 @@ function isEnabled( )
   if( !rez || this.pg ) return rez;
   return this.parent.isEnabled();
 }
+
+/***************************************************
+ * pwDescr - Page/widget descriptor object         *
+ ***************************************************/
 function pwDescr( pgAddr, pg, parent )
 {
   this.addr = pgAddr;
@@ -1593,6 +1766,7 @@ function pwDescr( pgAddr, pg, parent )
   this.pg = pg;
   this.parent = parent;
   this.window = null;
+  this.windowExt = false;
   this.place = null;
   this.callPage = callPage;
   this.findOpenPage = findOpenPage;
@@ -1609,20 +1783,26 @@ function pwDescr( pgAddr, pg, parent )
  ***************************************************/
 function makeUI()
 {
+  prcCnt++;
   //> Get open pages list
   var pgNode = servGet('/'+sessId,'com=pgOpenList&tm='+tmCnt);
-  if( pgNode )
+  if(pgNode)
   {
     //>> Check for delete pages
-    for( var i_p = 0; i_p < pgList.length; i_p++ )
+    for(var i_p = 0; i_p < pgList.length; i_p++)
     {
       var opPg; var i_ch;
-      for( i_ch = 0; i_ch < pgNode.childNodes.length; i_ch++ )
-	if( pgNode.childNodes[i_ch].nodeName == 'pg' && nodeText(pgNode.childNodes[i_ch]) == pgList[i_p] )
+      for(i_ch = 0; i_ch < pgNode.childNodes.length; i_ch++)
+	if(pgNode.childNodes[i_ch].nodeName == 'pg' && nodeText(pgNode.childNodes[i_ch]) == pgList[i_p])
 	  break;
-      if( i_ch < pgNode.childNodes.length || !(opPg=masterPage.findOpenPage(pgList[i_p])) ) continue;
-      if( opPg.window ) { opPg.window.close(); delete opPg.parent.pages[pgList[i_p]]; }
-      else if( opPg.parent && opPg.parent.inclOpen && opPg.parent.inclOpen == pgList[i_p] )
+      if(i_ch < pgNode.childNodes.length || !(opPg=masterPage.findOpenPage(pgList[i_p]))) continue;
+      if(opPg.window)
+      {
+        if(opPg.windowExt) opPg.window.close();
+        else document.getElementById('mainCntr').removeChild(opPg.window);
+        delete opPg.parent.pages[pgList[i_p]];
+      }
+      else if(opPg.parent && opPg.parent.inclOpen && opPg.parent.inclOpen == pgList[i_p])
       { opPg.parent.attrs['pgOpenSrc'] = ''; opPg.parent.makeEl(null,true); }
     }
     //>> Process opened pages
@@ -1633,7 +1813,7 @@ function makeUI()
 	var prPath = nodeText(pgNode.childNodes[i]);
 	//>>> Check for closed window
 	var opPg = masterPage.findOpenPage(prPath);
-	if( opPg && opPg.window && opPg.window.closed )
+	if(opPg && opPg.window && opPg.windowExt && opPg.window.closed)
 	{ servSet(prPath,'com=pgClose',''); delete opPg.parent.pages[prPath]; continue; }
 	//>>> Call page
 	pgList.push(prPath);
@@ -1642,7 +1822,7 @@ function makeUI()
     tmCnt = parseInt(pgNode.getAttribute('tm'));
   }
   //> Update some widgets
-  for( var i in perUpdtWdgs ) perUpdtWdgs[i].perUpdt();
+  for(var i in perUpdtWdgs) perUpdtWdgs[i].perUpdt();
   setTimeout(makeUI,1000);
 }
 
@@ -1651,10 +1831,10 @@ function makeUI()
  ***************************************************/
 function setStatus( mess, tm )
 {
-  window.status = mess ? mess : '###Ready###';
-  if( !mess ) return;
-  if( stTmID ) clearTimeout(stTmID);
-  if( !tm || tm > 0 ) stTmID = setTimeout('setStatus(null)',tm?tm:1000);
+    window.status = mess ? mess : '###Ready###';
+    if(!mess) return;
+    if(stTmID) clearTimeout(stTmID);
+    if(!tm || tm > 0) stTmID = setTimeout('setStatus(null)',tm?tm:1000);
 }
 
 /**************************************************
@@ -1693,6 +1873,7 @@ document.body.onmouseup = function(e)
   if( popUpMenu ) popUpMenu.style.visibility = 'hidden';
 }
 
+prcCnt = 0;				//Process counter
 tmCnt = 0;				//Call counter
 pgList = new Array();			//Opened pages list
 pgCache = new Object();			//Cached pages' data
