@@ -35,8 +35,8 @@
 #define MOD_TYPE	SUI_ID
 #define VER_TYPE	SUI_VER
 #define SUB_TYPE	"WWW"
-#define MOD_VER		"0.9.1"
-#define AUTORS		_("Roman Savochenko")
+#define MOD_VER		"1.0.0"
+#define AUTHORS		_("Roman Savochenko")
 #define DEVELOPERS	_("Roman Savochenko, Lysenko Maxim, Yashina Kseniya")
 #define DESCRIPTION	_("Web operation user interface for visual control area (VCA) projects playing.")
 #define LICENSE		"GPL2"
@@ -72,7 +72,7 @@ TWEB::TWEB( string name ) : TUI(MOD_ID), mTSess(10), mSessLimit(5), chck_st(fals
     mName	= MOD_NAME;
     mType	= MOD_TYPE;
     mVers	= MOD_VER;
-    mAutor	= AUTORS;
+    mAuthor	= AUTHORS;
     mDescr	= DESCRIPTION;
     mLicense	= LICENSE;
     mSource	= name;
@@ -248,7 +248,7 @@ TWEB::TWEB( string name ) : TUI(MOD_ID), mTSess(10), mSessLimit(5), chck_st(fals
     //> Default CSS init
     mCSStables =
 	"hr { width: 95%; }\n"
-	"body { background-color: #818181; margin: 0px; }\n"
+	"body { background-color: #B0B0B0; margin: 0px; }\n"
 	"h1.head { text-align:center; color:#ffff00; }\n"
 	"h2.title { text-align:center; font-style: italic; margin: 0px; padding: 0px; border-width:0; }\n"
 	"table.work { background-color: #9999ff; border: 3px ridge #a9a9a9; padding: 2px; }\n"
@@ -262,7 +262,8 @@ TWEB::TWEB( string name ) : TUI(MOD_ID), mTSess(10), mSessLimit(5), chck_st(fals
 	"table.prot td { border: 1px solid black; white-space: nowrap; }\n"
 	"table.prot td.hd { background-color: #E6E6E6; font-weight: bold; text-align: center; }\n"
 	"#popupmenu { position: absolute; border: 0; width: 150px; height: 150px; overflow : auto; z-index: 1000; }\n"
-	"#popupmenu select { background-color: #E6E6E6; border: 1px solid black; padding: 1px; }\n";
+	"#popupmenu select { background-color: #E6E6E6; border: 1px solid black; padding: 1px; }\n"
+	"input[type=\"checkbox\"] { margin: 0 4px 0 0; float: left; }\n";
 
 #if 0
     char mess[][100] = { _("Date and time"), _("Level"), _("Category"), _("Message"), _("mcsec"), _("Ready") };
@@ -302,7 +303,7 @@ string TWEB::optDescr( )
 
     snprintf(buf,sizeof(buf),_(
 	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n"
+	"---------- Parameters of the module section '%s' in config-file ----------\n"
 	"SessTimeLife <time>      Time of the session life, minutes (default 10).\n\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
 
@@ -331,7 +332,7 @@ void TWEB::load_( )
 	}
     } while(next_opt != -1);
 
-    //> Load parameters from config file
+    //> Load parameters from config-file
     setSessTime(atoi(TBDS::genDBGet(nodePath()+"SessTimeLife",TSYS::int2str(sessTime())).c_str()));
     setSessLimit(atoi(TBDS::genDBGet(nodePath()+"SessLimit",TSYS::int2str(sessLimit())).c_str()));
 }
@@ -391,12 +392,12 @@ void TWEB::TaskSessCheck( union sigval obj )
 
 string TWEB::httpHead( const string &rcode, int cln, const string &cnt_tp, const string &addattr )
 {
-    return "HTTP/1.0 "+rcode+"\r\n"
-	"Server: "+PACKAGE_STRING+"\r\n"
-	"Accept-Ranges: bytes\r\n"
-	"Content-Length: "+TSYS::int2str(cln)+"\r\n"
-	"Connection: close\r\n"
-	"Content-Type: "+cnt_tp+"; charset="+Mess->charset()+"\r\n"+addattr+"\r\n";
+    return "HTTP/1.0 "+rcode+"\x0D\x0A"
+	"Server: "+PACKAGE_STRING+"\x0D\x0A"
+	"Accept-Ranges: bytes\x0D\x0A"
+	"Content-Length: "+TSYS::int2str(cln)+"\x0D\x0A"
+	"Connection: close\x0D\x0A"
+	"Content-Type: "+cnt_tp+"; charset="+Mess->charset()+"\x0D\x0A"+addattr+"\x0D\x0A";
 }
 
 string TWEB::pgHead( const string &head_els, const string &title )
@@ -526,7 +527,7 @@ void TWEB::HttpGet( const string &url, string &page, const string &sender, vecto
 		    vector<string> vcaLs;
 		    vcaSesList(vcaLs);
 		    if((int)vcaLs.size() >= mod->sessLimit())
-		        messPost(ses.page,nodePath(),_("Sorry, openned sessions number reach limit!"),TWEB::Warning);
+		        messPost(ses.page,nodePath(),_("Sorry, opened sessions number reach limit!"),TWEB::Warning);
 		    else
 		    {
 			sesRes.request(true);
@@ -572,7 +573,7 @@ void TWEB::HttpGet( const string &url, string &page, const string &sender, vecto
 		page = ses.page;
 		return;
 	    }
-            else mess_err(nodePath().c_str(),_("An unaccessible request is received: '%s'"),zero_lev.c_str());
+            else mess_err(nodePath().c_str(),_("An inaccessible request is received: '%s'"),zero_lev.c_str());
 	}
     }catch(TError err)
     {
@@ -601,41 +602,13 @@ void TWEB::getAbout( SSess &ses )
 	"<TR class='content'><TD>\n"
 	"<table border='0'>\n"
 	"<TR><TD><font color='Blue'>"+_("Name: ")+"</font></TD><TD>"+_(MOD_NAME)+"</TD></TR>"
-	"<TR><TD><font color='Blue'>"+_("Desription: ")+"</font></TD><TD>"+_(DESCRIPTION)+"</TD></TR>"
+	"<TR><TD><font color='Blue'>"+_("Description: ")+"</font></TD><TD>"+_(DESCRIPTION)+"</TD></TR>"
 	"<TR><TD><font color='Blue'>"+_("License: ")+"</font></TD><TD>"+_(LICENSE)+"</TD></TR>"
-	"<TR><TD><font color='Blue'>"+_("Author: ")+"</font></TD><TD>"+_(AUTORS)+"</TD></TR>"
+	"<TR><TD><font color='Blue'>"+_("Author: ")+"</font></TD><TD>"+_(AUTHORS)+"</TD></TR>"
 	"</table>\n"
 	"</TD></TR>\n</table><br/></center>\n";
 }
 
-void TWEB::getAuth( SSess &ses )
-{
-    ses.page = ses.page+"<center><table class='page_auth'>"
-	"<tr><td><b>"+_("Enter to module")+"</b></td></tr>\n"
-	"<tr class='content'> <td align='center'>\n"
-	"<form method='post' action='/"MOD_ID+ses.url+"' enctype='multipart/form-data'>\n"
-	"<table cellpadding='3px'>\n"
-	"<tr><td><b>"+_("User name")+"</b></td><td><input type='text' name='user' size='20'/></td></tr>\n"
-	"<tr><td><b>"+_("Password")+"</b></td><td><input type='password' name='pass' size='20'/></td></tr>\n"
-	"<tr><td colspan='2' align='center'><input type='submit' name='auth_enter' value='"+_("Enter")+"'/>\n"
-	"<input type='reset' name='clean' value='"+_("Clean")+"'/>\n"
-	"</td></tr></table>\n</form>\n"
-	"</td></tr></table></center>\n";
-}
-
-string TWEB::getCookie( string name, vector<string> &vars )
-{
-    for( unsigned i_var = 0; i_var < vars.size(); i_var++)
-	if( vars[i_var].substr(0, vars[i_var].find(":",0)) == "Cookie" )
-	{
-	    size_t i_beg = vars[i_var].find(name+"=",0);
-	    if( i_beg == string::npos ) return "";
-	    i_beg += name.size()+1;
-	    return vars[i_var].substr(i_beg,vars[i_var].find(";",i_beg)-i_beg);
-	}
-    return "";
-}
-
 void TWEB::HttpPost( const string &url, string &page, const string &sender, vector<string> &vars, const string &user )
 {
     map<string,string>::iterator cntEl;
@@ -719,6 +692,69 @@ void TWEB::cntrCmdProc( XMLNode *opt )
     else TUI::cntrCmdProc(opt);
 }
 
+void TWEB::imgConvert(SSess &ses)
+{
+    map<string,string>::iterator prmEl;
+    gdImagePtr sim = NULL;
+    string itp;
+    int newImgH = 0, newImgW = 0;
+
+    if(ses.page.empty() || (ses.prm.find("size") == ses.prm.end() && ses.prm.find("filtr") == ses.prm.end()))   return;
+
+    if((sim=gdImageCreateFromPngPtr(ses.page.size(),(char*)ses.page.data())))		itp = "png";
+    else if((sim=gdImageCreateFromJpegPtr(ses.page.size(),(char*)ses.page.data())))	itp = "jpg";
+    else if((sim=gdImageCreateFromGifPtr(ses.page.size(),(char*)ses.page.data())))	itp = "gif";
+    //if(sim) gdImageAlphaBlending(sim, 0);
+
+    //>> Check for resize icon
+    if(sim && (prmEl=ses.prm.find("size")) != ses.prm.end() && (newImgH=atoi(prmEl->second.c_str())) > 0 && gdImageSY(sim) > newImgH)
+    {
+        newImgW = gdImageSX(sim)*newImgH/gdImageSY(sim);
+        gdImagePtr dim = gdImageCreateTrueColor(newImgW,newImgH);
+        gdImageAlphaBlending(dim,0);
+        gdImageFilledRectangle(dim,0,0,newImgW-1,newImgH-1,gdImageColorResolveAlpha(dim,0,0,0,127));
+        gdImageCopyResampled(dim,sim,0,0,0,0,newImgW,newImgH,gdImageSX(sim),gdImageSY(sim));
+        gdImageDestroy(sim);
+        sim = dim;
+    }
+
+    //>> Check for disable icon make
+    if(sim && (prmEl = ses.prm.find("filtr")) != ses.prm.end() && (prmEl->second == "gray" || prmEl->second == "unact"))
+    {
+        gdImagePtr dim = gdImageCreateTrueColor(gdImageSX(sim),gdImageSY(sim));
+        gdImageAlphaBlending(dim,0);
+        bool isUnAct = (prmEl->second == "unact");
+        for(int i_y = 0; i_y < gdImageSY(sim); i_y++)
+            for(int i_x = 0; i_x < gdImageSX(sim); i_x++)
+            {
+                int c = gdImageGetPixel(sim,i_x,i_y);
+                int y = (int)(0.3*gdImageRed(sim,c)+0.59*gdImageGreen(sim,c)+0.11*gdImageBlue(sim,c));
+                if(isUnAct) y = 255-(255-y)/2;
+                c = (int)gdImageColorResolveAlpha(dim,y,y,y,gdImageAlpha(sim,c));
+                gdImageSetPixel(dim,i_x,i_y,c);
+            }
+        gdImageDestroy(sim);
+        sim = dim;
+    }
+
+    //>> Save result
+    if(sim)
+    {
+        int img_sz;
+        char *img_ptr = NULL;
+        gdImageSaveAlpha(sim, 1);
+        if(itp == "png")        img_ptr = (char *)gdImagePngPtr(sim,&img_sz);
+        else if(itp == "jpg")   img_ptr = (char *)gdImageJpegPtr(sim,&img_sz,-1);
+        else if(itp == "gif")   img_ptr = (char *)gdImageGifPtr(sim,&img_sz);
+        if(img_ptr)
+        {
+            ses.page.assign(img_ptr,img_sz);
+            gdFree(img_ptr);
+        }
+        gdImageDestroy(sim);
+    }
+}
+
 int TWEB::colorParse( const string &tclr )
 {
     string clr = tclr;
@@ -794,7 +830,7 @@ SSess::SSess( const string &iurl, const string &isender, const string &iuser, ve
     //> Content parse
     string boundary;
     const char *c_bound = "boundary=";
-    const char *c_term = "\r\n";
+    const char *c_term = "\x0D\x0A";
     const char *c_end = "--";
     const char *c_fd = "Content-Disposition";
     const char *c_name = "name=\"";
