@@ -41,8 +41,6 @@ using namespace OSCADA;
 //************************************************
 //* TArchiveS                                    *
 //************************************************
-unsigned TArchiveS::max_req_mess = 3000;
-
 TArchiveS::TArchiveS( ) :
     TSubSYS(SARH_ID,"Archives",true), elMess(""), elVal(""), elAval(""), bufErr(0), mMessPer(10), prcStMess(false),
     headBuf(0), headLstread(0), mValPer(1000), mValPrior(10), prcStVal(false), endrunReqVal(false)
@@ -79,8 +77,8 @@ TArchiveS::TArchiveS( ) :
     elAval.fldAdd( new TFld("VTYPE",_("Value type"),TFld::Integer,0,"1") );
     elAval.fldAdd( new TFld("BPER",_("Buffer period (sec)"),TFld::Real,0,"9.6","1","0;10000") );
     elAval.fldAdd( new TFld("BSIZE",_("Buffer size (items)"),TFld::Integer,0,"6","100","0;1000000") );
-    elAval.fldAdd( new TFld("BHGRD",_("Buffer hard time griding"),TFld::Boolean,0,"1","1") );
-    elAval.fldAdd( new TFld("BHRES",_("Buffer high time resolution"),TFld::Boolean,0,"1","0") );
+    elAval.fldAdd( new TFld("BHGRD",_("Buffer in hard time grid"),TFld::Boolean,0,"1","1") );
+    elAval.fldAdd( new TFld("BHRES",_("Buffer in high time resolution"),TFld::Boolean,0,"1","0") );
     elAval.fldAdd( new TFld("ArchS",_("Process into archivators"),TFld::String,0,"500") );
 
     setMessBufLen( BUF_SIZE_DEF );
@@ -101,7 +99,7 @@ TArchiveS::~TArchiveS(  )
     timer_delete(tmIdMess);
 
     //> Stop values archiving task
-    if( prcStVal ) SYS->taskDestroy( nodePath('.',true)+".vals", &prcStVal, &endrunReqVal );
+    if(prcStVal) SYS->taskDestroy(nodePath('.',true)+".vals", &endrunReqVal);
 
     //> Free other resources
     nodeDelAll();
@@ -138,11 +136,11 @@ void TArchiveS::load_( )
     setMessPeriod( atoi(TBDS::genDBGet(nodePath()+"MessPeriod",TSYS::int2str(mMessPer)).c_str()) );
     setValPeriod( atoi(TBDS::genDBGet(nodePath()+"ValPeriod",TSYS::int2str(mValPer)).c_str()) );
     setValPrior( atoi(TBDS::genDBGet(nodePath()+"ValPriority",TSYS::int2str(mValPrior)).c_str()) );
-    max_req_mess = vmax(100,atoi(TBDS::genDBGet(nodePath()+"MaxReqMess",TSYS::int2str(max_req_mess)).c_str()));
 
     //> LidDB
     //>> Message archivators load
     string id,type;
+    map<string, bool>	itReg;
     try
     {
 	TConfig c_el(&elMess);
@@ -151,22 +149,30 @@ void TArchiveS::load_( )
 
 	//>> Search int DB and create new archivators
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_mess_proc","",fld_cnt++,c_el); )
+	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_mess_proc",nodePath()+subId()+"_mess_proc",fld_cnt++,c_el); )
 	    {
 		id = c_el.cfg("ID").getS();
 		type = c_el.cfg("MODUL").getS();
 		if(modPresent(type) && !at(type).at().messPresent(id))
 		    at(type).at().messAdd(id,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+		itReg[type+"."+id] = true;
 	    }
-	//>> Search into config file and create new archivators
-	if(SYS->chkSelDB("<cfg>"))
-	    for(int fld_cnt=0; SYS->db().at().dataSeek("",nodePath()+subId()+"_mess_proc",fld_cnt++,c_el); )
-	    {
-		id = c_el.cfg("ID").getS();
-		type = c_el.cfg("MODUL").getS();
-		if(modPresent(type) && !at(type).at().messPresent(id)) at(type).at().messAdd(id,"*.*");
-	    }
+
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            vector<string> m_ls;
+            modList(m_ls);
+            for(unsigned i_m = 0; i_m < m_ls.size(); i_m++)
+            {
+                at(m_ls[i_m]).at().messList(db_ls);
+                for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                    if(itReg.find(m_ls[i_m]+"."+db_ls[i_it]) == itReg.end() && SYS->chkSelDB(at(m_ls[i_m]).at().messAt(db_ls[i_it]).at().DB()))
+                        at(m_ls[i_m]).at().messDel(db_ls[i_it]);
+            }
+        }
     }catch( TError err )
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -179,25 +185,34 @@ void TArchiveS::load_( )
 	TConfig c_el(&elVal);
 	c_el.cfgViewAll(false);
 	vector<string> db_ls;
+	itReg.clear();
 
 	//>> Search into DB and create new archivators
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_val_proc","",fld_cnt++,c_el); )
+	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_val_proc",nodePath()+subId()+"_val_proc",fld_cnt++,c_el); )
 	    {
 		id = c_el.cfg("ID").getS();
 		type = c_el.cfg("MODUL").getS();
 		if(modPresent(type) && !at(type).at().valPresent(id))
 		    at(type).at().valAdd(id,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+		itReg[type+"."+id] = true;
 	    }
-	//>> Search into config file and create new archivators
-	if(SYS->chkSelDB("<cfg>"))
-	    for(int fld_cnt=0; SYS->db().at().dataSeek("",nodePath()+subId()+"_val_proc",fld_cnt++,c_el); )
-	    {
-		id = c_el.cfg("ID").getS();
-		type = c_el.cfg("MODUL").getS();
-		if(modPresent(type) && !at(type).at().valPresent(id)) at(type).at().valAdd(id,"*.*");
-	    }
+
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            vector<string> m_ls;
+            modList(m_ls);
+            for(unsigned i_m = 0; i_m < m_ls.size(); i_m++)
+            {
+                at(m_ls[i_m]).at().valList(db_ls);
+                for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                    if(itReg.find(m_ls[i_m]+"."+db_ls[i_it]) == itReg.end() && SYS->chkSelDB(at(m_ls[i_m]).at().valAt(db_ls[i_it]).at().DB()))
+                        at(m_ls[i_m]).at().valDel(db_ls[i_it]);
+            }
+        }
     }catch( TError err )
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str()); 
@@ -210,22 +225,27 @@ void TArchiveS::load_( )
 	TConfig c_el(&elAval);
 	c_el.cfgViewAll(false);
 	vector<string> db_ls;
+	itReg.clear();
 
 	//>> Search into DB and create new archives
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_val","",fld_cnt++,c_el); )
+	    for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_val",nodePath()+subId()+"_val",fld_cnt++,c_el); )
 	    {
 	        id = c_el.cfg("ID").getS();
 	        if(!valPresent(id)) valAdd(id,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+	        itReg[id] = true;
 	    }
-	//>> Search into config file and create new archives
-	if(SYS->chkSelDB("<cfg>"))
-	    for(int fld_cnt=0; SYS->db().at().dataSeek("",nodePath()+subId()+"_val",fld_cnt++,c_el); )
-	    {
-		id = c_el.cfg("ID").getS();
-		if(!valPresent(id)) valAdd(id,"*.*");
-	    }
+
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            valList(db_ls);
+            for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                if(itReg.find(db_ls[i_it]) == itReg.end() && SYS->chkSelDB(valAt(db_ls[i_it]).at().DB()))
+                    valDel(db_ls[i_it]);
+        }
     }catch(TError err)
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -242,7 +262,6 @@ void TArchiveS::save_( )
     TBDS::genDBSet(nodePath()+"MessPeriod",TSYS::int2str(messPeriod()));
     TBDS::genDBSet(nodePath()+"ValPeriod",TSYS::int2str(valPeriod()));
     TBDS::genDBSet(nodePath()+"ValPriority",TSYS::int2str(valPrior()));
-    TBDS::genDBSet(nodePath()+"MaxReqMess",TSYS::int2str(max_req_mess));
 }
 
 void TArchiveS::valAdd( const string &iid, const string &idb )
@@ -256,10 +275,10 @@ string TArchiveS::optDescr(  )
     char buf[STR_BUF_LEN];
     snprintf(buf,sizeof(buf),_(
 	"======================== Subsystem \"Archives\" options ===================\n"
-	"------------ Parameters of section <%s> in config file -----------\n"
+	"------------ Parameters of section '%s' in config-file -----------\n"
 	"MessBufSize   <items>       Messages buffer size.\n"
-	"MessPeriod    <sec>         Message arhiving period.\n"
-	"ValPeriod     <msec>        Values arhiving period.\n"
+	"MessPeriod    <sec>         Message archiving period.\n"
+	"ValPeriod     <msec>        Values archiving period.\n"
 	"ValPriority   <level>	     Values task priority level.\n"
 	"MaxReqMess    <items>       Maximum request messages.\n"
 	"MaxReqVals    <items>       Maximum request values.\n\n"
@@ -286,12 +305,12 @@ void TArchiveS::subStart( )
 	for(unsigned i_o = 0; i_o < o_lst.size(); i_o++)
 	{
 	    AutoHD<TMArchivator> mess = mod.at().messAt(o_lst[i_o]);
-	    if( !mess.at().startStat() && mess.at().toStart() )
+	    if( /*!mess.at().startStat() &&*/ mess.at().toStart() )
 		try{ mess.at().start(); }
 		catch(TError err)
 		{
-		    mess_err(err.cat.c_str(),"%s",err.mess.c_str()); 
-		    mess_err(nodePath().c_str(),_("Message archivator <%s> start error."),o_lst[i_o].c_str());
+		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
+		    mess_err(nodePath().c_str(),_("Message archivator '%s' start error."),o_lst[i_o].c_str());
 		}
 	}
 	//> Value archivators start
@@ -299,12 +318,12 @@ void TArchiveS::subStart( )
 	for(unsigned i_o = 0; i_o < o_lst.size(); i_o++)
 	{
 	    AutoHD<TVArchivator> val = mod.at().valAt(o_lst[i_o]);
-	    if( !val.at().startStat() && val.at().toStart() )
+	    if( /*!val.at().startStat() &&*/ val.at().toStart() )
 		try{ val.at().start(); }
 		catch(TError err)
 		{
 		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		    mess_err(nodePath().c_str(),_("Value archivator <%s> start error."),val.at().workId().c_str());
+		    mess_err(nodePath().c_str(),_("Value archivator '%s' start error."),val.at().workId().c_str());
 		}
 	}
     }
@@ -314,12 +333,12 @@ void TArchiveS::subStart( )
     for(unsigned i_o = 0; i_o < o_lst.size(); i_o++)
     {
 	AutoHD<TVArchive> aval = valAt(o_lst[i_o]);
-	if( !aval.at().startStat() && aval.at().toStart() )
+	if( /*!aval.at().startStat() &&*/ aval.at().toStart() )
 	    try{ aval.at().start(); }
 	    catch(TError err)
 	    {
 		mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		mess_err(nodePath().c_str(),_("Value archive <%s> start error."),o_lst[i_o].c_str());
+		mess_err(nodePath().c_str(),_("Value archive '%s' start error."),o_lst[i_o].c_str());
 	    }
     }
 
@@ -330,7 +349,7 @@ void TArchiveS::subStart( )
     timer_settime(tmIdMess, 0, &itval, NULL);
 
     //> Start values acquisition task
-    if( !prcStVal ) SYS->taskCreate( nodePath('.',true)+".vals", valPrior(), TArchiveS::ArhValTask, this, &prcStVal );
+    if(!prcStVal) SYS->taskCreate(nodePath('.',true)+".vals", valPrior(), TArchiveS::ArhValTask, this);
 
     TSubSYS::subStart( );
 
@@ -350,11 +369,11 @@ void TArchiveS::subStop( )
     itval.it_interval.tv_sec = itval.it_interval.tv_nsec =
 	itval.it_value.tv_sec = itval.it_value.tv_nsec = 0;
     timer_settime(tmIdMess, 0, &itval, NULL);
-    if( TSYS::eventWait( prcStMess, false, nodePath()+"mess_stop",10) )
+    if(TSYS::eventWait( prcStMess, false, nodePath()+"mess_stop",10))
 	throw TError(nodePath().c_str(),_("Archive messages thread is not stopped!"));
 
     //> Values acquisition task stop
-    if( prcStVal ) SYS->taskDestroy( nodePath('.',true)+".vals", &prcStVal, &endrunReqVal );
+    if(prcStVal) SYS->taskDestroy(nodePath('.',true)+".vals", &endrunReqVal);
 
     //> Last messages archivation call
     sigval obj; obj.sival_ptr = this;
@@ -375,7 +394,7 @@ void TArchiveS::subStop( )
 		catch(TError err)
 		{
 		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		    mess_err(nodePath().c_str(),_("Value archivator <%s> stop error."),o_lst[i_o].c_str());
+		    mess_err(nodePath().c_str(),_("Value archivator '%s' stop error."),o_lst[i_o].c_str());
 		}
 	}
 	// Message archivators stop
@@ -388,7 +407,7 @@ void TArchiveS::subStop( )
 		catch(TError err)
 		{
 		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		    mess_err(nodePath().c_str(),_("Message archivator <%s> stop error."),o_lst[i_o].c_str());
+		    mess_err(nodePath().c_str(),_("Message archivator '%s' stop error."),o_lst[i_o].c_str());
 		}
 	}
     }
@@ -403,7 +422,7 @@ void TArchiveS::subStop( )
 	    catch(TError err)
 	    {
 		mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		mess_err(nodePath().c_str(),_("Value archive <%s> stop error."),o_lst[i_o].c_str());
+		mess_err(nodePath().c_str(),_("Value archive '%s' stop error."),o_lst[i_o].c_str());
 	    }
     }
 }
@@ -437,7 +456,7 @@ void TArchiveS::messPut( time_t tm, int utm, const string &categ, int8_t level,
 	{
 	    bufErr |= 0x02;
 	    res.release();
-	    mess_warning(nodePath().c_str(),_("Messagess buffer filling is too fast!"));
+	    mess_warning(nodePath().c_str(),_("Messages buffer filling is too fast!"));
 	    res.request(true);
 	}
     }
@@ -455,18 +474,20 @@ void TArchiveS::messPut( const vector<TMess::SRec> &recs )
 	messPut(recs[i_r].time,recs[i_r].utime,recs[i_r].categ,recs[i_r].level,recs[i_r].mess);
 }
 
-void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category, int8_t level, const string &arch )
+void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category, int8_t level, const string &arch, time_t upTo )
 {
     recs.clear();
 
     ResAlloc res(mRes,false);
+    if(!upTo) upTo = time(NULL)+STD_INTERF_TM;
+    TRegExp re(category, "p");
 
     //> Get records from buffer
     unsigned i_buf = headBuf;
-    while(level >= 0 && (!arch.size() || arch==BUF_ARCH_NM))
+    while(level >= 0 && (!arch.size() || arch==BUF_ARCH_NM) && time(NULL) < upTo)
     {
 	if(mBuf[i_buf].time >= b_tm && mBuf[i_buf].time != 0 && mBuf[i_buf].time <= e_tm &&
-		mBuf[i_buf].level >= level && TMess::chkPattern(mBuf[i_buf].categ, category))
+		mBuf[i_buf].level >= level && re.test(mBuf[i_buf].categ))
 	    recs.push_back(mBuf[i_buf]);
 	if(++i_buf >= mBuf.size()) i_buf = 0;
 	if(i_buf == headBuf) break;
@@ -478,7 +499,7 @@ void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, c
     for(unsigned i_t = 0; level >= 0 && i_t < t_lst.size(); i_t++)
     {
 	at(t_lst[i_t]).at().messList(o_lst);
-	for(unsigned i_o = 0; i_o < o_lst.size(); i_o++)
+	for(unsigned i_o = 0; i_o < o_lst.size() && time(NULL) < upTo; i_o++)
 	{
 	    AutoHD<TMArchivator> archtor = at(t_lst[i_t]).at().messAt(o_lst[i_o]);
 	    if(archtor.at().startStat() && (!arch.size() || arch==archtor.at().workId()))
@@ -489,11 +510,11 @@ void TArchiveS::messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, c
     //> Alarms request processing
     if(level < 0)
     {
-	vector< pair<long long,TMess::SRec* > > mb;
-	for(map<string,TMess::SRec>::iterator p = mAlarms.begin(); p != mAlarms.end(); p++)
+	vector< pair<int64_t,TMess::SRec* > > mb;
+	for(map<string,TMess::SRec>::iterator p = mAlarms.begin(); p != mAlarms.end() && time(NULL) < upTo; p++)
 	    if((p->second.time >= b_tm || b_tm == e_tm) && p->second.time <= e_tm &&
-		    p->second.level >= abs(level) && Mess->chkPattern(p->second.categ,category))
-		mb.push_back(pair<long long,TMess::SRec* >(FTM(p->second),&p->second));
+		    p->second.level >= abs(level) && re.test(p->second.categ))
+		mb.push_back(pair<int64_t,TMess::SRec* >(FTM(p->second),&p->second));
 	sort(mb.begin(),mb.end());
 	for(unsigned i_b = 0; i_b < mb.size(); i_b++) recs.push_back(*mb[i_b].second);
     }
@@ -615,7 +636,7 @@ void TArchiveS::ArhMessTask( union sigval obj )
     if( arh.prcStMess )  return;
     arh.prcStMess = true;
 
-    //> Message bufer read
+    //> Message buffer read
     if( arh.headLstread != arh.headBuf )
 	try
 	{
@@ -648,7 +669,7 @@ void TArchiveS::ArhMessTask( union sigval obj )
 	catch(TError err)
 	{
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-	    mess_err(arh.nodePath().c_str(),_("Message bufer read error."));
+	    mess_err(arh.nodePath().c_str(),_("Message buffer read error."));
 	}
 
     arh.prcStMess = false;
@@ -662,7 +683,7 @@ void *TArchiveS::ArhValTask( void *param )
 
     while( !arh.endrunReqVal )
     {
-	long long work_tm = SYS->curTime();
+	int64_t work_tm = SYS->curTime();
 
 	arh.vRes.resRequestR( );
 	for(unsigned i_arh = 0; i_arh < arh.actUpSrc.size(); i_arh++)
@@ -675,7 +696,7 @@ void *TArchiveS::ArhValTask( void *param )
 	    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 	arh.vRes.resRelease( );
 
-	TSYS::taskSleep((long long)arh.valPeriod()*1000000);
+	TSYS::taskSleep((int64_t)arh.valPeriod()*1000000);
     }
 
     arh.prcStVal = false;
@@ -685,7 +706,7 @@ void *TArchiveS::ArhValTask( void *param )
 
 TVariant TArchiveS::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
 {
-    // Area messGet(int btm, int etm, string cat = "", int lev = 0, string arch = ""); - request of the system messages for the time from <btm>
+    // Array messGet(int btm, int etm, string cat = "", int lev = 0, string arch = ""); - request of the system messages for the time from <btm>
     //       to <etm> for the category <cat>, level <lev> and archiver <arch>
     //  btm - begin time
     //  etm - end time
@@ -753,17 +774,18 @@ void TArchiveS::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("grp",opt,-1,"/br/va_",_("Value archive"),RWRWR_,"root",SARH_ID,2,"idm","1","idSz","20");
 	if(ctrMkNode("area",opt,1,"/m_arch",_("Messages archive"),R_R_R_,"root",SARH_ID))
 	{
-	    ctrMkNode("fld",opt,-1,"/m_arch/max_am_req",_("Maximum requested messages"),RWRWR_,"root",SARH_ID,1,"tp","dec");
 	    ctrMkNode("fld",opt,-1,"/m_arch/size",_("Messages buffer size"),RWRWR_,"root",SARH_ID,2,"tp","dec","min",TSYS::int2str(BUF_SIZE_DEF).c_str());
 	    ctrMkNode("fld",opt,-1,"/m_arch/per",_("Archiving period (s)"),RWRWR_,"root",SARH_ID,1,"tp","dec");
 	    if(ctrMkNode("area",opt,-1,"/m_arch/view",_("View messages"),R_R___,"root",SARH_ID))
 	    {
 		ctrMkNode("fld",opt,-1,"/m_arch/view/tm",_("Time"),RWRW__,"root",SARH_ID,1,"tp","time");
 		ctrMkNode("fld",opt,-1,"/m_arch/view/size",_("Size (s)"),RWRW__,"root",SARH_ID,1,"tp","dec");
-		ctrMkNode("fld",opt,-1,"/m_arch/view/cat",_("Category pattern"),RWRW__,"root",SARH_ID,2,"tp","str",
-		    "help",_("Messages category. Use template symbols for group selection:\n  '*' - any substring;\n  '?' - any symbol."));
+		ctrMkNode("fld",opt,-1,"/m_arch/view/cat",_("Category pattern"),RWRW__,"root",SARH_ID,2,"tp","str","help",
+		    _("Messages category template or regular expression.\n"
+            	      "Use template symbols for group selection:\n  '*' - any substring;\n  '?' - any symbol.\n"
+                      "Regular expression enclosed in symbols '/' (/mod_(System|LogicLev)/)."));
 		ctrMkNode("fld",opt,-1,"/m_arch/view/lvl",_("Level"),RWRW__,"root",SARH_ID,4,"tp","dec","min","-7","max","7",
-		    "help",_("Get messages for level more and equaly it."));
+		    "help",_("Get messages for level more and equal it."));
 		ctrMkNode("fld",opt,-1,"/m_arch/view/archtor",_("Archivator"),RWRW__,"root",SARH_ID,4,"tp","str","dest","select","select","/m_arch/lstAMess",
 		    "help",_("Messages archivator.\nNo set archivator for process by buffer and all archivators.\nSet '<buffer>' for process by buffer."));
 		if(ctrMkNode("table",opt,-1,"/m_arch/view/mess",_("Messages"),R_R___,"root",SARH_ID))
@@ -788,12 +810,7 @@ void TArchiveS::cntrCmdProc( XMLNode *opt )
     }
 
     //> Process command to page
-    if(a_path == "/m_arch/max_am_req")
-    {
-	if(ctrChkNode(opt,"get",RWRWR_,"root",SARH_ID,SEC_RD))	opt->setText(TSYS::int2str(max_req_mess));
-	if(ctrChkNode(opt,"set",RWRWR_,"root",SARH_ID,SEC_WR))	{ max_req_mess = vmax(100,atoi(opt->text().c_str())); modif(); }
-    }
-    else if(a_path == "/m_arch/per")
+    if(a_path == "/m_arch/per")
     {
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SARH_ID,SEC_RD))	opt->setText(TSYS::int2str(messPeriod()));
 	if(ctrChkNode(opt,"set",RWRWR_,"root",SARH_ID,SEC_WR))	setMessPeriod(atoi(opt->text().c_str()));
@@ -931,14 +948,12 @@ TArchiveS &TTipArchivator::owner()
 
 void TTipArchivator::messAdd(const string &name, const string &idb )
 {
-    if( chldPresent(mMess,name) ) return;
-    chldAdd(mMess,AMess(name,idb));
+    chldAdd(mMess, AMess(name,idb));
 }
 
 void TTipArchivator::valAdd( const string &iid, const string &idb )
 {
-    if( chldPresent(mVal,iid) ) return;
-    chldAdd(mVal,AVal(iid,idb));
+    chldAdd(mVal, AVal(iid,idb));
 }
 
 void TTipArchivator::cntrCmdProc( XMLNode *opt )
@@ -1077,13 +1092,29 @@ bool TMArchivator::chkMessOK( const string &icateg, TMess::Type ilvl )
 
     categ(cat_ls);
 
-    if( ilvl >= level() )
-	for( unsigned i_cat = 0; i_cat < cat_ls.size(); i_cat++ )
-	    if( TMess::chkPattern(icateg,cat_ls[i_cat]) )
+    if(ilvl >= level())
+	for(unsigned i_cat = 0; i_cat < cat_ls.size(); i_cat++)
+	    if(TRegExp(cat_ls[i_cat], "p").test(icateg))
 		return true;
    return false;
 }
 
+TVariant TMArchivator::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    // bool status() - get archivator status.
+    if(iid == "status") return startStat();
+    // int end() - get archivator data end time.
+    if(iid == "end") return (int)end();
+    // int begin() - get archivator data begin time.
+    if(iid == "begin") return (int)begin();
+
+    //> Configuration functions call
+    TVariant cfRez = objFunc(iid, prms, user);
+    if(!cfRez.isNull()) return cfRez;
+
+    return TCntrNode::objFuncCall(iid, prms, user);
+}
+
 void TMArchivator::cntrCmdProc( XMLNode *opt )
 {
     //> Get page info
@@ -1095,23 +1126,24 @@ void TMArchivator::cntrCmdProc( XMLNode *opt )
 	{
 	    if(ctrMkNode("area",opt,-1,"/prm/st",_("State")))
 	    {
-		ctrMkNode("fld",opt,-1,"/prm/st/st",_("Runing"),RWRWR_,"root",SARH_ID,1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/prm/st/st",_("Running"),RWRWR_,"root",SARH_ID,1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/prm/st/db",_("Archivator DB"),RWRWR_,"root","root",4,
 		    "tp","str","dest","select","select","/db/list","help",TMess::labDB());
 		ctrMkNode("fld",opt,-1,"/prm/st/end",_("End"),R_R_R_,"root","root",1,"tp","time");
 		ctrMkNode("fld",opt,-1,"/prm/st/beg",_("Begin"),R_R_R_,"root","root",1,"tp","time");
 	    }
-	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Configuration")))
 	    {
 		ctrMkNode("fld",opt,-1,"/prm/cfg/id",cfg("ID").fld().descr(),R_R_R_,"root",SARH_ID,1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/nm",cfg("NAME").fld().descr(),RWRWR_,"root",SARH_ID,2,"tp","str","len","50");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/dscr",cfg("DESCR").fld().descr(),RWRWR_,"root",SARH_ID,3,"tp","str","cols","90","rows","3");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),RWRWR_,"root",SARH_ID,1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/lvl",cfg("LEVEL").fld().descr(),RWRWR_,"root",SARH_ID,2,"tp","dec",
-		    "help",_("Get messages for level more and equaly it."));
+		    "help",_("Get messages for level more and equal it."));
 		ctrMkNode("fld",opt,-1,"/prm/cfg/cats",cfg("CATEG").fld().descr(),RWRWR_,"root",SARH_ID,2,"tp","str",
-		    "help",_("Messages categories to processing by archivator, separated by symbol ';'.\n"
-			   "Use template symbols for group selection:\n  '*' - any substring;\n  '?' - any symbol."));
+		    "help",_("Messages category template or regular expression to processing by archivator, separated by symbol ';'.\n"
+			   "Use template symbols for group selection:\n  '*' - any substring;\n  '?' - any symbol.\n"
+			   "Regular expression enclosed in symbols '/' (/mod_(System|LogicLev)/)."));
 		ctrMkNode("fld",opt,-1,"/prm/cfg/start",_("To start"),RWRWR_,"root",SARH_ID,1,"tp","bool");
 	    }
 	}
@@ -1119,10 +1151,12 @@ void TMArchivator::cntrCmdProc( XMLNode *opt )
 	{
 	    ctrMkNode("fld",opt,-1,"/mess/tm",_("Time"),RWRW__,"root",SARH_ID,1,"tp","time");
 	    ctrMkNode("fld",opt,-1,"/mess/size",_("Size (s)"),RWRW__,"root",SARH_ID,1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/mess/cat",_("Category pattern"),RWRW__,"root",SARH_ID,2,"tp","str",
-		"help",_("Messages category. Use template symbols for group selection:\n  '*' - any substring;\n  '?' - any symbol."));
+	    ctrMkNode("fld",opt,-1,"/mess/cat",_("Category pattern"),RWRW__,"root",SARH_ID,2,"tp","str","help",
+		_("Messages category template or regular expression.\n"
+            	  "Use template symbols for group selection:\n  '*' - any substring;\n  '?' - any symbol.\n"
+                  "Regular expression enclosed in symbols '/' (/mod_(System|LogicLev)/)."));
 	    ctrMkNode("fld",opt,-1,"/mess/lvl",_("Level"),RWRW__,"root",SARH_ID,4,"tp","dec","min","0","max","7",
-		"help",_("Get messages for level more and equaly it."));
+		"help",_("Get messages for level more and equal it."));
 	    if(ctrMkNode("table",opt,-1,"/mess/mess",_("Messages"),R_R___,"root",SARH_ID))
 	    {
 		ctrMkNode("list",opt,-1,"/mess/mess/0",_("Time"),R_R___,"root",SARH_ID,1,"tp","time");
