@@ -51,32 +51,29 @@ TBDS::TBDS( ) : TSubSYS(SDB_ID,_("Data Bases"),true), mSYSStPref(true)
 
 TBDS::~TBDS(  )
 {
-    ResAlloc res(nodeRes(),true);
-    while( genDBCache.size() )
-    {
-	delete genDBCache.front();
-	genDBCache.pop_front();
-    }
-    res.release();
+
 }
 
 string TBDS::realDBName( const string &bdn )
 {
-    string bd_t = TSYS::strSepParse(bdn,0,'.');
-    string bd_n = TSYS::strSepParse(bdn,1,'.');
-    string bd_tbl = TSYS::strSepParse(bdn,2,'.');
+    int off = 0;
+    string bd_t = TSYS::strParse(bdn,0,".",&off);
+    string bd_n = TSYS::strParse(bdn,0,".",&off);
+    string bd_tbl = TSYS::strParse(bdn,0,".",&off);
 
-    return ((bd_t=="*") ? TSYS::strSepParse(SYS->workDB(),0,'.') : bd_t)+"."+
-	   ((bd_n=="*") ? TSYS::strSepParse(SYS->workDB(),1,'.') : bd_n)+(bd_tbl.empty() ? "" : "."+bd_tbl);
+    if(bd_t == "<cfg>")	return bdn;
+    if(SYS->workDB() == "<cfg>" && bd_t == "*" && bd_n == "*")	return "<cfg>."+bd_tbl;
+    return ((bd_t=="*") ? TSYS::strParse(SYS->workDB(),0,".") : bd_t)+"."+
+	   ((bd_n=="*") ? TSYS::strParse(SYS->workDB(),1,".") : bd_n)+(bd_tbl.empty() ? "" : "."+bd_tbl);
 }
 
 void TBDS::dbList( vector<string> &ls, bool checkSel )
 {
     ls.clear();
 
-    if( checkSel && !SYS->selDB().empty() )
+    if(checkSel && !SYS->selDB().empty())
     {
-	if( SYS->selDB() != "<cfg>" ) ls.push_back(SYS->selDB());
+	if(SYS->selDB() != "<cfg>") ls.push_back(SYS->selDB());
 	return;
     }
 
@@ -90,8 +87,9 @@ void TBDS::dbList( vector<string> &ls, bool checkSel )
     }
 }
 
-void TBDS::closeOldTables( int secOld )
+void TBDS::perSYSCall( unsigned int cnt )
 {
+    int secOld = 600;
     vector<string> tdbs, dbs, tbls;
 
     try
@@ -132,9 +130,10 @@ AutoHD<TTable> TBDS::open( const string &bdn, bool create )
 	string bd_tbl = TSYS::strSepParse(bdn,2,'.');
 	if(bd_t == "*") bd_t = TSYS::strSepParse(SYS->workDB(),0,'.');
 	if(bd_n == "*") bd_n = TSYS::strSepParse(SYS->workDB(),1,'.');
-	if( at(bd_t).at().at(bd_n).at().enableStat() )
+	if(bd_t == "<cfg>") return tbl;
+	if(at(bd_t).at().at(bd_n).at().enableStat())
 	{
-	    if( !at(bd_t).at().at(bd_n).at().openStat(bd_tbl) )
+	    if(!at(bd_t).at().at(bd_n).at().openStat(bd_tbl))
 		at(bd_t).at().at(bd_n).at().open(bd_tbl,create);
 	    tbl = at(bd_t).at().at(bd_n).at().at(bd_tbl);
 	}
@@ -156,14 +155,14 @@ void TBDS::close( const string &bdn, bool del )
 	string bd_tbl = TSYS::strSepParse(bdn,2,'.');
 	if(bd_t == "*") bd_t = TSYS::strSepParse(SYS->workDB(),0,'.');
 	if(bd_n == "*") bd_n = TSYS::strSepParse(SYS->workDB(),1,'.');
-	if( at(bd_t).at().at(bd_n).at().enableStat()&&
-		at(bd_t).at().at(bd_n).at().openStat(bd_tbl) &&
-		at(bd_t).at().at(bd_n).at().at(bd_tbl).at().nodeUse() == 1 )
+	if(bd_t == "<cfg>") return;
+	if(at(bd_t).at().at(bd_n).at().enableStat() && at(bd_t).at().at(bd_n).at().openStat(bd_tbl) &&
+		at(bd_t).at().at(bd_n).at().at(bd_tbl).at().nodeUse() == 1)
 	    at(bd_t).at().at(bd_n).at().close(bd_tbl,del);
     }catch(TError err)
     {
 	mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
-	mess_warning(nodePath().c_str(),_("Close DB <%s> error!"),bdn.c_str());
+	mess_warning(nodePath().c_str(),_("Close DB '%s' error!"),bdn.c_str());
     }
 }
 
@@ -177,55 +176,60 @@ string TBDS::fullDB()
     return SYS->workDB()+".DB";
 }
 
-bool TBDS::dataSeek( const string &bdn, const string &path, int lev, TConfig &cfg )
+bool TBDS::dataSeek( const string &ibdn, const string &path, int lev, TConfig &cfg, bool forceCfg )
 {
     int c_lev = 0;
+    string bdn = realDBName(ibdn);
 
-    if( path.size() )
+    if(path.size() && (forceCfg || ibdn.empty() || TSYS::strParse(bdn,0,".") == "<cfg>"))
     {
-	try
-	{
-	    ResAlloc res(SYS->nodeRes(),false);
-	    XMLNode *nd = ctrId(&SYS->cfgRoot(),path);
-	    for(unsigned i_fld = 0, i_el; i_fld < nd->childSize(); i_fld++)
-	    {
-		XMLNode *el = nd->childGet(i_fld);
-		if( el->name() == "fld" )
-		{
-		    //> Check keywords
-		    vector<string> cf_el;
-		    cfg.cfgList(cf_el);
+	ResAlloc res(SYS->nodeRes(),false);
+	XMLNode *nd, *fnd, *el;
+	string vl, vl_tr;
+	vector<string> cf_el;
 
-		    //> Check keywords
+	nd = SYS->cfgNode(path);
+	for(unsigned i_fld = 0, i_el; nd && i_fld < nd->childSize(); i_fld++)
+	{
+	    el = nd->childGet(i_fld);
+	    if(el->name() == "fld")
+	    {
+		cfg.cfgList(cf_el);
+
+		//> Check keywords
+		for(i_el = 0; i_el < cf_el.size(); i_el++)
+		    if(cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key &&
+		        cfg.cfg(cf_el[i_el]).keyUse() &&
+		        cfg.cfg(cf_el[i_el]).getS() != el->attr(cf_el[i_el])) break;
+		if(i_el == cf_el.size() && lev <= c_lev++)
+		{
 		    for(i_el = 0; i_el < cf_el.size(); i_el++)
-			if(cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key &&
-			    cfg.cfg(cf_el[i_el]).keyUse() &&
-			    cfg.cfg(cf_el[i_el]).getS() != el->attr(cf_el[i_el])) break;
-		    if(i_el == cf_el.size() && lev <= c_lev++)
 		    {
-			for(i_el = 0; i_el < cf_el.size(); i_el++)
+			TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
+			vl = el->attr(cf_el[i_el]);
+			//>> Check for field's tag, for store big values
+			if(vl.empty() && (fnd=el->childGet(cf_el[i_el],0,true))) vl = fnd->text(true);
+			//>> Check for translation
+			if(!cfg.noTransl() && u_cfg.fld().flg()&TCfg::TransltText &&
+			    (Mess->lang2CodeBase().empty() || Mess->lang2Code() != Mess->lang2CodeBase()))
 			{
-			    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
-			    u_cfg.setS(el->attr(cf_el[i_el]));
-			    if(!cfg.noTransl() && u_cfg.fld().flg()&TCfg::TransltText &&
-				(Mess->lang2CodeBase().empty() || Mess->lang2Code() != Mess->lang2CodeBase()))
-			    {
-				string vl = el->attr(cf_el[i_el]+"_"+Mess->lang2Code());
-				if(!vl.empty()) u_cfg.setS(vl);
-			    }
+			    vl_tr = el->attr(cf_el[i_el]+"_"+Mess->lang2Code());
+			    //>> Check for field's tag, for store big values
+			    if(vl_tr.empty() && (fnd=el->childGet(cf_el[i_el]+"_"+Mess->lang2Code(),0,true))) vl_tr = fnd->text(true);
+			    if(!vl_tr.empty()) vl = vl_tr;
 			}
-			return true;
+			u_cfg.setS(vl);
 		    }
+		    return true;
 		}
 	    }
 	}
-	catch(...){  }
     }
 
-    if(bdn.size())
+    if(bdn.size() && TSYS::strParse(bdn,0,".") != "<cfg>")
     {
 	AutoHD<TTable> tbl = open(bdn);
-	if( !tbl.freeStat() )
+	if(!tbl.freeStat())
 	{
 	    bool rez = tbl.at().fieldSeek(lev-c_lev,cfg);
 	    //tbl.free(); close(bdn);
@@ -236,39 +240,44 @@ bool TBDS::dataSeek( const string &bdn, const string &path, int lev, TConfig &cf
     return false;
 }
 
-bool TBDS::dataGet( const string &bdn, const string &path, TConfig &cfg )
+bool TBDS::dataGet( const string &ibdn, const string &path, TConfig &cfg )
 {
-    AutoHD<TTable> tbl = open(bdn);
+    string bdn = realDBName(ibdn);
 
     //> Load from DB
-    if( !tbl.freeStat() )
+    if(bdn.size() && TSYS::strParse(bdn,0,".") != "<cfg>")
     {
-	bool db_true = true;
-	try{ tbl.at().fieldGet(cfg); }
-	catch(TError err)
+	AutoHD<TTable> tbl = open(bdn);
+	if(!tbl.freeStat())
 	{
-	    if( err.cod != TSYS::DBRowNoPresent )
-		mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
-	    db_true = false;
+	    bool db_true = true;
+	    try{ tbl.at().fieldGet(cfg); }
+	    catch(TError err)
+	    {
+		if(err.cod != TSYS::DBRowNoPresent) mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
+		db_true = false;
+	    }
+	    if(db_true) return true;
 	}
-	if(db_true) return true;
     }
-    //> Load from Config file if tbl no present
-    XMLNode *nd;
-    ResAlloc res(SYS->nodeRes(),false);
-    try{ nd = ctrId(&SYS->cfgRoot(),path); }
-    catch(...){ return false; }
 
-    //>> Scan fields and fill Config
-    for(unsigned i_fld = 0, i_el; i_fld < nd->childSize(); i_fld++)
+    //> Load from Config-file if tbl no present
+    ResAlloc res(SYS->nodeRes(),false);
+    XMLNode *nd, *fnd, *el;
+    string vl, vl_tr;
+    vector<string> cf_el;
+
+    nd = SYS->cfgNode(path);
+
+    //>> Scan fields and fill Configuration
+    for(unsigned i_fld = 0, i_el; nd && i_fld < nd->childSize(); i_fld++)
     {
-	XMLNode *el = nd->childGet(i_fld);
+	el = nd->childGet(i_fld);
 	if(el->name() == "fld")
 	{
-	    //Check keywords
-	    vector<string> cf_el;
 	    cfg.cfgList(cf_el);
 
+	    //Check keywords
 	    for(i_el = 0; i_el < cf_el.size(); i_el++)
 		if(cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key &&
 		    cfg.cfg(cf_el[i_el]).getS() != el->attr(cf_el[i_el])) break;
@@ -277,80 +286,187 @@ bool TBDS::dataGet( const string &bdn, const string &path, TConfig &cfg )
 		for(i_el = 0; i_el < cf_el.size(); i_el++)
 		{
 		    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
-		    u_cfg.setS(el->attr(cf_el[i_el]));
+		    vl = el->attr(cf_el[i_el]);
+		    //>> Check for field's tag, for store big values
+		    if(vl.empty() && (fnd=el->childGet(cf_el[i_el],0,true))) vl = fnd->text(true);
+		    //>> Check for translation
 		    if(!cfg.noTransl() && u_cfg.fld().flg()&TCfg::TransltText &&
 			(Mess->lang2CodeBase().empty() || Mess->lang2Code() != Mess->lang2CodeBase()))
 		    {
-			string vl = el->attr(cf_el[i_el]+"_"+Mess->lang2Code());
-			if(!vl.empty()) u_cfg.setS(vl);
+			vl_tr = el->attr(cf_el[i_el]+"_"+Mess->lang2Code());
+			//>> Check for field's tag, for store big values
+			if(vl_tr.empty() && (fnd=el->childGet(cf_el[i_el]+"_"+Mess->lang2Code(),0,true))) vl_tr = fnd->text(true);
+			if(!vl_tr.empty()) vl = vl_tr;
 		    }
+		    u_cfg.setS(vl);
 		}
 		return true;
 	    }
 	}
     }
-    //throw TError(nodePath().c_str(),"Field <%s> no present.",path.c_str());
+    //throw TError(nodePath().c_str(),"Field '%s' no present.",path.c_str());
     return false;
 }
 
-bool TBDS::dataSet( const string &bdn, const string &path, TConfig &cfg )
+bool TBDS::dataSet( const string &ibdn, const string &path, TConfig &cfg )
 {
-    AutoHD<TTable> tbl = open(bdn,true);
+    string bdn = realDBName(ibdn);
 
-    if( !tbl.freeStat() )
+    //> Save to DB
+    if(bdn.size() && TSYS::strParse(bdn,0,".") != "<cfg>")
     {
-	bool db_true = true;
-	try{ tbl.at().fieldSet(cfg); }
-	catch(TError err) { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); db_true = false; }
-	return db_true;
-    }
-    //throw TError("BD","Field no present.");
-    return false;
-}
-
-bool TBDS::dataDel( const string &bdn, const string &path, TConfig &cfg, bool useKeyAll )
-{
-    vector<string> cels;
-    AutoHD<TTable> tbl = open(bdn);
-
-    if(!tbl.freeStat())
-    {
-	bool db_true = true;
-	try
+	AutoHD<TTable> tbl = open(bdn,true);
+	if(!tbl.freeStat())
 	{
-	    //> Select for using all keys
-	    if(useKeyAll)
+	    bool db_true = true;
+	    try{ tbl.at().fieldSet(cfg); }
+	    catch(TError err) { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); db_true = false; }
+	    return db_true;
+	}
+    }
+
+    //> Save to config
+    if(TSYS::strParse(bdn,0,".") == "<cfg>")
+    {
+	ResAlloc res(SYS->nodeRes(),false);
+	XMLNode *nd, *wel = NULL, *fnd;
+	vector<string> cf_el;
+	string vnm;
+
+	nd = SYS->cfgNode(path,true);
+
+	if(nd)
+	{
+	    cfg.cfgList(cf_el);
+	    if(nd->name() != "tbl")	nd->setName("tbl");
+	    //>> Search present field
+	    for(unsigned i_fld = 0, i_el; i_fld < nd->childSize(); i_fld++)
 	    {
-		cfg.cfgList(cels);
-		for(unsigned i_el = 0; i_el < cels.size(); )
+		XMLNode *el = nd->childGet(i_fld);
+		if(el->name() != "fld")	continue;
+		//Check keywords
+		for(i_el = 0; i_el < cf_el.size(); i_el++)
+		    if(cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key && cfg.cfg(cf_el[i_el]).getS() != el->attr(cf_el[i_el])) break;
+		if(i_el == cf_el.size()) { wel = el; break; }
+	    }
+	    bool isCreate = !wel;
+	    if(!wel) wel = nd->childAdd("fld");
+	    for(unsigned i_el = 0; i_el < cf_el.size(); i_el++)
+	    {
+		vnm = cf_el[i_el];
+		TCfg &u_cfg = cfg.cfg(vnm);
+		bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && !u_cfg.noTransl() &&
+				Mess->lang2CodeBase().size() && Mess->lang2Code() != Mess->lang2CodeBase());
+		if(isCreate || !isTransl)
 		{
-		    if((cfg.cfg(cels[i_el]).fld().flg()&TCfg::Key) && !cfg.cfg(cels[i_el]).keyUse())
-			cfg.cfg(cels[i_el]).setKeyUse(true);
-		    else { cels.erase(cels.begin()+i_el); continue; }
-		    i_el++;
+		    if(u_cfg.getS().size() < 100)
+		    {
+			wel->setAttr(vnm, u_cfg.getS());
+			if((fnd=wel->childGet(vnm,0,true))) wel->childDel(fnd);
+		    }
+		    else
+		    {
+			if(!(fnd=wel->childGet(vnm,0,true))) fnd = wel->childAdd(vnm);
+			fnd->setText(u_cfg.getS(),true);
+			wel->setAttr(vnm,"");
+		    }
+		}
+		if(isTransl)
+		{
+		    vnm = cf_el[i_el]+"_"+Mess->lang2Code();
+		    if(u_cfg.getS().size() < 100)
+		    {
+			wel->setAttr(vnm, u_cfg.getS());
+			if((fnd=wel->childGet(vnm,0,true))) wel->childDel(fnd);
+		    }
+		    else
+		    {
+			if(!(fnd=wel->childGet(vnm,0,true))) fnd = wel->childAdd(vnm);
+			fnd->setText(u_cfg.getS(),true);
+			wel->setAttr(vnm,"");
+		    }
 		}
 	    }
-
-	    tbl.at().fieldDel(cfg);
-
-	    //> Restore not using keys selection
-	    if(useKeyAll)
-		for(unsigned i_el = 0; i_el < cels.size(); i_el++)
-		    cfg.cfg(cels[i_el]).setKeyUse(false);
+	    SYS->modifCfg();
+	    return true;
 	}
-	catch(TError err) { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); db_true = false; }
-	//tbl.free(); close(bdn);
-	return db_true;
     }
 
     return false;
 }
 
+bool TBDS::dataDel( const string &ibdn, const string &path, TConfig &cfg, bool useKeyAll, bool forceCfg )
+{
+    vector<string> cels;
+    string bdn = realDBName(ibdn);
+    bool db_true = false;
+
+    if(bdn.size() && TSYS::strParse(bdn,0,".") != "<cfg>")
+    {
+	AutoHD<TTable> tbl = open(bdn);
+	if(!tbl.freeStat())
+	{
+
+	    try
+	    {
+		//> Select for using all keys
+		if(useKeyAll)
+		{
+		    cfg.cfgList(cels);
+		    for(unsigned i_el = 0; i_el < cels.size(); )
+		    {
+			if((cfg.cfg(cels[i_el]).fld().flg()&TCfg::Key) && !cfg.cfg(cels[i_el]).keyUse())
+			    cfg.cfg(cels[i_el]).setKeyUse(true);
+			else { cels.erase(cels.begin()+i_el); continue; }
+			i_el++;
+		    }
+		}
+
+		tbl.at().fieldDel(cfg);
+
+		//> Restore not using keys selection
+		if(useKeyAll)
+		    for(unsigned i_el = 0; i_el < cels.size(); i_el++)
+			cfg.cfg(cels[i_el]).setKeyUse(false);
+	    }
+	    catch(TError err) { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); }
+	    //tbl.free(); close(bdn);
+	    db_true = true;
+	}
+    }
+
+    //> Delete from config
+    if(path.size() && (forceCfg || ibdn.empty() || TSYS::strParse(bdn,0,".") == "<cfg>"))
+    {
+	ResAlloc res(SYS->nodeRes(),false);
+	XMLNode *nd = SYS->cfgNode(path,true);
+	vector<string> cf_el;
+	//>> Search present field
+	for(unsigned i_fld = 0, i_el; nd && i_fld < nd->childSize(); i_fld++)
+	{
+	    XMLNode *el = nd->childGet(i_fld);
+	    if(el->name() != "fld")	continue;
+	    //Check keywords
+	    cfg.cfgList(cf_el);
+	    for(i_el = 0; i_el < cf_el.size(); i_el++)
+	        if(cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key && cfg.cfg(cf_el[i_el]).getS() != el->attr(cf_el[i_el])) break;
+	    if(i_el == cf_el.size())
+	    {
+	        nd->childDel(i_fld);
+	        SYS->modifCfg();
+	        return true;
+	    }
+	}
+    }
+
+    return db_true;
+}
+
 void TBDS::genDBSet(const string &path, const string &val, const string &user, char rFlg )
 {
-    string rez;
+    bool bd_ok = false;
 
-    //Set to DB
+    //> Set to DB
     if(SYS->present(SDB_ID))
     {
 	AutoHD<TBDS> dbs = SYS->db();
@@ -369,27 +485,24 @@ void TBDS::genDBSet(const string &path, const string &val, const string &user, c
 	    }
 	    db_el.cfg("val").setS(val);
 
-	    try{ tbl.at().fieldSet(db_el); }
+	    try
+	    {
+		tbl.at().fieldSet(db_el);
+		bd_ok = true;
+	    }
 	    catch(TError err){ }
 	}
-	else
-	{
-	    //Put to cache
-	    ResAlloc res(dbs.at().nodeRes(),true);
-	    for(unsigned i_cel = 0; i_cel < dbs.at().genDBCache.size(); i_cel++)
-		if(dbs.at().genDBCache[i_cel]->cfg("user").getS() == user && dbs.at().genDBCache[i_cel]->cfg("id").getS() == path)
-		{ dbs.at().genDBCache[i_cel]->cfg("val").setS(val); return; }
-	    TConfig *db_el = new TConfig(&dbs.at());
-	    db_el->cfg("user").setS(user);
-	    db_el->cfg("id").setS(path);
-	    db_el->cfg("val").setS(val);
-	    dbs.at().genDBCache.push_front(db_el);
-	    while( dbs.at().genDBCache.size() > 100 )
-	    {
-		delete dbs.at().genDBCache.back();
-		dbs.at().genDBCache.pop_back();
-	    }
-	}
+    }
+
+    //> Set to config
+    if(!bd_ok && SYS->workDB() == "<cfg>")
+    {
+	ResAlloc res(SYS->nodeRes(),true);
+	XMLNode *tgtN = NULL;
+	if(rFlg&TBDS::UseTranslate && Mess->lang2Code().size())
+	    tgtN = SYS->cfgNode(path+"_"+Mess->lang2Code(),true);
+	if(!tgtN) tgtN = SYS->cfgNode(path,true);
+	if(tgtN) { tgtN->setText(val,true); SYS->modifCfg(); }
     }
 }
 
@@ -427,25 +540,13 @@ string TBDS::genDBGet(const string &path, const string &oval, const string &user
 
     if(!bd_ok)
     {
-	//> Get from cache
-	if(SYS->present(SDB_ID))
-	{
-	    AutoHD<TBDS> dbs = SYS->db();
-	    ResAlloc res(dbs.at().nodeRes(),false);
-	    for(unsigned i_cel = 0; i_cel < dbs.at().genDBCache.size(); i_cel++)
-		if(dbs.at().genDBCache[i_cel]->cfg("user").getS() == user && dbs.at().genDBCache[i_cel]->cfg("id").getS() == path)
-		    return dbs.at().genDBCache[i_cel]->cfg("val").getS();
-	}
-
-	//> Get from config file
+	//> Get from config-file
 	ResAlloc res(SYS->nodeRes(),false);
-	XMLNode *tgtN = TCntrNode::ctrId(&SYS->cfgRoot(),path,true);
-	if(tgtN) rez = tgtN->text();
-	if(rFlg&TBDS::UseTranslate)
-	{
-	    tgtN = TCntrNode::ctrId(&SYS->cfgRoot(),path+"_"+Mess->lang2Code(),true);
-	    if(tgtN) rez = tgtN->text();
-	}
+	XMLNode *tgtN = NULL;
+	if(rFlg&TBDS::UseTranslate && Mess->lang2Code().size())
+	    tgtN = SYS->cfgNode(path+"_"+Mess->lang2Code());
+	if(!tgtN) tgtN = SYS->cfgNode(path);
+	if(tgtN) rez = tgtN->text(true);
     }
 
     return rez;
@@ -456,7 +557,7 @@ string TBDS::optDescr(  )
     char buf[STR_BUF_LEN];
     snprintf(buf,sizeof(buf),_(
 	"========================= Subsystem \"DB\" options ========================\n"
-	"----------- The config file station <%s> parameters -----------\n"
+	"----------- The config-file station '%s' parameters -----------\n"
 	"SYSStPref    <1>   Use station id prefix into generic (SYS) table.\n\n"
 	),nodePath().c_str());
 
@@ -485,33 +586,33 @@ void TBDS::load_( )
 	}
     } while(next_opt != -1);
 
-    //> Load parameters from config file
+    //> Load parameters from config-file
     mSYSStPref = (bool)atoi(TBDS::genDBGet(nodePath()+"SYSStPref",(mSYSStPref?"1":"0"),"root",TBDS::OnlyCfg).c_str());
 
     //> DB open
     //>> Open, load and enable generic DB
     string db_tp = TSYS::strSepParse(fullDB(),0,'.');
     string db_nm = TSYS::strSepParse(fullDB(),1,'.');
-    if( !at(db_tp).at().openStat(db_nm) )
+    if(modPresent(db_tp) && !at(db_tp).at().openStat(db_nm))
     {
 	at(db_tp).at().open(db_nm);
 	at(db_tp).at().at(db_nm).at().load();
 	at(db_tp).at().at(db_nm).at().enable();
     }
 
-    //>> Open other DB stored into table 'DB' and config file
+    //>> Open other DB stored into table 'DB' and config-file
     try
     {
 	string id,type;
-	if( SYS->chkSelDB(fullDB()) )
+	if(SYS->chkSelDB(fullDB()))
 	{
 	    TConfig c_el(&el_db);
 	    c_el.cfgViewAll(false);
-	    for( int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),nodePath()+"DB/",fld_cnt++,c_el); )
+	    for(int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),nodePath()+"DB/",fld_cnt++,c_el,true); )
 	    {
 		id = c_el.cfg("ID").getS();
 		type = c_el.cfg("TYPE").getS();
-		if( (type+"."+id) != SYS->workDB() && modPresent(type) && !at(type).at().openStat(id) )
+		if((type+"."+id) != SYS->workDB() && modPresent(type) && !at(type).at().openStat(id))
 		    at(type).at().open(id);
 	    }
 	}
@@ -696,8 +797,7 @@ void TBD::disable( )
 
 void TBD::open( const string &table, bool create )
 {
-    if(chldPresent(mTbl,table)) return;
-    chldAdd(mTbl,openTable(table, create));
+    if(!chldPresent(mTbl,table)) chldAdd(mTbl,openTable(table, create));
 }
 
 void TBD::load_( )
@@ -736,6 +836,11 @@ TVariant TBD::objFuncCall( const string &iid, vector<TVariant> &prms, const stri
 	}catch(...){ }
 	return rez;
     }
+
+    //> Configuration functions call
+    TVariant cfRez = objFunc(iid, prms, user);
+    if(!cfRez.isNull()) return cfRez;
+
     return TCntrNode::objFuncCall(iid,prms,user);
 }
 
@@ -771,7 +876,7 @@ void TBD::cntrCmdProc( XMLNode *opt )
 		    "tp","br","br_pref","tbl_","s_com","del","help",_("Tables which are in the DB, tables which are not opened at that moment."));
 		ctrMkNode("comm",opt,-1,"/prm/st/load",_("Load system from this DB"),RWRW__,"root","root");
 	    }
-	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Configuration")))
 	    {
 		ctrMkNode("fld",opt,-1,"/prm/cfg/id",cfg("ID").fld().descr(),R_R_R_,"root",SDB_ID,1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/nm",cfg("NAME").fld().descr(),RWRWR_,"root",SDB_ID,2,"tp","str","len","50");
@@ -890,7 +995,7 @@ void TBD::cntrCmdProc( XMLNode *opt )
 //************************************************
 //* TTable                                       *
 //************************************************
-TTable::TTable( const string &name ) :  mName(name)
+TTable::TTable( const string &name ) :  mName(name), notFullShow(false), tblOff(0)
 {
     modifClr();
     mLstUse = time(NULL);
@@ -941,6 +1046,7 @@ TVariant TTable::objFuncCall( const string &iid, vector<TVariant> &prms, const s
 		    case TFld::Integer:	stp = "int";	break;
 		    case TFld::Real:	stp = "real";	break;
 		    case TFld::String:	stp = "str";	break;
+		    default: break;
 	        }
 		XMLNodeObj *el = new XMLNodeObj(icfg.name());
 		el->propSet("type",stp);
@@ -1010,8 +1116,9 @@ void TTable::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Table: ")+name(),RWRW__,"root",SDB_ID);
 	if(ctrMkNode("area",opt,0,"/prm",_("Table")))
 	{
-	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Configuration")))
 		ctrMkNode("fld",opt,-1,"/prm/cfg/nm",_("Name"),R_R___,"root",SDB_ID,1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/prm/tblOff",_("Table offset"),RWRW__,"root",SDB_ID,2,"tp","dec","min","0");
 	    XMLNode *tbl;
 	    if((tbl=ctrMkNode("table",opt,-1,"/prm/tbl",_("Data"),RWRW__,"root",SDB_ID,1,"s_com","add,del")))
 	    {
@@ -1034,20 +1141,29 @@ void TTable::cntrCmdProc( XMLNode *opt )
     //Process command to page
     string a_path = opt->attr("path");
     if(a_path == "/prm/cfg/nm" && ctrChkNode(opt,"get",R_R___,"root",SDB_ID,SEC_RD)) opt->setText(name());
+    else if(a_path == "/prm/tblOff")
+    {
+	if(ctrChkNode(opt,"get",RWRW__,"root",SDB_ID))	opt->setText(TSYS::int2str(tblOff));
+	if(ctrChkNode(opt,"set",RWRW__,"root",SDB_ID))	tblOff = atoi(opt->text().c_str());
+    }
     else if(a_path == "/prm/tbl")
     {
 	TConfig req;
 	string eid;
 	fieldStruct(req);
 	if(ctrChkNode(opt,"get",RWRW__,"root",SDB_ID,SEC_RD))
-	    for(unsigned i_r = 0; fieldSeek(i_r,req); i_r++)
+	{
+	    time_t upTo = time(NULL)+STD_INTERF_TM;
+	    bool firstRow = true;
+	    for(unsigned i_r = vmax(0,tblOff); time(NULL) < upTo && fieldSeek(i_r,req); i_r++, firstRow = false)
 	        for(unsigned i_f = 0; i_f < req.elem().fldSize(); i_f++)
 		{
 		    eid = req.elem().fldAt(i_f).name();
-		    if( i_r == 0 )	//Prepare columns
-			ctrMkNode("list",opt,-1,("/prm/tbl/"+eid).c_str(),"",RWRWR_);
+		    if(firstRow) ctrMkNode("list",opt,-1,("/prm/tbl/"+eid).c_str(),"",RWRWR_);
 		    opt->childGet(i_f)->childAdd("el")->setText(req.cfg(eid).getS());
 		}
+	    notFullShow = (time(NULL) >= upTo);
+	}
 	if(ctrChkNode(opt,"add",RWRW__,"root",SDB_ID,SEC_WR))
 	{
 	    for(unsigned i_f = 0; i_f < req.elem().fldSize(); i_f++)
