@@ -75,6 +75,13 @@ void TCntrNode::nodeDelAll( )
 	}
 }
 
+void TCntrNode::setNodeMode( char mode )
+{
+    hd_res.resRequestW();
+    m_flg = (m_flg&(~0x03))|(mode&0x03);
+    hd_res.resRelease();
+}
+
 XMLNode *TCntrNode::ctrId( XMLNode *inf, const string &name_id, bool noex )
 {
     int l_off = 0;
@@ -133,7 +140,7 @@ void TCntrNode::cntrCmd( XMLNode *opt, int lev, const string &ipath, int off )
 	    opt->setAttr("path",s_br);
 	    cntrCmdProc(opt);
 	    if( opt->attr("rez") != "0" )
-		throw TError("ContrItfc",_("%s:%s:> Control element <%s> error!"),opt->name().c_str(),path.c_str(),s_br.c_str());
+		throw TError("ContrItfc",_("%s:%s:> Control element '%s' error!"),opt->name().c_str(),path.c_str(),s_br.c_str());
 	}
     }
     catch(TError err)
@@ -151,77 +158,68 @@ void TCntrNode::cntrCmd( XMLNode *opt, int lev, const string &ipath, int off )
 //* Resource section                              *
 void TCntrNode::nodeEn( int flag )
 {
-    if( nodeMode() == Enable )	return;		//throw TError(nodePath().c_str(),"Node already enabled!");
-    if( nodeMode() != Disable )	throw TError(nodePath().c_str(),_("Node already in process!"));
+    if(nodeMode() == Enable)	return;		//throw TError(nodePath().c_str(),"Node already enabled!");
+    if(nodeMode() != Disable)	throw TError(nodePath().c_str(),_("Node already in process!"));
 
-    ResAlloc res( hd_res, true );
-    setNodeMode( MkEnable );
-    res.release( );
+    setNodeMode(MkEnable);
 
-    preEnable( flag );
+    preEnable(flag);
 
     TMap::iterator p;
-    for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
-	for( p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
-	    if( p->second->nodeMode( ) == Disable )
+    for(unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++)
+	for(p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p)
+	    if(p->second->nodeMode( ) == Disable)
 		p->second->nodeEn( flag );
 
-    res.request( true );
-    setNodeMode( Enable );
-    res.release( );
+    setNodeMode(Enable);
 
-    try{ postEnable( flag ); }
-    catch( TError err )	{ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+    try{ postEnable(flag); }
+    catch(TError err)	{ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 };
 
 void TCntrNode::nodeDis( long tm, int flag )
 {
-    if( nodeMode() == Disable )	return;		//throw TError(nodePath().c_str(),"Node already disabled!");
-    if( nodeMode() != Enable )	throw TError(nodePath().c_str(),_("Node already in process!"));
+    if(nodeMode() == Disable)	return;		//throw TError(nodePath().c_str(),"Node already disabled!");
+    if(nodeMode() != Enable)	throw TError(nodePath().c_str(),_("Node already in process!"));
 
-    preDisable( flag );
+    preDisable(flag);
 
-    ResAlloc res( hd_res, true );
-    setNodeMode( MkDisable );
-    res.release( );
+    setNodeMode(MkDisable);
 
     try
     {
 	TMap::iterator p;
-	for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
-	    for( p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
-		if( p->second->nodeMode() == Enable )
-		    p->second->nodeDis( tm, flag );
+	for(unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++)
+	    for(p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p)
+		if(p->second->nodeMode() == Enable)
+		    p->second->nodeDis(tm, flag);
 
-	//- Wait of free node -
+	//> Wait of free node
 	time_t t_cur = time(NULL);
 	while(1)
 	{
-	    if( !mUse )	break;
+	    if(!mUse)	break;
 #if OSC_DEBUG >= 1
             mess_debug(nodePath().c_str(),_("Waiting for freeing by %d users!"),mUse);
 #endif
 	    //Check timeout
-	    if( tm && time(NULL) > t_cur+tm)
+	    if(tm && time(NULL) > t_cur+tm)
 	    {
-		if( !TSYS::finalKill )
+		if(!TSYS::finalKill)
 		    throw TError(nodePath().c_str(),_("Timeouted of wait. Object is used by %d users. Free object first!"),mUse);
-		mess_err(nodePath().c_str(),_("Blocking node error. Inform developpers please!"));
+		mess_err(nodePath().c_str(),_("Blocking node error. Inform developers please!"));
 		break;
 	    }
-	    usleep( STD_WAIT_DELAY*1000 );
+	    usleep(STD_WAIT_DELAY*1000);
 	}
-	res.request( true );
-	setNodeMode( Disable );
-	modif( );
-	res.release( );
+
+	setNodeMode(Disable);
+	modif();
     }catch(TError err)
     {
 	mess_warning(err.cat.c_str(),_("Node disable error. Restore node enabling."));
-	res.request( true );
-	setNodeMode( Disable );
-	res.release( );
-	nodeEn( NodeRestore|(flag<<8) );
+	setNodeMode(Disable);
+	nodeEn(NodeRestore|(flag<<8));
 	throw;
     }
     try{ postDisable(flag); }
@@ -346,7 +344,7 @@ int8_t TCntrNode::grpId( const string &sid )
 
 TCntrNode::GrpEl &TCntrNode::grpAt( int8_t iid )
 {
-    if( iid < 0 || iid >= grpSize( ) )	throw TError(nodePath().c_str(),_("Branche group '%d' error."),iid);
+    if( iid < 0 || iid >= grpSize( ) )	throw TError(nodePath().c_str(),_("Branch group '%d' error."),iid);
     return (*chGrp)[iid];
 }
 
@@ -393,40 +391,40 @@ bool TCntrNode::chldPresent( int8_t igr, const string &name )
     return false;
 }
 
-void TCntrNode::chldAdd( int8_t igr, TCntrNode *node, int pos )
+void TCntrNode::chldAdd( int8_t igr, TCntrNode *node, int pos, bool noExp )
 {
     ResAlloc res(hd_res,false);
-    if(!chGrp || igr >= (int)chGrp->size()) throw TError(nodePath().c_str(),_("Group of childs %d error!"),igr);
-    if(nodeMode() != Enable)	throw TError(nodePath().c_str(),_("Node is not enabled!"));
+    if(!chGrp || igr >= (int)chGrp->size())
+    { delete node; throw TError(nodePath().c_str(),_("Group of childs %d error!"),igr); }
+    if(nodeMode() != Enable)
+    { delete node; throw TError(nodePath().c_str(),_("Node is not enabled!")); }
 
     TMap::iterator p;
-    if( TSYS::strNoSpace(node->nodeName()).empty() )
-    {
-	delete node;
-	throw TError(nodePath().c_str(),_("Add child id is empty!"));
-    }
+    if(TSYS::strNoSpace(node->nodeName()).empty())
+    { delete node; throw TError(nodePath().c_str(),_("Add child id is empty!")); }
 
-    if( (p = (*chGrp)[igr].elem.find(node->nodeName())) != (*chGrp)[igr].elem.end() )
+    if((p=(*chGrp)[igr].elem.find(node->nodeName())) != (*chGrp)[igr].elem.end())
     {
 	delete node;
-	if( p->second->nodeMode() == Disable )	p->second->nodeEn( TCntrNode::NodeRestore );
+	if(p->second->nodeMode() == Disable)	p->second->nodeEn(TCntrNode::NodeRestore);
+	if(!noExp) throw TError(nodePath().c_str(),_("Node '%s' is already present."),p->first.c_str());
 	return;
     }
 
     res.request(true);
     node->prev.node = this;
     node->prev.grp = igr;
-    if( (*chGrp)[igr].ordered )
+    if((*chGrp)[igr].ordered)
     {
 	pos = (pos < 0 || pos > (int)(*chGrp)[igr].elem.size()) ? (int)(*chGrp)[igr].elem.size() : pos;
 	node->mOi = pos;
-	for( p = (*chGrp)[igr].elem.begin(); p != (*chGrp)[igr].elem.end(); p++ )
-	    if( p->second->mOi >= pos ) p->second->mOi++;
+	for(p = (*chGrp)[igr].elem.begin(); p != (*chGrp)[igr].elem.end(); p++)
+	    if(p->second->mOi >= pos) p->second->mOi++;
     }
     (*chGrp)[igr].elem.insert(std::pair<string,TCntrNode*>(node->nodeName(),node));
     res.release();
 
-    if( node->nodeMode() == Disable )	node->nodeEn( TCntrNode::NodeConnect );
+    if(node->nodeMode() == Disable)	node->nodeEn(TCntrNode::NodeConnect);
 }
 
 void TCntrNode::chldDel( int8_t igr, const string &name, long tm, int flag, bool shDel )
@@ -435,11 +433,11 @@ void TCntrNode::chldDel( int8_t igr, const string &name, long tm, int flag, bool
     ResAlloc res(hd_res,false);
     if(!chGrp || igr >= (int)chGrp->size()) throw TError(nodePath().c_str(),_("Group of childs %d error!"),igr);
     if(!(nodeMode() == Enable || nodeMode() == Disable))
-        throw TError(nodePath().c_str(),_("Node is beign processed now!"));
+        throw TError(nodePath().c_str(),_("Node is begin processed now!"));
 
     TMap::iterator p = (*chGrp)[igr].elem.find(name);
     if(p == (*chGrp)[igr].elem.end())
-	throw TError(nodePath().c_str(),_("Child <%s> is not present!"), name.c_str());
+	throw TError(nodePath().c_str(),_("Child '%s' is not present!"), name.c_str());
 
     if( p->second->nodeMode() == Enable )
 	p->second->nodeDis( tm, (flag<<8)|(shDel?NodeShiftDel:0) );
@@ -460,17 +458,17 @@ void TCntrNode::chldDel( int8_t igr, const string &name, long tm, int flag, bool
     }
 }
 
-unsigned TCntrNode::nodeUse(  )
+unsigned TCntrNode::nodeUse( bool selfOnly )
 {
     ResAlloc res(hd_res,false);
-    if( nodeMode() == Disable )	throw TError(nodePath().c_str(),"Node is disabled!");
+    //if(nodeMode() == Disable)	throw TError(nodePath().c_str(),"Node is disabled!");
 
     unsigned i_use = mUse;
     TMap::iterator p;
-    for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
-     for( p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
-	if( p->second->nodeMode() != Disable )
-	    i_use+=p->second->nodeUse();
+    for(unsigned i_g = 0; !selfOnly && chGrp && i_g < chGrp->size(); i_g++)
+	for(p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p)
+	    if(p->second->nodeMode() != Disable)
+		i_use += p->second->nodeUse();
 
     return i_use;
 }
@@ -513,7 +511,7 @@ AutoHD<TCntrNode> TCntrNode::chldAt( int8_t igr, const string &name, const strin
 
     TMap::iterator p = (*chGrp)[igr].elem.find(name);
     if(p == (*chGrp)[igr].elem.end() || p->second->nodeMode() == Disable)
-	throw TError(nodePath().c_str(),_("Element <%s> is not present or disabled!"), name.c_str());
+	throw TError(nodePath().c_str(),_("Element '%s' is not present or disabled!"), name.c_str());
 
     return AutoHD<TCntrNode>(p->second,user);
 }
@@ -660,7 +658,7 @@ TVariant TCntrNode::objFuncCall( const string &iid, vector<TVariant> &prms, cons
     {
 	try
 	{
-	    AutoHD<TCntrNode> nd = nodeAt(prms[0].getS());
+	    AutoHD<TCntrNode> nd = nodeAt(prms[0].getS(),0,(prms.size()>=2 && prms[1].getS().size())?prms[1].getS()[0]:0);
 	    return new TCntrNodeObj(nd,user);
 	}
 	catch(TError)	{ }
@@ -673,6 +671,11 @@ TVariant TCntrNode::objFuncCall( const string &iid, vector<TVariant> &prms, cons
 	if(prev) return new TCntrNodeObj(AutoHD<TCntrNode>(prev), user);
 	return false;
     }
+    // string nodePath(string sep = "", bool from_root = true) - get the node path into OpenSCADA objects tree
+    //  sep - Separator symbol for separated path
+    //  from_root - path forming from root tree and do not include station ID.
+    if(iid == "nodePath")
+	return nodePath(((prms.size() && prms[0].getS().size()) ? prms[0].getS()[0] : 0), ((prms.size() >= 2) ? prms[1].getB() : true));
 
     throw TError(nodePath().c_str(),_("Function '%s' error or not enough parameters."),iid.c_str());
 }
@@ -714,7 +717,7 @@ XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const cha
 	if( obj1 ) { obj = obj1; continue; }
 	//int wofft = woff;
 	if( TSYS::pathLev(path,0,true,&woff).size() )
-	    throw TError("ContrItfc",_("Some tags on path <%s> are missed!"),req.c_str());
+	    throw TError("ContrItfc",_("Some tags on path '%s' are missed!"),req.c_str());
 	obj = obj->childIns(pos);
     }
     obj->setName(n_nd)->setAttr("id",reqt1)->setAttr("dscr",dscr)->setAttr("acs",TSYS::int2str(n_acs));
@@ -760,9 +763,9 @@ bool TCntrNode::ctrChkNode( XMLNode *nd, const char *cmd, int perm, const char *
 {
     if( nd->name() != cmd ) return false;
     if( ((char)perm&mode) != mode && SYS->security().at().access(nd->attr("user"),mode,user,grp,perm) != mode )
-	throw TError("ContrItfc",_("Error access to element <%s>!"),nd->attr("path").c_str());
+	throw TError("ContrItfc",_("Error access to element '%s'!"),nd->attr("path").c_str());
     if( warn && !atoi(nd->attr("force").c_str()) )
-	throw TError("warning",_("Element <%s> warning! %s"),nd->attr("path").c_str(),warn);
+	throw TError("warning",_("Element '%s' warning! %s"),nd->attr("path").c_str(),warn);
     nd->setAttr("rez","0");
     return true;
 }
@@ -825,8 +828,9 @@ void TCntrNode::cntrCmdProc( XMLNode *opt )
 	bool tblList = a_path == "/db/tblList";
 	vector<string> c_list;
 	SYS->db().at().dbList(c_list);
-	opt->childAdd("el")->setText(tblList ? _("*.*.<TableName>") : "*.*");
+	opt->childAdd("el")->setText(tblList ? _("*.*.[TableName]") : "*.*");
+	opt->childAdd("el")->setText(tblList ? _("<cfg>.[TableName]") : "<cfg>");
 	for(unsigned i_db = 0; i_db < c_list.size(); i_db++)
-	    opt->childAdd("el")->setText( tblList ? c_list[i_db]+"."+_("<TableName>") : c_list[i_db] );
+	    opt->childAdd("el")->setText( tblList ? c_list[i_db]+"."+_("[TableName]") : c_list[i_db] );
     }
 }
