@@ -123,23 +123,28 @@ string TController::tbl( )	{ return owner().owner().subId()+"_"+owner().modId();
 
 string TController::getStatus( )
 {
-    string rez;
-    if( startStat() )
+    string rez, mess;
+    if(startStat())
     {
 	rez = string("0:")+_("Started. ");
-	if( owner().redntAllow( ) && redntUse( ) )
+	if(owner().redntAllow() && redntUse())
 	{
-	    rez += _("Geting data from remote station: ");
+	    mess = _("Getting data from remote station: ");
 	    string rSt = mRedntSt.getVal();
-	    if( !rSt.empty() )
+	    if(!rSt.empty())
 	    {
-		int rOff = 0;
-		rez.replace(0,1,TSYS::strSepParse(rSt,0,':',&rOff));
-		rez.append(rSt.substr(rOff));
+		if(rSt.find(mess) == string::npos)
+		{
+		    int rOff = 0;
+		    rez.replace(0,1,TSYS::strSepParse(rSt,0,':',&rOff));
+		    mess.append(rSt.substr(rOff));
+		}
+		else mess = _("Your redundancy configuration wrong and controller often enable-disable redundancy!");
 	    }
+	    rez += mess;
 	}
     }
-    else if( enableStat() ) rez = string("1:")+_("Enabled. ");
+    else if(enableStat()) rez = string("1:")+_("Enabled. ");
     else rez = string("2:")+_("Disabled. ");
 
     return rez;
@@ -149,10 +154,11 @@ void TController::load_( )
 {
     if(!SYS->chkSelDB(DB())) return;
 
-    mess_info(nodePath().c_str(),_("Load controller's configs!"));
+    mess_info(nodePath().c_str(),_("Load controller's configurations!"));
 
     bool en_st_prev = en_st, run_st_prev = run_st;
 
+    cfgViewAll(true);
     SYS->db().at().dataGet(fullDB(),owner().nodePath()+"DAQ",*this);
 
     //> Load parameters if enabled
@@ -164,7 +170,7 @@ void TController::load_( )
 
 void TController::save_( )
 {
-    mess_info(nodePath().c_str(),_("Save controller's configs!"));
+    mess_info(nodePath().c_str(),_("Save controller's configurations!"));
 
     //> Update type controller bd record
     SYS->db().at().dataSet(fullDB(),owner().nodePath()+"DAQ",*this);
@@ -231,7 +237,7 @@ void TController::enable( )
 	    catch(TError err)
 	    {
 		mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
-		mess_warning(nodePath().c_str(),_("Enable parameter <%s> error."),prm_list[i_prm].c_str());
+		mess_warning(nodePath().c_str(),_("Enable parameter '%s' error."),prm_list[i_prm].c_str());
 		enErr = true;
 	    }
 
@@ -258,7 +264,7 @@ void TController::disable( )
 	    catch(TError err)
 	    {
 		mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
-		mess_warning(nodePath().c_str(),_("Disable parameter <%s> error."),prm_list[i_prm].c_str());
+		mess_warning(nodePath().c_str(),_("Disable parameter '%s' error."),prm_list[i_prm].c_str());
 	    }
 
     //> Disable for children
@@ -270,6 +276,8 @@ void TController::disable( )
 
 void TController::LoadParmCfg(  )
 {
+    map<string, bool>   itReg;
+
     //> Search and create new parameters
     for(unsigned i_tp = 0; i_tp < owner().tpPrmSize(); i_tp++)
     {
@@ -279,17 +287,20 @@ void TController::LoadParmCfg(  )
 	    TConfig c_el(&owner().tpPrmAt(i_tp));
 	    c_el.cfgViewAll(false);
 
+	    //>>> Search new into DB and Config-file
 	    for(int fld_cnt = 0; SYS->db().at().dataSeek(DB()+"."+cfg(owner().tpPrmAt(i_tp).db).getS(),
 					   owner().nodePath()+cfg(owner().tpPrmAt(i_tp).db).getS(),fld_cnt++,c_el); )
 	    {
 		try
 		{
-		    if( !present(c_el.cfg("SHIFR").getS()) ) add( c_el.cfg("SHIFR").getS(), i_tp );
+		    string shfr = c_el.cfg("SHIFR").getS();
+		    if(!present(shfr))	add(shfr, i_tp);
+		    itReg[shfr] = true;
 		}
 		catch(TError err)
 		{
 		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		    mess_err(nodePath().c_str(),_("Add parameter <%s> error."),c_el.cfg("SHIFR").getS().c_str());
+		    mess_err(nodePath().c_str(),_("Add parameter '%s' error."),c_el.cfg("SHIFR").getS().c_str());
 		}
 	    }
 	}catch(TError err)
@@ -299,7 +310,17 @@ void TController::LoadParmCfg(  )
 	}
     }
 
-    //- Force load present parameters -
+    //>>> Check for remove items removed from DB
+    if(!SYS->selDB().empty())
+    {
+	vector<string> it_ls;
+    	list(it_ls);
+    	for(unsigned i_it = 0; i_it < it_ls.size(); i_it++)
+    	    if(itReg.find(it_ls[i_it]) == itReg.end())
+            	del(it_ls[i_it]);
+    }
+
+    //> Force load present parameters
     vector<string> prm_ls;
     list(prm_ls);
     for(unsigned i_p = 0; i_p < prm_ls.size(); i_p++)
@@ -311,8 +332,7 @@ void TController::LoadParmCfg(  )
 
 void TController::add( const string &name, unsigned type )
 {
-    if( chldPresent(mPrm,name) ) return;
-    chldAdd(mPrm,ParamAttach( name, type ));
+    chldAdd(mPrm, ParamAttach(name, type));
 }
 
 TParamContr *TController::ParamAttach( const string &name, int type)
@@ -363,7 +383,7 @@ void TController::redntDataUpdate( )
 	    if(!vl.at().arch().freeStat() || vl.at().reqFlg()) { prmNd->childAdd("el")->setAttr("id",listV[iV]); rC++; }
 	    if(!vl.at().arch().freeStat())
 		prmNd->childAdd("ael")->setAttr("id",listV[iV])->setAttr("tm",TSYS::ll2str(vmax(vl.at().arch().at().end(""),
-						    TSYS::curTime()-(long long)(3.6e9*owner().owner().rdRestDtTm()))));
+						    TSYS::curTime()-(int64_t)(3.6e9*owner().owner().rdRestDtTm()))));
 	}
 	if(rC > listV.size()/2)
 	{
@@ -390,8 +410,8 @@ void TController::redntDataUpdate( )
 	    { vl.at().setS(aNd->text(),atoll(aNd->attr("tm").c_str()),true); vl.at().setReqFlg(false); }
 	    else if( aNd->name() == "ael" && !vl.at().arch().freeStat() && aNd->childSize() )
 	    {
-		long long btm = atoll(aNd->attr("tm").c_str());
-		long long per = atoll(aNd->attr("per").c_str());
+		int64_t btm = atoll(aNd->attr("tm").c_str());
+		int64_t per = atoll(aNd->attr("per").c_str());
 		TValBuf buf(vl.at().arch().at().valType(),0,per,false,true);
 		for(unsigned i_v = 0; i_v < aNd->childSize(); i_v++)
 		    buf.setS(aNd->childGet(i_v)->text(),btm+per*i_v);
@@ -401,6 +421,11 @@ void TController::redntDataUpdate( )
     }
 }
 
+void TController::alarmSet( const string &mess, int lev, const string &prm )
+{
+    message(("al"+owner().modId()+":"+id()+(prm.size()?("."+prm):"")).c_str(), lev, mess.c_str());
+}
+
 TVariant TController::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
 {
     // string name() - get controller name.
@@ -409,6 +434,28 @@ TVariant TController::objFuncCall( const string &iid, vector<TVariant> &prms, co
     if(iid == "descr")	return descr();
     // string status() - get controller status.
     if(iid == "status")	return getStatus();
+    // bool alarmSet(string mess, int lev = -5, string prm = "") - set alarm to message <mess> and level <lev> for parameter <prm>.
+    if(iid == "alarmSet" && prms.size() >= 1)
+    {
+	alarmSet(prms[0].getS(), (prms.size() >= 2) ? prms[1].getI() : -TMess::Crit, (prms.size() >= 3) ? prms[2].getS() : "");
+	return true;
+    }
+    // bool enable(bool newSt = EVAL) - get enable status or change it by argument 'newSt' assign.
+    if(iid == "enable")
+    {
+	if(prms.size())	{ prms[0].getB() ? enable() : disable(); }
+	return enableStat();
+    }
+    // bool start(bool newSt = EVAL) - get start status or change it by argument 'newSt' assign.
+    if(iid == "start")
+    {
+	if(prms.size())	{ prms[0].getB() ? start() : stop(); }
+	return startStat();
+    }
+
+    //> Configuration functions call
+    TVariant cfRez = objFunc(iid, prms, user);
+    if(!cfRez.isNull()) return cfRez;
 
     return TCntrNode::objFuncCall(iid,prms,user);
 }
@@ -431,12 +478,11 @@ void TController::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/cntr/st/db",_("Controller DB"),RWRWR_,"root",SDAQ_ID,4,
 		    "tp","str","dest","select","select","/db/list","help",TMess::labDB());
 	    }
-	    if(ctrMkNode("area",opt,-1,"/cntr/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/cntr/cfg",_("Configuration")))
 	    {
 		TConfig::cntrCmdMake(opt,"/cntr/cfg",0,"root",SDAQ_ID,RWRWR_);
-		//>> Append configuration properties
-		XMLNode *xt = ctrId(opt->childGet(0),"/cntr/cfg/REDNT_RUN",true);
-		if(xt) xt->setAttr("dest","select")->setAttr("select","/cntr/redRunLst");
+		ctrRemoveNode(opt,"/cntr/cfg/REDNT");
+		ctrRemoveNode(opt,"/cntr/cfg/REDNT_RUN");
 	    }
 	}
 	if(owner().tpPrmSize())
@@ -516,15 +562,5 @@ void TController::cntrCmdProc( XMLNode *opt )
 		if(owner().tpPrmAt(i_t).db == TSYS::pathLev(a_path,2))
 		     modifG( );
     }
-    else if(a_path == "/cntr/redRunLst" && ctrChkNode(opt))
-    {
-	opt->childAdd("el")->setAttr("id","<high>")->setText(_("<High level>"));
-	opt->childAdd("el")->setAttr("id","<low>")->setText(_("<Low level>"));
-	opt->childAdd("el")->setAttr("id","<optimal>")->setText(_("<Optimal>"));
-	vector<string> sls;
-	owner().owner().rdStList(sls);
-	for(unsigned i_s = 0; i_s < sls.size(); i_s++)
-	    opt->childAdd("el")->setAttr("id",sls[i_s])->setText(SYS->transport().at().extHostGet("*",sls[i_s]).name);
-    }
     else TCntrNode::cntrCmdProc(opt);
 }
