@@ -49,7 +49,8 @@ TDAQS::TDAQS( ) : TSubSYS(SDAQ_ID,_("Data acquisition"),true), el_err("Error"),
     el_tmpl.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
     el_tmpl.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
     el_tmpl.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::FullText|TCfg::TransltText,"200") );
-    el_tmpl.fldAdd( new TFld("PROGRAM",_("Template programm"),TFld::String,TCfg::TransltText,"10000") );
+    el_tmpl.fldAdd( new TFld("MAXCALCTM",_("Maximum calculate time (sec)"),TFld::Integer,TFld::NoFlag,"4","10","0;3600") );
+    el_tmpl.fldAdd( new TFld("PROGRAM",_("Template program"),TFld::String,TCfg::TransltText,"10000") );
 
     //> Parameter template IO DB structure
     el_tmpl_io.fldAdd( new TFld("TMPL_ID",_("Template ID"),TFld::String,TCfg::Key,"20") );
@@ -66,7 +67,7 @@ TDAQS::TDAQS( ) : TSubSYS(SDAQ_ID,_("Data acquisition"),true), el_err("Error"),
 
 TDAQS::~TDAQS( )
 {
-    if( prcStRd ) subStop();
+    if(prcStRd) subStop();
 
     nodeDelAll();
 }
@@ -128,6 +129,7 @@ void TDAQS::load_( )
 	}
     } while(next_opt != -1);
 
+    map<string, bool>   itReg;
     //> Load templates libraries of parameter
     try
     {
@@ -138,20 +140,23 @@ void TDAQS::load_( )
 
 	//>> Search into DB
 	SYS->db().at().dbList(db_ls,true);
+	db_ls.push_back("<cfg>");
 	for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+tmplLibTable(),"",lib_cnt++,c_el); )
+	    for(int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+tmplLibTable(),nodePath()+"tmplib",lib_cnt++,c_el); )
 	    {
 		string l_id = c_el.cfg("ID").getS();
 		if(!tmplLibPresent(l_id)) tmplLibReg(new TPrmTmplLib(l_id.c_str(),"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]));
+		itReg[l_id] = true;
 	    }
 
-	//>> Search into config file
-	if( SYS->chkSelDB("<cfg>") )
-	    for( int lib_cnt = 0; SYS->db().at().dataSeek("",nodePath()+"tmplib",lib_cnt++,c_el); )
-	    {
-		string l_id = c_el.cfg("ID").getS();
-		if(!tmplLibPresent(l_id)) tmplLibReg(new TPrmTmplLib(l_id.c_str(),"","*.*"));
-	    }
+	//>>> Check for remove items removed from DB
+        if(!SYS->selDB().empty())
+        {
+            tmplLibList(db_ls);
+            for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                if(itReg.find(db_ls[i_it]) == itReg.end() && SYS->chkSelDB(tmplLibAt(db_ls[i_it]).at().DB()))
+                    tmplLibUnreg(db_ls[i_it]);
+        }
     }catch( TError err )
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -170,39 +175,35 @@ void TDAQS::load_( )
 	    wmod = at(mod_ls[i_md]);
 	    TConfig g_cfg(&wmod.at());
 	    g_cfg.cfgViewAll(false);
+	    itReg.clear();
 
 	    //>> Search into DB and create new controllers
 	    SYS->db().at().dbList(db_ls,true);
+	    db_ls.push_back("<cfg>");
 	    for(unsigned i_db = 0; i_db < db_ls.size(); i_db++)
-		for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_"+wmod.at().modId(),"",fld_cnt++,g_cfg); )
+		for(int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_"+wmod.at().modId(),wmod.at().nodePath()+"DAQ",fld_cnt++,g_cfg); )
 		{
 		    string m_id = g_cfg.cfg("ID").getS();
 		    try
 		    {
-			if( !wmod.at().present(m_id) )
-			    wmod.at().add(m_id,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+			if(!wmod.at().present(m_id)) wmod.at().add(m_id,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+			itReg[m_id] = true;
 		    }catch(TError err)
 		    {
 			mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-			mess_err(nodePath().c_str(),_("Add controller <%s> error."),m_id.c_str());
-		    }
-		}
-	    //>> Search into config file and create new controllers
-	    if( SYS->chkSelDB("<cfg>") )
-		for( int fld_cnt=0; SYS->db().at().dataSeek("",wmod.at().nodePath()+"DAQ",fld_cnt++,g_cfg); )
-		{
-		    string m_id = g_cfg.cfg("ID").getS();
-		    try { if( !wmod.at().present(m_id) ) wmod.at().add(m_id,"*.*"); }
-		    catch(TError err)
-		    {
-			mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-			mess_err(nodePath().c_str(),_("Add controller <%s> error."),m_id.c_str());
+			mess_err(wmod.at().nodePath().c_str(),_("Add controller '%s' error."),m_id.c_str());
 		    }
 		}
+
+	    //>>> Check for remove items removed from DB
+	    wmod.at().list(db_ls);
+	    for(unsigned i_it = 0; i_it < db_ls.size(); i_it++)
+                if(itReg.find(db_ls[i_it]) == itReg.end() && SYS->chkSelDB(wmod.at().at(db_ls[i_it]).at().DB()))
+                    wmod.at().del(db_ls[i_it]);
 	}
     }catch(TError err) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 
-    //> Load parameters from config file and SYS DB
+    //> Load parameters from config-file and SYS DB
     setRdStLevel(atoi(TBDS::genDBGet(nodePath()+"RdStLevel",TSYS::int2str(rdStLevel())).c_str()));
     setRdTaskPer(atof(TBDS::genDBGet(nodePath()+"RdTaskPer",TSYS::real2str(rdTaskPer())).c_str()));
     setRdRestConnTm(atoi(TBDS::genDBGet(nodePath()+"RdRestConnTm",TSYS::int2str(rdRestConnTm())).c_str()));
@@ -229,6 +230,88 @@ void TDAQS::save_( )
     res.release();
 }
 
+TVariant TDAQS::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    // bool funcCall(string progLang, TVarObj args, string prog) - call function text <prog> whith arguments <args> for program language <progLang>.
+    //		Return "true" on well call.
+    //  progLang - program procedure language;
+    //  args - function arguments;
+    //  prog - function text.
+    if(iid == "funcCall" && prms.size() >= 3 && prms[1].type() == TVariant::Object)
+    {
+	string langMod = TSYS::strParse(prms[0].getS(),0,".");
+	if(!modPresent(langMod)) return false;
+	TVariant aVal;
+	TVarObj *args = prms[1].getO();
+
+	try
+	{
+	    //> Prepare arguments structure
+	    TFunction argStr("<auto>");
+	    vector<string> als;
+	    args->propList(als);
+	    for(unsigned i_a = 0; i_a < als.size(); i_a++)
+	    {
+		aVal = args->propGet(als[i_a]);
+		IO::Type tp = IO::String;
+		switch(aVal.type())
+		{
+        	    case TVariant::Boolean:	tp = IO::Boolean;	break;
+        	    case TVariant::Integer:	tp = IO::Integer;	break;
+		    case TVariant::Real:	tp = IO::Real;		break;
+        	    case TVariant::String:	tp = IO::String;	break;
+        	    case TVariant::Object:	tp = IO::Object;	break;
+		    default:	break;
+		}
+		argStr.ioAdd(new IO(als[i_a].c_str(),als[i_a].c_str(),tp,IO::Default));
+	    }
+
+	    //> Get function id and compile.
+	    string faddr = at(langMod).at().compileFunc(TSYS::strParse(prms[0].getS(),1,"."), argStr, prms[2].getS());
+	    AutoHD<TFunction> wFnc = SYS->nodeAt(faddr);
+	    TValFunc wCtx("UserFunc", &wFnc.at(), true, user);
+
+	    //> Load values
+	    for(unsigned i_a = 0; i_a < als.size(); i_a++)
+		switch((aVal=args->propGet(als[i_a])).type())
+		{
+        	    case TVariant::Boolean:	wCtx.setB(i_a, aVal.getB());	break;
+        	    case TVariant::Integer:	wCtx.setI(i_a, aVal.getI());	break;
+		    case TVariant::Real:	wCtx.setR(i_a, aVal.getR());	break;
+        	    case TVariant::String:	wCtx.setS(i_a, aVal.getS());	break;
+        	    case TVariant::Object:	wCtx.setO(i_a, aVal.getO());	break;
+		    default:	break;
+		}
+
+	    //> Call function
+	    wCtx.calc();
+
+	    //> Place call result and remove function object.
+	    for(int i_a = 0; i_a < wCtx.ioSize(); i_a++)
+		switch(wCtx.ioType(i_a))
+		{
+        	    case IO::Boolean:	args->propSet(als[i_a], wCtx.getB(i_a));	break;
+        	    case IO::Integer:	args->propSet(als[i_a], wCtx.getI(i_a));	break;
+		    case IO::Real:	args->propSet(als[i_a], wCtx.getR(i_a));	break;
+        	    case IO::String:	args->propSet(als[i_a], wCtx.getS(i_a));	break;
+        	    case IO::Object:	args->propSet(als[i_a], wCtx.getO(i_a));	break;
+		    default:	break;
+		}
+
+	    //> Remove compiled function object
+	    wCtx.setFunc(NULL);
+	    wFnc.free();
+	    SYS->nodeDel(faddr);
+
+	    return true;
+	}catch(TError err) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+
+	return false;
+    }
+
+    return TCntrNode::objFuncCall(iid,prms,user);
+}
+
 void TDAQS::subStart(  )
 {
 #if OSC_DEBUG >= 1
@@ -251,7 +334,7 @@ void TDAQS::subStart(  )
 		if(try_cnt)
 		{
 		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		    mess_err(nodePath().c_str(),_("Start template library <%s> error."),tmpl_lst[i_lb].c_str());
+		    mess_err(nodePath().c_str(),_("Start template library '%s' error."),tmpl_lst[i_lb].c_str());
 		}
 		reply = true;
 	    }
@@ -272,7 +355,7 @@ void TDAQS::subStart(  )
 			if( try_cnt )
 			{
 			    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-			    mess_err(nodePath().c_str(),_("Enable controller <%s> error."),(m_l[i_m]+"."+c_l[i_c]).c_str());
+			    mess_err(nodePath().c_str(),_("Enable controller '%s' error."),(m_l[i_m]+"."+c_l[i_c]).c_str());
 			}
 			reply = true;
 		    }
@@ -286,7 +369,7 @@ void TDAQS::subStart(  )
     if(!SYS->archive().at().subStartStat() || !SYS->stopSignal()) SYS->archive().at().subStart( );
 
     //> Redundant task start
-    if( !prcStRd ) SYS->taskCreate( nodePath('.',true)+".redundant", 5, TDAQS::RdTask, this, &prcStRd );
+    if(!prcStRd) SYS->taskCreate(nodePath('.',true)+".redundant", 5, TDAQS::RdTask, this);
 
     //> Controllers start
     TSubSYS::subStart( );
@@ -298,7 +381,7 @@ void TDAQS::subStop( )
     mess_debug(nodePath().c_str(),_("Stop subsystem."));
 #endif
 
-    if( prcStRd ) SYS->taskDestroy( nodePath('.',true)+".redundant", &prcStRd, &endrunRd );
+    if(prcStRd) SYS->taskDestroy(nodePath('.',true)+".redundant", &endrunRd);
 
     vector<string> m_l;
 
@@ -316,7 +399,7 @@ void TDAQS::subStop( )
 		catch(TError err)
 		{
 		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		    mess_err(nodePath().c_str(),_("Stop controller <%s> error."),(m_l[i_m]+"."+c_l[i_c]).c_str());
+		    mess_err(nodePath().c_str(),_("Stop controller '%s' error."),(m_l[i_m]+"."+c_l[i_c]).c_str());
 		}
 	}
     }
@@ -333,7 +416,7 @@ void TDAQS::subStop( )
 		catch(TError err)
 		{
 		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-		    mess_err(nodePath().c_str(),_("Disable controller <%s> error."),(m_l[i_m]+"."+c_l[i_c]).c_str());
+		    mess_err(nodePath().c_str(),_("Disable controller '%s' error."),(m_l[i_m]+"."+c_l[i_c]).c_str());
 		}
 	}
     }
@@ -346,6 +429,24 @@ void TDAQS::subStop( )
     TSubSYS::subStop( );
 }
 
+AutoHD<TValue> TDAQS::prmAt( const string &path, char sep, bool noex )
+{
+    AutoHD<TValue> rez;
+    try { rez = nodeAt(path, 0, sep); }
+    catch(TError err) { if(!noex) throw; }
+
+    return rez;
+}
+
+AutoHD<TVal> TDAQS::attrAt( const string &path, char sep, bool noex )
+{
+    AutoHD<TVal> rez;
+    try { rez = nodeAt(path, 0, sep); }
+    catch(TError err) { if(!noex) throw; }
+
+    return rez;
+}
+
 bool TDAQS::rdActive( )
 {
     ResAlloc res(nodeRes(),false);
@@ -408,7 +509,7 @@ void *TDAQS::RdTask( void *param )
     while( !daq.endrunRd )
     try
     {
-	long long work_tm = SYS->curTime();
+	int64_t work_tm = SYS->curTime();
 
 	//> Update wait time for dead stations and process connections to stations
 	ResAlloc res(daq.nodeRes(),false);
@@ -504,7 +605,7 @@ void *TDAQS::RdTask( void *param )
 
 	daq.mRdPrcTm = SYS->curTime()-work_tm;
 
-	TSYS::taskSleep((long long)(daq.rdTaskPer()*1e9));
+	TSYS::taskSleep((int64_t)(daq.rdTaskPer()*1e9));
     } catch( TError err ) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 
     daq.prcStRd = false;
@@ -516,8 +617,8 @@ string TDAQS::optDescr( )
 {
     return TSYS::strMess(_(
 	"=================== Subsystem \"Data acquisition\" options ================\n"
-	"------------ Parameters of section <%s> in config file -----------\n"
-	"RdStLevel    <lev>  The curent station redundant level.\n"
+	"------------ Parameters of section '%s' in config-file -----------\n"
+	"RdStLevel    <lev>  The current station redundant level.\n"
 	"RdTaskPer    <s>    The redundant task call period.\n"
 	"RdRestConnTm <s>    Restore connection timeout to dead reserve stations.\n"
 	"RdRestDtTm   <hour> Restore data archive depth from a reserve station after deadline.\n"
@@ -551,7 +652,7 @@ void TDAQS::cntrCmdProc( XMLNode *opt )
     {
 	TSubSYS::cntrCmdProc(opt);
 	ctrMkNode("grp",opt,-1,"/br/tmplb_",_("Template library"),RWRWR_,"root",SDAQ_ID,2,"idm","1","idSz","20");
-	if( ctrMkNode("area",opt,0,"/redund",_("Redundance")) )
+	if( ctrMkNode("area",opt,0,"/redund",_("Redundancy")) )
 	{
 	    ctrMkNode("fld",opt,-1,"/redund/status",_("Status"),R_R_R_,"root",SDAQ_ID,1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/redund/statLev",_("Station level"),RWRWR_,"root",SDAQ_ID,1,"tp","dec");
