@@ -27,8 +27,8 @@ using namespace OSCADA;
 //*************************************************
 //* TPrmTempl                                     *
 //*************************************************
-TPrmTempl::TPrmTempl( const char *iid, const char *iname ) :
-    TFunction(string("tmpl_")+iid), TConfig(&SYS->daq().at().tplE()), m_id(cfg("ID").getSd()), m_prog(cfg("PROGRAM").getSd())
+TPrmTempl::TPrmTempl( const string &iid, const string &iname ) :
+    TFunction("tmpl_"+iid), TConfig(&SYS->daq().at().tplE()), m_id(cfg("ID").getSd()), m_prog(cfg("PROGRAM").getSd())
 {
     m_id = iid;
     setName(iname);
@@ -92,17 +92,26 @@ string TPrmTempl::descr( )		{ return cfg("DESCR").getS(); }
 
 void TPrmTempl::setDescr( const string &idsc )	{ cfg("DESCR").setS(idsc); modif(); }
 
-string TPrmTempl::progLang()		{ return m_prog.substr(0,m_prog.find("\n")); }
+int TPrmTempl::maxCalcTm( )		{ return cfg("MAXCALCTM").getI(); }
+
+void TPrmTempl::setMaxCalcTm( int vl )	{ cfg("MAXCALCTM").setI(vl); modif(); }
+
+string TPrmTempl::progLang()
+{
+    string tPrg = m_prog.getVal();
+    return tPrg.substr(0,tPrg.find("\n"));
+}
 
 string TPrmTempl::prog( )
 {
-    size_t lngEnd = m_prog.find("\n");
-    return m_prog.substr( (lngEnd==string::npos)?0:lngEnd+1 );
+    string tPrg = m_prog.getVal();
+    size_t lngEnd = tPrg.find("\n");
+    return tPrg.substr((lngEnd==string::npos)?0:lngEnd+1);
 }
 
 void TPrmTempl::setProgLang( const string &ilng )
 {
-    if( startStat() ) setStart(false);
+    if(startStat()) setStart(false);
 
     m_prog = ilng+"\n"+prog();
     modif();
@@ -110,7 +119,7 @@ void TPrmTempl::setProgLang( const string &ilng )
 
 void TPrmTempl::setProg( const string &iprg )
 {
-    if( startStat() ) setStart(false);
+    if(startStat()) setStart(false);
 
     m_prog = progLang()+"\n"+iprg;
     modif();
@@ -118,22 +127,28 @@ void TPrmTempl::setProg( const string &iprg )
 
 void TPrmTempl::setStart( bool vl )
 {
-    if( startStat() == vl ) return;
+    if(startStat() == vl) return;
     if(vl)
     {
+	//> Check for doubled attributes clear
+	std::map<string,bool> ioIds;
+	for(int id = 0; id < ioSize(); )
+	    if(ioIds.find(io(id)->id()) != ioIds.end()) { ioDel(id); modif(); }
+	    else { ioIds[io(id)->id()] = true; id++; }
+
 	//> Compile new function
 	if(prog().size())
 	    work_prog = SYS->daq().at().at(TSYS::strSepParse(progLang(),0,'.')).at().
-					compileFunc(TSYS::strSepParse(progLang(),1,'.'),*this,prog());
+					compileFunc(TSYS::strSepParse(progLang(),1,'.'),*this,prog(),"",maxCalcTm());
     }
     TFunction::setStart(vl);
 }
 
 AutoHD<TFunction> TPrmTempl::func()
 {
-    if(!startStat())	throw TError(nodePath().c_str(),_("Tempate is disabled."));
+    if(!startStat())	throw TError(nodePath().c_str(),_("Template is disabled."));
     if(!prog().size())	return AutoHD<TFunction>(this);
-    return SYS->nodeAt(work_prog,1);
+    return SYS->nodeAt(work_prog);
 }
 
 void TPrmTempl::load_( )
@@ -147,7 +162,7 @@ void TPrmTempl::load_( )
     vector<string> u_pos;
     TConfig cfg(&owner().owner().tplIOE());
     cfg.cfg("TMPL_ID").setS(id(),true);
-    for( int io_cnt = 0; SYS->db().at().dataSeek(owner().fullDB()+"_io",owner().owner().nodePath()+owner().tbl()+"_io",io_cnt++,cfg); )
+    for(int io_cnt = 0; SYS->db().at().dataSeek(owner().fullDB()+"_io",owner().owner().nodePath()+owner().tbl()+"_io",io_cnt++,cfg); )
     {
 	string sid = cfg.cfg("ID").getS();
 
@@ -206,10 +221,10 @@ void TPrmTempl::save_( )
     }
     //> Clear IO
     cfg.cfgViewAll(false);
-    for( int fld_cnt=0; SYS->db().at().dataSeek(w_db+"_io",w_cfgpath+"_io",fld_cnt++,cfg); )
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(w_db+"_io",w_cfgpath+"_io",fld_cnt++,cfg); )
     {
 	string sio = cfg.cfg("ID").getS( );
-	if( ioId(sio) < 0 || io(ioId(sio))->flg()&TPrmTempl::LockAttr )
+	if(ioId(sio) < 0 || io(ioId(sio))->flg()&TPrmTempl::LockAttr)
 	{
 	    SYS->db().at().dataDel(w_db+"_io",w_cfgpath+"_io",cfg,true);
 	    fld_cnt--;
@@ -222,6 +237,15 @@ void TPrmTempl::preIOCfgChange()
     if(startStat()) setStart(false);
 }
 
+TVariant TPrmTempl::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    //> Configuration functions call
+    TVariant cfRez = objFunc(iid, prms, user);
+    if(!cfRez.isNull()) return cfRez;
+
+    return TCntrNode::objFuncCall(iid, prms, user);
+}
+
 void TPrmTempl::cntrCmdProc( XMLNode *opt )
 {
     //> Get page info
@@ -236,11 +260,12 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/tmpl/st/st",_("Accessing"),RWRWR_,"root",SDAQ_ID,1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/tmpl/st/use",_("Used"),R_R_R_,"root",SDAQ_ID,1,"tp","dec");
 	    }
-	    if(ctrMkNode("area",opt,-1,"/tmpl/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/tmpl/cfg",_("Configuration")))
 	    {
 		ctrMkNode("fld",opt,-1,"/tmpl/cfg/id",_("Id"),R_R_R_,"root",SDAQ_ID,1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/tmpl/cfg/name",_("Name"),RWRWR_,"root",SDAQ_ID,2,"tp","str","len","50");
 		ctrMkNode("fld",opt,-1,"/tmpl/cfg/descr",_("Description"),RWRWR_,"root",SDAQ_ID,3,"tp","str","cols","100","rows","4");
+		ctrMkNode("fld",opt,-1,"/tmpl/cfg/m_calc_tm",_("Maximum calculate time (sec)"),RWRWR_,"root",SDAQ_ID,3,"tp","dec","min","0","max","3600");
 	    }
 	}
 	if(ctrMkNode("area",opt,-1,"/io",_("IO")))
@@ -250,8 +275,8 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/io/io/0",_("Id"),RWRWR_,"root",SDAQ_ID,1,"tp","str");
 		ctrMkNode("list",opt,-1,"/io/io/1",_("Name"),RWRWR_,"root",SDAQ_ID,1,"tp","str");
 		ctrMkNode("list",opt,-1,"/io/io/2",_("Type"),RWRWR_,"root",SDAQ_ID,5,"tp","dec","idm","1","dest","select",
-		    "sel_id",(TSYS::int2str(IO::Real)+";"+TSYS::int2str(IO::Integer)+";"+TSYS::int2str(IO::Boolean)+";"+TSYS::int2str(IO::String)).c_str(),
-		    "sel_list",_("Real;Integer;Boolean;String"));
+		    "sel_id",(TSYS::int2str(IO::Real)+";"+TSYS::int2str(IO::Integer)+";"+TSYS::int2str(IO::Boolean)+";"+TSYS::int2str(IO::String)+";"+TSYS::int2str(IO::Object)).c_str(),
+		    "sel_list",_("Real;Integer;Boolean;String;Object"));
 		ctrMkNode("list",opt,-1,"/io/io/3",_("Mode"),RWRWR_,"root",SDAQ_ID,5,"tp","dec","idm","1","dest","select",
 		    "sel_id",(TSYS::int2str(IO::Default)+";"+TSYS::int2str(IO::Output)).c_str(),"sel_list",_("Input;Output"));
 		ctrMkNode("list",opt,-1,"/io/io/4",_("Attribute"),RWRWR_,"root",SDAQ_ID,5,"tp","dec","idm","1","dest","select",
@@ -262,8 +287,8 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 		    "sel_list",_("Constant;Public constant;Link"));
 		ctrMkNode("list",opt,-1,"/io/io/6",_("Value"),RWRWR_,"root",SDAQ_ID,1,"tp","str");
 	    }
-	    ctrMkNode("fld",opt,-1,"/io/prog_lang",_("Programm language"),RWRW__,"root",SDAQ_ID,3,"tp","str","dest","sel_ed","select","/io/plang_ls");
-	    ctrMkNode("fld",opt,-1,"/io/prog",_("Programm"),RWRW__,"root",SDAQ_ID,3,"tp","str","rows","10","SnthHgl","1");
+	    ctrMkNode("fld",opt,-1,"/io/prog_lang",_("Program language"),RWRW__,"root",SDAQ_ID,3,"tp","str","dest","sel_ed","select","/io/plang_ls");
+	    ctrMkNode("fld",opt,-1,"/io/prog",_("Program"),RWRW__,"root",SDAQ_ID,3,"tp","str","rows","10","SnthHgl","1");
 	}
 	return;
     }
@@ -288,6 +313,11 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD))	opt->setText(descr());
 	if(ctrChkNode(opt,"set",RWRWR_,"root",SDAQ_ID,SEC_WR))	setDescr(opt->text());
     }
+    else if(a_path == "/tmpl/cfg/m_calc_tm")
+    {
+        if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD))  opt->setText(TSYS::int2str(maxCalcTm()));
+        if(ctrChkNode(opt,"set",RWRWR_,"root",SDAQ_ID,SEC_WR))  setMaxCalcTm(atoi(opt->text().c_str()));
+    }
     else if(a_path == "/io/io")
     {
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD))
@@ -392,7 +422,7 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 //*************************************************
 //* TPrmTmplLib                                   *
 //*************************************************
-TPrmTmplLib::TPrmTmplLib( const char *id, const char *name, const string &lib_db ) :
+TPrmTmplLib::TPrmTmplLib( const string &id, const string &name, const string &lib_db ) :
     TConfig(&SYS->daq().at().elLib()), run_st(false), m_id(cfg("ID").getSd()), m_db(cfg("DB").getSd()), work_lib_db(lib_db)
 {
     m_id = id;
@@ -411,7 +441,7 @@ TCntrNode &TPrmTmplLib::operator=( TCntrNode &node )
     TPrmTmplLib *src_n = dynamic_cast<TPrmTmplLib*>(&node);
     if( !src_n ) return *this;
 
-    //- Configuration copy -
+    //> Configuration copy
     string tid = id();
     *(TConfig*)this = *(TConfig*)src_n;
     m_id = tid;
@@ -440,7 +470,7 @@ void TPrmTmplLib::postDisable(int flag)
     if( flag )
     {
 	//> Delete libraries record
-	SYS->db().at().dataDel(work_lib_db+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this,true);
+	SYS->db().at().dataDel(work_lib_db.getVal()+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this,true);
 
 	//> Delete temlate librarie's DBs
 	SYS->db().at().open(fullDB());
@@ -463,24 +493,36 @@ void TPrmTmplLib::setDescr( const string &vl )	{ cfg("DESCR").setS(vl); modif();
 
 void TPrmTmplLib::setFullDB( const string &vl )
 {
-    work_lib_db = TSYS::strSepParse(vl,0,'.')+"."+TSYS::strSepParse(vl,1,'.');
-    m_db = TSYS::strSepParse(vl,2,'.');
+    size_t dpos = vl.rfind(".");
+    work_lib_db = (dpos!=string::npos) ? vl.substr(0,dpos) : "";
+    m_db = (dpos!=string::npos) ? vl.substr(dpos+1) : "";
     modifG();
 }
 
 void TPrmTmplLib::load_( )
 {
-    if( !SYS->chkSelDB(DB()) ) return;
+    if(!SYS->chkSelDB(DB())) return;
 
     SYS->db().at().dataGet(DB()+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this);
 
     //> Load templates
+    map<string, bool>	itReg;
     TConfig c_el(&owner().tplE());
     c_el.cfgViewAll(false);
-    for( int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),owner().nodePath()+tbl(), fld_cnt++,c_el); )
+    for(int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),owner().nodePath()+tbl(), fld_cnt++,c_el); )
     {
 	string f_id = c_el.cfg("ID").getS();
-	if( !present(f_id) )	add(f_id.c_str());
+	if(!present(f_id)) add(f_id.c_str());
+	itReg[f_id] = true;
+    }
+
+    //>>> Check for remove items removed from DB
+    if(!SYS->selDB().empty())
+    {
+	vector<string> it_ls;
+        list(it_ls);
+        for(unsigned i_it = 0; i_it < it_ls.size(); i_it++)
+            if(itReg.find(it_ls[i_it]) == itReg.end())	del(it_ls[i_it]);
     }
 }
 
@@ -508,9 +550,18 @@ void TPrmTmplLib::start( bool val )
     if( isErr )	throw TError(nodePath().c_str(),_("Some templates start error."));
 }
 
-void TPrmTmplLib::add( const char *id, const char *name )
+void TPrmTmplLib::add( const string &id, const string &name )
 {
-    chldAdd(m_ptmpl,new TPrmTempl(id,name));
+    chldAdd(m_ptmpl, new TPrmTempl(id,name));
+}
+
+TVariant TPrmTmplLib::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
+{
+    //> Configuration functions call
+    TVariant cfRez = objFunc(iid, prms, user);
+    if(!cfRez.isNull()) return cfRez;
+
+    return TCntrNode::objFuncCall(iid, prms, user);
 }
 
 void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
@@ -530,7 +581,7 @@ void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/lib/st/db",_("Library BD"),RWRWR_,"root",SDAQ_ID,4,"tp","str","dest","sel_ed","select","/db/tblList",
 		    "help",_("DB address in format [<DB module>.<DB name>.<Table name>].\nFor use main work DB set '*.*'."));
 	    }
-	    if(ctrMkNode("area",opt,-1,"/lib/cfg",_("Config")))
+	    if(ctrMkNode("area",opt,-1,"/lib/cfg",_("Configuration")))
 	    {
 		ctrMkNode("fld",opt,-1,"/lib/cfg/id",_("Id"),R_R_R_,"root",SDAQ_ID,1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/lib/cfg/name",_("Name"),RWRWR_,"root",SDAQ_ID,2,"tp","str","len","50");
