@@ -65,19 +65,19 @@ void TValue::addFld( TElem *el, unsigned id_val )
 
 void TValue::delFld( TElem *el, unsigned id_val )
 {
-    if( nodeMode() == TCntrNode::Enable && chldPresent(m_vl,el->fldAt(id_val).name()) ) 
+    if( nodeMode() == TCntrNode::Enable && chldPresent(m_vl,el->fldAt(id_val).name()) )
 	chldDel(m_vl,el->fldAt(id_val).name());
 }
 
 void TValue::setVlCfg( TConfig *cfg )
 {
     vector<string> list;
-    //> Detach old configs
-    if( mCfg )
+    //> Detach old configurations
+    if(mCfg)
     {
-	mCfg->cfgList( list );
-	for( unsigned i_cf = 0; i_cf < list.size(); i_cf++ )
-	    if( !(mCfg->cfg(list[i_cf]).fld().flg()&TCfg::NoVal) && vlPresent(list[i_cf]) )
+	mCfg->cfgList(list);
+	for(unsigned i_cf = 0; i_cf < list.size(); i_cf++)
+	    if(!(mCfg->cfg(list[i_cf]).fld().flg()&TCfg::NoVal) && vlPresent(list[i_cf]))
 	    {
 		chldDel(m_vl,list[i_cf]);
 		l_cfg--;
@@ -88,8 +88,8 @@ void TValue::setVlCfg( TConfig *cfg )
     if( cfg )
     {
 	cfg->cfgList( list );
-	for( unsigned i_cf = 0; i_cf < list.size(); i_cf++ )
-	    if( !(cfg->cfg(list[i_cf]).fld().flg()&TCfg::NoVal) && !vlPresent(list[i_cf]) )
+	for(unsigned i_cf = 0; i_cf < list.size(); i_cf++)
+	    if(!(cfg->cfg(list[i_cf]).fld().flg()&TCfg::NoVal) && !vlPresent(list[i_cf]))
 		//chldAdd(m_vl, new TVal(cfg->cfg(list[i_cf]),this),l_cfg++);
 	    {
 		TVal *vl = vlNew();
@@ -127,7 +127,7 @@ void TValue::vlElemDet( TElem *ValEl )
 	    elem.erase(elem.begin()+i_e);
 	    return;
 	}
-    //throw TError(nodePath().c_str(),"Element <%s> no present!",ValEl->elName().c_str());
+    //throw TError(nodePath().c_str(),"Element '%s' no present!",ValEl->elName().c_str());
 }
 
 TElem &TValue::vlElem( const string &name )
@@ -135,7 +135,7 @@ TElem &TValue::vlElem( const string &name )
     for(unsigned i_e = 0; i_e < elem.size(); i_e++)
 	if( elem[i_e]->elName() == name )
 	    return *elem[i_e];
-    throw TError(nodePath().c_str(),"Element <%s> is not present!",name.c_str());
+    throw TError(nodePath().c_str(),"Element '%s' is not present!",name.c_str());
 }
 
 void TValue::cntrCmdProc( XMLNode *opt )
@@ -163,7 +163,7 @@ void TValue::cntrCmdProc( XMLNode *opt )
 	}
 	if(ctrChkNode(opt,"get",RWRWRW,"root",SDAQ_ID,SEC_RD))		//All attributes values
 	{
-	    long long vtm;
+	    int64_t vtm;
 	    string svl;
 	    AutoHD<TVal> vl;
 	    XMLNode *aNd;
@@ -201,10 +201,10 @@ void TValue::cntrCmdProc( XMLNode *opt )
 		if(vl.at().arch().freeStat()) { opt->childDel(aNd); i_a--; continue; }
 
 		AutoHD<TVArchive> arch = vl.at().arch();
-		long long vper = arch.at().period(BUF_ARCH_NM);
-		long long reqBeg = (atoll(aNd->attr("tm").c_str())/vper+1)*vper;
-		long long vbeg = vmax(reqBeg,arch.at().begin(BUF_ARCH_NM));
-		long long vend = arch.at().end(BUF_ARCH_NM);
+		int64_t vper = arch.at().period(BUF_ARCH_NM);
+		int64_t reqBeg = (atoll(aNd->attr("tm").c_str())/vper+1)*vper;
+		int64_t vbeg = vmax(reqBeg,arch.at().begin(BUF_ARCH_NM));
+		int64_t vend = arch.at().end(BUF_ARCH_NM);
 
 		//>>> Longing to equivalent archivators
 		if(vbeg == arch.at().begin(BUF_ARCH_NM))
@@ -341,14 +341,14 @@ void TValue::cntrCmdProc( XMLNode *opt )
 		string rez_nm = a_nm;
 		for(int p_cnt = 0; SYS->archive().at().valPresent(rez_nm); p_cnt++)
 		{
+		    if(SYS->archive().at().valAt(rez_nm).at().srcData() == vlAt(attr).at().nodePath('.',true)) break;
 		    rez_nm = a_nm+TSYS::int2str(p_cnt);
 		    if(rez_nm.size() > 20) rez_nm = a_nm.substr(0,a_nm.size()-(rez_nm.size()-20))+TSYS::int2str(p_cnt);
 		}
 		//>>>> Create new archive
-		SYS->archive().at().valAdd(rez_nm);
+		if(!SYS->archive().at().valPresent(rez_nm)) SYS->archive().at().valAdd(rez_nm);
 		SYS->archive().at().valAt(rez_nm).at().setValType(vlAt(attr).at().fld().type());
-		SYS->archive().at().valAt(rez_nm).at().setSrcMode(TVArchive::PassiveAttr,
-		    vlAt(attr).at().nodePath('.').substr(SYS->id().size()+1));
+		SYS->archive().at().valAt(rez_nm).at().setSrcMode(TVArchive::PassiveAttr,vlAt(attr).at().nodePath('.',true));
 		SYS->archive().at().valAt(rez_nm).at().setToStart(true);
 		SYS->archive().at().valAt(rez_nm).at().start();
 		vlArchMake(vlAt(attr).at());
@@ -432,6 +432,7 @@ void TVal::setFld( TFld &fld )
 	    val.val_r = src.fld->def().empty() ? EVAL_REAL : atof(src.fld->def().c_str());	break;
 	case TFld::Boolean:
 	    val.val_b = src.fld->def().empty() ? EVAL_BOOL : atoi(src.fld->def().c_str());	break;
+	default: break;
     }
 
     mCfg = false;
@@ -464,7 +465,7 @@ AutoHD<TVArchive> TVal::arch()	{ return mArch; }
 
 void TVal::setArch( const AutoHD<TVArchive> &vl )	{ mArch = vl; }
 
-string TVal::getSEL( long long *tm, bool sys )
+string TVal::getSEL( int64_t *tm, bool sys )
 {
     if(!(fld().flg()&TFld::Selected))	throw TError("Val",_("Not select type!"));
     switch(fld().type())
@@ -473,11 +474,25 @@ string TVal::getSEL( long long *tm, bool sys )
 	case TFld::Integer:	return fld().selVl2Nm(getI(tm,sys));
 	case TFld::Real:	return fld().selVl2Nm(getR(tm,sys));
 	case TFld::Boolean:	return fld().selVl2Nm(getB(tm,sys));
+	default: break;
     }
     return EVAL_STR;
 }
 
-string TVal::getS( long long *tm, bool sys )
+TVariant TVal::get( int64_t *tm, bool sys )
+{
+    switch(fld().type())
+    {
+	case TFld::Integer:	return getI(tm,sys);
+	case TFld::Real:	return getR(tm,sys);
+	case TFld::Boolean:	return getB(tm,sys);
+	case TFld::String:	return getS(tm,sys);
+	default: break;
+    }
+    return EVAL_STR;
+}
+
+string TVal::getS( int64_t *tm, bool sys )
 {
     switch(fld().type())
     {
@@ -490,23 +505,24 @@ string TVal::getS( long long *tm, bool sys )
 	case TFld::String:
 	    setReqFlg(true);
 	    //> Get from archive
-	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() )
-		return mArch.at().getS(tm);
+	    if(tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period())
+		return mArch.at().getVal(tm).getS();
 	    //> Get value from config
-	    if( mCfg )
+	    if(mCfg)
 	    {
 		if(tm) *tm = TSYS::curTime();
-		return src.cfg->getS( );
+		return src.cfg->getS();
 	    }
 	    //> Get current value
-	    if( fld().flg()&TVal::DirRead && !sys )	owner().vlGet( *this );
-	    if( tm ) *tm = time();
+	    if(fld().flg()&TVal::DirRead && !sys) owner().vlGet(*this);
+	    if(tm) *tm = time();
 	    return val.val_s->getVal();
+	default: break;
     }
     return EVAL_STR;
 }
 
-int TVal::getI( long long *tm, bool sys )
+int TVal::getI( int64_t *tm, bool sys )
 {
     switch(fld().type())
     {
@@ -519,23 +535,24 @@ int TVal::getI( long long *tm, bool sys )
 	case TFld::Integer:
 	    setReqFlg(true);
 	    //> Get from archive
-	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() ) 
-		return mArch.at().getI(tm);
+	    if(tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period())
+		return mArch.at().getVal(tm).getI();
 	    //> Get value from config
-	    if( mCfg )
+	    if(mCfg)
 	    {
 		if(tm) *tm = TSYS::curTime();
-		return src.cfg->getI( );
+		return src.cfg->getI();
 	    }
 	    //> Get current value
-	    if( fld().flg()&TVal::DirRead && !sys )	owner().vlGet( *this );
-	    if( tm ) *tm = time();
+	    if(fld().flg()&TVal::DirRead && !sys) owner().vlGet(*this);
+	    if(tm) *tm = time();
 	    return val.val_i;
+	default: break;
     }
     return EVAL_INT;
 }
 
-double TVal::getR( long long *tm, bool sys )
+double TVal::getR( int64_t *tm, bool sys )
 {
     switch(fld().type())
     {
@@ -548,23 +565,24 @@ double TVal::getR( long long *tm, bool sys )
 	case TFld::Real:
 	    setReqFlg(true);
 	    //> Get from archive
-	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() )
-		return mArch.at().getR(tm);
+	    if(tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period())
+		return mArch.at().getVal(tm).getR();
 	    //> Get value from config
-	    if( mCfg )
+	    if(mCfg)
 	    {
 		if(tm) *tm = TSYS::curTime();
-		return src.cfg->getR( );
+		return src.cfg->getR();
 	    }
 	    //> Get current value
-	    if( fld().flg()&TVal::DirRead && !sys )	owner().vlGet( *this );
-	    if( tm ) *tm = time();
+	    if(fld().flg()&TVal::DirRead && !sys) owner().vlGet(*this);
+	    if(tm) *tm = time();
 	    return val.val_r;
+	default: break;
     }
     return EVAL_REAL;
 }
 
-char TVal::getB( long long *tm, bool sys )
+char TVal::getB( int64_t *tm, bool sys )
 {
     switch(fld().type())
     {
@@ -577,23 +595,24 @@ char TVal::getB( long long *tm, bool sys )
 	case TFld::Boolean:
 	    setReqFlg(true);
 	    //> Get from archive
-	    if( tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period() )
-		return mArch.at().getB(tm);
+	    if(tm && (*tm) && !mArch.freeStat() && *tm/mArch.at().period() < time()/mArch.at().period())
+		return mArch.at().getVal(tm).getB();
 	    //> Get value from config
-	    if( mCfg )
+	    if(mCfg)
 	    {
 		if(tm) *tm = TSYS::curTime();
-		return src.cfg->getB( );
+		return src.cfg->getB();
 	    }
 	    //> Get current value
-	    if( fld().flg()&TVal::DirRead && !sys )	owner().vlGet( *this );
-	    if( tm ) *tm = time();
+	    if(fld().flg()&TVal::DirRead && !sys) owner().vlGet(*this);
+	    if(tm) *tm = time();
 	    return val.val_b;
+	default: break;
     }
     return EVAL_BOOL;
 }
 
-void TVal::setSEL( const string &value, long long tm, bool sys )
+void TVal::setSEL( const string &value, int64_t tm, bool sys )
 {
     if( !(fld().flg()&TFld::Selected) )	throw TError("Val",_("Not select type!"));
     switch( fld().type() )
@@ -602,10 +621,23 @@ void TVal::setSEL( const string &value, long long tm, bool sys )
 	case TFld::Integer:	setI(fld().selNm2VlI(value),tm,sys);	break;
 	case TFld::Real:	setR(fld().selNm2VlR(value),tm,sys);	break;
 	case TFld::Boolean:	setB(fld().selNm2VlB(value),tm,sys);	break;
+	default: break;
     }
 }
 
-void TVal::setS( const string &value, long long tm, bool sys )
+void TVal::set( const TVariant &value, int64_t tm, bool sys )
+{
+    switch(fld().type())
+    {
+	case TFld::Integer:	setI(value.getI(), tm, sys);	break;
+	case TFld::Real:	setR(value.getR(), tm, sys);	break;
+	case TFld::Boolean:	setB(value.getB(), tm, sys);	break;
+	case TFld::String:	setS(value.getS(), tm, sys);	break;
+	default: break;
+    }
+}
+
+void TVal::setS( const string &value, int64_t tm, bool sys )
 {
     switch( fld().type() )
     {
@@ -616,6 +648,7 @@ void TVal::setS( const string &value, long long tm, bool sys )
 	case TFld::Boolean:
 	    setB( (value!=EVAL_STR) ? (bool)atoi(value.c_str()) : EVAL_BOOL, tm, sys );	break;
 	case TFld::String:
+	{
 	    //> Set value to config
 	    if( mCfg )	{ src.cfg->setS( value ); return; }
 	    //> Check to write
@@ -629,10 +662,13 @@ void TVal::setS( const string &value, long long tm, bool sys )
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
 		try{ mArch.at().setS(value,time()); }
 		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
+	    break;
+	}
+	default: break;
     }
 }
 
-void TVal::setI( int value, long long tm, bool sys )
+void TVal::setI( int value, int64_t tm, bool sys )
 {
     switch( fld().type() )
     {
@@ -659,11 +695,13 @@ void TVal::setI( int value, long long tm, bool sys )
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
 		try{ mArch.at().setI(value,time()); }
 		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
+	    break;
 	}
+	default: break;
     }
 }
 
-void TVal::setR( double value, long long tm, bool sys )
+void TVal::setR( double value, int64_t tm, bool sys )
 {
     switch( fld().type() )
     {
@@ -690,11 +728,13 @@ void TVal::setR( double value, long long tm, bool sys )
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
 		try{ mArch.at().setR(value,time()); }
 		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
+	    break;
 	}
+	default: break;
     }
 }
 
-void TVal::setB( char value, long long tm, bool sys )
+void TVal::setB( char value, int64_t tm, bool sys )
 {
     switch( fld().type() )
     {
@@ -719,7 +759,9 @@ void TVal::setB( char value, long long tm, bool sys )
 	    if( !mArch.freeStat() && mArch.at().srcMode() == TVArchive::PassiveAttr )
 		try{ mArch.at().setB(value,time()); }
 		catch(TError err){ mess_err(nodePath().c_str(),_("Write value to archive error: %s"),err.mess.c_str()); }
+	    break;
 	}
+	default: break;
     }
 }
 
@@ -733,8 +775,8 @@ TVariant TVal::objFuncCall( const string &iid, vector<TVariant> &prms, const str
 	try
 	{
 	    TVariant rez;
-	    long long tm = 0;
-	    if(prms.size() >= 1) tm = (long long)prms[0].getI()*1000000;
+	    int64_t tm = 0;
+	    if(prms.size() >= 1) tm = (int64_t)prms[0].getI()*1000000;
 	    if(prms.size() >= 2) tm += prms[1].getI();
 	    bool isSys = false;
 	    if(prms.size() >= 3) isSys = prms[2].getB();
@@ -744,6 +786,7 @@ TVariant TVal::objFuncCall( const string &iid, vector<TVariant> &prms, const str
 		case TFld::Integer:	rez = getI(&tm,isSys);	break;
 		case TFld::Real:	rez = getR(&tm,isSys);	break;
 		case TFld::String:	rez = getS(&tm,isSys);	break;
+		default: break;
 	    }
 	    if(prms.size() >= 1)	{ prms[0].setI(tm/1000000); prms[0].setModify(); }
 	    if(prms.size() >= 2)	{ prms[1].setI(tm%1000000); prms[1].setModify(); }
@@ -760,8 +803,8 @@ TVariant TVal::objFuncCall( const string &iid, vector<TVariant> &prms, const str
     {
 	try
 	{
-	    long long tm = 0;
-	    if(prms.size() >= 2) tm = (long long)prms[1].getI()*1000000;
+	    int64_t tm = 0;
+	    if(prms.size() >= 2) tm = (int64_t)prms[1].getI()*1000000;
 	    if(prms.size() >= 3) tm += prms[2].getI();
 	    bool isSys = false;
 	    if(prms.size() >= 4) isSys = prms[3].getB();
@@ -771,11 +814,19 @@ TVariant TVal::objFuncCall( const string &iid, vector<TVariant> &prms, const str
 		case TFld::Integer:	setI(prms[0].getI(),tm,isSys);	break;
 		case TFld::Real:	setR(prms[0].getR(),tm,isSys);	break;
 		case TFld::String:	setS(prms[0].getS(),tm,isSys);	break;
+		default: break;
 	    }
 	    return false;
 	}catch(...){ }
 	return true;
     }
+    // TCntrNodeObj arch() - get current archive object for value
+    if(iid == "arch")
+    {
+	AutoHD<TVArchive> aobj = arch();
+	if(aobj.freeStat()) return false;
+	return new TCntrNodeObj(aobj, user);
+    }
     // string descr() - get attribute description
     if(iid == "descr")	return fld().descr();
     // int time(int utm) - get last attribute's value time
@@ -821,7 +872,7 @@ void TVal::cntrCmdProc( XMLNode *opt )
 	{
 	    if(!atoll(opt->attr("tm_grnd").c_str()) && opt->attr("arch").empty())
 	    {
-		long long tm = atoll(opt->attr("tm").c_str());
+		int64_t tm = atoll(opt->attr("tm").c_str());
 		opt->setText(getS(&tm));
 		opt->setAttr("tm",TSYS::ll2str(tm));
 	    }
