@@ -55,9 +55,9 @@ TVariant::~TVariant( )				{ setType(TVariant::Null); }
 
 void TVariant::setType( Type tp )
 {
-    if( tp == type() )	return;
+    if(tp == type()) return;
 
-    if( type() == TVariant::Object && getO() && !getO()->disconnect() ) delete getO( );
+    if(type() == TVariant::Object && getO() && !getO()->disconnect()) delete getO();
 
     switch(tp)
     {
@@ -78,6 +78,7 @@ void TVariant::setType( Type tp )
 	    break;
 	case TVariant::Object:
 	    vl.assign(1,(char)TVariant::Object);
+	    vl.reserve(1+sizeof(void*));
 	    break;
 	default: break;
     }
@@ -88,7 +89,9 @@ void TVariant::setModify( bool iv )
     vl[0] = iv ? vl[0]|0x80 : vl[0]&(~0x80);
 }
 
-bool TVariant::operator==( TVariant &vr )	{ return vr.vl == vl; }
+bool TVariant::operator==( const TVariant &vr )	{ return vr.vl == vl; }
+
+bool TVariant::operator!=( const TVariant &vr )	{ return vr.vl != vl; }
 
 TVariant &TVariant::operator=( const TVariant &vr )
 {
@@ -98,6 +101,20 @@ TVariant &TVariant::operator=( const TVariant &vr )
     return *this;
 }
 
+bool TVariant::isEVal( ) const
+{
+    switch(type())
+    {
+	case TVariant::String:	return (getS()==EVAL_STR);
+	case TVariant::Integer:	return (getI()==EVAL_INT);
+	case TVariant::Real:	return (getR()==EVAL_REAL);
+	case TVariant::Boolean:	return (getB()==EVAL_BOOL);;
+	case TVariant::Object:	return !getO();
+	default: break;
+    }
+    return true;
+}
+
 char TVariant::getB( ) const
 {
     switch(type())
@@ -116,7 +133,7 @@ int TVariant::getI( ) const
 {
     switch( type() )
     {
-	case TVariant::String:	return (getS()==EVAL_STR) ? EVAL_INT: atoi(getS().c_str());
+	case TVariant::String:	return (getS()==EVAL_STR) ? EVAL_INT : atoi(getS().c_str());
 	case TVariant::Integer:	return TSYS::getUnalignInt(vl.data()+1);
 	case TVariant::Real:	return (getR()==EVAL_REAL) ? EVAL_INT : (int)getR();
 	case TVariant::Boolean:	return (getB()==EVAL_BOOL) ? EVAL_INT : getB();
@@ -427,17 +444,26 @@ bool TArrayObj::compareLess( const TVariant &v1, const TVariant &v2 )
 //* TRegExp                                                 *
 //*   Regular expression object                             *
 //***********************************************************
-TRegExp::TRegExp( const string &rule, const string &flg ) : lastIndex(0), pattern(rule), regex(NULL), vSz(90), capv(NULL)
+TRegExp::TRegExp( const string &rule, const string &flg ) :
+    lastIndex(0), pattern(rule), global(false), ignoreCase(false), multiline(false), isSimplePat(false), regex(NULL), vSz(90), capv(NULL)
 {
     global = (flg.find('g')!=string::npos);
     ignoreCase = (flg.find('i')!=string::npos);
     multiline = (flg.find('m')!=string::npos);
+    if(flg.find('p') != string::npos)
+    {
+	isSimplePat = !(rule.size() > 2 && rule[0] == '/' && rule[rule.size()-1] == '/');
+	if(!isSimplePat) pattern = rule.substr(1,rule.size()-2);
+    }
 
-    const char *terr;
-    int erroff;
-    regex = pcre_compile(pattern.c_str(),PCRE_DOTALL|(ignoreCase?PCRE_CASELESS:0)|(multiline?PCRE_MULTILINE:0),&terr,&erroff,NULL);
-    if(!regex) err = terr;
-    else capv = new int[90];
+    if(!isSimplePat && pattern.size())
+    {
+	const char *terr;
+	int erroff;
+	regex = pcre_compile(pattern.c_str(),PCRE_DOTALL|(ignoreCase?PCRE_CASELESS:0)|(multiline?PCRE_MULTILINE:0),&terr,&erroff,NULL);
+	if(!regex) err = terr;
+	else capv = new int[90];
+    }
 }
 
 TRegExp::~TRegExp( )
@@ -469,10 +495,10 @@ string TRegExp::replace( const string &vl, const string &str )
 {
     string rez = vl, repl;
     if(!regex) return rez;
-    for(int curPos = 0, n; (!curPos || global) && (n=pcre_exec(regex,NULL,vl.data(),vl.size(),curPos,0,capv,vSz)) > 0; curPos = capv[0]+repl.size())
+    for(int curPos = 0, n; (!curPos || global) && (n=pcre_exec(regex,NULL,rez.data(),rez.size(),curPos,0,capv,vSz)) > 0; curPos = capv[0]+repl.size())
     {
-	repl = substExprRepl(str,vl,capv,n);
-	rez.replace(capv[0],(capv[1]-capv[0]),repl);
+	repl = substExprRepl(str,rez,capv,n);
+	rez.replace(capv[0],capv[1]-capv[0],repl);
     }
     return rez;
 }
@@ -494,6 +520,38 @@ TArrayObj *TRegExp::split( const string &vl, int limit )
 
 bool TRegExp::test( const string &vl )
 {
+    //> Check by simple pattern
+    if(isSimplePat)
+    {
+	bool mult_s = false;
+	int v_cnt = 0, p_cnt = 0;
+	int v_bck = -1, p_bck = -1;
+
+	while(true)
+        {
+            if(p_cnt >= (int)pattern.size() ) return true;
+            if(pattern[p_cnt] == '?')	{ v_cnt++; p_cnt++; mult_s = false; continue; }
+            if(pattern[p_cnt] == '*')	{ p_cnt++; mult_s = true; v_bck = -1; continue; }
+            if(pattern[p_cnt] == '\\')	p_cnt++;
+            if(v_cnt >= (int)vl.size())  break;
+            if(pattern[p_cnt] == vl[v_cnt])
+            {
+                if(mult_s && v_bck < 0 )	{ v_bck = v_cnt+1; p_bck = p_cnt; }
+                v_cnt++; p_cnt++;
+            }
+            else
+            {
+                if(mult_s)
+        	{
+                    if(v_bck >= 0) { v_cnt = v_bck; p_cnt = p_bck; v_bck = -1; }
+                    else v_cnt++;
+                }
+                else break;
+            }
+        }
+        return false;
+    }
+    //> Check by regular expression
     if(!regex) return false;
     int n = pcre_exec(regex, NULL, vl.data(), vl.size(), (global?lastIndex:0), 0, capv, vSz);
     if(global) lastIndex = (n>0) ? capv[1] : 0;
@@ -586,6 +644,7 @@ XMLNodeObj::~XMLNodeObj( )
 
 void XMLNodeObj::childAdd(XMLNodeObj *nd)
 {
+    if(nd == this) return;
     mChilds.push_back(nd);
     nd->parent = this;
     nd->connect();
@@ -593,6 +652,7 @@ void XMLNodeObj::childAdd(XMLNodeObj *nd)
 
 void XMLNodeObj::childIns(unsigned id, XMLNodeObj *nd)
 {
+    if(nd == this) return;
     if(id < 0) id = mChilds.size();
     id = vmin(id,mChilds.size());
     mChilds.insert(mChilds.begin()+id,nd);
@@ -604,6 +664,7 @@ void XMLNodeObj::childDel( unsigned id )
 {
     if(id < 0 || id >= mChilds.size()) throw TError("XMLNodeObj",_("Deletion child '%d' error."),id);
     if(!mChilds[id]->disconnect()) delete mChilds[id];
+    else if(mChilds[id]->parent == this) mChilds[id]->parent = NULL;
     mChilds.erase(mChilds.begin()+id);
 }
 
@@ -613,7 +674,18 @@ XMLNodeObj *XMLNodeObj::childGet(unsigned id)
     return mChilds[id];
 }
 
-string XMLNodeObj::getStrXML(const string &oid)	{ return ""; }
+string XMLNodeObj::getStrXML(const string &oid)
+{
+    string nd("<XMLNodeObj:"+name());
+    for(map<string,TVariant>::iterator ip = mProps.begin(); ip != mProps.end(); ip++)
+	nd += " "+ip->first+"=\""+TSYS::strEncode(ip->second.getS(),TSYS::Html)+"\"";
+    nd += ">"+TSYS::strEncode(text(),TSYS::Html)+"\n";
+    for(unsigned i_ch = 0; i_ch < mChilds.size(); i_ch++)
+	nd += mChilds[i_ch]->getStrXML();
+    nd += "</XMLNodeObj:"+name()+">\n";
+
+    return nd;
+}
 
 TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
 {
@@ -623,7 +695,12 @@ TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
     if(id == "text")	return text();
     // string attr(string id) - get node attribute
     //  id - attribute identifier
-    if(id == "attr" && prms.size())	return propGet(prms[0].getS()).getS();
+    if(id == "attr" && prms.size())
+    {
+	TVariant avl = propGet(prms[0].getS());
+	if(avl.isEVal() || avl.isNull()) return "";
+	return avl;
+    }
     // XMLNodeObj setName(string vl) - set node name
     //  vl - value for node name
     if(id == "setName" && prms.size())	{ setName(prms[0].getS()); return this; }
@@ -637,6 +714,7 @@ TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
     // int childSize( ) - return childs counter for node
     if(id == "childSize")	return (int)childSize();
     // XMLNodeObj childAdd(ElTp no = XMLNodeObj) - add node <no> as child to the node
+    // XMLNodeObj childAdd(string nd) - add node with name <nd>
     //  no - node object or name for new node
     if(id == "childAdd")
     {
@@ -649,6 +727,7 @@ TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
 	return no;
     }
     // XMLNodeObj childIns(int id, ElTp no = XMLNodeObj) - insert node <no> as child to the node
+    // XMLNodeObj childIns(int id, string nd) - insert node with name <nd>
     //  id - insert position
     //  no - node object or name for new node
     if(id == "childIns" && prms.size())
@@ -669,18 +748,20 @@ TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
     if(id == "childGet" && prms.size())	return childGet(prms[0].getI());
     // XMLNodeObj parent() - get parent node
     if(id == "parent")	return parent ? TVariant(parent) : TVariant(false);
-    // string load(string str, bool file = false) - load XML tree from XML-stream from string or file
-    //  str - source stream string or file name, for <file> = true
-    //  file - load XML-tree from file (true) or stram (false)
+    // string load(string str, bool file = false, bool full = false, string cp = "UTF-8") - load XML tree from XML-stream from string or file
+    //  str - source stream string or file name, for <file> = true;
+    //  file - load XML-tree from file (true) or stram (false);
+    //  full - node's text and comments load into separated nodes "<*>" and "<!>";
+    //  cp - source codepage.
     if(id == "load" && prms.size())
     {
 	XMLNode nd;
 	//> Load from file
-	if( prms.size() >= 2 && prms[1].getB() )
+	if(prms.size() >= 2 && prms[1].getB())
 	{
 	    string s_buf;
 	    int hd = open(prms[0].getS().c_str(),O_RDONLY);
-	    if( hd < 0 ) return TSYS::strMess(_("2:Open file <%s> error: %s"),prms[0].getS().c_str(),strerror(errno));
+	    if(hd < 0) return TSYS::strMess(_("2:Open file '%s' error: %s"),prms[0].getS().c_str(),strerror(errno));
 	    int cf_sz = lseek(hd,0,SEEK_END);
 	    if(cf_sz > 0)
 	    {
@@ -693,17 +774,17 @@ TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
 	    }
 	    close(hd);
 
-	    try{ nd.load(s_buf); }
-	    catch( TError err ) { return "1:"+err.mess; }
+	    try{ nd.load(s_buf, ((prms.size()>=3)?prms[2].getB():false), ((prms.size()>=4)?prms[3].getS():string("UTF-8"))); }
+	    catch(TError err) { return "1:"+err.mess; }
 	}
 	//> Load from string
 	else
-	    try{ nd.load(prms[0].getS()); }
-	    catch( TError err ) { return "1:"+err.mess; }
+	    try{ nd.load(prms[0].getS(), ((prms.size()>=3)?prms[2].getB():false)); }
+	    catch(TError err) { return "1:"+err.mess; }
 	fromXMLNode(nd);
 	return string("0");
     }
-    // string save(int opt = 0, string path = "") - save XML-tree to string or file stream
+    // string save(int opt = 0, string path = "", string cp = "UTF-8") - save XML-tree to string or file stream
     //  opt - save options:
     //   0x01 - interrupt the string before the opening tag;
     //   0x02 - interrupt the string after the opening tag;
@@ -712,11 +793,12 @@ TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
     //   0x10 - interrupt the string after the instruction;
     //   0x1E - interrupt the string after all.
     //  path - file path for save to file
-    if( id == "save" )
+    //  cp - target codepage
+    if(id == "save")
     {
 	XMLNode nd;
 	toXMLNode(nd);
-	string s_buf = nd.save( (prms.size()>=1)?prms[0].getI():0 );
+	string s_buf = nd.save(((prms.size()>=1)?prms[0].getI():0), (prms.size()>=3)?prms[2].getS():string("UTF-8"));
 	//> Save to file
 	if( prms.size() >= 2 )
 	{
@@ -726,6 +808,15 @@ TVariant XMLNodeObj::funcCall(const string &id, vector<TVariant> &prms)
 	}
 	return s_buf;
     }
+    // XMLNodeObj getElementBy( string val, string attr = "id" ) - get element from the tree by attribute <attr> value <val>.
+    //  val - attribute value for find;
+    //  attr - attribute name for find it value.
+    if(id == "getElementBy" && prms.size())
+    {
+	XMLNodeObj *rez = getElementBy(((prms.size() >= 2) ? prms[1].getS() : "id"), prms[0].getS());
+	if(!rez) return TVariant();
+	return rez;
+    }
 
     throw TError("XMLNodeObj",_("Function '%s' error or not enough parameters."),id.c_str());
 }
@@ -760,6 +851,17 @@ void XMLNodeObj::fromXMLNode(XMLNode &nd)
     }
 }
 
+XMLNodeObj *XMLNodeObj::getElementBy( const string &attr, const string &val )
+{
+    if(propGet(attr).getS() == val)	return this;
+
+    XMLNodeObj *rez = NULL;
+    for(unsigned i_ch = 0; !rez && i_ch < childSize(); i_ch++)
+	rez = childGet(i_ch)->getElementBy(attr,val);
+
+    return rez;
+}
+
 //***********************************************************
 //* TCntrNodeObj                                            *
 //*   TCntrNode object for access to system's objects       *
